﻿#Requires -Version 2
$ErrorActionPreference = "Stop";
Set-PSDebug -Strict;
$Error.Clear();

function Expand-String([string] $source)
{
    $source = (($source -replace '&quot;', '`"') -replace '''', '`''');
    if ($PSVersionTable.PSVersion -eq '2.0') {        
        $target = $ExecutionContext.InvokeCommand.ExpandString($source);
    } else {
        
        try {
            # If the source string contains a Powershell expression, we have to use Invoke-Expression because ExpandString is broken in PS v3.0
            if($source.StartsWith('$(') -and $source.EndsWith(')')) {
                Write-Host "Calling Invoke-Expession " $source -ForegroundColor DarkGreen;
                $target = Invoke-Expression $source;
            }
		    else {
                $target =  $ExecutionContext.InvokeCommand.ExpandString($source);
            }
        }
        catch {
            Write-Host 'Assuming $source is a plain old string -> value: ' $source -ForegroundColor DarkGreen;
            $target = $ExecutionContext.InvokeCommand.ExpandString($source);
        }        
    }    
    Write-Host '$target set to ' $target -ForegroundColor Green;
    return $target;
} 

function New-Feature()
{
  param (
  [string] $identity, 
  [bool] $deleteIfExists,
  [bool] $reactivate,
  [bool] $activateUsingHttpGet
  )

  $feature = new-object PSObject

  $feature | add-member -type NoteProperty -Name identity -Value $identity
  $feature | add-member -type NoteProperty -Name deleteIfExists -Value $deleteIfExists
  $feature | add-member -type NoteProperty -Name reactivate -Value $reactivate
  $feature | add-member -type NoteProperty -Name activateUsingHttpGet -Value $activateUsingHttpGet
  
  return $feature
}

$DEPLOYMENT_DIRECTORY=(Split-Path -Path $MyInvocation.MyCommand.Definition -Parent);

. ( Join-Path -Path $DEPLOYMENT_DIRECTORY -ChildPath "UtilityScripts\CommerceServer-Configure.ps1");
. ( Join-Path -Path $DEPLOYMENT_DIRECTORY -ChildPath "UtilityScripts\CommerceServer_Profiles.ps1");
. ( Join-Path -Path $DEPLOYMENT_DIRECTORY -ChildPath "UtilityScripts\CommerceServer_Catalog.ps1");
. ( Join-Path -Path $DEPLOYMENT_DIRECTORY -ChildPath "UtilityScripts\CommerceServer_MSCSAdmin.ps1");
. ( Join-Path -Path $DEPLOYMENT_DIRECTORY -ChildPath "UtilityScripts\IIS.ps1");
. ( Join-Path -Path $DEPLOYMENT_DIRECTORY -ChildPath "UtilityScripts\Windows.ps1");
. ( Join-Path -Path $DEPLOYMENT_DIRECTORY -ChildPath "UtilityScripts\File.ps1");
. ( Join-Path -Path $DEPLOYMENT_DIRECTORY -ChildPath "UtilityScripts\AZMan.ps1");
. ( Join-Path -Path $DEPLOYMENT_DIRECTORY -ChildPath "UtilityScripts\SQL.ps1");

<#
.Synopsis
	Using the Environment.xml file, create a Commerce Server site
.Description
	
.Example
	Deploy-CommerceServer -ConfigurationIdentity "Domain.Dev.Unpup" -PreCompile;
	Runs the Domain.Dev.Unpup configuration and all actions that normally happen under preCompile
.Example
	Deploy-CommerceServer -ConfigurationIdentities @("Domain.Dev.Unpup","Domain.Dev")
	Runs the Domain.Dev.Unpup and Domain.Dev configurations
.Parameter Precompile
	Runs precompile for the solution.
.Parameter Postcompile
	Runs postcompile for the solution.	
.Parameter ConfigurationIdentities
	An array of strings that coorespond to one or more of the identity attributes of the <Configurations><Configuration> node of the Environment.xml.
#>
function Deploy-CommerceServer
(
	[switch]$Precompile = $false,
	[switch]$Postcompile = $false,
	[switch]$GenerateWSP = $false,
	[switch]$Debug=$false,
	[switch]$SP2013=$false,
	[string]$ConfigurationIdentity="Domain.Dev",
	$DeploymentIdentities=@()
)
{
	# if COMMERCE_SERVER_ROOT is not defined we should not do anything
	if(((Test-Path Env:\COMMERCE_SERVER_ROOT) -eq $false) -or ($($env:COMMERCE_SERVER_ROOT) -eq $null) -or ($($env:COMMERCE_SERVER_ROOT) -eq "")){

		Write-Host "The COMMERCE_SERVER_ROOT environmental variable has not been definied, please ensure Commerce Server is installed and the variable is registed. If it is installed try closing and re-opening all Powershell and Command windows, if this does not work you may need to reboot this machine." -ForegroundColor Red;
		exit 1;
	}

	Clear-Host;
	
	$scriptStartDate = Get-Date;
	$transactionLogPath = ( Join-Path -Path $DEPLOYMENT_DIRECTORY -ChildPath "Deploy-CommerceServer.log" );
	$success = "FAILED";

	try
	{
        try {
		    Start-Transcript -Force -Path $transactionLogPath -Debug -Confirm:$false -ErrorAction SilentlyContinue;
        }
        catch { 
            # to allow hosts which do not support transcription, you must catch terminating errors ... -ErrorAction SilentlyContinue doesn't work
        }

		Write-Host "Script started @ $($scriptStartDate).";

		Clear-EventLog -LogName "Application";
		Clear-EventLog -LogName "System";
		Clear-EventLog -LogName "Windows PowerShell";

		Write-Host "Restarting IIS";
		#IISRESET
		
		$environmentXml = [xml]( Get-Content -Path ( Join-Path -Path $DEPLOYMENT_DIRECTORY -ChildPath "Environment.xml" ) );

		$configuration = $environmentXml.CommerceServer.Configurations.Configuration | where { $_.identity -ieq $ConfigurationIdentity };
		if( $configuration )
		{
			Write-Host "Using configuration $($configuration.identity)";

			Write-Host "Loading variables";
			
			$variables = $configuration.SelectNodes("Variables/Variable");

			foreach( $variable in $variables )
			{
                $variableValue = Expand-String $variable.value

                Write-Host "$($variable.identity) =" $variableValue -ForegroundColor Green;			
				Set-Variable -Name $variable.identity -Value $variableValue -Scope Global;
			}
            foreach( $node in $configuration.SelectNodes( ".//*" ) )
			{
				foreach( $attribute in $node.Attributes )
				{
                    # The act of expanding the string here will have side-effects of evaluating $($variable)) and $env:variables.
					$attribute.Value = Expand-String $attribute.Value
				}
			}
		}
		else
		{
			Write-Error "No configuration with the identity $($ConfigurationIdentity) was found in the Environment.xml file.";
		}

		#Get-Variable | ft Name, Value -auto
		Write-Host "Writing complete variable list..." -ForegroundColor Green;
		Get-Variable | out-default;

		if( $Precompile )
		{
            $iisVersion = Get-ItemProperty "HKLM:\software\microsoft\InetStp";
            $iis8 = $false;
            # Windows Server 2008
            if ($iisVersion.MajorVersion -eq 7 -and $iisVersion.MinorVersion -lt 5 -and (-not (Get-PSSnapIn | Where {$_.Name -eq "WebAdministration";})) ) {
                Add-PSSnapIn WebAdministration;                
            }
            else # Windows Server 2008 R2 (IIS 7.5) and Windows 8 (IIS 8)
            {
                Import-Module WebAdministration;
                $iis8 = $true;
            }
			
			Stop-Website -Name "Default Web Site";
			
			if($configuration.UserGroups -and $configuration.UserGroups.UserGroup){
			
				Write-Host "Adding users to groups..." -ForegroundColor Green;
				
				$userGroups = $configuration.UserGroups.SelectNodes("UserGroup");
				
				foreach( $userGroup in $userGroups )
				{
					if($userGroup.Users.User){
						$users = $userGroup.Users.SelectNodes("User");
						
						foreach( $user in $users )
						{						
							$actualUser = $configuration.UserAccounts.UserAccount | where { $_.identity -ieq $($user.userAccountIdentity) }
							Write-Host "`$user.userAccountIdentity = $user.userAccountIdentity";
							Windows-AddUserToLocalGroup -LocalGroupName $userGroup.name -UserName "$($actualUser.domain)\$($actualUser.username)" -Password $actualUser.password;
						}
					}
				}
			}	
			
			if($configuration.Events -and $configuration.Events.Definition){
			
				Write-Host "Registering new events for the event log..." -ForegroundColor Green;
				
				$events = $configuration.Events.SelectNodes("Definition");
				
				foreach( $event in $events )
				{
					Add-NewEventLogSource $event.EventSource;
				}
			}			
			
            if($configuration.CommerceServer.Pup) {
			    $pupLocation = $configuration.CommerceServer.Pup.pupLocation;
                $iniLocation = $configuration.CommerceServer.Pup.iniLocation;
					
				Write-Host "Deleting existing site before unpuping" -ForegroundColor Green;
				$returnValue = Start-Process -FilePath  (Join-Path -Path $DEPLOYMENT_DIRECTORY -ChildPath "\tools\deletesite.exe") -ArgumentList @("$($PROJECT_NAME)") -NoNewWindow -Wait -PassThru;
				
                if( $returnValue.ExitCode -ne 0 )
				{
				    Write-Host "Program cleaning up existing site." -ForegroundColor Red;
					Write-Host "Program Exit Code was $($returnValue.ExitCode), aborting.`r$($returnValue.StandardError)" -ForegroundColor Red;
                }
            }
			
            if($configuration.IIS){
				# IIS Sites
				if( $configuration.IIS.Websites -and $configuration.IIS.Websites.Website ){
					$websites = $configuration.IIS.SelectNodes("Websites/Website");
					
					Write-Host "Creating sites..." -ForegroundColor Green;
					foreach( $website in $websites )
					{
						Write-Host "Working on $($website.identity) site..." -ForegroundColor Yellow;
						$siteInstance = Get-WebSite | where { $_.Name -eq $($website.identity) };

						if($siteInstance -ne $null){
							if($website.deleteIfExists -eq "true"){
								Write-Host "Site $($website.identity) exists, deleting...";

								Remove-Item "IIS:\Sites\$($website.identity)" -recurse;

								$siteInstance = $null;
							}
						}
						else{
							Write-Host "Site $($website.identity) does not exist.";
						}

						if($siteInstance -eq $null)
						{
							if($website.create -eq "true"){
								Write-Host "Site $($website.identity) does not exist, creating...";
                                
                                $physicalPath = $website.path;
                                Write-Host "Physical Path: " $physicalPath -ForegroundColor Green;
                                $siteIdentity = $website.identity;
                                Write-Host "Site identity: " $siteIdentity -ForegroundColor Green;
                                $protocol = $($website.protocol);
                                # Null or empty = http
                                $protocol = @{$true=$protocol;$false="http"}[$protocol -ne $null -and $protocol -ne ""];                               
                                Write-Host "Protocol: " $protocol;
                                $ssl = @{$true=$true;$false=$false}[$protocol -eq "https"];
                                Write-Host "SSL: " $ssl;
                                $port = $website.port;
                                Write-Host "Port: " $port;
                                
                                # On Windows 8, there is no protocol property on the website object.
                                if($iis8) {
                                    # must create the -PhysicalPath if it does not exist
                                    if(-not (Test-Path -Path $physicalPath)) {
                                        New-Item -Path $physicalPath -ItemType Directory;
                                    }

                                    if($ssl) {
                                        $siteInstance = New-Website $siteIdentity -PhysicalPath $physicalPath -Ssl -Port $port;
                                        #New-WebBinding -Name $siteIdentity -Port 443 -Protocol https;
                                    } else {
                                        $siteInstance = New-Website $siteIdentity -PhysicalPath $physicalPath -Port $port;
                                        #New-WebBinding -Name $siteIdentity -Port $port -Protocol http;
                                    }
                                                                        
                                }
                                else {
								    $siteInstance = New-Item IIS:\Sites\CSServices -PhysicalPath c:\inetpub\csservices -bindings @{protocol="$($website.protocol)";bindingInformation="*:$($website.port):"};
                                }
							
                                Assign-AppPoolToSite $configuration $website.applicationPool $siteIdentity;
							}
							else{
								Write-Host "Will not create $($website.identity).";
							}
						}
						
						if($siteInstance -and $website.VirtualDirectories -and $website.VirtualDirectories.VirtualDirectory){
							$vds = $website.SelectNodes("VirtualDirectories/VirtualDirectory");
							
							foreach($vd in $vds){
								$vdInstance = Get-ChildItem "IIS:\Sites\$($website.identity)" | where { $_.Name -eq $($vd.identity) };
								
								if($vdInstance -ne $null){
									if($vd.deleteIfExists -eq "true"){
										Write-Host "Virtual Directory $($vd.identity) exists, deleting...";

										Remove-Item "IIS:\Sites\$($website.identity)\$($vd.identity)" -recurse;

										$vdInstance = $null;
									}
									else{
										Write-Host "Virtual Directory $($vd.identity) exists.";
									}
								}
								
								if($vdInstance -eq $null)
								{
									if($vd.create -eq "true"){
										Write-Host "Creating Virtual Directory $($vd.identity)";
										
										if($vd.applicationPool -ne $null){
											$vdInstance = New-Item "IIS:\Sites\$($website.identity)\$($vd.identity)" -Type Application -PhysicalPath "$($vd.path)";
										
											Assign-AppPoolToSite $configuration $vd.applicationPool "$($website.identity)\$($vd.identity)" $true;
										}
										else{
											$vdInstance = New-Item "IIS:\Sites\$($website.identity)\$($vd.identity)" -Type VirtualDirectory -PhysicalPath "$($vd.path)";
										}
									}
								}						
							}
						}
						else{
							Write-Host "$($website.identity) does not exist so cannot create virtual directories." -ForegroundColor Yellow;
						}
					}
				
					Write-Host "Done." -ForegroundColor Yellow;
				}
				
				# IIS Certificates
				if( $configuration.IIS.Certificates -and $configuration.IIS.Certificates.Certificate ){
					$certificates = $configuration.IIS.SelectNodes("Certificates/Certificate");
					
					Write-Host "Importing certificates..." -ForegroundColor Green;
					foreach( $iisCertificate in $certificates )
					{
						$cert = ( Get-ChildItem -Path "cert:\localmachine\MY" | where { $_.FriendlyName -ieq $iisCertificate.identity -or $_.Subject -ieq "CN=$($iisCertificate.identity)" });

						if( $cert -ne $null )
						{
							Write-Host "Deleting pre-existing certificate: $($iisCertificate.identity)";
							
							$store = New-Object System.Security.Cryptography.X509Certificates.X509Store "My", "localmachine"
							$store.Open("ReadWrite")
							$store.Remove($cert)
							$store.Close()
						}

						if($iisCertificate.selfSigned -and $iisCertificate.selfSigned -eq "true"){
							Create-SelfSignedCert $iisCertificate.identity;
							continue;
						}
						
						Write-Host "Importing $($iisCertificate.identity)";
						$certPath = $iisCertificate.path;
						
						Import-IIS7PfxCertificate -certPath "$certPath" -pfxPass $iisCertificate.password;
					}
				
					Write-Host "Done." -ForegroundColor Yellow;
				}

				# IIS Bindings
				if( $configuration.IIS.WebBindings -and $configuration.IIS.WebBindings.WebBinding )
				{
					Write-Host "Importing IIS bindings...";
							
					foreach( $iisWebBinding in $configuration.IIS.WebBindings.WebBinding )
					{
						Import-Module -Name WebAdministration;
																
						$existingBindingConfig = "$($iisWebBinding.ip):$($iisWebBinding.port):$($iisWebBinding.hostheader)";
						$existingBinding = Get-WebBinding | where {$_.bindingInformation -eq $existingBindingConfig -and $_.protocol -eq "$($iisWebBinding.protocol)"};							
								
						if($existingBinding -ne $null){
							Write-Host "Deleting already existing binding $($existingBindingConfig )";
							(Get-WebBinding | where {$_.bindingInformation -eq $existingBindingConfig -and $_.protocol -eq "$($iisWebBinding.protocol)"}) | Remove-WebBinding;
						}
						else{
							Write-Host "A binding with this config does not already exist: $($existingBindingConfig)";
						}
								
						Write-Host "Creating an IIS binding";
						New-WebBinding -Name $iisWebBinding.webApplicationIdentity -Port $iisWebBinding.port -HostHeader $iisWebBinding.hostheader -Protocol $iisWebBinding.protocol -IP $iisWebBinding.ip;

						Write-Host "Adding binding to hosts file";
						Windows-UpdateHostsFile 127.0.0.1 $($iisWebBinding.hostheader);
									
						if($iisWebBinding.protocol -eq "https")
						{
							Write-Host "Adding certificate to binding...";
							$commerceCert = ( Get-ChildItem -Path "cert:\localmachine\MY" | where { $_.FriendlyName -ieq $iisWebBinding.certificate -or $_.Subject -ieq "CN=$($iisWebBinding.certificate)" });
							$sslBinding = Get-WebBinding -Name $iisWebBinding.webApplicationIdentity -Port $iisWebBinding.port -HostHeader $iisWebBinding.hostheader -Protocol $iisWebBinding.protocol -IP $iisWebBinding.ip;
							$sslBinding.AddSslCertificate( $commerceCert.ThumbPrint, "MY" );
						}
					}

					Write-Host "Done." -ForegroundColor Yellow;
				}
				else
				{
					Write-Host "No SPWebApplication IIS web bindings defined in the Environment.xml file." -ForegroundColor Yellow;
				}

				if($configuration.IIS.WAMREG -and $configuration.IIS.WAMREG.UserAccounts -and $configuration.IIS.WAMREG.UserAccounts.UserAccount){
					$userAccounts = $configuration.IIS.WAMREG.UserAccounts.SelectNodes("UserAccount");				
										
					Write-Host "Setting permissions on IIS WAMREG admin service..." -ForegroundColor Green;
					foreach( $userAccount in $userAccounts )
					{
						$user = $configuration.UserAccounts.UserAccount | where { $_.identity -ieq $($userAccount.userAccountIdentity) }

						Write-Host "DCOM rights for `"IIS WAMREG admin service`" needs to be adjusted to allow RuntimeUser to activate locally.";
						Write-Host "Adding rights for $($user.domain)\$($user.username)";
						$returnValue = Start-Process -FilePath  (Join-Path -Path $DEPLOYMENT_DIRECTORY -ChildPath "\tools\dcomperm.exe") -ArgumentList @("-al '{61738644-F196-11D0-9953-00C04FD919C1}'", "set $($user.domain)\$($user.username) permit level:l") -NoNewWindow -Wait -PassThru;
						
						if( $returnValue.ExitCode -ne 0 )
						{
							Write-Error "Program Exit Code was $($returnValue.ExitCode), aborting.`r$($returnValue.StandardError)";
						}						
					}					
				}
			}

			if($configuration.CommerceServer){
				if($configuration.CommerceServer.Configure){
					$stagingUser = $configuration.UserAccounts.UserAccount | where { $_.identity -ieq $($configuration.CommerceServer.Configure.stagingUser) };
					$mailerUser = $configuration.UserAccounts.UserAccount | where { $_.identity -ieq $($configuration.CommerceServer.Configure.mailerUser) }
				
					CS2007-Configure $($CS_SQL_Server) $stagingUser.domain $stagingUser.username $stagingUser.password $mailerUser.domain $mailerUser.username $mailerUser.password
				}
			
				if($configuration.CommerceServer.Pup) {		
					
					Write-Host "Performing un-pup with $($pupLocation) and $($iniLocation)" -ForegroundColor Green;
					
					CS2007-UnpupWebServices $PROJECT_NAME $pupLocation $iniLocation;
				}
				
				if($configuration.CommerceServer.ProfileKeys -and $configuration.CommerceServer.ProfileKeys.create){
					if($configuration.CommerceServer.ProfileKeys.create -eq "true"){

						Write-Host "Creating profile keys...";
						Write-Host "Creating file..." -ForegroundColor Green;
						$returnValue = Start-Process -FilePath ( Join-Path $($Env:COMMERCE_SERVER_ROOT) -ChildPath "Tools\ProfileKeyManager.exe") -ArgumentList @( "/kn", "/o `"$($DEPLOYMENT_DIRECTORY)\ProfilesKeys.xml`"", "/f" ) -NoNewWindow -Wait -PassThru;
						
						if( $returnValue.ExitCode -ne 0 )
						{
							Write-Error "Program Exit Code was $($returnValue.ExitCode), aborting.`r$($returnValue.StandardError)";
						}						
						
						Write-Host "Setting 32bit registry..." -ForegroundColor Green;
						$returnValue = Start-Process -FilePath ( Join-Path $($Env:COMMERCE_SERVER_ROOT) -ChildPath "Tools\ProfileKeyManager.exe") -ArgumentList @( "/ke", "/kf `"$($DEPLOYMENT_DIRECTORY)\ProfilesKeys.xml`"", "/reg HKEY_LOCAL_MACHINE\SOFTWARE\CommerceServer\Encryption\Keys\$($PROJECT_NAME)", "/f" ) -NoNewWindow -Wait -PassThru;
						
						if( $returnValue.ExitCode -ne 0 )
						{
							Write-Error "Program Exit Code was $($returnValue.ExitCode), aborting.`r$($returnValue.StandardError)";
						}						
						
						Write-Host "Setting 64bit registry..." -ForegroundColor Green;
						$returnValue = Start-Process -FilePath ( Join-Path $($Env:COMMERCE_SERVER_ROOT) -ChildPath "Tools\ProfileKeyManager.exe") -ArgumentList @( "/ke", "/kf `"$($DEPLOYMENT_DIRECTORY)\ProfilesKeys.xml`"", "/reg HKEY_LOCAL_MACHINE\SOFTWARE\Wow6432Node\CommerceServer\Encryption\Keys\$($PROJECT_NAME)", "/f" ) -NoNewWindow -Wait -PassThru;
						
						if( $returnValue.ExitCode -ne 0 )
						{
							Write-Error "Program Exit Code was $($returnValue.ExitCode), aborting.`r$($returnValue.StandardError)";
						}					
					}
				}				
				
				if($configuration.CommerceServer.Azman){
					Write-Host "Updating AzMan Files" -ForegroundColor Green;
					
					$azManEntries = $configuration.CommerceServer.Azman.SelectNodes("Definition");
					
					foreach( $azman in $azManEntries )
					{
						if(Test-Path $($azman.FilePath))
						{
							Write-Host "Adding entry to $($azman.FilePath)" -ForegroundColor Green;
							
							Set-ItemProperty $($azman.FilePath) -name IsReadOnly -value $false;
							
							$azmanDomain = "";
							$azmanUser = "";
							
							if($($azman.userAccountIdentity) -eq $null -or $($azman.userAccountIdentity) -eq ""){
								$azmanDomain = $azman.UserOrGroupDomain;
								$azmanUser = $azman.UserOrGroupName;							
							}
							else{
								$azmanUser = $configuration.UserAccounts.UserAccount | where { $_.identity -ieq $($azman.userAccountIdentity) }
								
								$azmanDomain = $azmanUser.domain;
								$azmanUser = $azmanUser.username;										
							}
							
							if($($azman.ScopeName) -eq $null -or $($azman.ScopeName) -eq ""){
								Add-AZAddToRole $($azman.FilePath) $($azman.ApplicationName) $($azman.RoleName) $($azmanDomain) $($azmanUser);
							}
							else{
								Add-AZAddToRoleInScope $($azman.FilePath) $($azman.ApplicationName) $($azman.ScopeName) $($azman.RoleName) $($azmanDomain) $($azmanUser);
							}
						}
						else{
							Write-Host "$($azman.FilePath) does not exist, skipping." -ForegroundColor Red;
						}
					}
				}
			
				if($configuration.CommerceServer.Databases -or $configuration.CommerceServer.WebServices -or $configuration.CommerceServer.Schemas -or$configuration.CommerceServer.DatabaseScripts.DatabaseScript){
					$csServiceWebsite = $configuration.IIS.Websites.Website | where { $_.identity -ieq "CSServices" };
					$bizDeskHost = "$($csServiceWebsite.serverName)";
					
					if( ( $csServiceWebsite.port ) -and ( $csServiceWebsite.port -ne "" ) )
					{
						$bizDeskHost = "$($bizDeskHost):$($csServiceWebsite.port)";
					}			
				
					if(($CS_DB_Folder -ne $null) -and ($CS_DB_Folder -ne ""))
					{
						if(!(Test-Path "$($CS_DB_Folder)"))
						{
							Write-Host "Creating folder for CS databases: $($CS_DB_Folder)" -ForegroundColor Green;
							New-Item -Path "$($CS_DB_Folder)" -type directory | Out-Null;
						}
						else
						{
							Write-Host "CS databases folder already exists." -ForegroundColor Green;;
						}
					}
					
					$SQL_VARIABLES_BASE=@(
						"CS_USER_RUNTIME=`"$(( $configuration.UserAccounts.UserAccount | where { $_.identity -ieq "CSRuntimeUser" }).domain )\$(( $configuration.UserAccounts.UserAccount | where { $_.identity -ieq "CSRuntimeUser" }).userName )`"",
						"CS_USER_FOUNDATION=`"$(( $configuration.UserAccounts.UserAccount | where { $_.identity -ieq "CSFoundationUser" }).domain )\$(( $configuration.UserAccounts.UserAccount | where { $_.identity -ieq "CSFoundationUser" }).userName )`"",
						"CS_USER_SPFARMADMIN=`"$(( $configuration.UserAccounts.UserAccount | where { $_.identity -ieq "SPFarmAdminUser" }).domain )\$(( $configuration.UserAccounts.UserAccount | where { $_.identity -ieq "SPFarmAdminUser" }).userName )`"",
						"CS_USER_CATALOG=`"$(( $configuration.UserAccounts.UserAccount | where { $_.identity -ieq "CSCatalogUser" }).domain )\$(( $configuration.UserAccounts.UserAccount | where { $_.identity -ieq "CSCatalogUser" }).userName )`"",
						"CS_USER_CATALOG_PASSWORD=`"$(( $configuration.UserAccounts.UserAccount | where { $_.identity -ieq "CSCatalogUser" }).password )`"",
						"CS_USER_MARKETING=`"$(( $configuration.UserAccounts.UserAccount | where { $_.identity -ieq "CSMarketingUser" }).domain )\$(( $configuration.UserAccounts.UserAccount | where { $_.identity -ieq "CSMarketingUser" }).userName )`"",
						"CS_USER_ORDERS=`"$(( $configuration.UserAccounts.UserAccount | where { $_.identity -ieq "CSOrdersUser" }).domain )\$(( $configuration.UserAccounts.UserAccount | where { $_.identity -ieq "CSOrdersUser" }).userName )`"",
						"CS_USER_PROFILES=`"$(( $configuration.UserAccounts.UserAccount | where { $_.identity -ieq "CSProfilesUser" }).domain )\$(( $configuration.UserAccounts.UserAccount | where { $_.identity -ieq "CSProfilesUser" }).userName )`"",
						"CS_USER_BIZTALK=`"$(( $configuration.UserAccounts.UserAccount | where { $_.identity -ieq "CSBizTalkUser" }).domain )\$(( $configuration.UserAccounts.UserAccount | where { $_.identity -ieq "CSBizTalkUser" }).userName )`"",
						"CS_USER_FARMADMIN=`"$(( $configuration.UserAccounts.UserAccount | where { $_.identity -ieq "SPFarmAdminUser" }).domain )\$(( $configuration.UserAccounts.UserAccount | where { $_.identity -ieq "SPFarmAdminUser" }).userName )`"",
						"CS_DB_NAME_CATALOG=`"$(( $configuration.CommerceServer.Databases.Database | where { $_.identity -ieq "CSCatalog" }).databaseName )`"",
						"CS_DB_NAME_MARKETING_LISTS=`"$(( $configuration.CommerceServer.Databases.Database | where { $_.identity -ieq "CSMarketingLists" }).databaseName )`"",
						"CS_DB_NAME_MARKETING=`"$(( $configuration.CommerceServer.Databases.Database | where { $_.identity -ieq "CSMarketing" }).databaseName )`"",
						"CS_DB_NAME_PROFILES=`"$(( $configuration.CommerceServer.Databases.Database | where { $_.identity -ieq "CSProfiles" }).databaseName )`"",
						"CS_DB_NAME_ORDERS=`"$(( $configuration.CommerceServer.Databases.Database | where { $_.identity -ieq "CSOrders" }).databaseName )`"",
						"CS_DB_NAME_ORDERS_CONFIG=`"$(( $configuration.CommerceServer.Databases.Database | where { $_.identity -ieq "CSOrdersConfig" }).databaseName )`"",
						"CS_DB_NAME_ADMIN=`"$(( $configuration.CommerceServer.Databases.Database | where { $_.identity -ieq "CSAdmin" }).databaseName )`"",
						"CS_DB_NAME_CATALOG_SCRATCH=`"$(( $configuration.CommerceServer.Databases.Database | where { $_.identity -ieq "CSCatalogScratch" }).databaseName )`"",
						"CS_DB_NAME_CSC_EXTENDED=`"$(( $configuration.CommerceServer.Databases.Database | where { $_.identity -ieq "CSCExtended" }).databaseName )`"",
						"CS_DB_Folder=`"$($CS_DB_Folder)`"",
						"SOLUTION_BASE_DIRECTORY=`"$($SOLUTION_BASE_DIRECTORY)`"",
						"PROJECT_NAME=`"$($PROJECT_NAME)`""
						"CS_BIZDESKHOST=`"$($bizDeskHost)`""
					);										
					
					if($configuration.CommerceServer.WebServices){
						Write-Host "Updating Web Services" -ForegroundColor Green;
						
						$directories = ( Get-ChildItem -Path ( Join-Path -Path $SOLUTION_BASE_DIRECTORY -ChildPath "WebServices" ) ) | Where {$_.psIsContainer -eq $true};
						foreach( $webService in $configuration.CommerceServer.WebServices.WebService )
						{
							$webSiteTargetDir = "";
							
							if($webService.targetPath){
								$webSiteTargetDir = $webService.targetPath;
							}
							else{
								$webSiteTargetDir = $webService.path;
							}
							
							Write-Host "Setting path for $($webService.virtualDirectoryPath) to be $webSiteTargetDir...";
							# get the website object from IIS and assign it's physical path
							$website = $configuration.IIS.Websites.Website | where { $_.identity -ieq $webService.websiteIdentity };
							Set-ItemProperty -Path "IIS:\\Sites\$($website.identity)\$($webService.virtualDirectoryPath)" -Name "physicalPath" -Value $webSiteTargetDir;
							
							# if the targetPath property is specified then the path should be copied to the targetPath
							if($webService.targetPath){
								Write-Host "Copying files from $($webService.path) to $webSiteTargetDir...";
								
								if(Test-Path "$webSiteTargetDir" -PathType Any){
									Write-Host "Deleting existing folder...";
									Remove-Item "$webSiteTargetDir" -Recurse -Force -Confirm:$false;
								}
								
								Copy-Item "$($webService.path)" "$webSiteTargetDir" -Recurse -Force -Confirm:$false;
							}
							
							# make sure that the web service has permissions to access the folder
							$iisWebApplication = Get-WebApplication -Site $website.identity -Name $webService.virtualDirectoryPath;
							$iisAppPoolName = $iisWebApplication.Attributes["applicationpool"].Value;
							$appPoolNode = Get-Item "IIS:\AppPools\$iisAppPoolName";
							$appPoolUserName = $appPoolNode.processModel.userName;
							
							Write-Host "Granting $appPoolUserName access to $($webSiteTargetDir)...";
							# we should look into making the permissions configurable
							Windows-GrantFullReadWriteAccessToFolder "$($webSiteTargetDir)" $appPoolUserName;
							
							#Write-Host "Removing read-only flag from AuthorizationStore.xml file...";
							# make sure the AuthorizationStore.xml file is not read only
							#$fullAuthPath = ( Join-Path -Path $webSiteTargetDir -ChildPath "*AuthorizationStore.xml");
							#$authFile = Get-Childitem "$fullAuthPath";
							#$authFile.IsReadonly = $false;
						}
					}
					
					if($configuration.CommerceServer.Schemas){
						
						Write-Host "Updating Profile Schemas";
						if( $configuration.CommerceServer.Schemas.ProfilesSchema )
						{
							foreach( $profilesSchema in $configuration.CommerceServer.Schemas.ProfilesSchema )
							{
								Write-Host "$($profilesSchema.path)" -ForegroundColor Green;
								Import-CS2007ProfileSchema -CommerceSitename $configuration.CommerceServer.identity -FilePath $profilesSchema.path;
							}
						}

						Write-Host "Updating Catalog Schemas";
						if( $configuration.CommerceServer.Schemas.CatalogSchema )
						{
							foreach( $catalogSchema in $configuration.CommerceServer.Schemas.CatalogSchema )
							{
								Write-Host "$($catalogSchema.path)" -ForegroundColor Green;
								Import-CS2007CatalogSchema -CommerceSitename $configuration.CommerceServer.identity -FilePath $catalogSchema.path -Replace ( $catalogSchema.replace -ieq "true" ) -Transacted ( $catalogSchema.transacted -ieq "true" ) -AllowSchemaUpdate ( $catalogSchema.allowSchemaUpate -ieq "true" );
								
								if($catalogSchema.AzMan -and $catalogSchema.AzMan.File){
									
									Write-Host "Updating AzMan files associated to catalog..." -ForegroundColor Green;
									
									foreach($file in $catalogSchema.AzMan.File){
										Update-CatalogAuthStore $file.siteName $file.name;
									}
								}
							}
						}

						Write-Host "Upating Catalog Inventory Schemas";
						if( $configuration.CommerceServer.Schemas.CatalogInventorySchema )
						{
							foreach( $catalogInventorySchema in $configuration.CommerceServer.Schemas.CatalogInventorySchema )
							{
								Write-Host "$($catalogInventorySchema.path)" -ForegroundColor Green;

								if($catalogInventorySchema.AzMan -and $catalogInventorySchema.AzMan.File){	
									Write-Host "Updating with AzMan file..." -ForegroundColor Green;							
									Import-CS2007InventorySchema -CommerceSitename $configuration.CommerceServer.identity -FilePath $catalogInventorySchema.path -AzManFile $catalogInventorySchema.AzMan.File.name;
								}
								else{
									Write-Host "Updating without AzMan file..." -ForegroundColor Green;							
									Import-CS2007InventorySchema -CommerceSitename $configuration.CommerceServer.identity -FilePath $catalogInventorySchema.path;
								}
							}
						}

#						Write-Host "Importing Marketing sample data";
#						$returnValue = Start-Process -FilePath ( Join-Path -Path $DEPLOYMENT_DIRECTORY -ChildPath "MarketingUtilities.exe" ) -Wait -NoNewWindow -WorkingDirectory $DEPLOYMENT_DIRECTORY -ArgumentList "/importmarketingschema" -PassThru;
#						Write-Host $returnValue.StandardOutput;
#						if( $returnValue.ExitCode -ne 0 )
#						{
#							Write-Error -Message $returnValue.StandardError -ErrorId $returnValue.ExitCode ;
#						}
					}

					if($configuration.CommerceServer.DatabaseScripts.DatabaseScript ){
						Write-Host "Updating CS databases...";
			
						Add-SQLPSSnapin;
						
						foreach( $databaseScript in $configuration.CommerceServer.DatabaseScripts.DatabaseScript )
						{
							Write-Host "$($databaseScript.path)" -ForegroundColor Green;
							$databaseIdentity = ( $configuration.CommerceServer.Databases.Database | where { $_.identity -ieq $databaseScript.databaseIdentity } ) ;

							$SQL_VARIABLES = $SQL_VARIABLES_BASE;
							$SQL_VARIABLES += "CS_DB_SERVER=`"$($databaseIdentity.serverName)`"";
							
							$SQL_VARIABLES_STRING = [System.String]::Join( " -v ", $SQL_VARIABLES );
							$returnValue = Start-Process -Wait -NoNewWindow -PassThru -WorkingDirectory $DEPLOYMENT_DIRECTORY -FilePath $SQLCMD_PATH -ArgumentList @( "-b", "-E", "-S $($databaseIdentity.serverName)", "-i `"$($databaseScript.path)`"", "-v $($SQL_VARIABLES_STRING)");
							Write-Host $returnValue.StandardOutput;
							if( $returnValue.ExitCode -ne 0 )
							{
								if( $Error )
								{
									Write-Host $Error -ForegroundColor Red;
								}
								Write-Error -Message $returnValue.StandardError -ErrorId $returnValue.ExitCode ;
							}
						}
					}	
				
					Write-Host "Setting the Commerce Server Display Out-of-Stock items flag" -ForegroundColor Green;
					Set-CSConfigInventoryPropertyFlag -CommerceSiteName $configuration.CommerceServer.identity -CommercePropertyName "f_display_oos_skus" -CommercePropertyValue $true;				
				}
			
				if($configuration.SharepointServices){
					if($configuration.SharepointServices.StateService){
						Write-Host "Setting up state service..." -ForegroundColor Yellow;
					
						$stateService = $configuration.SharepointServices.StateService;
						Setup-SPStateService -StateServiceDisplayName "$($stateService.displayName)" -StateServiceProxyDisplayName "$($stateService.proxyName)" -StateServiceDatabaseName "$($stateService.dbName)";
					}
			
					if($configuration.SharepointServices.AnalyticsService){
						Write-Host "Setting up analytics service..." -ForegroundColor Yellow;
					
						$analyticsService = $configuration.SharepointServices.AnalyticsService;
						Setup-SPWebAnalyticsWebServiceApplication -AppPoolDisplayName "$($analyticsService.appPoolName)" -AppPoolUserName "$($analyticsService.appPoolUserName)" -WebAnalyticsServiceDisplayName "$($analyticsService.displayName)" -SharePointDatabaseServer "$($analyticsService.dbServer)" -WebAnalyticsStagingDatabaseName "$($analyticsService.stagingDBName)" -WebAnalyticsReportingDatabaseName "$($analyticsService.reportingDBName)";
					}
				
					if($configuration.SharepointServices.Backups -and $configuration.SharepointServices.Backups.Backup){
						foreach($backup in $configuration.SharepointServices.Backups.Backup){
					
							Write-Host "Backing up SharePoint site using $($backup.settings) and $($backup.backupFile)..." -ForegroundColor Yellow;
						
							$exportSettings = $( Join-Path -Path $SP_BACKUP_WORKING_FOLDER -ChildPath $backup.settings);
						
							Export-SharePointContent $TFS $TFS_Project $SP_BACKUP_WORKING_FOLDER $backup.backupFile $SP_BACKUP_OUTPUT_FOLDER $exportSettings $SP_STSADM_PATH;					
						}
					}
				
					if($configuration.SharepointServices.BusinessDataConnectivityService){
						Write-Host "Setting up business data connectivity service..." -ForegroundColor Yellow;
					
						$bcsService = $configuration.SharepointServices.BusinessDataConnectivityService;
					
						$userAccount = $configuration.UserAccounts.UserAccount | where { $_.identity -ieq $($bcsService.appPoolUserName)};
						$bcsUserName = "$($userAccount.domain)\$($userAccount.username)";					
					
						Write-Host "Checking to see if the $($bcsUserName) user is set up as a managed account...";
						$managedAccount = Get-SPManagedAccount | where {$_.UserName -eq $bcsUserName};
					
						if($managedAccount -eq $null){
							Write-Host "User is not setup, registering..." -ForegroundColor Yellow;
						
							$password =  ( ConvertTo-SecureString $userAccount.password -AsPlainText -Force );
							$credentials = New-Object -typename System.Management.Automation.PSCredential -argumentlist $bcsUserName, $password ;
							$managedAccount = New-SPManagedAccount -Credential $credentials;
						}
						else{
							Write-Host "User is already registered." -ForegroundColor Green;
						}
					
						Setup-SPBusinessDataConnectivityServiceApplication -AppPoolDisplayName "$($bcsService.appPoolName)" -AppPoolUserName "$($bcsUserName)" -DisplayName "$($bcsService.displayName)" -DatabaseServer "$($bcsService.dbServer)" -DatabaseName "$($bcsService.dbName)" -ServiceContext "$($bcsService.serviceContext)" ;
					}
				
			}		
					
			# File tokens
			if( $configuration.FileTokens -and $configuration.FileTokens.FileToken ){
				
				$files = $configuration.SelectNodes("FileTokens/FileToken");
				
				$TOKENS=@(
						"#CatalogDBName=$(( $configuration.CommerceServer.Databases.Database | where { $_.identity -ieq "CSCatalog" }).databaseName )",
						"#PrivateSiteDomain=$(( $configuration.Variables.Variable | where { $_.identity -ieq "PrivateUrlDomain" }).value )",
						"#ExtendedDBName=$(( $configuration.CommerceServer.Databases.Database | where { $_.identity -ieq "CSExtended" }).databaseName )",
						"#ProjectName=$($PROJECT_NAME)",
						"#DomainName=$($DOMAIN_NAME)",
						"#RuntimeUser=$(( $configuration.UserAccounts.UserAccount | where { $_.identity -ieq "CSRuntimeUser" }).username )"
						 );
				
				Write-Host "Replacing file tokens";				
				foreach( $file in $files)
				{
					Write-Host "Removing read-only flag for $($file.filePath)" ;
					# make sure the  file is not read only
					Set-ItemProperty ($file.filePath) -name IsReadOnly -value $false
											
					Write-Host "$($file.filePath)" -ForegroundColor Green;
					foreach ( $token in $TOKENS ) 
					{		
						$content = Get-Content -path $file.filePath;
						$keyValue = $token -split "=";			
						$content -replace $keyValue[0], $keyValue[1] | Set-Content $file.filePath;
					}
				}
			}

			#create any required folders and assign permissions
			if($configuration.Folders -and $configuration.Folders.Folder){
				Write-Host "Creating folders..." -ForegroundColor Green;
				
				$folders = $configuration.SelectNodes("Folders/Folder");
			
				foreach($folder in $folders){
					if ((Test-Path $folder.path) -ne $true){
						Write-Host "Creating $($folder.path)";
						
						New-Item $folder.path -type directory > $null;											
					}
					else{
						Write-Host "$($folder.path) already exists will not re-create.";
					}
					
					if($folder.UserAccounts -and $folder.UserAccounts.UserAccount){
						Write-Host "Creating permissions for $($folder.path)";
						
						$users = $folder.SelectNodes("UserAccounts/UserAccount");
					
						foreach($user in $users){
							$userAccount = $configuration.UserAccounts.UserAccount | where { $_.identity -ieq $user.userAccountIdentity};
							
							Write-Host "Creating permissions for $($userAccount.domain)\$($userAccount.username) on $($folder.path)";
							Windows-GrantFullReadWriteAccessToFolder "$($folder.path)" "$($userAccount.domain)\$($userAccount.username)";
						}
					}					
				}
			}

			Write-Host "Done." -ForegroundColor Yellow;
		}

		# Begin SharePoint Stuff
		foreach( $deploymentIdentity in $DeploymentIdentities )
		{
			Write-Host "Deploying $($deploymentIdentity)";

			foreach( $deployment in $environmentXml.CommerceServer.Deployments.Deployment | where { $_.identity -ieq $deploymentIdentity } )
			{
				# Update the variables in the xml file in memory.
				foreach( $node in $deployment.SelectNodes( ".//*" ) )
				{
					foreach( $attribute in $node.Attributes )
					{
						$attribute.Value = Expand-String( ( ( ( $attribute.Value -replace '&quot;', '`"') -replace '''', '`''') ) )
					}
				}

				if( $deployment.SPFarm )
				{
					#$Host.Runspace.ThreadOptions = [System.Management.Automation.Runspaces.PSThreadOptions]::ReuseThread;
					if( -not (  Get-PSSnapin | where { $_.Name -eq "Microsoft.SharePoint.PowerShell" } ) )
					{
						Add-PSSnapin Microsoft.SharePoint.PowerShell;
					}

					Start-SPAssignment -Global;

					$spFarm = $deployment.SPFarm;
					
					# Generate the WSP files from code
					if( $GenerateWSP )
					{
						if( $spFarm.Solutions.Solution )
						{
							foreach( $solution in $spFarm.Solutions.Solution | where { $_.generate -ieq "true" } )
							{
								Write-Host "Generating $($solution.identity)";

								$parentDir = (Split-Path -Path ( Split-Path -Path $solution.path -Parent ) -Parent);
								$csProj = "`"" + ( Join-Path -Path $parentDir -ChildPath $($solution.identity)) + ".csproj`"";

								if($SP2013)
								{
									$args = @($csProj, "/p:IsPackaging=True", "/p:Configuration=DebugV4", "/nologo");
								}
								else
								{
									$args = @($csProj, "/p:IsPackaging=True", "/p:Configuration=Debug", "/nologo");
								}
								

								Write-Host "Parent Dir: $($parentDir)";
								Write-Host "MSBUILD args: $($args)";
								
								$returnValue = Start-Process -FilePath $MSBUILD_PATH -Wait -NoNewWindow -WorkingDirectory $parentDir -PassThru -ArgumentList $args;
								Write-Host $returnValue.StandardOutput;
								if( $returnValue.ExitCode -ne 0 )
								{
									Write-Host $returnValue.StandardError
									# Write-Error -Message $returnValue.StandardError -ErrorId $returnValue.ExitCode ;
								}
							}
						}
					}

					Write-Host "Restarting timer...";
					Restart-Service -Name "SPTimerv4";

					foreach( $spWebApplication in $spFarm.SPWebApplications.SPWebApplication )
					{
						if( $spWebApplication.deleteIfExists -and ( $spWebApplication.deleteIfExists -ieq "true" ) )
						{
							if( Get-SPWebApplication | where { $_.DisplayName -ieq $spWebApplication.identity } )
							{
								Write-Host "$($spWebApplication.identity)" -ForegroundColor Green;
								if( $spWebApplication.DeploySolutions.Solution )
								{
									Write-Host "Retracting Web Application solutions";
									
									if($Debug)
									{
										Write-Host "About to retract solutions.  Press any key to continue..."
										$x = $host.UI.RawUI.ReadKey("NoEcho,IncludeKeyDown")
									}

									$solutionNames = @();
									foreach( $spWebApplicationDeployedSolution in $spWebApplication.DeploySolutions.Solution )
									{
										$spFarmSolution = $spFarm.Solutions.Solution | where { $_.identity -ieq $spWebApplicationDeployedSolution.identity };
										$wspName = ( Split-Path -Path $spFarmSolution.path -Leaf );

										$solutionNames += $wspName;
									}

									# Undeploy the solutions in reverse order.
									[array]::Reverse($solutionNames);
									foreach( $solutionName in  $solutionNames)
									{
										UndeployAndWait-SPSolution -WSPName $solutionName -WebApplication $spWebApplication.identity;
									}

									Write-Host "Done." -ForegroundColor Yellow;

									if($Debug)
									{
										Write-Host "Retracted solutions, check state.  Press any key to continue..."
										$x = $host.UI.RawUI.ReadKey("NoEcho,IncludeKeyDown")
									}
								}
								else
								{
									Write-Host "No SPWebApplication deploy solutions defined in the Environment.xml file." -ForegroundColor Yellow;
								}

								Write-Host "Stopping timer service" -ForegroundColor Green;
								# This needs to be done to ensure Remove-SPWebApplication works
								# See http://support.microsoft.com/kb/939308
								Stop-Service -Name "SPTimerV4";

								Write-Host "Cleaning up SharePoint local config cache." -ForegroundColor Green;
								Get-ChildItem ( Join-Path $Env:ALLUSERSPROFILE -ChildPath "Microsoft\SharePoint\Config\*" ) -Recurse -Include *.xml | Remove-Item;
								$cacheFiles = ( Get-ChildItem -Path ( Join-Path $Env:ALLUSERSPROFILE -ChildPath "\Microsoft\SharePoint\Config\*" ) -Include "cache.ini" -Recurse ); 
								foreach( $cacheFile in $cacheFiles )
								{
									Set-Content -Path $cacheFile -Value "1";
								}

								Write-Host "Starting timer service" -ForegroundColor Green;
								Start-Service -Name "SPTimerV4";

								Write-Host "Removing $($spWebApplication.identity)";
								Remove-SPWebApplication -identity $spWebApplication.identity -DeleteIISSite -RemoveContentDatabases -Confirm:$false;
								Write-Host "Re-starting timer service" -ForegroundColor Green;
								ReStart-Service -Name "SPTimerV4";
							}
							else
							{
								Write-Host "Web application $($spWebApplication.identity) is not found in the Farm" -ForegroundColor Yellow;
							}
						}
					}

					if( $spFarm.DeploySolutions -and $spFarm.DeploySolutions.Solution )
					{
						Write-Host "Retracting Farm solutions";

						$solutionNames = @();
						foreach( $spFarmDeploySolution in $spFarm.DeploySolutions.Solution )
						{
							Write-Host "$($spFarmDeploySolution.identity)";

							$spFarmSolution = $spFarm.Solutions.Solution | where { $_.identity -ieq $spFarmDeploySolution.identity };

							# Only undeploy if the deleteIfExists attribute is true or the GenerateWSP parameter is set and the generate attribute is true.
							if( ( ( $spFarmSolution.generate -and ( $spFarmSolution.generate -ieq "true" ) -and $GenerateWSP ) ) -or ( $spFarmSolution.deleteIfExists -and ( $spFarmSolution.deleteIfExists -ieq "true" ) ) )
							{
								$wspName = ( Split-Path -Path $spFarmSolution.path -Leaf );

								$solutionNames += $wspName;
							}
						}

						# Retract the solutions in reverse order.
						[array]::Reverse($solutionNames);
						foreach( $solutionName in  $solutionNames)
						{	
							if($Debug)
							{
								Write-Host "About to retract solution $($solutionName).  Press any key to continue..."
								$x = $host.UI.RawUI.ReadKey("NoEcho,IncludeKeyDown")
							}

							UndeployAndWait-SPSolution -WSPName $solutionName;
						}

						if($Debug)
						{
							Write-Host "Retracted solutions.  Check state.  Press any key to continue..."
							$x = $host.UI.RawUI.ReadKey("NoEcho,IncludeKeyDown")
						}

						Write-Host "Done." -ForegroundColor Yellow;
					}
					else
					{
						Write-Host "No SPFarm deploy solutions defined in the Environment.xml file." -ForegroundColor Yellow;
					}
					
					if( $spFarm.Solutions -and $spFarm.Solutions.Solution )
					{
						Write-Host "Remove Farm solutions";
						$solutionNames = @();
						foreach( $spFarmSolution in $spFarm.Solutions.Solution )
						{
							# Only remove if the deleteIfExists attribute is true or the GenerateWSP parameter is set and the generate attribute is true.
							if( ( ( $spFarmSolution.generate -and ( $spFarmSolution.generate -ieq "true" ) -and $GenerateWSP ) ) -or ( $spFarmSolution.deleteIfExists -and ( $spFarmSolution.deleteIfExists -ieq "true" ) ) )
							{
								$wspName = ( Split-Path -Path $spFarmSolution.path -Leaf );
								$solutionNames += $wspName;
							}
						}

						# Retract the solutions in reverse order.
						[array]::Reverse($solutionNames);
						foreach( $solutionName in  $solutionNames)
						{
							Write-Host "$($solutionName)" -ForegroundColor Green;
							$solution = Get-SPSolution | where { $_.Name -ieq $solutionName };
							
							if($Debug)
							{
								Write-Host "About to remove solution $($solutionName).  Press any key to continue..."
								$x = $host.UI.RawUI.ReadKey("NoEcho,IncludeKeyDown")
							}

							if($solution -ne $null )
							{
								Remove-SPSolution -Identity $solution -Force -Confirm:$false;
							}
							else
							{
								Write-Host "Solution $($wspName) was not found in the solution store." -ForegroundColor Yellow;
							}
						}

						Write-Host "Done." -ForegroundColor Yellow;
					}
					else
					{
						Write-Host "No SPFarm solutions defined in the Environment.xml file." -ForegroundColor Yellow;
					}

					# Add the solutions to the solution store
					if( $spFarm.Solutions.Solution )
					{
						Write-Host "Adding SPFarm solutions...";
						foreach( $spFarmSolution in $spFarm.Solutions.Solution )
						{
							if( ( $GenerateWSP -and ( $spFarmSolution.generate -and ( $spFarmSolution.generate -ieq "true" ) ) ) -or ( $spFarmSolution.deleteIfExists -and ( $spFarmSolution.deleteIfExists -ieq "true" ) ) )
							{
								if($Debug)
								{
									Write-Host "Adding solution $($spFarmSolution.identity).  Press any key to continue..."
									$x = $host.UI.RawUI.ReadKey("NoEcho,IncludeKeyDown")
								}

								Write-Host "$($spFarmSolution.identity)..." -ForegroundColor Green;
								$solution = Add-SPSolution -LiteralPath  $spFarmSolution.path;
							}
						}

						if($Debug)
						{
							Write-Host "Added solutions. Check state.  Press any key to continue..."
							$x = $host.UI.RawUI.ReadKey("NoEcho,IncludeKeyDown")
						}

						Write-Host "Done." -ForegroundColor Yellow;
					}
					else
					{
						Write-Host "No SPFarm solutions defined in the Environment.xml file." -ForegroundColor Yellow;
					}
						
					# Deploy the Farm level solutions
					if( $spFarm.DeploySolutions.Solution )
					{
						Write-Host "Deploying SPFarm solutions...";
						foreach( $spFarmDeploySolution in $spFarm.DeploySolutions.Solution )
						{
							Write-Host "$($spFarmDeploySolution.identity)" -ForegroundColor Green;

							$spFarmSolution = $spFarm.Solutions.Solution | where { $_.identity -ieq $spFarmDeploySolution.identity };
							
							if( ( $GenerateWSP -and ( $spFarmSolution.generate -and ( $spFarmSolution.generate -ieq "true" ) ) ) -or ( $spFarmSolution.deleteIfExists -and ( $spFarmSolution.deleteIfExists -ieq "true" ) ) )
							{
								if($Debug)
								{
									Write-Host "Deploying solution $($spFarmDeploySolution.identity).  Press any key to continue..."
									$x = $host.UI.RawUI.ReadKey("NoEcho,IncludeKeyDown")
								}

								
								if($SP2013)
								{
									DeployAndWait-SPSolution -WSPName ( Split-Path -Path $spFarmSolution.path -Leaf ) -SP2013;
								}
								else
								{
									DeployAndWait-SPSolution -WSPName ( Split-Path -Path $spFarmSolution.path -Leaf );
								}
							}
						}

						Write-Host "Done." -ForegroundColor Yellow;
					}
					else
					{
						Write-Host "No SPFarm solutions defined in the Environment.xml file." -ForegroundColor Yellow;
					}
					
					# Activate the Farm level features
					if( $spFarm.ActivateFeatures.Feature )
					{
						Write-Host "Activating SPFarm features";

						# WORKAROUND - The Site Definitions cache does not get refreshed in the current session
						# 	If you want to create a new site collection based on a site definition file (ONET.xml) that you deployed during the current PowerShell session, you need to spawn a new session.
						#	See http://social.msdn.microsoft.com/Forums/en/sharepoint2010general/thread/5ea46e07-a4b0-4bab-b698-9b6eeb695607

						# Copy stuff to the list
						$farmFeatures = @()
						foreach( $spFarmActivateFeature in $spFarm.ActivateFeatures.Feature )
						{
							Write-Host "Feature Name $($spFarmActivateFeature.identity)";
							$feature = New-Feature($spFarmActivateFeature.identity, $spFarmActivateFeature.deleteIfExists, $spFarmActivateFeature.reactivate, $spFarmActivateFeature.activateUsingHttpGet)
							$farmFeatures += $feature
						}
						Write-Host "------"

						$job = Start-Job -ArgumentList @( $farmFeatures, $DEPLOYMENT_DIRECTORY ) -ScriptBlock {
						
							PARAM
							(
								$farmFeatures, $DEPLOYMENT_DIRECTORY
							)

							if( -not ( Get-PSSnapin  | where { $_.Name -eq "Microsoft.SharePoint.PowerShell" } ) )
							{
								Add-PSSnapin Microsoft.SharePoint.PowerShell;
							}
							
							. ( Join-Path -Path $DEPLOYMENT_DIRECTORY -ChildPath "UtilityScripts\IIS.ps1");
							. ( Join-Path -Path $DEPLOYMENT_DIRECTORY -ChildPath "SharePoint2010.ps1");
							
							foreach( $spFarmActivateFeature in $farmFeatures )
							{
								if($spFarmActivateFeature.identity){
									if( $spFarmActivateFeature.deleteIfExists -and ( $spFarmActivateFeature.deleteIfExists -ieq "true" ) )
									{
									<#
										if($Debug)
										{
											Write-Host "Disabling feature $($spFarmActivateFeature.identity).  Press any key to continue..."
											$x = $host.UI.RawUI.ReadKey("NoEcho,IncludeKeyDown")
										}

										#>
										DisableAndWait-SPFeature -FeatureName $spFarmActivateFeature.identity;
										
										$reactivate = "true";
										if($spFarmActivateFeature.reactivate){
											$reactivate = $spFarmActivateFeature.reactivate;
										}
										
										if($reactivate -ieq "true"){
											<#
											if($Debug)
											{
												Write-Host "Reactivating feature $($spFarmActivateFeature.identity).  Press any key to continue..."
												$x = $host.UI.RawUI.ReadKey("NoEcho,IncludeKeyDown")
											}
											#>

											EnableAndWait-SPFeature -FeatureName $spFarmActivateFeature.identity.Trim() -ActivateUsingHttpGet ( $spFarmActivateFeature.activateUsingHttpGet -and ( $spFarmActivateFeature.activateUsingHttpGet -ieq "true" ) );
										}										
									}
									else{
										<#
										if($Debug)
										{
											Write-Host "Enabling feature $($spFarmActivateFeature.identity).  Press any key to continue..."
											$x = $host.UI.RawUI.ReadKey("NoEcho,IncludeKeyDown")
										}
										#>

										EnableAndWait-SPFeature -FeatureName $spFarmActivateFeature.identity -ActivateUsingHttpGet ( $spFarmActivateFeature.activateUsingHttpGet -and ( $spFarmActivateFeature.activateUsingHttpGet -ieq "true" ) );
									}
								}
							}

							#if($Debug)
							#{
						#		Write-Host "Farm features enabled.  Check state.  Press any key to continue..."
						#		$x = $host.UI.RawUI.ReadKey("NoEcho,IncludeKeyDown")
						#	}
						}
																	
						while( ( $job.State -ne "Completed" ) -or $job.HasMoreData )
						{
							Receive-Job -Job $job;
							Start-Sleep -Seconds 1;
						}
						Remove-Job -Job $job;
					}
					else
					{
						Write-Host "No SPFarm features defined in the Environment.xml file." -ForegroundColor Yellow;
					}

					Write-Host "Finished with SPFarm features." -ForegroundColor Green;

					# Create the Web Application
					foreach( $spWebApplication in $spFarm.SPWebApplications.SPWebApplication )
					{
						# Create the web application only if the create flag is true
						if( $spWebApplication.create -and ( $spWebApplication.create -ieq "true" ) )
						{
							if($Debug)
							{
								Write-Host "Creating application solution $($spWebApplication.identity).  Press any key to continue..."
								$x = $host.UI.RawUI.ReadKey("NoEcho,IncludeKeyDown")
							}

							Write-Host "Creating SPWebApplication"
							$userAccount = $configuration.UserAccounts.UserAccount | where { $_.identity -ieq $spWebApplication.iisAppPoolUserAccountIdentity};
							$userName="$($userAccount.domain)\$($userAccount.username)";
							
							$managedAccount = Get-SPManagedAccount | where {$_.UserName -ieq $userName };
							if( -not $managedAccount )
							{	
								$password =  ( ConvertTo-SecureString $userAccount.password -AsPlainText -Force );
								$credentials = New-Object -typename System.Management.Automation.PSCredential -argumentlist $userName, $password ;

								if($Debug)
								{
									Write-Host "Creating managed account  for $($userName).  Press any key to continue..."
									$x = $host.UI.RawUI.ReadKey("NoEcho,IncludeKeyDown")
								}

								$managedAccount = New-SPManagedAccount -Credential $credentials;
							}
							
							$app = $null;

							$winAuth = New-SPAuthenticationProvider;
							Write-Host "Creating web application (Default Zone)..." -ForegroundColor Green;
							
							if(-not(Get-Module -name "WebAdministration"))
							{
								if(Get-Module -ListAvailable | Where-Object { $_.name -eq "WebAdministration" })
								{
									Write-Host "Importing IIS module...";
									Import-Module -Name WebAdministration;
								}
							}
							
							$iisAppPoolNode = Get-Item "IIS:\AppPools\";
														
							if(($iisAppPoolNode -ne $null) -and ($iisAppPoolNode.Children -ne $null) -and ($iisAppPoolNode.Children.ContainsKey($spWebApplication.iisAppPoolName) -eq $true) ){
								Write-Host "The app pool $($spWebApplication.iisAppPoolName) can only be used by the web application";
								
								throw;
							}
							else{
								Write-Host "Creating the web application with a  new app pool...";

								if($Debug)
								{
									Write-Host "Creating SP Web Application $($spWebApplication.identity).  Press any key to continue..."
									$x = $host.UI.RawUI.ReadKey("NoEcho,IncludeKeyDown")
								}

								$app = New-SPWebApplication -Name $spWebApplication.identity -ApplicationPool $spWebApplication.iisAppPoolName -ApplicationPoolAccount $managedAccount -Url $spWebApplication.url -AuthenticationProvider $winAuth -DatabaseName $spWebApplication.contentDatabaseName -HostHeader $spWebApplication.hostheader -Port $spWebApplication.port -AllowAnonymous:$false;

								if($Debug)
								{
									Write-Host "SP Web Application created.  Check state.  Press any key to continue..."
									$x = $host.UI.RawUI.ReadKey("NoEcho,IncludeKeyDown")
								}
							}


							if(!$SP2013){
								Write-Host "Setting the portalsuperuseraccount and portalsuperreaderaccount properties of the web application." -ForegroundColor Green;
								$app.Properties["portalsuperuseraccount"] = "NT AUTHORITY\LOCAL SERVICE";
								$app.Properties["portalsuperreaderaccount"] = "SHAREPOINT\system";
								$app.PresenceEnabled=$false;
								$app.Update();
							}
							


							$providerName = "CS_" + $app.Id.ToString("D") + "_Internet";
							$roleProviderName = "Role_" + $app.Id.ToString("D") + "_Internet";

							Write-Host "Auth Provider: $providerName";
							Write-Host "Role Provider: $roleProviderName";

							# Uncomment to enable multi-authentication
							# $defaultZoneFormsAuthProvider = New-SPAuthenticationProvider -ASPNETMembershipProvider $providerName -ASPNETRoleProviderName $roleProviderName
							# Set-SPwebApplication $app -AuthenticationProvider ((New-SPAuthenticationProvider),$defaultZoneFormsAuthProvider) -Zone "Default";
							
							if($Debug)
							{
								Write-Host "Creating Authentication Provider $($providerName).  Press any key to continue..."
								$x = $host.UI.RawUI.ReadKey("NoEcho,IncludeKeyDown")
							}

							$commerceAuthProvider = New-SPAuthenticationProvider -ASPNETMembershipProvider $providerName -ASPNETRoleProviderName $roleProviderName;
							$commerceAuthProvider.RoleProvider = $roleProviderName;
							
							if($Debug)
							{
								Write-Host "Created Authentication Provider $($providerName).  Press any key to continue..."
								$x = $host.UI.RawUI.ReadKey("NoEcho,IncludeKeyDown")
							}

							# Uncomment to enable multi-authenication
							# Need to explictly set Anonymous to App Pool Identity  - SharePoint sets it correctly 50% of the time.
							# Set-WebConfigurationProperty -filter /system.WebServer/security/authentication/AnonymousAuthentication -name enabled -value true -location $spWebApplication.identity
							# Set-WebConfigurationProperty -filter /system.WebServer/security/authentication/AnonymousAuthentication -name userName -value "" -location $spWebApplication.identity
							# Set-WebConfigurationProperty -filter /system.WebServer/security/authentication/AnonymousAuthentication -name password -value "" -location $spWebApplication.identity
							# Set-WebConfigurationProperty -filter /system.WebServer/security/authentication/AnonymousAuthentication -name logonMethod -value 3 -location $spWebApplication.identity

							if($spWebApplication.customErrors -and ($spWebApplication.customErrors -ne "")){
								Write-Host "Setting custom errors to $($spWebApplication.customErrors)";
								$iisWebSite = Get-Item "IIS:\\Sites\$($spWebApplication.identity)";
								
								if($iisWebSite -ne $null){
									$iisWebSitePath = (Join-Path -Path $iisWebSite.PhysicalPath -ChildPath "web.config" );
									
									Update-XmlAttribute -FilePath $iisWebSitePath -XPath "/configuration/system.web/customErrors" -UpdateAttribute -AttributeName "mode" -AttributeValue $spWebApplication.customErrors -Namespaces $null;
								}
								else{
									Write-Host "Could not find web.config" -ForegroundColor Red;
								}
							}

							# Moving host head stuff here...
							Windows-UpdateHostsFile 127.0.0.1 $spWebApplication.hostheader
							
							if($spWebApplication.trust -and ($spWebApplication.trust -ne "")){
								Write-Host "Setting trust to $($spWebApplication.trust)";
								$iisWebSite = Get-Item "IIS:\\Sites\$($spWebApplication.identity)";
								
								if($iisWebSite -ne $null){
									$iisWebSitePath = (Join-Path -Path $iisWebSite.PhysicalPath -ChildPath "web.config" );
									
									Update-XmlAttribute -FilePath $iisWebSitePath -XPath "/configuration/system.web/trust" -UpdateAttribute -AttributeName "level" -AttributeValue $spWebApplication.trust -Namespaces $null;
								}
								else{
									Write-Host "Could not find web.config" -ForegroundColor Red;
								}								
							}							

							# SP Alternate Web Application Extensions
							if( $spWebApplication.SPWebApplicationExtensions.SPWebApplicationExtension )
							{
								foreach( $spWebApplicationExtension in $spWebApplication.SPWebApplicationExtensions.SPWebApplicationExtension )
								{
									Write-Host "Creating web application (Internet Zone)..." -ForegroundColor Green;

									if($Debug)
									{
										Write-Host "Creating web application extension $($spWebApplicationExtension.identity).  Press any key to continue..."
										$x = $host.UI.RawUI.ReadKey("NoEcho,IncludeKeyDown")
									}

									New-SPWebApplicationExtension -Identity $spWebApplication.identity -Name $spWebApplicationExtension.identity -Zone $spWebApplicationExtension.zone -AuthenticationProvider $commerceAuthProvider -Confirm:$false -Url $spWebApplicationExtension.url -HostHeader $spWebApplicationExtension.hostheader -Port $spWebApplicationExtension.port -AllowAnonymous;

									# Set Use App Pool Identity explictly
									Set-WebConfigurationProperty -filter /system.WebServer/security/authentication/AnonymousAuthentication -name enabled -value true -location $spWebApplicationExtension.identity
									Set-WebConfigurationProperty -filter /system.WebServer/security/authentication/AnonymousAuthentication -name userName -value "" -location $spWebApplicationExtension.identity
									Set-WebConfigurationProperty -filter /system.WebServer/security/authentication/AnonymousAuthentication -name password -value "" -location $spWebApplicationExtension.identity
									Set-WebConfigurationProperty -filter /system.WebServer/security/authentication/AnonymousAuthentication -name logonMethod -value 3 -location $spWebApplicationExtension.identity
									
									if($Debug)
									{
										Write-Host "Application extension and IIS settings created for $($spWebApplicationExtension.identity).  Press any key to continue..."
										$x = $host.UI.RawUI.ReadKey("NoEcho,IncludeKeyDown")
									}

									if($spWebApplicationExtension.customErrors -and ($spWebApplicationExtension.customErrors -ne "")){
										Write-Host "Setting custom errors to $($spWebApplicationExtension.customErrors)";
										$iisWebSite = Get-Item "IIS:\\Sites\$($spWebApplicationExtension.identity)";
										
										if($iisWebSite -ne $null){
											$iisWebSitePath = (Join-Path -Path $iisWebSite.PhysicalPath -ChildPath "web.config" );
											
											Update-XmlAttribute -FilePath $iisWebSitePath -XPath "/configuration/system.web/customErrors" -UpdateAttribute -AttributeName "mode" -AttributeValue $spWebApplication.customErrors -Namespaces $null;
										}
										else{
											Write-Host "Could not find web.config" -ForegroundColor Red;
										}
									}

									# Move host header for extension here...
									Windows-UpdateHostsFile 127.0.0.1 $spWebApplicationExtension.hostheader

									if($spWebApplicationExtension.trust -and ($spWebApplicationExtension.trust -ne "")){
										Write-Host "Setting trust to $($spWebApplicationExtension.trust)";
										$iisWebSite = Get-Item "IIS:\\Sites\$($spWebApplicationExtension.identity)";
										
										if($iisWebSite -ne $null){
											$iisWebSitePath = (Join-Path -Path $iisWebSite.PhysicalPath -ChildPath "web.config" );
											
											Update-XmlAttribute -FilePath $iisWebSitePath -XPath "/configuration/system.web/trust" -UpdateAttribute -AttributeName "level" -AttributeValue $spWebApplication.trust -Namespaces $null;
										}
										else{
											Write-Host "Could not find web.config" -ForegroundColor Red;
										}								
									}									
								}

								Write-Host "Done." -ForegroundColor Yellow;
							}
							else
							{
								Write-Host "No SPWebApplication extended sites defined in the Environment.xml file." -ForegroundColor Yellow;
							}
							
							# SP Alternate URLs
							if( $spWebApplication.SPAlternateUrls.SPAlternateUrl )
							{
								foreach( $spAlternateUrl in $spWebApplication.SPAlternateUrls.SPAlternateUrl )
								{
									if($Debug)
									{
										Write-Host "Setting up alternate URL $($spAlternateUrl.url).  Press any key to continue..."
										$x = $host.UI.RawUI.ReadKey("NoEcho,IncludeKeyDown")
									}

									if( $spAlternateUrl.internal -ieq "true" )
									{
										$alternateUrl = New-SPAlternateUrl -Url $spAlternateUrl.url -WebApplication $spWebApplication.identity -Zone $spAlternateUrl.zone -Internal;
									}
									else
									{
										$alternateUrl = New-SPAlternateUrl -Url $spAlternateUrl.url -WebApplication $spWebApplication.identity -Zone $spAlternateUrl.zone;
									}

									if($Debug)
									{
										Write-Host "Created up alternate URL $($spAlternateUrl.url).  Check state and then hit any key"
										$x = $host.UI.RawUI.ReadKey("NoEcho,IncludeKeyDown")
									}
								}
							
								Write-Host "Done." -ForegroundColor Yellow;
							}
							else
							{
								Write-Host "No SPWebApplication alternate Urls defined in the Environment.xml file.." -ForegroundColor Yellow;
							}
														
							# SP Managed Paths
							if( $spWebApplication.SPManagedPaths.SPManagedPath )
							{
								foreach( $spManagedPath in $spWebApplication.SPManagedPaths.SPManagedPath )
								{
									if($Debug)
									{
										Write-Host "Setting up managed path $($spManagedPath.relativeUrl).  Press any key to continue..."
										$x = $host.UI.RawUI.ReadKey("NoEcho,IncludeKeyDown")
									}

									$mamangedPath = New-SPManagedPath -RelativeURL $spManagedPath.relativeUrl -WebApplication $spWebApplication.identity -Explicit:($spManagedPath.explicit -and ( $spManagedPath.explicit -ieq "true" ) );

									if($Debug)
									{
										Write-Host "Set up managed path $($spManagedPath.relativeUrl).  Check state and press any key to continue..."
										$x = $host.UI.RawUI.ReadKey("NoEcho,IncludeKeyDown")
									}
								}

								Write-Host "Done." -ForegroundColor Yellow;
							}
							else
							{
								Write-Host "No SPWebApplication managed paths defined in the Environment.xml file." -ForegroundColor Yellow;
							}

							# SP Web Application smtp server settings
							if( $spWebApplication.Smtp )
							{
								if ( $CONFIGURE_SMTP_SERVER -ne "false" )
								{
									Write-Host "Setting outgoing SMTP settings";
									Set-SPWebApplication -Identity $spWebApplication.identity -SmtpServer $spWebApplication.Smtp.server -OutgoingEmailAddress $spWebApplication.Smtp.fromAddress;
									
									Write-Host "Done." -ForegroundColor Yellow;
								}
								else
								{
									Write-Host "The SMTP server will not be configured because CONFIGURE_SMTP_SERVER is false." -ForegroundColor Yellow;
								}
							}
							else
							{
								Write-Host "No SPWebApplication smtp servers defined in the Environment.xml file." -ForegroundColor Yellow;
							}

							if($Debug)
							{
								Write-Host "Turning off NTLM for public site.  Press any key to continue..."
								$x = $host.UI.RawUI.ReadKey("NoEcho,IncludeKeyDown")
							}

							Write-Host "Turning off NTLM for the public site." -ForegroundColor Green;
							$webApplication = Get-SPWebApplication | where { $_.Name -ieq $spWebApplication.identity };
							$ZoneType = "Internet";
							$Zone = [microsoft.sharepoint.administration.SPURLZone]::$ZoneType;
							$iisSettings = $webApplication.GetIisSettingsWithFallback($Zone);
							$iisSettings.UseWindowsIntegratedAuthentication=$false;
							$webApplication.Update();

							if($Debug)
							{
								Write-Host "NTLM should now be turned off for public site.  Press any key to continue..."
								$x = $host.UI.RawUI.ReadKey("NoEcho,IncludeKeyDown")
							}

							Write-Host "Setting SPWebApplicationPermissiveBrowserFileHandling";
							Set-SPWebApplicationPermissiveBrowserFileHandling -WebApplication $spWebApplication.url;
						}
						
						
						# IIS Certificates
						if( $spWebApplication.IIS.Certificates -and $spWebApplication.IIS.Certificates.Certificate )
						{
							Write-Host "Setting up certificates...";
							
							$certificates = $spWebApplication.IIS.SelectNodes("Certificates/Certificate");
								
							foreach( $iisCertificate in $certificates )
							{
								$cert = ( Get-ChildItem -Path "cert:\localmachine\MY" | where { $_.FriendlyName -ieq $iisCertificate.identity -or $_.Subject -ieq "CN=$($iisCertificate.identity)" });

								if( $cert -ne $null )
								{
									Write-Host "Deleting pre-existing certificate: $($iisCertificate.identity)";
							
									$store = New-Object System.Security.Cryptography.X509Certificates.X509Store "My", "localmachine"
									$store.Open("ReadWrite")
									$store.Remove($cert)
									$store.Close()
								}							

								if($iisCertificate.selfSigned -and $iisCertificate.selfSigned -eq "true"){
									Create-SelfSignedCert $iisCertificate.identity;
									continue;
								}

								Write-Host "Assign the certificate to the https binding";
								$certPath = $iisCertificate.path;
								
								Import-IIS7PfxCertificate -certPath "$certPath" -pfxPass $iisCertificate.password;
							}
							
							Write-Host "Done." -ForegroundColor Yellow;
						}
						else
						{
							Write-Host "No SPWebApplication IIS certificates defined in the Environment.xml file." -ForegroundColor Yellow;
						}							

						# IIS Bindings
						if( $spWebApplication.IIS.WebBindings.WebBinding )
						{
							Write-Host "Importing IIS bindings...";
							
							foreach( $iisWebBinding in $spWebApplication.IIS.WebBindings.WebBinding )
							{
								Import-Module -Name WebAdministration;
																
								$existingBindingConfig = "$($iisWebBinding.ip):$($iisWebBinding.port):$($iisWebBinding.hostheader)";
								$existingBinding = Get-WebBinding | where {$_.bindingInformation -eq $existingBindingConfig -and $_.protocol -eq "$($iisWebBinding.protocol)"};							
								
								if($existingBinding -ne $null){
									Write-Host "Deleting already existing binding $($existingBindingConfig )";
									(Get-WebBinding | where {$_.bindingInformation -eq $existingBindingConfig -and $_.protocol -eq "$($iisWebBinding.protocol)"}) | Remove-WebBinding;
								}
								else{
									Write-Host "A binding with this config does not already exist: $($existingBindingConfig)";
								}
								
								Write-Host "Creating an IIS binding";
								New-WebBinding -Name $iisWebBinding.webApplicationIdentity -Port $iisWebBinding.port -HostHeader $iisWebBinding.hostheader -Protocol $iisWebBinding.protocol -IP $iisWebBinding.ip;
								
								Write-Host "Adding binding to hosts file";
								Windows-UpdateHostsFile 127.0.0.1 $($iisWebBinding.hostheader);
							
								if($iisWebBinding.protocol -eq "https")
								{
									Write-Host "Adding certificate to binding...";
									$commerceCert = ( Get-ChildItem -Path "cert:\localmachine\MY" | where { $_.FriendlyName -ieq $iisWebBinding.certificate });
									$sslBinding = Get-WebBinding -Name $iisWebBinding.webApplicationIdentity -Port $iisWebBinding.port -HostHeader $iisWebBinding.hostheader -Protocol $iisWebBinding.protocol -IP $iisWebBinding.ip;
									$sslBinding.AddSslCertificate( $commerceCert.ThumbPrint, "MY" );
								}
							}

							Write-Host "Done." -ForegroundColor Yellow;
						}
						else
						{
							Write-Host "No SPWebApplication IIS web bindings defined in the Environment.xml file." -ForegroundColor Yellow;
						}

						# SP Web Application solutions
						if( $spWebApplication.DeploySolutions.Solution )
						{
							Write-Host "Deploying SPWebApplication solutions";
							foreach( $spWebApplicationDeploySolution in $spWebApplication.DeploySolutions.Solution )
							{
								Write-Host "$($spWebApplicationDeploySolution.identity)";

								$spFarmSolution = $spFarm.Solutions.Solution | where { $_.identity -ieq $spWebApplicationDeploySolution.identity };
								
								if( ( $GenerateWSP -and ( $spFarmSolution.generate -and ( $spFarmSolution.generate -ieq "true" ) ) ) -or ( $spFarmSolution.deleteIfExists -and ( $spFarmSolution.deleteIfExists -ieq "true" ) ) )
								{
									$wspName = ( Split-Path -Path $spFarmSolution.path -Leaf );

									if($Debug)
									{
										Write-Host "Deploying WSP $($wspName).  Press any key to continue..."
										$x = $host.UI.RawUI.ReadKey("NoEcho,IncludeKeyDown")
									}

									if($SP2013)
									{
										DeployAndWait-SPSolution -WSPName $wspName -WebApplication $spWebApplication.identity -SP2013;
									}
									else
									{
										DeployAndWait-SPSolution -WSPName $wspName -WebApplication $spWebApplication.identity;
									}
									

									if($Debug)
									{
										Write-Host "Deployed WSP $($wspName).  Check state and press any key to continue..."
										$x = $host.UI.RawUI.ReadKey("NoEcho,IncludeKeyDown")
									}
								}
							}

							Write-Host "Done." -ForegroundColor Yellow;
						}
						else
						{
							Write-Host "No SPWebApplication solutions to deploy." -ForegroundColor Yellow;
						}

						# Update any feature XML files with the proper values based on the Environment.xml file.
						if( $spWebApplication.UpdateXmls.UpdateXml )
						{
							foreach( $updateXml in $spWebApplication.UpdateXmls.UpdateXml )
							{
								$namespaces = @{};
								if( $updateXml.Namespaces.Namespace)
								{
									foreach( $updateXmlNamespace in $updateXml.Namespaces.Namespace)
									{
										$namespaces.Add( $updateXmlNamespace.key, $updateXmlNamespace.value );
									}
								}
								
								switch( $updateXml.type )
								{
									"UpdateAttribute"
									{
										Update-XmlAttribute -FilePath $updateXml.filePath -XPath $updateXml.xpath -UpdateAttribute -AttributeName $updateXml.attributeName -AttributeValue $updateXml.attributeValue -Namespaces $namespaces;
									}
									"RemoveAttribute"
									{
										Update-XmlAttribute -FilePath $updateXml.filePath -XPath $updateXml.xpath -RemoveAttribute -AttributeName $updateXml.attributeName -Namespaces $namespaces;
									}
								}
							}
						}
						else
						{
							Write-Host "No xml files to update." -ForegroundColor Yellow;
						}

						if( $spWebApplication.ActivateFeatures.Feature )
						{
							Write-Host "Activating SPWebApplication features";

							# WORKAROUND - The Site Definitions cache does not get refreshed in the current session
							# 	If you want to create a new site collection based on a site definition file (ONET.xml) that you deployed during the current PowerShell session, you need to spawn a new session.
							#	See http://social.msdn.microsoft.com/Forums/en/sharepoint2010general/thread/5ea46e07-a4b0-4bab-b698-9b6eeb695607
							
							# Copy stuff to the list
							$applicationFeatures = @()
							foreach( $spFarmActivateFeature in $spWebApplication.ActivateFeatures.Feature )
							{
								$feature = New-Feature($spFarmActivateFeature.identity, $spFarmActivateFeature.deleteIfExists, $spFarmActivateFeature.reactivate, $spFarmActivateFeature.activateUsingHttpGet)
								$applicationFeatures += $feature
							}

							$job = Start-Job -ArgumentList @( $applicationFeatures, $DEPLOYMENT_DIRECTORY, $spWebApplication.url ) -ScriptBlock {
							
								PARAM
								(
									$applicationFeatures, $DEPLOYMENT_DIRECTORY, $url
								)

								if( -not ( Get-PSSnapin  | where { $_.Name -eq "Microsoft.SharePoint.PowerShell" } ) )
								{
									Add-PSSnapin Microsoft.SharePoint.PowerShell;
								}
								
								. ( Join-Path -Path $DEPLOYMENT_DIRECTORY -ChildPath "UtilityScripts\IIS.ps1");
								. ( Join-Path -Path $DEPLOYMENT_DIRECTORY -ChildPath "SharePoint2010.ps1");

								foreach( $spWebApplicationActivateFeature in $applicationFeatures )
								{
									if($spWebApplicationActivateFeature.identity){
										if( $spWebApplicationActivateFeature.deleteIfExists -and ( $spWebApplicationActivateFeature.deleteIfExists -ieq "true" ) )
										{
											DisableAndWait-SPFeature -FeatureName $spWebApplicationActivateFeature.identity -Url $spWebApplication.url;
											
											$reactivate = "true";
											if($spWebApplicationActivateFeature.reactivate){
												$reactivate = $spWebApplicationActivateFeature.reactivate;
											}
											
											if($reactivate -ieq "true"){
												EnableAndWait-SPFeature -FeatureName $spWebApplicationActivateFeature.identity -Url $url -ActivateUsingHttpGet ( $spWebApplicationActivateFeature.activateUsingHttpGet -and ( $spWebApplicationActivateFeature.activateUsingHttpGet -ieq "true" ) );
											}											
										}
										else{
											EnableAndWait-SPFeature -FeatureName $spWebApplicationActivateFeature.identity -Url $url -ActivateUsingHttpGet ( $spWebApplicationActivateFeature.activateUsingHttpGet -and ( $spWebApplicationActivateFeature.activateUsingHttpGet -ieq "true" ) );
										}
									}
								}
							}
																		
							while( ( $job.State -ne "Completed" ) -or $job.HasMoreData )
							{
								Receive-Job -Job $job;
								Start-Sleep -Seconds 1;
							}
							Remove-Job -Job $job;
							
							Write-Host "Done." -ForegroundColor Yellow;
						}
						else
						{
							Write-Host "No SPWebApplication defined in the Environment.xml file." -ForegroundColor Yellow;
						}
						
						if( $spWebApplication.SPSites.SPSite )
						{
							foreach( $spSite in $spWebApplication.SPSites.SPSite )
							{
								if( $spSite.deleteIfExists -and ( $spSite.deleteIfExists -ieq "true" ) )
								{
									if( Get-SPSite | where { $_.Url -ieq $spSite.url } )
									{
										Write-Host "Removing SPSite $($spSite.Identity)";
										
										if($Debug)
										{
											Write-Host "Removing old SPSite  $($spSite.Identity).  Press any key to continue..."
											$x = $host.UI.RawUI.ReadKey("NoEcho,IncludeKeyDown")
										}

										Remove-SPSite -Identity $spSite.url -Confirm:$false;

										if($Debug)
										{
											Write-Host "Removed old SPSite  $($spSite.Identity).  Check state and press any key to continue..."
											$x = $host.UI.RawUI.ReadKey("NoEcho,IncludeKeyDown")
										}
									}
								}

								# Create the web application only if the create flag is true
								if( $spSite.create -and ( $spSite.create -ieq "true" ) )
								{

									$siteOwnerUserAccount = $configuration.UserAccounts.UserAccount | where { $_.identity -ieq $spSite.SiteOwner.userAccountIdentity};

									if($Debug)
									{
										Write-Host "Creating new SPSite  $($spSite.Identity).  Press any key to continue..."
										$x = $host.UI.RawUI.ReadKey("NoEcho,IncludeKeyDown")
									}

									# WORKAROUND - The Site Definitions cache does not get refreshed in the current session
									# 	If you want to create a new site collection based on a site definition file (ONET.xml) that you deployed during the current PowerShell session, you need to spawn a new session.
									#	See http://social.msdn.microsoft.com/Forums/en/sharepoint2010general/thread/5ea46e07-a4b0-4bab-b698-9b6eeb695607
									$job = Start-Job -ArgumentList @($spSite.identity, $spSite.url, $spSite.language, $spSite.template, $siteOwnerUserAccount, $siteOwnerUserAccount.domain, $siteOwnerUserAccount.username, $siteOwnerUserAccount.email, $SP2013) -ScriptBlock  {
										
										PARAM
										(
											$identity,
											$url, 
											$language,
											$template,
											$siteOwnerUserAccount,
											$domain,
											$username,
											$email,
											$SP2013 
											
										)

										if( -not ( Get-PSSnapIn | where { $_.Name -eq "Microsoft.SharePoint.PowerShell" } ) )
										{
											Add-PSSnapin Microsoft.SharePoint.PowerShell;
										}
										
										Write-Host "Creating SPSite $($identity) with url $($url)";
										Write-Host "....with $($domain)\$($username)";
										if($SP2013)
										{
											$site = New-SPSite -Url $url -OwnerAlias "$($domain)\$($username)" -Language $language -Name $identity -Template $template -OwnerEmail $email -CompatibilityLevel 14;
										}
										else
										{
											$site = New-SPSite -Url $url -OwnerAlias "$($domain)\$($username)" -Language $language -Name $identity -Template $template -OwnerEmail $email 
										}
										
										$site.Dispose();

										if( $Error )
										{
											foreach( $errorRecord in $Error )
											{
												Write-Host -Object $errorRecord -ForegroundColor Red;
												Write-Host -Object $errorRecord.InvocationInfo.PositionMessage -ForegroundColor Red;
											}
											
											throw;
										}
									}
									
									while( ( $job.State -ne "Completed" ) -or $job.HasMoreData )
									{
										Receive-Job -Job $job;
										Start-Sleep -Seconds 1;
									}
									Remove-Job -Job $job;

									if($Debug)
									{
										Write-Host "Was the SPSite created?  $($spSite.Identity).  Check state and press any key to continue..."
										$x = $host.UI.RawUI.ReadKey("NoEcho,IncludeKeyDown")
									}

									$site = Get-SPSite $spSite.url
									$web = $site.RootWeb;

									if($Debug)
									{
										Write-Host "About to add the catalog user to the  $($spSite.identity) Members Group.  Press any key to continue..."
										$x = $host.UI.RawUI.ReadKey("NoEcho,IncludeKeyDown")
									}

									$catalogUserAccount = $configuration.UserAccounts.UserAccount | where { $_.identity -ieq "CSCatalogUser"};
									Write-Host "Adding $($catalogUserAccount.domain)\$($catalogUserAccount.username) to $($spSite.identity) Members Group";
									$newUser = New-SPUser -UserAlias "i:0#.w|$($catalogUserAccount.domain)\$($catalogUserAccount.username)" -Web $spSite.url -Group "$($spSite.identity) Members";
									$web.SiteGroups["$($spSite.identity) Members"].AddUser($newUser);

									if($Debug)
									{
										Write-Host "About to add the All Authenticated Users to the  $($spSite.identity) Members Group.  Press any key to continue..."
										$x = $host.UI.RawUI.ReadKey("NoEcho,IncludeKeyDown")
									}

									Write-Host "Adding All Authenticated Users to $($spSite.identity) Members Group";
									$newUser = New-SPUser -UserAlias "c:0(.s|true" -Web $spSite.url -Group "$($spSite.identity) Members";
									$web.SiteGroups["$($spSite.identity) Members"].AddUser($newUser);
									
									if($Debug)
									{
										Write-Host "Adding SignedInClient permission level and role definition.  Press any key to continue..."
										$x = $host.UI.RawUI.ReadKey("NoEcho,IncludeKeyDown")
									}

									$permissionLevelName ="SignedInClient"
									$permissionLevelDescription ="A permission for signed In users"
									$groupName = "$($spSite.identity) Members"
									$spRoleDefinition = New-Object Microsoft.SharePoint.SPRoleDefinition
									$spRoleDefinition.Name = $permissionLevelName
									$spRoleDefinition.Description = $permissionLevelDescription
									$spRoleDefinition.BasePermissions = "ViewListItems, ViewPages, Open"
									$web.RoleDefinitions.Add($spRoleDefinition)

									if($Debug)
									{
										Write-Host "Adding role assignment for that stuff.  Press any key to continue..."
										$x = $host.UI.RawUI.ReadKey("NoEcho,IncludeKeyDown")
									}

									$newPermissionLevel = $web.RoleDefinitions[$permissionLevelName]
									$group = $web.SiteGroups[$groupName]
									$roleAssignment = New-Object Microsoft.SharePoint.SPRoleAssignment -argumentlist $group
									$roleAssignment.RoleDefinitionBindings.Add($newPermissionLevel)
									$web.RoleAssignments.Add($roleAssignment)

									if($Debug)
									{
										Write-Host "About to save the SPWeb.  Press any key to continue..."
										$x = $host.UI.RawUI.ReadKey("NoEcho,IncludeKeyDown")
									}

									$web.Update();
									$web.Dispose();
									$site.Dispose();

									if($Debug)
									{
										Write-Host "Saved, now check the state and press any key to continue..."
										$x = $host.UI.RawUI.ReadKey("NoEcho,IncludeKeyDown")
									}


								}
								
								if($spSite.anonymousState -and ($spSite.anonymousState -ne "")){
									$site = Get-SPSite $spSite.url
									$web = $site.RootWeb;
									
									switch($spSite.anonymousState.ToLower())
									{
										"on"
										{
											$web.AnonymousState = 2;
											Write-Host "Set $($spSite.url) anonymous state to On." -ForegroundColor Green;
										}
										"enabled"
										{
											$web.AnonymousState = 1;
											Write-Host "Set $($spSite.url) anonymous state to Enabled." -ForegroundColor Green;
										}
										"disabled"
										{
											$web.AnonymousState = 0;
											Write-Host "Set $($spSite.url) anonymous state to Disabled." -ForegroundColor Green;
										}											
									}
									
									$web.Update();
									$web.Dispose();
									$site.Dispose();
								}
								
								if($Debug)
								{
									Write-Host "Setting custom master page $($spSite.customMasterUrl). Press any key to continue..."
									$x = $host.UI.RawUI.ReadKey("NoEcho,IncludeKeyDown")
								}

								if($spSite.customMasterUrl -and ($spSite.customMasterUrl -ne "")){
									$site = Get-SPSite $spSite.url
									$web = $site.RootWeb;
									
									Write-Host "Setting masterpage MasterUrl to $($spSite.customMasterUrl)" -ForegroundColor Green;
									#$web.AllowUnsafeUpdates = $true;
									#$web.MasterUrl = $spSite.masterUrl;
									$web.CustomMasterUrl = $spSite.customMasterUrl;
									$web.Update();
									#$web.AllowUnsafeUpdates = $false;	
									$web.Dispose();
									$site.Dispose();									
								}

								if($Debug)
								{
									Write-Host "Set the master page. Check the state and press any key to continue..."
									$x = $host.UI.RawUI.ReadKey("NoEcho,IncludeKeyDown")
								}
								
								Write-Host "Activating SPSite features";
								if( $spSite.ActivateFeatures.Feature )
								{

									if($Debug)
									{
										Write-Host "Activating Site Features.  Press any key to continue..."
										$x = $host.UI.RawUI.ReadKey("NoEcho,IncludeKeyDown")
									}

									# WORKAROUND - The Site Definitions cache does not get refreshed in the current session
									# 	If you want to create a new site collection based on a site definition file (ONET.xml) that you deployed during the current PowerShell session, you need to spawn a new session.
									#	See http://social.msdn.microsoft.com/Forums/en/sharepoint2010general/thread/5ea46e07-a4b0-4bab-b698-9b6eeb695607

									$siteFeatures = @()
									foreach( $spSiteActivateFeature in $spSite.ActivateFeatures.Feature )
									{
										$feature = New-Feature($spSiteActivateFeature.identity, $spSiteActivateFeature.deleteIfExists, $spSiteActivateFeature.reactivate, $spSiteActivateFeature.activateUsingHttpGet)
										$siteFeatures += $feature
									}

									$job = Start-Job -ArgumentList @( $siteFeatures, $DEPLOYMENT_DIRECTORY, $spSite.url ) -ScriptBlock {
										
										PARAM
										(
											$siteFeatures, $DEPLOYMENT_DIRECTORY, $url
										)

										if( -not ( Get-PSSnapin  | where { $_.Name -eq "Microsoft.SharePoint.PowerShell" } ) )
										{
											Add-PSSnapin Microsoft.SharePoint.PowerShell;
										}
										
										. ( Join-Path -Path $DEPLOYMENT_DIRECTORY -ChildPath "UtilityScripts\IIS.ps1");
										. ( Join-Path -Path $DEPLOYMENT_DIRECTORY -ChildPath "SharePoint2010.ps1");

										foreach( $spSiteActivateFeature in $siteFeatures )
										{
											if($spSiteActivateFeature.identity){
												if( $spSiteActivateFeature.deleteIfExists -and ( $spSiteActivateFeature.deleteIfExists -ieq "true" ) )
												{
													DisableAndWait-SPFeature -FeatureName $spSiteActivateFeature.identity -Url $spSite.url;
													
													$reactivate = "true";
													if($spSiteActivateFeature.reactivate){
														$reactivate = $spSiteActivateFeature.reactivate;
													}
													
													if($reactivate -ieq "true"){
														EnableAndWait-SPFeature -FeatureName $spSiteActivateFeature.identity -Url $url -ActivateUsingHttpGet ( $spSiteActivateFeature.activateUsingHttpGet -and ( $spSiteActivateFeature.activateUsingHttpGet -ieq "true" ) );
													}
												}
												else{
													EnableAndWait-SPFeature -FeatureName $spSiteActivateFeature.identity -Url $url -ActivateUsingHttpGet ( $spSiteActivateFeature.activateUsingHttpGet -and ( $spSiteActivateFeature.activateUsingHttpGet -ieq "true" ) );
												}
											}
										}

										if( $Error )
										{
											foreach( $errorRecord in $Error )
											{
												Write-Host -Object $errorRecord -ForegroundColor Red;
												Write-Host -Object $errorRecord.InvocationInfo.PositionMessage -ForegroundColor Red;
											}
											
											throw;
										}
									}
																				
									while( ( $job.State -ne "Completed" ) -or $job.HasMoreData )
									{
										Receive-Job -Job $job;
										Start-Sleep -Seconds 1;
									}
									Remove-Job -Job $job;
									
									if($Debug)
									{
										Write-Host "Site features should now be active - go check and then press any key to continue..."
										$x = $host.UI.RawUI.ReadKey("NoEcho,IncludeKeyDown")
									}

									Write-Host "Done." -ForegroundColor Yellow;
								}
								else
								{
									Write-Host "No SPSite activate features defined in the Environment.xml file." -ForegroundColor Yellow;
								}
							
								if( $spSite.VariationLabels )
								{
									$rootWeb = ( ( Get-SPSite -Identity $spSite.url ).OpenWeb() );
									if( $spSite.VariationLabels )
									{
										$variationLabels = @();
										foreach( $variationLabel in $spSite.VariationLabels.VariationLabel )
										{
											$variationLabels += New-Object Object | 
												Add-Member -MemberType NoteProperty -Name "Title" -Value $variationLabel.title -PassThru |
												Add-Member -MemberType NoteProperty -Name "Description" -Value $variationLabel.descripion -PassThru |
												Add-Member -MemberType NoteProperty -Name "FlagControlDisplayName" -Value $variationLabel.flagControlDisplayName -PassThru |
												Add-Member -MemberType NoteProperty -Name "Language" -Value $variationLabel.language -PassThru |
												Add-Member -MemberType NoteProperty -Name "Locale" -Value $variationLabel.locale -PassThru |
												Add-Member -MemberType NoteProperty -Name "HierarchyCreationMode" -Value $variationLabel.hierarchyCreationMode -PassThru |
												Add-Member -MemberType NoteProperty -Name "IsSource" -Value ( $variationLabel.isSource -ieq "true" ) -PassThru |
												Add-Member -MemberType NoteProperty -Name "SourceVarRootWebTemplatePropertyName" -Value $variationLabel.sourceSiteTemplate -PassThru;
										}
									}
									
									Create-SPVariations -Site ( Get-SPSite -Identity $spSite.url ) -RootWeb $rootWeb -VariantLabels $variationLabels;
								}
								
								$isVariationSite = $false;
								if( $spSite.isVariationSite -and ( $spSite.isVariationSite -ieq "true" ))
								{
									Disable-VariationsPageJobs -Site ( Get-SPSite -Identity $spSite.url );
									$isVariationSite = $true;
								}

								if( $spSite.SPWebs.SPWeb )
								{
									foreach( $spWeb in $spSite.SPWebs.SPWeb )
									{
										if( $spWeb.deleteIfExists -and ( $spWeb.deleteIfExists -ieq "true" ) )
										{
											if( Get-SPWeb -Site $spSite.url | where { $_.Url -ieq $spWeb.url } )
											{
												Write-Host "Removing SPWeb $($spWeb.Identity)";
												Remove-SPWeb -Identity $spWeb.url -Confirm:$false;
											}
										}

										# The create attribute is set to 'false' if this web has already been created via a variant.
										if( $spWeb.create -and ( $spWeb.create -ieq "true" ) )
										{
											if($Debug)
											{
												Write-Host "Creating the SPWeb $($spWeb.Identity).  Press any key to continue..."
												$x = $host.UI.RawUI.ReadKey("NoEcho,IncludeKeyDown")
											}

											Write-Host "Creating SPWeb $($spWeb.Identity)";
											# WORKAROUND - The Site Definitions cache does not get refreshed in the current session
											# 	If you want to create a new site collection based on a site definition file (ONET.xml) that you deployed during the current PowerShell session, you need to spawn a new session.
											#	See http://social.msdn.microsoft.com/Forums/en/sharepoint2010general/thread/5ea46e07-a4b0-4bab-b698-9b6eeb695607
											$job = Start-Job -ArgumentList @($spWeb.url, $spWeb.identity, $spWeb.template) -ScriptBlock {
											
												PARAM
												(
													$url,
													$identity, 
													$template							
												)

												if( -not ( Get-PSSnapin | where { $_.Name -eq "Microsoft.SharePoint.PowerShell" } )  )
												{
													Add-PSSnapin Microsoft.SharePoint.PowerShell;
												}

												Write-Host "Creating $($identity) web at $($url)";
												$web = New-SPWeb -Url $url -Name $identity -Template $template -AddToTopNav;
												$web.Dispose();

												if( $Error )
												{
													foreach( $errorRecord in $Error )
													{
														Write-Host -Object $errorRecord -ForegroundColor Red;
														Write-Host -Object $errorRecord.InvocationInfo.PositionMessage -ForegroundColor Red;
													}
													
													throw;
												}
											}
										
											while( ( $job.State -ne "Completed" ) -or $job.HasMoreData )
											{
												Receive-Job -Job $job;
												Start-Sleep -Seconds 1;
											}
											Remove-Job -Job $job;

											if($Debug)
											{
												Write-Host "Created SPWeb $($spWeb.Identity), go take a look and then press any key to continue..."
												$x = $host.UI.RawUI.ReadKey("NoEcho,IncludeKeyDown")
											}
										}

										Write-Host "Activating SPWeb features";
										if( $spWeb.ActivateFeatures.Feature )
										{
											if($Debug)
											{
												Write-Host "Activating Site Featuress.  Press any key to continue..."
												$x = $host.UI.RawUI.ReadKey("NoEcho,IncludeKeyDown")
											}

											# 	If you want to create a new site collection based on a site definition file (ONET.xml) that you deployed during the current PowerShell session, you need to spawn a new session.
											#	See http://social.msdn.microsoft.com/Forums/en/sharepoint2010general/thread/5ea46e07-a4b0-4bab-b698-9b6eeb695607

											$webFeatures = @()
											foreach( $spWebActivateFeature in $spWeb.ActivateFeatures.Feature )
											{
												$feature = New-Feature($spWebActivateFeature.identity, $spWebActivateFeature.deleteIfExists, $spWebActivateFeature.reactivate, $spWebActivateFeature.activateUsingHttpGet)
												$webFeatures += $feature
											}

											$job = Start-Job -ArgumentList @( $webFeatures, $DEPLOYMENT_DIRECTORY, $spWeb.url, $spSite.url, $spSite.isSource ) -ScriptBlock {
											
												PARAM
												(
													$webFeatures, $DEPLOYMENT_DIRECTORY, $url, $siteUrl, $isSource
												)

												if( -not ( Get-PSSnapin  | where { $_.Name -eq "Microsoft.SharePoint.PowerShell" } ) )
												{
													Add-PSSnapin Microsoft.SharePoint.PowerShell;
												}
												
												. ( Join-Path -Path $DEPLOYMENT_DIRECTORY -ChildPath "UtilityScripts\IIS.ps1");
												. ( Join-Path -Path $DEPLOYMENT_DIRECTORY -ChildPath "SharePoint2010.ps1");

												foreach( $spWebActivateFeature in $webFeatures )
												{
													if($spWebActivateFeature.identity){
														if( $spWebActivateFeature.deleteIfExists -and ( $spWebActivateFeature.deleteIfExists -ieq "true" ) )
														{
															DisableAndWait-SPFeature -FeatureName $spWebActivateFeature.identity -Url $url;
															
															$reactivate = "true";
															if($spWebActivateFeature.reactivate){
																$reactivate = $spWebActivateFeature.reactivate;
															}
															
															if($reactivate -ieq "true"){
																EnableAndWait-SPFeature -FeatureName $spWebActivateFeature.identity -Url $url -ActivateUsingHttpGet ( $spWebActivateFeature.activateUsingHttpGet -and ( $spWebActivateFeature.activateUsingHttpGet -ieq "true" ) );
															}															
														}
														else{
															EnableAndWait-SPFeature -FeatureName $spWebActivateFeature.identity -Url $url -ActivateUsingHttpGet ( $spWebActivateFeature.activateUsingHttpGet -and ( $spWebActivateFeature.activateUsingHttpGet -ieq "true" ) );
														}
													}
												}
												
												$rootWeb = ( ( Get-SPSite -Identity $spSite.url ).OpenWeb() );
												if($spWeb.isSource -and ($spWeb.isSource -eq "true")){
													Run-VariationsPropagateSiteJob -Site ( Get-SPSite -Identity $spSite.url ) -RootWeb $rootWeb;												   
												}
												
												if( $Error )
												{
													foreach( $errorRecord in $Error )
													{
														Write-Host -Object $errorRecord -ForegroundColor Red;
														Write-Host -Object $errorRecord.InvocationInfo.PositionMessage -ForegroundColor Red;
													}
													
													throw;
												}
											}
											
											while( ( $job.State -ne "Completed" ) -or $job.HasMoreData )
											{
												Receive-Job -Job $job;
												Start-Sleep -Seconds 1;
											}
											Remove-Job -Job $job;
											
											if($Debug)
											{
												Write-Host "Activated Site Features - check state and then press any key to continue..."
												$x = $host.UI.RawUI.ReadKey("NoEcho,IncludeKeyDown")
											}

											Write-Host "Done." -ForegroundColor Yellow;
										}
										else
										{
											Write-Host "No SPWeb activate features defined in the Environment.xml file." -ForegroundColor Yellow;
										}
									}
								}
								else
								{
									Write-Host "No SPWebs defined in the Environment.xml file." -ForegroundColor Yellow;
								}

								
								if( $isVariationSite -eq $true)
								{
									Enable-VariationsPageJobs -Site ( Get-SPSite -Identity $spSite.url );	
								}
								
								if($spSite.Restore -and $spSite.Restore.settings -and $spSite.Restore.backupFile){
									$settingsFullPath = ( Join-Path -Path $SP_BACKUP_WORKING_FOLDER -ChildPath $spSite.Restore.settings);
								
									Import-SharePointContent $settingsFullPath $SP_BACKUP_WORKING_FOLDER $spSite.Restore.backupFile $spSite.url $SP_STSADM_PATH;
									
									if($spSite.Restore.backupUser -and $spSite.Restore.backupUser -ne ""){
										Write-Host "Adding SharePoint backup user.";
										
										$backupUserAccount = $configuration.UserAccounts.UserAccount | where { $_.identity -ieq $spSite.Restore.backupUser };
										$backupUserAccountLogin = $backupUserAccount.domain + "\" + $backupUserAccount.username;
										
										$webApplication = Get-SPWebApplication | where { $_.Name -ieq $spWebApplication.identity };
										$policy = $webApplication.Policies.Add($backupUserAccountLogin, "SharePoint Backup");
										$policy.PolicyRoleBindings.Add($webApplication.PolicyRoles.GetSpecialRole([Microsoft.SharePoint.Administration.SPPolicyRoleType]::FullControl));
										$webApplication.Update(); 
									}
								}
							}
						}
						else
						{
							Write-Host "No SPSites defined in the Environment.xml file." -ForegroundColor Yellow;
						}
					}
					
					Stop-SPAssignment -Global;
				}
			}
		}
		# End SharePoint Stuff
		
		if( $configuration.CommerceServer -and $configuration.CommerceServer.Pipelines)
		{
			Write-Host "Registering pipelines";
			
			$pipelines = $configuration.CommerceServer.SelectNodes("Pipelines/Pipeline");
			
			foreach( $pipeline in $pipelines )
			{
				$dllPath = "$($pipeline.dllPath)"
				$tlbPath32 = "$($pipeline.dllPath).32.tlb"
				$tlbPath64 = "$($pipeline.dllPath).64.tlb"
				$argumentList64 = @( "`"$dllPath`"", "/codebase", "/tlb:`"$tlbPath64`"" );
				$argumentList32 = @( "`"$dllPath`"", "/codebase", "/tlb:`"$tlbPath32`"" );
				$argumentListReg = @( "/s", $pipeline.regPath );

				if (Test-Path $dllPath)
				{
					Write-Host "Registering $($pipeline.identity)";
					Write-Host "64 bit" -ForegroundColor Green;
					$returnValue = Start-Process -FilePath $REGASM_64_PATH -Wait -NoNewWindow -ArgumentList $argumentList64 -PassThru;
					Write-Host $returnValue.StandardOutput;
					if( $returnValue.ExitCode -ne 0 )
					{
						Write-Error -Message $returnValue.StandardError -ErrorId $returnValue.ExitCode ;
					}

					Write-Host "32 bit" -ForegroundColor Green;
					$returnValue = Start-Process -FilePath $REGASM_PATH -Wait -NoNewWindow -ArgumentList $argumentList32 -PassThru;
					Write-Host $returnValue.StandardOutput;
					if( $returnValue.ExitCode -ne 0 )
					{
						Write-Error -Message $returnValue.StandardError -ErrorId $returnValue.ExitCode ;
					}

					Write-Host "Adding registry entries";
					Write-Host "64 bit" -ForegroundColor Green;
					$returnValue = Start-Process -FilePath $REGEDIT_64_PATH -Wait -NoNewWindow -ArgumentList $argumentListReg -PassThru;
					Write-Host $returnValue.StandardOutput;
					if( $returnValue.ExitCode -ne 0 )
					{
						Write-Error -Message $returnValue.StandardError -ErrorId $returnValue.ExitCode ;
					}

					Write-Host "32 bit" -ForegroundColor Green;
					$returnValue = Start-Process -FilePath $REGEDIT_PATH -Wait -NoNewWindow -ArgumentList  $argumentListReg -PassThru;
					Write-Host $returnValue.StandardOutput;
					if( $returnValue.ExitCode -ne 0 )
					{
						Write-Error -Message $returnValue.StandardError -ErrorId $returnValue.ExitCode ;
					}
				}
				else
				{
					Write-Host -Message "Could not register pipeline components in '$dllPath' - file not found." -ForegroundColor Yellow;
				}
			}
		}		

		if($configuration.Files -and $configuration.Files.File){
			
			$files = $configuration.Files.SelectNodes("File");
		
			foreach($file in $files){
				if($file.UserAccounts -and $file.UserAccounts.UserAccount){
					Write-Host "Creating permissions for $($file.path)";
					
					$users = $file.SelectNodes("UserAccounts/UserAccount");
				
					foreach($user in $users){
						$userAccount = $configuration.UserAccounts.UserAccount | where { $_.identity -ieq $user.userAccountIdentity};
						
						Write-Host "Creating permissions for $($userAccount.domain)\$($userAccount.username) on $($file.path)";
						Windows-GrantFullReadWriteAccessToFile "$($file.path)" "$($userAccount.domain)\$($userAccount.username)";
					}
				}				
			}
		}
		
		if($configuration.Registry -and $configuration.Registry.File){
		
			Write-Host "Running registry files..." -ForegroundColor Green;
			
			$files = $configuration.Registry.SelectNodes("File");
			
			foreach( $file in $files )
			{
				$argumentListReg = @( "/s", $file.path );
			
				Write-Host "Running file $($file.path)";
				Write-Host "64 bit" -ForegroundColor Green;
				$returnValue = Start-Process -FilePath $REGEDIT_64_PATH -Wait -NoNewWindow -ArgumentList $argumentListReg -PassThru;
				Write-Host $returnValue.StandardOutput;
				if( $returnValue.ExitCode -ne 0 )
				{
					Write-Error -Message $returnValue.StandardError -ErrorId $returnValue.ExitCode ;
				}

				Write-Host "32 bit" -ForegroundColor Green;
				$returnValue = Start-Process -FilePath $REGEDIT_PATH -Wait -NoNewWindow -ArgumentList  $argumentListReg -PassThru;
				Write-Host $returnValue.StandardOutput;
				if( $returnValue.ExitCode -ne 0 )
				{
					Write-Error -Message $returnValue.StandardError -ErrorId $returnValue.ExitCode ;
				}
			}
		}		

		if( $Postcompile ){
			Write-Host "Running post-compile." -ForegroundColor Green;
			
			# File tokens
			if( $configuration.FileTokens -and $configuration.FileTokens.FileToken ){
				
				$files = $configuration.SelectNodes("FileTokens/FileToken");
				
				$TOKENS=@(
						"#CatalogDBName=$(( $configuration.CommerceServer.Databases.Database | where { $_.identity -ieq "CSCatalog" }).databaseName )",
						"#PrivateSiteDomain=$(( $configuration.Variables.Variable | where { $_.identity -ieq "PrivateUrlDomain" }).value )",
						"#ExtendedDBName=$(( $configuration.CommerceServer.Databases.Database | where { $_.identity -ieq "CSExtended" }).databaseName )",
						"#ProjectName=$($PROJECT_NAME)",
						"#DomainName=$($DOMAIN_NAME)",
						"#RuntimeUser=$(( $configuration.UserAccounts.UserAccount | where { $_.identity -ieq "CSRuntimeUser" }).username )"
						 );
				
				Write-Host "Replacing file tokens";				
				foreach( $file in $files)
				{
					Write-Host "Removing read-only flag for $($file.filePath)" ;
					# make sure the  file is not read only
					Set-ItemProperty ($file.filePath) -name IsReadOnly -value $false
											
					Write-Host "$($file.filePath)" -ForegroundColor Green;
					foreach ( $token in $TOKENS ) 
					{		
						$content = Get-Content -path $file.filePath;
						$keyValue = $token -split "=";			
						$content -replace $keyValue[0], $keyValue[1] | Set-Content $file.filePath;
					}
				}
			}
		}
		
		$success = "SUCCESSFUL";
	}
	}
	catch
	{
		foreach( $errorRecord in $Error )
		{
			Write-Host -Object $errorRecord -ForegroundColor Red;
			Write-Host -Object $errorRecord.InvocationInfo.PositionMessage -ForegroundColor Red;
		}
	}
	finally
	{
		$scriptEndDate = Get-Date;
		$scriptDuration = New-TimeSpan -Start $scriptStartDate -End $scriptEndDate;
		Write-Host "Script ended @ $($scriptEndDate) ( $($scriptDuration.Minutes) minutes, $($scriptDuration.Seconds) seconds. ).";

		if( Get-WinEvent -ErrorAction SilentlyContinue -MaxEvents 1 -FilterHashTable @{LogName="*"; StartTime=$scriptStartDate} -Oldest | where { $_.LevelDisplayName -ieq 'Error' -or $_.LevelDisplayName -eq 'Critical' } )
		{
			Get-WinEvent -ErrorAction SilentlyContinue -FilterHashTable @{LogName="*"; StartTime=$scriptStartDate} -Oldest | where { $_.LevelDisplayName -ieq 'Error' -or $_.LevelDisplayName -eq 'Critical' } | Format-List;
		}
		
		if( Get-Job )
		{
			Stop-Job -Name *;
		}

        try {
		    Stop-Transcript -ErrorAction SilentlyContinue;
        }
        catch {
            # workaround for hosts which don't support transcription
        }
	}
	
}