﻿#Requires -Version 2
$ErrorActionPreference = "Stop";
Set-PSDebug -Strict

# Adapted from an example on technet.

<#
.Synopsis
    Creates a farm
.Description
    Creates a farm using the provided parameters
.Example
    New-SharePointFarm -DatabaseAccessUsername "DOMAIN\username" -DatabaseAccessPassword "Cactus123" -DatabaseServer SQL01\instancename -FarmName TestFarm
.Parameter DatabaseAccessUsername
    The farm account username.  Needs to be a string.
.Parameter DatabaseAccessPassword
    The farm account password.  Needs to be a string.    
.Parameter DatabaseServer
    The SQL server name
.Parameter Passphrase
    The secret string that must be used to add servers to the farm.
.Parameter Port
	The port for Central Administration.    
.Parameter AdminAuthMethod
    The authentication method for the central admin site
.Parameter FarmName
    The name of the farm
.Link
    Install-SharePoint
    Join-SharePointFarm
#>
function New-SharePointFarm
{
    [CmdletBinding()]
    param 
    (    
        [Parameter(Mandatory=$true)][ValidateNotNullOrEmpty()]
        [String]$DatabaseAccessUsername,
        
        [Parameter(Mandatory=$true)][ValidateNotNullOrEmpty()]
        [System.Security.SecureString]$DatabaseAccessPassword,
        
        [Parameter(Mandatory=$true)][ValidateNotNullOrEmpty()]
        [String]$DatabaseServer,
        
        [Parameter(Mandatory=$false)][ValidateScript({$_.Length -ne 0})]
        [System.Security.SecureString]$Passphrase,
        
        [Parameter(Mandatory=$false)][ValidateRange(1024, 65535)]
        [int]$Port = "1111",
        
        [Parameter(Mandatory=$false)][ValidateSet("NTLM", "Kerberos")]
        [String]$AdminAuthMethod = "NTLM",
        
        [Parameter(Mandatory=$false)][ValidateNotNullOrEmpty()]
        [String]$FarmName = $env:COMPUTERNAME
    )
    
	if( -not ( Get-PSSnapIn | where { $_.Name -eq "Microsoft.SharePoint.PowerShell" } )  )
	{
		Add-PSSnapin Microsoft.SharePoint.PowerShell;
	}
        
    $Sucessful = $false
    $Verbose = $false 
    
    $Activity = "Creating SharePoint Farm"
    $ScriptLog = ("$env:TEMP\New-SharePointFarm_{0}.ps1.log" -f (Get-Date -Format 'dd_MMM_yyyy_HH_mm_ss'))
   
	Write-Output "Initial Bound parameters:" | Out-File $ScriptLog -Append
	Write-Output $MyInvocation.BoundParameters | Out-File $ScriptLog -Append   
    
    #region Settings Object
    $FarmInfoSettings = @"
        using System;
        namespace SPModule.setup {
            public class CreateFarmInformation {
                public DateTime StartTime {get; set;}
				public TimeSpan TotalTime {get; set; }
                public String   DatabaseAccessUser {get; set;}
                public String   DatabaseAccessPassword {get; set;}
                public String   DatabaseServer {get; set;}
                public String   FarmPassphrase {get; set;}
                public String   ConfigurationDatabaseName {get; set;}
                public String   AdminDatabaseName {get; set;}
                public int      AdminPort {get; set;}
				public String   CommandLine {get; set;}
            }
        }
"@
    Add-Type $FarmInfoSettings -Language CSharpVersion3
    #endregion 
    
    $RunSettings = New-Object SPModule.setup.CreateFarmInformation
    
    $RunSettings.StartTime = Get-Date -Format 'dd-MMM-yyyy HH:mm:ss'
    $RunSettings.DatabaseAccessUser = $DatabaseAccessUsername
    $RunSettings.DatabaseAccessPassword = (ConvertFrom-SecureString $DatabaseAccessPassword)
    $RunSettings.DatabaseServer = $DatabaseServer
    $RunSettings.ConfigurationDatabaseName = ("{0}_SharePoint_Configuration_Database" -f $FarmName)
    $RunSettings.AdminDatabaseName = ("{0}_SharePoint_Administration_Content_Database" -f $FarmName)
    $RunSettings.AdminPort = $Port
	$RunSettings.CommandLine = $MyInvocation.Line
	
    if (($Passphrase.Length -eq 0) -or ($Passphrase -eq $null))
    {
        $Passphrase = $DatabaseAccessPassword
    }
    
    $RunSettings.FarmPassphrase = (ConvertFrom-SecureString $Passphrase)
    
    $RunSettings | Out-File $ScriptLog -Append
      
    $ErrorActionPreference = "Stop"
    
    try 
    {
        # Create a new config db 
        Write-Progress -Activity $Activity -Status ("Creating new configuration database: {0}..." -f $RunSettings.ConfigurationDatabaseName)
        $credentials = New-Object -typename System.Management.Automation.PSCredential -argumentlist $DatabaseAccessUsername, $DatabaseAccessPassword
        New-SPConfigurationDatabase -DatabaseServer $DatabaseServer -DatabaseName $RunSettings.ConfigurationDatabaseName -AdministrationContentDatabaseName $RunSettings.AdminDatabaseName -FarmCredentials $credentials -Passphrase $Passphrase -Verbose:$Verbose
        if (-not $?) { throw "Failed to create Configuration Database."    }
        
        #install help collections
        Write-Progress -Activity $Activity -Status "Installing Help Collections..." 
        Install-SPHelpCollection -All -Verbose:$Verbose
        if (-not $?) { throw "Failed to install Help Collections."    }
        
        #Secure resources
        Write-Progress -Activity $Activity -Status "Securing SharePoint Resources..."  
        Initialize-SPResourceSecurity -Verbose:$Verbose
        if (-not $?) { throw "Failed to Secure Resources."    }
        
        #Install Services
        Write-Progress -Activity $Activity -Status "Installing Services..." 
        Install-SPService -Verbose:$Verbose
        if (-not $?) { throw "Failed to Install Services."    }
                
        #Install Features
        Write-Progress -Activity $Activity -Status "Installing Features..." 
        $InstalledFeatures = Install-SPFeature -AllExistingFeatures -Verbose:$Verbose
        if (-not $?) { throw "Failed to Install Features."    }
        
        Write-Verbose ("Installed {0} features" -f $InstalledFeatures.Count)
        $InstalledFeatures | Out-File $ScriptLog -Append
        Write-Output ("Installed {0} features" -f $InstalledFeatures.Count) | Out-File $ScriptLog -Append
        
        #Provision Central Administration
        Write-Progress -Activity $Activity -Status "Provisioning Central Administration..." 
        New-SPCentralAdministration -Port $RunSettings.AdminPort -WindowsAuthProvider $AdminAuthMethod -Verbose:$Verbose
                
        #Install Application content
        Write-Progress -Activity $Activity -Status "Installing Application Content..." 
        Install-SPApplicationContent -Verbose:$Verbose
        if (-not $?) { throw "Failed to Install Application Content."    }
        
		$Sucessful = $true
    }
    catch
    {
		Write-Output $_ | Out-File $ScriptLog -Append
        Write-Error $_
		
        throw
    }
    finally
	{
		if ($Sucessful)
		{
			$TotalTime = [System.DateTime]::Now - $RunSettings.StartTime
			$RunSettings.TotalTime = $TotalTime
		}
		
		Write-Verbose $RunSettings
		
        Remove-PSSnapin Microsoft.SharePoint.Powershell
    }
   
}

<#
.SYNOPSIS
	Configure Search for SP2010 Server
.NOTES
	Only SharePoint Server 2010 is supported, SharePoint Foundation 2010 is not yet supported.
#>
function SharePointServer-Configure()
{
	PARAM
	(
	    [String]$SPUsernameFarmService=$(throw 'Parameter -SPUsernameFarmService is missing!'),
		[String]$ProjectName=$(throw 'Parameter -ProjectName is missing!')
	)

	if( -not ( Get-PSSnapIn | where { $_.Name -eq "Microsoft.SharePoint.PowerShell" } ) )
	{
		Add-PSSnapin Microsoft.SharePoint.PowerShell;
	}

	if($MOSS_CONFIGURATION)
    {
		Write-Host "Configuring SharePoint Server";
		Write-Host "--- Start Excel Services";
		Get-SPServiceInstance | where-object {$_.TypeName -eq "Excel Calculation Services"} | Start-SPServiceInstance

		# Start the Search Query and Site Settings Service
		Get-SPServiceInstance | where-object {$_.TypeName -eq "Search Query and Site Settings Service"} | Start-SPServiceInstance

		$SSI = get-spenterprisesearchserviceinstance -local 
		$searchAppPool = New-SPServiceApplicationPool -name "$($ProjectName)AppPool_Search" -account $SPUsernameFarmService
		$ssa =  New-SPEnterpriseSearchServiceApplication -Name "$($ProjectName) Search Service Application" -ApplicationPool $searchAppPool
		$SSAProxy = new-spenterprisesearchserviceapplicationproxy -name "$($ProjectName) Search Service Application Proxy" -Uri $ssa.Uri.AbsoluteURI 
		Get-SPServiceInstance | where-object {$_.TypeName -eq "SharePoint Server Search"} | Start-SPServiceInstance
		# need to wait until the Search service is provsioned
		
		do 
		{ 
			$err = "test" 
			if ((Get-SPServiceInstance | where-object {$_.TypeName -eq "SharePoint Server Search"}).Status -eq "Online") 
			{ 
				$err = $null 
			} 
			Start-Sleep -Seconds 1 
		} 
		until ($err -eq $null) 
		
		Set-SPenterpriseSearchAdministrationComponent -searchapplication $ssa  -searchserviceinstance $SSI 
		$CrawlTopo = $ssa | New-SPEnterpriseSearchCrawlTopology 
		$CrawlStore = $ssa | Get-SPEnterpriseSearchCrawlDatabase 
		New-SPEnterpriseSearchCrawlComponent -CrawlTopology $CrawlTopo -CrawlDatabase $CrawlStore -SearchServiceInstance $SSI 
		
		do 
		{ 
			$err = $null 
			$CrawlTopo | Set-SPEnterpriseSearchCrawlTopology -Active -ErrorVariable err 
			if ($CrawlTopo.State -eq "Active") 
			{ 
				$err = $null 
			} 
			Start-Sleep -Seconds 1 
		} 
		until ($err -eq $null) 
		
		$QueryTopo = $ssa | New-SPenterpriseSearchQueryTopology -partitions 1 
		$Partition1 = ($QueryTopo | Get-SPEnterpriseSearchIndexPartition) 
		New-SPEnterpriseSearchQueryComponent -indexpartition $Partition1 -QueryTopology $QueryTopo -SearchServiceInstance $SSI 
		$PropDB = $ssa | Get-SPEnterpriseSearchPropertyDatabase 
		$Partition1 | Set-SPEnterpriseSearchIndexPartition -PropertyDatabase $PropDB 
		do 
		{ 
			$err = $null 
			$QueryTopo | Set-SPEnterpriseSearchQueryTopology -Active -ErrorVariable err -ErrorAction SilentlyContinue 
			Start-Sleep -Seconds 1 
			if ($QueryTopo.State -eq "Active") 
			{ 
				$err = $null 
			} 
		} 
		until ($err -eq $null) 
	}
}


<#
.SYNOPSIS
	Creates a new SPWebApplication of the specified name with the authentication provider required for CS
.NOTES
	Currently, the New-SPWebApplication command, when an authentication provider is passed, causes Anonymous to be turned on for both the private and public sites.
.PARAMETER ApplicationName
	The dislay name of the Default zone of the web application.
.PARAMETER ApplicationPoolName
	The name of the application pool which the site runs under.
.PARAMETER ApplicationPoolAccount
	The account under which the application pool defined in ApplicationPoolName runs under.
.PARAMETER ApplicationPoolPassword
	The password of the account under which the application pool defined in ApplicationPoolName runs under.
.PARAMETER DefaultPort
	The port of the Default zone of the web application
.PARAMETER ExtendedPort
	The port of the Internet zone of the web application
.PARAMETER ExtendedSiteName
	The dislay name of the Internet zone of the web application.
.PARAMETER Url
	The url of the Default zone of the web application
.PARAMETER ExtendedUrl
	The url of the Internet zone of the web application
.PARAMETER ContentDatabaseName
	The name of the content database to be created when the web application is created.
.EXAMPLE
	TODO
	Create-CommerceSPWebApplication -ApplicationName "MySite - Public" -ApplicationPoolName "MySiteApplicationPool" -ApplicationPoolAccount "MYDOMAIN\MyUser" -ApplicationPoolPassword 
#>
function Create-CommerceSPWebApplication
{
	param
	(
		[String][Parameter( Mandatory = $true )]$DefaultName,
		[String][Parameter( Mandatory = $true )]$DefaultUrl,
		[Int]$DefaultPort = 80,
		[String][Parameter( Mandatory = $true )]$DefaultHostHeader,

		[String][Parameter( Mandatory = $true )]$ExtendedName,
		[String][Parameter( Mandatory = $true )]$ExtendedUrl,
		[String]$ExtendedUrlSecure,
		[Int]$ExtendedPort = 80,
		[String][Parameter( Mandatory = $true )]$ExtendedHostHeader,

		[String][Parameter( Mandatory = $true )]$ApplicationPoolName,
		[String][Parameter( Mandatory = $true )]$ApplicationPoolAccount,
		[String][Parameter( Mandatory = $true )]$ApplicationPoolPassword,
		
		[String][Parameter( Mandatory = $true )]$ContentDatabaseName
	)
	
	if( -not ( Get-PSSnapin | where { $_.Name -eq "Microsoft.SharePoint.PowerShell" } ) )
	{
		Add-PSSnapin Microsoft.SharePoint.PowerShell;
	}

	$managedAccount = Get-SPManagedAccount | where {$_.UserName -eq $ApplicationPoolAccount};
	if($managedAccount -eq $null)
	{	
		$password =  ( ConvertTo-SecureString $ApplicationPoolPassword -AsPlainText -Force );
		$credentials = New-Object -typename System.Management.Automation.PSCredential -argumentlist $ApplicationPoolAccount, $password ;
		$managedAccount = New-SPManagedAccount -Credential $credentials;
	}
	
	$appPool = $null;
	$app = $null;
	
	if( ( Get-SPServiceApplicationPool | where { $_.Name -ieq $ApplicationPoolName  } ) -eq $null )
	{
		Write-Host "Creating new application Pool..." -ForegroundColor Green;
		$appPool = New-SPServiceApplicationPool -Name $ApplicationPoolName -Account $managedAccount; 
	}

	$winAuth = New-SPAuthenticationProvider;
	Write-Host "Creating web application (Default Zone)..." -ForegroundColor Green;
	
	if($appPool){
		Write-Host "`tCreating with existing app pool" -ForegroundColor Yellow;
		$app = New-SPWebApplication -Name $DefaultName -ApplicationPool $ApplicationPoolName -Url $DefaultUrl -AuthenticationProvider $winAuth -DatabaseName $ContentDatabaseName -HostHeader $DefaultHostHeader -Port $DefaultPort -AllowAnonymous:$false;
	}
	else{
		Write-Host "`tCreating with new app pool" -ForegroundColor Yellow;
		$app = New-SPWebApplication -Name $DefaultName -ApplicationPool $ApplicationPoolName -ApplicationPoolAccount $managedAccount -Url $DefaultUrl -AuthenticationProvider $winAuth -DatabaseName $ContentDatabaseName -HostHeader $DefaultHostHeader -Port $DefaultPort -AllowAnonymous:$false;
	}

	Write-Host "`tSetting the portalsuperuseraccount and portalsuperreaderaccount properties of the web application." -ForegroundColor Green;
	$app.Properties["portalsuperuseraccount"] = "NT AUTHORITY\LOCAL SERVICE";
	$app.Properties["portalsuperreaderaccount"] = "SHAREPOINT\system";
	$app.Update();
	
	$providerName = "UpmProvider_" + $app.Id.ToString("B");
	$commerceAuthProvider = New-SPAuthenticationProvider -ASPNETMembershipProvider $providerName -ASPNETRoleProviderName "SingleRoleProvider";
	$commerceAuthProvider.RoleProvider = "";
	
	Write-Host "Creating web application (Internet Zone)..." -ForegroundColor Green;
	New-SPWebApplicationExtension -Identity $DefaultName -Name $ExtendedName -Zone "Internet" -AuthenticationProvider $commerceAuthProvider -Confirm:$false -Url $ExtendedUrl -HostHeader $ExtendedHostHeader -Port $ExtendedPort -AllowAnonymous -SignInRedirectUrl "/Pages/Signin.aspx";

	if( $ExtendedUrlSecure )
	{
		New-SPAlternateUrl -Url $ExtendedUrlSecure -WebApplication $DefaultName -Zone "Internet" -Internal;
	}
	
	Write-Host "Creating an IIS binding for HTTPs";
	Import-Module -Name WebAdministration;

	Write-Host "Assign the certificate to the https binding";
	Write-Host "`tPublic SIte" -ForegroundColor Green;
	if( ( Get-ChildItem -Path "cert:\localmachine\MY" | where { $_.FriendlyName -ieq "www.commercelive.com" }) -eq $null )
	{
		Import-IIS7PfxCertificate -certPath ( Join-Path -Path $DEPLOYMENT_DIRECTORY -ChildPath "www.commercelive.com.pfx" ) -pfxPass "cactus123";
	}
	$commerceLiveCert = ( Get-ChildItem -Path "cert:\localmachine\MY" | where { $_.FriendlyName -ieq "www.commercelive.com" });

	New-WebBinding -Name $ExtendedName -Port 443 -HostHeader $ExtendedHostHeader -Protocol https -IP "*";
	$sslBinding = Get-WebBinding -Name $ExtendedName -Port 443 -HostHeader $ExtendedHostHeader -Protocol https -IP "*";
	$sslBinding.AddSslCertificate( $commerceLiveCert.ThumbPrint, "MY" ) ;
	
	Write-Host "`tTurning off NTLM for the public site." -ForegroundColor Green;
	$webApplication = Get-SPWebApplication
	$ZoneType = "Internet"
	$Zone = [microsoft.sharepoint.administration.SPURLZone]::$ZoneType
	$iisSettings = $webApplication.GetIisSettingsWithFallback($Zone)
	$iisSettings.UseWindowsIntegratedAuthentication=$false
	$webApplication.Update()
}

<#
.SYNOPSIS
	Adds a WSP to the solution and deploys it.
.NOTES
.PARAMETER WSPPath
	The full path to the WSP to be added to the solution store.
.PARAMETER WebApplication
	If the WSP contains artifacts that need to be deployed to a web application, this is the 
.EXAMPLE
	TODO
	Create-CommerceSPWebApplication -ApplicationName "MySite - Public" -ApplicationPoolName "MySiteApplicationPool" -ApplicationPoolAccount "MYDOMAIN\MyUser" -ApplicationPoolPassword 
#>
function DeployAndWait-SPSolution()
{
	param 
	(
		[Parameter( Mandatory = $true )][String]$WSPName,
		[string]$WebApplication = $null,
		[switch]$Wait = $true,
		[switch]$SP2013=$false
	)
	
	if( -not ( Get-PSSnapin | where { $_.Name -eq "Microsoft.SharePoint.PowerShell" } ) )
	{
		Add-PSSnapin Microsoft.SharePoint.PowerShell;
	}
	
	Write-Host "Deploying $($WSPName)" -ForegroundColor Green;

	if($SP2013)
	{
		if($WebApplication)
		{
			Install-SPSolution -Identity $WSPName -WebApplication $WebApplication -FullTrustBinDeployment;
		}
		else
		{
			Install-SPSolution -Identity $WSPName -GACDeployment;
		}
	}
	else
	{
		if($WebApplication)
		{
			Install-SPSolution -Identity $WSPName -CASPolicies -WebApplication $WebApplication;
		}
		else
		{
			Install-SPSolution -Identity $WSPName -GACDeployment -CASPolicies;
		}
	}

	
	$solution = Get-SPSolution | where { $_.Name -ieq $WSPName };
	
	if( $solution -and $Wait )
	{
		Write-Host -NoNewLine "Waiting for deployment job to complete." -ForegroundColor Green;
		while ($solution.JobExists) 
		{
			Write-Host -NoNewLine "." -ForegroundColor Green;
			Start-Sleep -Seconds 1;
		}
	    
		Write-Host "." -ForegroundColor Green;
		
		if( $solution.LastOperationResult -ne [Microsoft.SharePoint.Administration.SPSolutionOperationResult]::DeploymentSucceeded )
		{
			Write-Error $solution.LastOperationDetails;
		}
	}
}

<#
.SYNOPSIS
	Adds a WSP to the solution and deploys it.
.NOTES
.PARAMETER WSPPath
	The full path to the WSP to be added to the solution store.
.PARAMETER WebApplication
	If the WSP contains artifacts that need to be deployed to a web application, this is the 
.EXAMPLE
	TODO
	Create-CommerceSPWebApplication -ApplicationName "MySite - Public" -ApplicationPoolName "MySiteApplicationPool" -ApplicationPoolAccount "MYDOMAIN\MyUser" -ApplicationPoolPassword 
#>
function UnDeployAndWait-SPSolution
(
	[Parameter( Mandatory = $true )][String]$WSPName,
	[string]$WebApplication = $null,
	[switch]$Wait = $true
)
{	
	if( -not ( Get-PSSnapin | where { $_.Name -eq "Microsoft.SharePoint.PowerShell" } ) )
	{
		Add-PSSnapin Microsoft.SharePoint.PowerShell;
	}

	$solution = Get-SPSolution | where { $_.Name -ieq $WSPName };
	
	if( $solution )
	{
		if( $solution.Deployed )
		{
			Write-Host "Undeploying $($WSPName)" -ForegroundColor Green;
			if($WebApplication)
			{
				Uninstall-SPSolution -Identity $WSPName -WebApplication $WebApplication -Confirm:$false;
			}
			else
			{
				Uninstall-SPSolution -Identity $WSPName -Confirm:$false;
			}

			if( $Wait )
			{
				Write-Host -NoNewLine "Waiting for deployment job to complete." -ForegroundColor Green;
				while ($solution.JobExists) 
				{
					Write-Host -NoNewLine "." -ForegroundColor Green;
					Start-Sleep -Seconds 1;
				}
			    
				Write-Host "." -ForegroundColor Green;
				
				if( $solution.LastOperationResult -ne [Microsoft.SharePoint.Administration.SPSolutionOperationResult]::RetractionSucceeded )
				{
					Write-Error $solution.LastOperationDetails;
				}
			}
		}
	}
}

<#
.SYNOPSIS
	Activates a feature and waits for any timer jobs associated with the feature to complete.
.NOTES
.PARAMETER FeatureNames
	An array of names of the feaure directories (under 14/Templates/Features) to deploy
.PARAMETER Url
	The url of the webapplication, site or web to deploy the features to.
.PARAMETER ActivateUsingHttpGet
	This is used when the feature activation requires an HTTP or Site context
.EXAMPLE
#>
function EnableAndWait-SPFeature
(
	[Parameter( Mandatory = $true )][string]$FeatureName,
	[string]$Url,
	[bool]$ActivateUsingHttpGet
)
{
	Write-Host "$($FeatureName)" -ForegroundColor Green;

	$feature = Get-SPFeature | where {$_.DisplayName -ieq $FeatureName.Trim() };
	
	if( $feature )
	{
		if( $ActivateUsingHttpGet )
		{
			$activateFeatureUrl = "$($Url)_layouts/FeatureManagement/ActivateFeature.aspx?featureScope=$($feature.Scope)&featureId=$($feature.Id)";
			Write-Host $activateFeatureUrl;
			$result = Execute-HTTPGetCommand -Url $activateFeatureUrl;
			Write-Host $result;
		}
		else
		{
			if( $Url  )
			{
				Enable-SPFeature -Identity $FeatureName -Url $Url;
			}
			else
			{
				Enable-SPFeature -Identity $FeatureName;
			}
		}
		
		$timerJob = Get-SPTimerJob | where { $_.Title -ilike "$($FeatureName)*" };

		# Does the job definition exist?
		if( $timerJob -ne $null )
		{
			# Is the job running?
			Write-Host  -NoNewLine "Waiting for timer jobs to complete..." -ForegroundColor Green;
			while( $timerJob -ne $null )
			{
				Write-Host -NoNewLine "." -ForegroundColor Green;
				Start-Sleep -Seconds 1;
				$timerJob = Get-SPTimerJob | where { $_.Title -ilike "$($FeatureName)*" };
			}
			
			Write-Host "." -ForegroundColor Green;
		}
	}
	else
	{
		Write-Error "SPFeature $($FeatureName) does not exist in the farm.";
	}
}

<#
.SYNOPSIS
	Deactivates a feature and waits for any timer jobs associated with the feature to complete.
.NOTES
.PARAMETER FeatureNames
	An array of names of the feaure directories (under 14/Templates/Features) to deploy
.PARAMETER Url
	The url of the webapplication, site or web to deploy the features to.
.EXAMPLE
	TODO
#>
function DisableAndWait-SPFeature
{
	param
	(
		[Parameter( Mandatory = $true )][string]$FeatureName,
		[string]$Url
	)

	Write-Host "$($FeatureName)" -ForegroundColor Green;

	$feature = Get-SPFeature | where  {$_.DisplayName -ieq $FeatureName };

	if( $feature.Scope -eq [Microsoft.SharePoint.SPFeatureScope]::Farm )
	{
		$activeFeature = Get-SPFeature -Farm | where { $_.DisplayName -ieq $FeatureName };

		if( $activeFeature )
		{
			Disable-SPFeature -Identity $activeFeature.Id -Confirm:$false;
		}
	}
	else
	{
		switch( $feature.Scope )
		{
			([Microsoft.SharePoint.SPFeatureScope]::Site)
			{
				$activeFeature = Get-SPFeature -Site $Url | where { $_.DisplayName -ieq $FeatureName };
			}
			([Microsoft.SharePoint.SPFeatureScope]::Web)
			{
				$activeFeature = Get-SPFeature -Web $Url | where { $_.DisplayName -ieq $FeatureName };
			}
			([Microsoft.SharePoint.SPFeatureScope]::WebApplication)
			{
				$activeFeature = Get-SPFeature -WebApplication $Url | where { $_.DisplayName -ieq $FeatureName };
			}
		}
		
		if( $activeFeature )
		{
			Disable-SPFeature -Identity $activeFeature.Id -Url $Url -Confirm:$false;
		}
	}
	
	if( $activeFeature )
	{
		$timerJob = Get-SPTimerJob | where { $_.Title -ilike "$($FeatureName)*" };

		# Does the job definition exist?
		if( $timerJob -ne $null )
		{
			# Is the job running?
			Write-Host  -NoNewLine "Waiting for timer jobs to complete..." -ForegroundColor Green;
			while( $timerJob -ne $null )
			{
				Write-Host -NoNewLine "." -ForegroundColor Green;
				Start-Sleep -Seconds 1;
				$timerJob = Get-SPTimerJob | where { $_.Title -ilike "$($FeatureName)*" };
			}
			
			Write-Host "." -ForegroundColor Green;
		}
	}
	else
	{
		Write-Host "SPFeature $($FeatureName) cannot be found at the specified scope." -ForegroundColor Red;
	}
}

function Set-SPWebApplicationPermissiveBrowserFileHandling
(
	[Parameter( Mandatory = $true )][String]$WebApplication
)
{
	if( -not ( Get-PSSnapIn | where { $_.Name -eq "Microsoft.SharePoint.PowerShell" } ) )
	{
		Add-PSSnapin Microsoft.SharePoint.PowerShell;
	}

	$app = ( Get-SPWebApplication -Identity $WebApplication );
	$app.BrowserFileHandling = "permissive";
	$app.update();
}

<#
.Synopsis
    Creates a variant root site and subsites
.Description
    
.Example
	$variationLabels = @();
	$variationLabels += New-Object Object | 
		Add-Member -MemberType NoteProperty -Name "Title" -Value "en-ca" -PassThru |
		Add-Member -MemberType NoteProperty -Name "Description" -Value "My Description" -PassThru |
		Add-Member -MemberType NoteProperty -Name "FlagControlDisplayName" -Value "English (Canada)" -PassThru |
		Add-Member -MemberType NoteProperty -Name "Language" -Value "en-US" -PassThru |
		Add-Member -MemberType NoteProperty -Name "Locale" -Value "en-CA" -PassThru |
		Add-Member -MemberType NoteProperty -Name "HierarchyCreationMode" -Value "Publishing Sites and All Pages" -PassThru |
		Add-Member -MemberType NoteProperty -Name "IsSource" -Value "true" -PassThru |
		Add-Member -MemberType NoteProperty -Name "SourceVarRootWebTemplatePropertyName" -Value "CommerceServerContribMOSSBlank#0" -PassThru;

	Create-SPVariations -Site ( Get-SPSite "http://private.commercelive.com/Sites/Canada/" ) -RootWeb ( Get-SPSite "http://private.commercelive.com/Sites/Canada/" ).OpenWeb() -VariantLabels $variationLabels;
	
	Creates a Canada variant root site with a Canada (English) subsite.
.Parameter Site
    The SPSite object to use as a root to create the root variant site.
.Parameter RootWeb
    The SPWeb object to use as a root to create the root variant site.
.Parameter VariantLabels
    An array of variant objects defined as follows:
	
	New-Object Object | 
		Add-Member -MemberType NoteProperty -Name "Title" -Value "en-ca" -PassThru |
		Add-Member -MemberType NoteProperty -Name "Description" -Value "My Description" -PassThru |
		Add-Member -MemberType NoteProperty -Name "FlagControlDisplayName" -Value "English (Canada)" -PassThru |
		Add-Member -MemberType NoteProperty -Name "Language" -Value "en-US" -PassThru |
		Add-Member -MemberType NoteProperty -Name "Locale" -Value "en-CA" -PassThru |
		Add-Member -MemberType NoteProperty -Name "HierarchyCreationMode" -Value "Publishing Sites and All Pages" -PassThru |
		Add-Member -MemberType NoteProperty -Name "IsSource" -Value "true" -PassThru |
		Add-Member -MemberType NoteProperty -Name "SourceVarRootWebTemplatePropertyName" -Value "CommerceServerContribMOSSBlank#0" -PassThru;
		
	I had some problems with the Language property.  I *think* it needs to be the same as the local of the site collection, otherwise the subsites do not get created.
	
	Valid values for HierarchyCreationMode are:
		"Publishing Sites and All Pages"
		"Publishing Sites Only"
		"Root Sites Only"
		
.Link
    http://blog.mastykarz.nl/programmatically-provisioning-variations-in-sharepoint-server-2010/
#>
function Create-SPVariations
(
	[Microsoft.SharePoint.SPSite]$Site,
	[Microsoft.SharePoint.SPWeb]$RootWeb,
	$VariantLabels
)
{
	if( -not ( Get-PSSnapIn | where { $_.Name -eq "Microsoft.SharePoint.PowerShell" } ) )
	{
		Add-PSSnapin Microsoft.SharePoint.PowerShell;
	}

	Write-Host "Configuring Varation Root Site" -ForegroundColor Green;
	$varRelationshipsListId = [System.Guid]($RootWeb.AllProperties["_VarRelationshipsListId"].ToString());
    $varRelationshipsList = $RootWeb.Lists[$varRelationshipsListId];
    $rootFolder = $varRelationshipsList.RootFolder;
    # Automatic creation
    $rootFolder.Properties["EnableAutoSpawnPropertyName"] = "true";
    # Recreate Deleted Target Page; set to false to enable recreation
    $rootFolder.Properties["AutoSpawnStopAfterDeletePropertyName"] = "false";
    # Update Target Page Web Parts
    $rootFolder.Properties["UpdateWebPartsPropertyName"] = "true";
    # Resources
    $rootFolder.Properties["CopyResourcesPropertyName"] = "true";
    # Notification
    $rootFolder.Properties["SendNotificationEmailPropertyName"] = "false";
    $rootFolder.Update();
 
    $item = $null;
    if ( $varRelationshipsList.Items | where { $_["GroupGuid"] -ieq [System.Guid]("F68A02C8-2DCC-4894-B67D-BBAED5A066F9") } )
    {
        $item = $varRelationshipsList.Items[0];
    }
    else
    {
        $item = $varRelationshipsList.Items.Add();
        $item["GroupGuid"] = [System.Guid]("F68A02C8-2DCC-4894-B67D-BBAED5A066F9");
    }
 
    $item["Deleted"] = $false;
    $item["ObjectID"] = $rootWeb.ServerRelativeUrl;
    $item["ParentAreaID"] = [System.String]::Empty;
    $item.Update();
	
	Write-Host "Creating Variation Labels" -ForegroundColor Green;
	foreach( $variantLabel in $VariantLabels )
	{
		Write-Host "Creating Variation $($variantLabel.Title)" -ForegroundColor Green;
	    $varListId = [System.Guid]( $rootWeb.AllProperties["_VarLabelsListId"] );
		$varList = $rootWeb.Lists[$varListId];

	    $item = $varList.Items.Add();
	    $item[[Microsoft.SharePoint.SPBuiltInFieldId]::Title] = $variantLabel.Title;
	    $item["Description"] = $variantLabel.Description;
	    $item["Flag Control Display Name"] = $variantLabel.FlagControlDisplayName;
	    $item["Language"] = $variantLabel.Language;
	    $item["Locale"] = $variantLabel.Locale;
	    $item["Hierarchy Creation Mode"] = $variantLabel.HierarchyCreationMode;
	    $item["Is Source"] = $variantLabel.IsSource;
	    $item["Hierarchy Is Created"] = $false;
	    $item.Update();

		if( $variantLabel.IsSource )
		{
			$varRelationshipsListId = [System.Guid]($rootWeb.AllProperties["_VarRelationshipsListId"].ToString());
		    $varRelationshipsList = $RootWeb.Lists[$varRelationshipsListId];
		    $rootFolder = $varRelationshipsList.RootFolder;
			$rootFolder.Properties["SourceVarRootWebTemplatePropertyName"] = $variantLabel.SourceVarRootWebTemplatePropertyName;
			$rootFolder.Update();
		}
	}
	
	Write-Host "Creating Hierarchies" -ForegroundColor Green;
	$Site.AddWorkItem([System.Guid]::Empty, [System.DateTime]::Now.ToUniversalTime(), [System.Guid]("e7496be8-22a8-45bf-843a-d1bd83aceb25"), $rootWeb.ID, $Site.ID, 1, $false, [System.Guid]::Empty, [System.Guid]::Empty, $rootWeb.CurrentUser.ID, $null, [System.String]::Empty, [System.Guid]::Empty, $false);
 	
	$timerJob = Get-SPTimerJob | where { $_.Title -eq "Variations Create Hierarchies Job Definition" };
	
	if($timerJob){
		if($timerJob.Count){
			$count = $timerJob.Count;
			
			Write-Host "`tFound $count timer jobs, picking the one that matces the current web application name." -ForegroundColor Yellow;
			
			$timerJob = $timerJob | where {$_.WebApplication.DisplayName -eq $Site.WebApplication.DisplayName}
		}
		else{
			Write-Host "`tFound timer job" -ForegroundColor Green;
		}
	}
	else{
		Write-Host "`tCould not find timer job" -ForegroundColor Red;
	}
	
	Write-Host "`tStarting timer job..." -ForegroundColor Green;
	Start-SPTimerJob -Identity $timerJob;

	# Sounds dumb, but I have to wait for the job to start, as it seems to take some time to fire up.
	Write-Host -NoNewLine "Waiting for timer jobs to start..." -ForegroundColor Green;
	while ( ! ( $Site.WebApplication.RunningJobs | where { $_.JobDefinitionTitle -eq "Variations Create Hierarchies Job Definition" } ) )
	{
		Write-Host -NoNewLine "." -ForegroundColor Green;
		Start-Sleep -Seconds 1;
	}
	Write-Host "." -ForegroundColor Green;

	Write-Host -NoNewLine "Waiting for timer jobs to complete..." -ForegroundColor Green;
	while( $Site.WebApplication.RunningJobs | where { $_.JobDefinitionTitle -eq "Variations Create Hierarchies Job Definition" } )
	{
		Write-Host -NoNewLine "." -ForegroundColor Green;
		Start-Sleep -Seconds 1;
	}	
	Write-Host "." -ForegroundColor Green;
}

<#
.Synopsis
    Runs Variations Propagate Site Job Definition
.Description
    
.Example

.Parameter Site
    The SPSite object to use as a root to find the job.
.Parameter RootWeb
    The SPWeb object to use as a root to find the job.	
.Link

#>
function Run-VariationsPropagateSiteJob
(
	[Microsoft.SharePoint.SPSite]$Site,
	[Microsoft.SharePoint.SPWeb]$RootWeb
)
{
	if( -not ( Get-PSSnapIn | where { $_.Name -eq "Microsoft.SharePoint.PowerShell" } ) )
	{
		Add-PSSnapin Microsoft.SharePoint.PowerShell;
	}

	Write-Host "Running Variations Propagate Site Job" -ForegroundColor Green;
	$Site.AddWorkItem([System.Guid]::Empty, [System.DateTime]::Now.ToUniversalTime(), [System.Guid]("2436D94B-D3EA-4e2c-80CD-67028655A138"), $rootWeb.ID, $Site.ID, 1, $false, [System.Guid]::Empty, [System.Guid]::Empty, $rootWeb.CurrentUser.ID, $null, [System.String]::Empty, [System.Guid]::Empty, $false);
 	$timerJob = Get-SPTimerJob | where { $_.Title -eq "Variations Propagate Site Job Definition" };
	
	if($timerJob){
		if($timerJob.Count){
			$count = $timerJob.Count;
			
			Write-Host "`tFound $count timer jobs, picking the one that matces the current web application name." -ForegroundColor Yellow;
			
			$timerJob = $timerJob | where {$_.WebApplication.DisplayName -eq $Site.WebApplication.DisplayName}
		}
		else{
			Write-Host "`tFound timer job" -ForegroundColor Green;
		}
	}
	else{
		Write-Host "`tCould not find timer job" -ForegroundColor Red;
	}
	
	Write-Host "`tStarting timer job..." -ForegroundColor Green;
	Start-SPTimerJob -Identity $timerJob;
	
	# Sounds dumb, but I have to wait for the job to start, as it seems to take some time to fire up.
	Write-Host -NoNewLine "Waiting for timer jobs to start..." -ForegroundColor Green;
	while ( ! ( $Site.WebApplication.RunningJobs | where { $_.JobDefinitionTitle -eq "Variations Propagate Site Job Definition" } ) )
	{
		Write-Host -NoNewLine "." -ForegroundColor Green;
		Start-Sleep -Seconds 1;
	}
	Write-Host "." -ForegroundColor Green;

	Write-Host -NoNewLine "Waiting for timer jobs to complete..." -ForegroundColor Green;
	while( $Site.WebApplication.RunningJobs | where { $_.JobDefinitionTitle -eq "Variations Propagate Site Job Definition" } )
	{
		Write-Host -NoNewLine "." -ForegroundColor Green;
		Start-Sleep -Seconds 1;
	}	
	Write-Host "." -ForegroundColor Green;
}

<#
.Synopsis
    Disables Variations Propagate Page Job Definition and Variations Create Page Job Definition
.Description
    
.Example

.Parameter Site
    The SPSite object to use as a root to find the job
.Link

#>
function Disable-VariationsPageJobs
(
	[Microsoft.SharePoint.SPSite]$Site
)
{
	if( -not ( Get-PSSnapIn | where { $_.Name -eq "Microsoft.SharePoint.PowerShell" } ) )
	{
		Add-PSSnapin Microsoft.SharePoint.PowerShell;
	}

	Write-Host "Disabling Variations Propagate Page Job Definition/Variations Create Page Job Definition" -ForegroundColor Green;
	$timerJob = Get-SPTimerJob | where { $_.Title -eq "Variations Propagate Page Job Definition" }
	if($timerJob){
		if($timerJob.Count){
			$count = $timerJob.Count;
			
			Write-Host "`tFound $count timer jobs, picking the one that matces the current web application name." -ForegroundColor Yellow;
			
			$timerJob = $timerJob | where {$_.WebApplication.DisplayName -eq $Site.WebApplication.DisplayName}
		}
		else{
			Write-Host "`tFound timer job" -ForegroundColor Green;
		}
	}
	else{
		Write-Host "`tCould not find timer job" -ForegroundColor Red;
	}
	
	Write-Host "Disabling Variations Propagate Page Job Definition" -ForegroundColor Green;
	Disable-SPTimerJob -Identity $timerJob;
	
	$timerJob = Get-SPTimerJob | where { $_.Title -eq "Variations Create Page Job Definition" }
	if($timerJob){
		if($timerJob.Count){
			$count = $timerJob.Count;
			
			Write-Host "`tFound $count timer jobs, picking the one that matces the current web application name." -ForegroundColor Yellow;
			
			$timerJob = $timerJob | where {$_.WebApplication.DisplayName -eq $Site.WebApplication.DisplayName}
		}
		else{
			Write-Host "`tFound timer job" -ForegroundColor Green;
		}
	}
	else{
		Write-Host "`tCould not find timer job" -ForegroundColor Red;
	}
	
	Write-Host "Disabling Variations Create Page Job Definition" -ForegroundColor Green;
	Disable-SPTimerJob -Identity $timerJob;
}

<#
.Synopsis
    Enables Variations Propagate Page Job Definition and Variations Create Page Job Definition
.Description
    
.Example

.Parameter Site
    The SPSite object to use as a root to find the job
.Link

#>
function Enable-VariationsPageJobs
(
	[Microsoft.SharePoint.SPSite]$Site
)
{
	if( -not ( Get-PSSnapIn | where { $_.Name -eq "Microsoft.SharePoint.PowerShell" } ) )
	{
		Add-PSSnapin Microsoft.SharePoint.PowerShell;
	}

	Write-Host "Enabling Variations Propagate Page Job Definition/Variations Create Page Job Definition" -ForegroundColor Green;
	$timerJob = Get-SPTimerJob | where { $_.Title -eq "Variations Propagate Page Job Definition" }
	if($timerJob){
		if($timerJob.Count){
			$count = $timerJob.Count;
			
			Write-Host "`tFound $count timer jobs, picking the one that matces the current web application name." -ForegroundColor Yellow;
			
			$timerJob = $timerJob | where {$_.WebApplication.DisplayName -eq $Site.WebApplication.DisplayName}
		}
		else{
			Write-Host "`tFound timer job" -ForegroundColor Green;
		}
	}
	else{
		Write-Host "`tCould not find timer job" -ForegroundColor Red;
	}
	
	Write-Host "Enabling Variations Propagate Page Job Definition" -ForegroundColor Green;
	Enable-SPTimerJob -Identity $timerJob;
	
	$timerJob = Get-SPTimerJob | where { $_.Title -eq "Variations Create Page Job Definition" }
	if($timerJob){
		if($timerJob.Count){
			$count = $timerJob.Count;
			
			Write-Host "`tFound $count timer jobs, picking the one that matces the current web application name." -ForegroundColor Yellow;
			
			$timerJob = $timerJob | where {$_.WebApplication.DisplayName -eq $Site.WebApplication.DisplayName}
		}
		else{
			Write-Host "`tFound timer job" -ForegroundColor Green;
		}
	}
	else{
		Write-Host "`tCould not find timer job" -ForegroundColor Red;
	}
	
	Write-Host "Enabling Variations Create Page Job Definition" -ForegroundColor Green;
	Enable-SPTimerJob -Identity $timerJob;
}

function Setup-SPStateService
(
	$StateServiceDisplayName,
	$StateServiceProxyDisplayName,
	$StateServiceDatabaseName
)
{
	if( -not ( Get-PSSnapIn | where { $_.Name -eq "Microsoft.SharePoint.PowerShell" } )  )
	{
		Add-PSSnapin Microsoft.SharePoint.PowerShell;
	}
	
	if( Get-SPStateServiceApplication | where { $_.Name -ieq $StateServiceDisplayName } )
	{
		Write-Host "State Service $($StateServiceDisplayName) already exists." -ForegroundColor Yellow;
	}
	else
	{
		$serviceApp = New-SPStateServiceApplication -Name $StateServiceDisplayName;
		New-SPStateServiceDatabase -Name $StateServiceDatabaseName -ServiceApplication $serviceApp;
		New-SPStateServiceApplicationProxy -Name $StateServiceProxyDisplayName -ServiceApplication $serviceApp -DefaultProxyGroup;
	}
}

function Setup-SPWebAnalyticsWebServiceApplication
(
	$AppPoolDisplayName,
	$AppPoolUserName,
	$WebAnalyticsServiceDisplayName,
	$SharePointDatabaseServer,
	$WebAnalyticsStagingDatabaseName,
	$WebAnalyticsReportingDatabaseName	
)
{
	if( -not ( Get-PSSnapIn | where { $_.Name -eq "Microsoft.SharePoint.PowerShell" } )  )
	{
		Add-PSSnapin Microsoft.SharePoint.PowerShell;
	}
	
	if( Get-SPWebAnalyticsServiceApplication -Identity $WebAnalyticsServiceDisplayName -ErrorAction SilentlyContinue )
	{
		Write-Host "Web Analytics Service Application $($WebAnalyticsServiceDisplayName) already exists." -ForegroundColor Yellow;
	}
	else
	{
		if( ( Get-SPServiceApplicationPool | where { $_.Name -ieq $AppPoolDisplayName  } ) -eq $null )
		{
			Write-Host "Creating new application Pool..." -ForegroundColor Green;
			$managedAccount = Get-SPManagedAccount | where {$_.UserName -eq $AppPoolUserName};
			New-SPServiceApplicationPool -Name $AppPoolDisplayName -Account $managedAccount;
		}

		$appPool = Get-SPServiceApplicationPool -Identity $AppPoolDisplayName -EA 0;
		$stagerSubscription = "<StagingDatabases><StagingDatabase ServerName='$SharePointDatabaseServer' DatabaseName='$WebAnalyticsStagingDatabaseName'/></StagingDatabases>";
		$reportingSubscription = "<ReportingDatabases><ReportingDatabase ServerName='$SharePointDatabaseServer' DatabaseName='$WebAnalyticsReportingDatabaseName'/></ReportingDatabases>";
		
		New-SPWebAnalyticsServiceApplication -Name $WebAnalyticsServiceDisplayName -ApplicationPool $AppPoolDisplayName -ReportingDataRetention 25 -SamplingRate 100 -ListOfReportingDatabases $reportingSubscription -ListOfStagingDatabases $stagerSubscription > $null;
		New-SPWebAnalyticsServiceApplicationProxy -Name $WebAnalyticsServiceDisplayName -ServiceApplication $WebAnalyticsServiceDisplayName > $null;
		Get-SPServiceInstance | where-object {$_.TypeName -eq "Web Analytics Web Service"} | Start-SPServiceInstance > $null;
		Get-SPServiceInstance | where-object {$_.TypeName -eq "Web Analytics Data Processing Service"} | Start-SPServiceInstance > $null;	
	}
}

function Setup-SPBusinessDataConnectivityServiceApplication
(
	$AppPoolDisplayName,
	$AppPoolUserName,
	$DisplayName,
	$DatabaseServer,
	$DatabaseName, 
	$ServiceContext
)
{
	if( -not ( Get-PSSnapIn | where { $_.Name -eq "Microsoft.SharePoint.PowerShell" } )  )
	{
		Add-PSSnapin Microsoft.SharePoint.PowerShell;
	}
		
	if( ( Get-SPServiceApplication | where {$_ -match $DisplayName} ) -ne $null )
	{
		Write-Host "Business Data Connectivity Service Application $($DisplayName) already exists." -ForegroundColor Yellow;
	}
	else
	{
		if( ( Get-SPServiceApplicationPool | where { $_.Name -ieq $AppPoolDisplayName  } ) -eq $null )
		{
			Write-Host "Creating new application Pool..." -ForegroundColor Green;
			$managedAccount = Get-SPManagedAccount | where {$_.UserName -eq $AppPoolUserName};
			New-SPServiceApplicationPool -Name $AppPoolDisplayName -Account $managedAccount;
		}

		$appPool = Get-SPServiceApplicationPool -Identity $AppPoolDisplayName -EA 0;
		
		New-SPBusinessDataCatalogServiceApplication -Name $DisplayName -ApplicationPool $AppPoolDisplayName -DatabaseServer $DatabaseServer -DatabaseName $DatabaseName > $null;	
		$bcs = Get-SPServiceApplication | where {$_ -match $DisplayName};
		$bcs.RevertToSelfAllowed = $true;
		$bcs.Update();
						
		Get-SPServiceInstance | where-object {$_.TypeName -eq "Business Data Connectivity Service"} | Start-SPServiceInstance > $null;	
	}
}

function Set-BDCModelsConfiguration{
    [CmdletBinding()]
    param 
    (
        [Parameter(Mandatory=$true)][ValidateNotNullOrEmpty()]
        [System.Xml.XmlElement]$configuration		
	)
	
	if( -not ( Get-PSSnapIn | where { $_.Name -eq "Microsoft.SharePoint.PowerShell" } )  )
	{
		Add-PSSnapin Microsoft.SharePoint.PowerShell;
	}
	
	Write-Host "Configuring bdc models..." -ForegroundColor Green;
	
	foreach( $bdcModel in $configuration.BDCModels.BDCModel ){

		if( $bdcModel.modelName -and $bdcModel.modelFilePath -and $bdcModel.serviceContext ){

			$modelName = $bdcModel.modelName;
			$serviceContext = $bdcModel.serviceContext;
			$modelFilePath = $bdcModel.modelFilePath;
						
			Write-Host "Configuring bdc model $($modelName)";
			
			$model = Get-SPBusinessDataCatalogMetadataObject -Name $modelName -BdcObjectType "Model" -ServiceContext $serviceContext;
				
			if( $bdcModel.deleteIfExists -and ( $bdcModel.deleteIfExists -ieq "true" ) -and ( $model -ne $null ) ){
				
				Write-Host "Removing bdc model $($modelName)";
				
				Remove-SPBusinessDataCatalogModel -Identity $model
			}
			
			if( $model -eq $null ){
				
				Write-Host "Importing bdc model $($modelName)";
				
				$metadataStore = Get-SPBusinessDataCatalogMetadataObject -BdcObjectType "Catalog" -ServiceContext $serviceContext;
				Import-SPBusinessDataCatalogModel -Path $modelFilePath -Identity $metadataStore -ModelsIncluded -PermissionsIncluded -PropertiesIncluded;
			}
			else{
				Write-Host "BDC model $($modelName) already exists.";
			}
		}
	}	
}