﻿//-----------------------------------------------------------------------
// <copyright file="CartController.cs" company="commerceserver.net">
//     Copyright © commerceserver.net.  All rights reserved.
// </copyright>
//-----------------------------------------------------------------------
namespace CommerceServerContrib.MVCSiteTemplate.Controllers
{
    using CommerceServerContrib.MVCSiteTemplate.Models;
    using CommerceServerContrib.MVCSiteTemplate.Models.Generated;
    using CommerceServerContrib.MVCSiteTemplate.RequestTemplates;
    using CommerceServerContrib.Web;
    using System;
    using System.Linq;
    using System.Web.Mvc;
    using System.Globalization;

    /// <summary>
    /// Handles all cart functions.
    /// </summary>
    public class CartController : BaseController
    {
        /// <summary>
        /// GET: /Cart/
        /// Gets the shopping cart, or Basket, for the current user or anonymous user.
        /// </summary>
        /// <param name="page">The page number, if any.</param>
        /// <returns>
        /// A View for a BasketViewModel.
        /// </returns>
        public ActionResult Index(string page)
        {
            var skip = Helpers.ItemsToSkip(page, 10);
            
            Basket basket = null;
            if (skip > 0)
            {
                var baskets = Orders.GetBasketPagedLineItems(null, Helpers.UserId(), "0", "true", skip.ToString(CultureInfo.InvariantCulture), "10", "true");
                basket = Helpers.GetDefaultBasketFromCollectionOfBaskets(baskets);
            }
            else
            {
                basket = BasketCachingHelper.GetMiniCartBasket(this.LargeBasketMode);
            }

            // need to calculate pages...
            ViewBag.CurrentPage = Helpers.CalculateCurrentPage(basket.LineItemsFirstItemIndex);
            ViewBag.TotalPages = Helpers.CalculateTotalPages(basket.LineItemsTotalItemCount);

            var cart = CreateBasketViewModel(basket);

            return View(cart);
        }

        /// <summary>
        /// Gets a BasketViewModel for the MiniCart.
        /// </summary>
        /// <returns>A PartialView for a BasketViewModel.</returns>
        public ActionResult MiniCart()
        {
            var basket = BasketCachingHelper.GetMiniCartBasket(this.LargeBasketMode);
            var cart = CreateBasketViewModel(basket);

            return PartialView("_MiniCart", cart);
        }

        /// <summary>
        /// Adds a LineItem to the user's Basket.
        /// </summary>
        /// <param name="viewModel">A LineItemViewModel representing the LineItem to be added to the Basket.</param>
        /// <returns>A message indicating if the operation was successful.</returns>
        [HttpPost]
        public bool AddToCart(LineItemViewModel viewModel)
        {
            try
            {
                BasketCachingHelper.InvalidateBasketCache();
                Basket basket = (Basket)(Orders.CreateLineItemWithPagedLineItems(
                    this.Context.DefaultBasketName,
                    this.UserId,
                    this.CartBasketType,
                    DoRefreshBasket,
                    this.DefaultCatalog,
                    viewModel.ProductId,
                    viewModel.VariantId,
                    viewModel.Quantity.ToString(),
                    "0",
                    "10",
                    "true").FirstOrDefault());
                BasketCachingHelper.AddBasketToCache(basket);

                return true;
            }
            catch (Exception ex)
            {
                CommerceServerContrib.Web.CommerceServerContribExceptionManager.HandleException(ex);
                return false;
            }
        }

        /// <summary>
        /// Removes a LineItem from the Basket.
        /// </summary>
        /// <param name="viewModel">The LineItem to remove.</param>
        /// <returns>True if successful.</returns>
        [HttpPost]
        [OutputCache(NoStore = true, Duration = 0, VaryByParam = "*")]
        public ActionResult DeleteLineItem(LineItemViewModel viewModel)
        {
            try
            {
				BasketCachingHelper.InvalidateBasketCache();
                var basket = (Basket)Orders.DeleteLineItem(this.Context.DefaultBasketName, this.UserId, this.CartBasketType, DoRefreshBasket, viewModel.Id)[0];
                BasketCachingHelper.AddBasketToCache(basket);
                var basketViewModel = CreateBasketViewModel(basket);

                ModelState.Clear();

                return PartialView("_cart", basketViewModel);
            }
            catch (Exception ex)
            {
                CommerceServerContribExceptionManager.HandleException(ex);
            }

            return null;
        }

        /// <summary>
        /// Updates a LineItem in the Basket.
        /// </summary>
        /// <param name="viewModel">The LineItem to update.</param>
        /// <returns>True if successful.</returns>
        [HttpPost]
        [OutputCache(NoStore = true, Duration = 0, VaryByParam = "*")]
        public ActionResult UpdateLineItem(LineItemViewModel viewModel)
        {
            try
            {
				BasketCachingHelper.InvalidateBasketCache();
                var basket = (Basket)Orders.UpdateLineItem(this.Context.DefaultBasketName, this.UserId, this.CartBasketType, DoRefreshBasket, viewModel.Id, viewModel.Quantity.ToString())[0];
                BasketCachingHelper.AddBasketToCache(basket);
                var basketViewModel = CreateBasketViewModel(basket);

                ModelState.Clear();

                return PartialView("_cart", basketViewModel);
            }
            catch (Exception ex)
            {
                CommerceServerContribExceptionManager.HandleException(ex);
            }

            return null;
        }

        private BasketViewModel CreateBasketViewModel(Basket basket)
        {
            if (basket == null)
            {
                return new BasketViewModel();
            }

            var basketViewModel = new BasketViewModel(basket) { BasketId = basket.Id };

            if (basket.BasketErrors != null)
            {
                basketViewModel.BasketErrors.AddRange(basket.BasketErrors.OfType<string>());
            }

            if (basket.LineItems != null && basket.LineItems.Any())
            {
                basketViewModel.LineItems.AddRange(
                    from li in basket.LineItems
                    select new LineItemViewModel
                        {
                            BasketId = basket.Id,
                            Id = li.Id,
                            ProductImagePath = li.ProductImageFilename,
                            ImageFilename = li.ProductImageFilename,
                            ProductId = li.ProductId,
                            DisplayName = li.DisplayName,
                            VariantId = li.VariantId,
                            Description = li.Description,
                            Quantity = li.Quantity.HasValue ? li.Quantity.Value : 0.0m,
                            ListPrice = li.ListPrice,
                            Total = li.Total.Value,
                            ProductViewModel = new ProductViewModel
                                {
                                    ImageFilename = li.ProductImageFilename
                                }
                        });
            }

            return basketViewModel;
        }
    }
}
