﻿//-----------------------------------------------------------------------
// <copyright file="CheckoutController.cs" company="commerceserver.net">
//     Copyright © commerceserver.net.  All rights reserved.
// </copyright>
//-----------------------------------------------------------------------
namespace CommerceServerContrib.MVCSiteTemplate.Controllers
{
    using CommerceServer.Foundation;
    using CommerceServerContrib.MVCSiteTemplate.Models;
    using CommerceServerContrib.MVCSiteTemplate.Models.Generated;
    using CommerceServerContrib.MVCSiteTemplate.RequestTemplates;
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Web.Mvc;
    using RT = CommerceServerContrib.MVCSiteTemplate.RequestTemplates;

    /// <summary>
    /// Handles all calls to checkout
    /// </summary>
    public class CheckoutController : BaseController
    {
        private const string ReadyForCheckout = "ReadyForCheckout";

        /// <summary>
        /// Handles the index view of the controller
        /// </summary>
        /// <returns>The action for this view</returns>
        [HttpGet]
        public ActionResult Index()
        {
            this.AddSupportingDataToViewBag();

            var baskets = RT.Orders.GetBasket(this.Context.DefaultBasketName, this.UserId, this.CartBasketType, DoRefreshBasket, "true");

            if (baskets != null && baskets.Count > 0)
            {
                var basket = (Basket)baskets[0];

                return View(new BasketViewModel(basket));
            }

            return View(new BasketViewModel());
        }

        /// <summary>
        /// Handles posting of baskets.
        /// </summary>
        /// <param name="basket">The basket.</param>
        /// <returns>Action result for either a submit, recalculate or error condition.</returns>
        [HttpPost]
        public ActionResult Index(BasketViewModel basket)
        {
            if (ModelState.IsValid)
            {
                bool cleanBasket = ShouldCleanBasket();

                if (this.IsCheckout())
                {
                    if (cleanBasket)
                    {
                        RT.Orders.CleanBasket(this.Context.DefaultBasketName, this.UserId, this.CartBasketType);
                    }

                    var result = UpdateBasket(basket, this.OrderBasketType, ReadyForCheckout);
                    if (result != null && result.Count > 0)
                    {
                        BasketCachingHelper.InvalidateBasketCache();
                        var resultBasket = (Basket)result[0];
                        return View("Submitted", new BasketViewModel(resultBasket));
                    }

                    return View(basket);
                }
                else
                {
                    this.AddSupportingDataToViewBag();
                    
                    if (cleanBasket)
                    {
                        RT.Orders.CleanBasket(this.Context.DefaultBasketName, this.UserId, this.CartBasketType);
                    }

                    var result = UpdateBasket(basket, this.CartBasketType, ReadyForCheckout);
                    if (result != null && result.Count > 0)
                    {
                        var resultBasket = (Basket)result[0];
                        //// Special handling for some state issues
                        var basketViewModel = new BasketViewModel(resultBasket);
                        BasketCachingHelper.AddBasketToCache(resultBasket);
                        basketViewModel.UseBillingAddressForShippingAddress = basket.UseBillingAddressForShippingAddress;
                        basketViewModel.CreditCardPayment = basket.CreditCardPayment;
                        return View(basketViewModel);
                    }

                    return View(basket);
                }
            }
            else
            {
                this.AddSupportingDataToViewBag();
                return View(basket);
            }
        }

        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Usage", "CA1806:DoNotIgnoreMethodResults", MessageId = "System.Boolean.TryParse(System.String,System.Boolean@)", Justification = "Do not care about the result.  The default value is fine.")]
        private bool ShouldCleanBasket()
        {
            var cleanBasketFormValue = Request.Form["cleanBasket"];
            bool cleanBasket = false;
            bool.TryParse(cleanBasketFormValue, out cleanBasket);
            return cleanBasket;
        }

        private bool IsCheckout()
        {
            if (Request.Form.AllKeys.Contains("Checkout"))
            {
                return true;
            }

            return false;
        }

        /// <summary>
        /// Updates the basket.
        /// </summary>
        /// <param name="basket">The basket.</param>
        /// <param name="basketType">Type of the basket.</param>
        /// <param name="basketStatus">The basket status.</param>
        /// <returns>Resulting baskets from the Update operation.</returns>
        private List<CommerceEntity> UpdateBasket(BasketViewModel basket, string basketType, string basketStatus)
        {
            var shippingAddressId = Guid.NewGuid().ToString();
            var billingAddressId = Guid.NewGuid().ToString();

            if (basket.UseBillingAddressForShippingAddress)
            {
                this.CopyBillingAddressIntoShippingAddress(basket);
            }

            BasketCachingHelper.InvalidateBasketCache();

            #region Update Basket
            var result = RT.Orders.UpdateBasket(
                this.Context.DefaultBasketName,
                this.UserId,
                "0",
                basketType,
                basketStatus,
                "true",
                shippingAddressId,
                basket.ShippingAddress.AddressName,
                basket.ShippingAddress.FirstName,
                basket.ShippingAddress.LastName,
                basket.ShippingAddress.Line1,
                basket.ShippingAddress.Line2,
                basket.ShippingAddress.City,
                basket.ShippingAddress.StateProvinceCode,
                basket.ShippingAddress.StateProvinceName,
                basket.ShippingAddress.ZipPostalCode,
                basket.ShippingAddress.CountryRegionCode,
                basket.ShippingAddress.CountryRegionName,
                basket.ShippingAddress.Telephone,
                billingAddressId,
                basket.BillingAddress.AddressName,
                basket.BillingAddress.FirstName,
                basket.BillingAddress.LastName,
                basket.BillingAddress.Line1,
                basket.BillingAddress.Line2,
                basket.BillingAddress.City,
                basket.BillingAddress.StateProvinceCode,
                basket.BillingAddress.StateProvinceName,
                basket.BillingAddress.ZipPostalCode,
                basket.BillingAddress.CountryRegionCode,
                basket.BillingAddress.CountryRegionName,
                basket.BillingAddress.Telephone,
                basket.ShippingMethodId,
                basket.CreditCardPayment.Month,
                basket.CreditCardPayment.Year,
                basket.CreditCardPayment.FirstName + " " + basket.CreditCardPayment.LastName,
                basket.CreditCardPayment.CreditCardNumber,
                basket.CreditCardPayment.CreditCardVerification,
                billingAddressId,
                basket.PaymentId);

            if (result != null && result.Count > 0)
            {
                var resultBasket = (Basket)result[0];
                BasketCachingHelper.AddBasketToCache(resultBasket);
            }
            #endregion

            return result;
        }

        private void CopyBillingAddressIntoShippingAddress(BasketViewModel basket)
        {
            var newShippingAddress = new AddressOrders();
            foreach (var property in basket.BillingAddress.Properties)
            {
                newShippingAddress.Properties[property.Key] = property.Value;
            }

            basket.ShippingAddress = newShippingAddress;
        }

        private void AddSupportingDataToViewBag()
        {
            ViewBag.PaymentMethods = RT.Profile.GetPaymentMethods();
            ViewBag.ShippingMethods = RT.Profile.GetShippingMethods();
        }
    }
}
