﻿//---------------------------------------------------------------------
// <copyright file="EntityExtensions.cs" company="commerceserver.net">
//     Copyright © commerceserver.net.  All rights reserved.
// </copyright>
// <summary>Extensions for dealing with Commerce Server entities.</summary>
//---------------------------------------------------------------------
namespace CommerceServerContrib.MVCSiteTemplate.Models
{
    using System.Collections.Generic;
    using System.ComponentModel;
    using System.Linq;
    using System.Reflection;

    /// <summary>
    /// A number of known image sizes.
    /// </summary>
    public enum ImageSize
    {
        /// <summary>
        /// The thumbnail image size.
        /// </summary>
        [Description("75/75/")]
        Thumbnail,

        /// <summary>
        /// The detail image size.  
        /// </summary>
        [Description("170/67/")]
        Detail,

        /// <summary>
        /// The medium image size.  
        /// </summary>
        [Description("480/240/")]
        Medium,

        /// <summary>
        /// The banner image size.
        /// </summary>
        [Description("1200/200/")]
        Banner,

        /// <summary>
        /// The large image size.
        /// </summary>
        [Description("1200/400/")]
        Large
    }

    /// <summary>
    /// Extensions for working with Commerce Entities in the MVC Site
    /// </summary>
    public static class EntityExtensions
    {
        /// <summary>
        /// Gets the full size image paths based on the BaseUrl in the app settings and the short path in the delimited property Image_Filename in the Product.
        /// </summary>
        /// <returns>
        /// The full size image paths.
        /// </returns>
        /// <param name="entity">An instance of a Commerce Server entity which implements the <see cref="IImageEntity"/> interface.</param>
        public static IEnumerable<string> MediumSizeImagePaths(this IImageEntity entity)
        {
            return entity.BuildImageList(ImageSize.Medium);
        }

        /// <summary>
        /// Gets the large size image paths based on BaseUrl in the app settings and the short path in the delimited property Image_Filename in the Product.
        /// </summary>
        /// <returns>
        /// The large size image paths.
        /// </returns>
        /// <param name="entity">An instance of a Commerce Server entity which implements the <see cref="IImageEntity"/> interface.</param>
        public static IEnumerable<string> LargeSizeImagePaths(this IImageEntity entity)
        {
            return entity.BuildImageList(ImageSize.Large);
        }

        /// <summary>
        /// Gets the thumbnail size image paths based on the BaseUrl in the app settings and the short path in the delimited file Image_Filename in the Product.
        /// </summary>
        /// <returns>
        /// The thumbnail image paths.
        /// </returns>
        /// <param name="entity">An instance of a Commerce Server entity which implements the <see cref="IImageEntity"/> interface.</param>
        public static IEnumerable<string> ThumbnailImagePaths(this IImageEntity entity)
        {
            return entity.BuildImageList(ImageSize.Thumbnail);
        }

        /// <summary>
        /// Gets the first images by index in the delimited Image_Filename property.
        /// </summary>
        /// <param name="entity">An instance of a Commerce Server entity which implements the <see cref="IImageEntity"/> interface.</param>
        /// <param name="size">The size of the image; must be one of the <see cref="ImageSize"/> constants.</param>
        /// <returns>
        /// Url to the image including the correct protocol.
        /// </returns>
        public static string FirstImagePath(this IImageEntity entity, ImageSize size)
        {
            var imageFileName = entity.ImageFilename;
            if (!string.IsNullOrWhiteSpace(imageFileName))
            {
                var splitImages = imageFileName.Split(';');
                if (splitImages != null && splitImages.Count() > 0)
                {
                    return string.Join(string.Empty, Helpers.GetImageBaseUrl(), size.GetDescription(), splitImages[0]);
                }
            }

            return string.Empty;
        }

        /* TODO:  Move to a shared extensions library?  Putting this in here for now. */

        /// <summary>
        /// Gets the value of the <see cref="T:System.ComponentModel.DescriptionAttribute"/> on an struct, including enums.  
        /// </summary>
        /// <typeparam name="T">The type of the struct.</typeparam>
        /// <param name="enumerationValue">A value of type <see cref="T:System.Enum"/></param>
        /// <returns>If the struct has a Description attribute, this method returns the description.  Otherwise it just calls ToString() on the struct.</returns>
        /// <remarks>Based on http://stackoverflow.com/questions/479410/enum-tostring/479417#479417, but useful for any struct.</remarks>
        public static string GetDescription<T>(this T enumerationValue) where T : struct
        {
            return enumerationValue.GetType().GetMember(enumerationValue.ToString())
                    .SelectMany(mi => mi.GetCustomAttributes<DescriptionAttribute>(false), (mi, ca) => ca.Description)
                    .FirstOrDefault() ?? enumerationValue.ToString();
        }

        private static string[] GetDelimitedListOfImages(this IImageEntity entity)
        {
            var images = entity.ImageFilename;
            if (!string.IsNullOrWhiteSpace(images))
            {
                return images.Split(';');
            }

            return null;
        }

        private static IEnumerable<string> BuildImageList(this IImageEntity entity, ImageSize size)
        {
            var baseUrl = Helpers.GetImageBaseUrl();

            if (entity != null)
            {
                var imagePaths = entity.GetDelimitedListOfImages();
                if (imagePaths != null)
                {
                    return imagePaths.Select(i => string.Join(string.Empty, baseUrl, size.GetDescription(), i));
                } 
            }

            return new List<string>() { string.Join(string.Empty, baseUrl, "/g/", size.GetDescription()) };
        }
    }
}
