﻿//-----------------------------------------------------------------------
// <copyright file="BasketCachingHelper.cs" company="commerceserver.net">
//     Copyright © commerceserver.net.  All rights reserved.
// </copyright>
//-----------------------------------------------------------------------

namespace CommerceServerContrib.MVCSiteTemplate.RequestTemplates
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using CommerceServer.Foundation;
    using CommerceServerContrib.MVCSiteTemplate.Models.Generated;
    using CommerceServerContrib.Web;

    /// <summary>
    /// Helper method of caching baskets to improve the performance of page loads with mini-carts.
    /// </summary>
    public static class BasketCachingHelper
    {
        private const string CacheGroupName = "MiniCart";
        private const string CachePrefix = "Default";
        private const string CacheName = "MiniCart";

        /// <summary>
        /// Invalidates the basket cache.
        /// </summary>
        public static void InvalidateBasketCache()
        {
            var cacheProvider = GetCacheProvider();
            cacheProvider.RemoveData(CachePrefix, CacheName, Helpers.UserId());
        }

        /// <summary>
        /// Adds the basket to cache.
        /// </summary>
        /// <param name="basket">The basket.</param>
        public static void AddBasketToCache(Basket basket)
        {
            var cacheProvider = GetCacheProvider();
            cacheProvider.AddData<Basket>(CachePrefix, CacheName, Helpers.UserId(), basket);
        }

        /// <summary>
        /// Gets the mini cart basket.
        /// </summary>
        /// <param name="largeBasketMode">if set to <c>true</c> the system is in large basket mode and a limited number of line items are returned.</param>
        /// <returns>
        /// A Basket which can be used to render the mini cart.
        /// </returns>
        public static Basket GetMiniCartBasket(bool largeBasketMode)
        {
            var cacheProvider = GetCacheProvider();
            Basket basket;
            if (!cacheProvider.TryGetData<Basket>(CachePrefix, CacheName, Helpers.UserId(), out basket))
            {
                /*
                 * Since we don't have the option to change the sort order of the line items I think we should have a perf
                 * mode in the config.  If it's enable than the mini-cart doesn't display the line items, 
                 * if it's disabled it displays all of the line items.
                 * */
                List<CommerceEntity> baskets = null;
                if (largeBasketMode)
                {
                    baskets = Orders.GetBasketPagedLineItems(null, Helpers.UserId(), "0", "false", "0", "10", "true");
                }
                else
                {
                    baskets = Orders.GetBasket(null, Helpers.UserId(), "0", "false", "true");
                }

                if (baskets != null && baskets.Any())
                {
                    // Find the first basket with name default
                    basket = Helpers.GetDefaultBasketFromCollectionOfBaskets(baskets);
                }
                else
                {
                    basket = new Basket();
                }

                cacheProvider.AddData(CachePrefix, CacheName, Helpers.UserId(), basket);
            }

            return basket;
        }

        private static ICacheProvider GetCacheProvider()
        {
            var cacheContext = PresentationCacheSettings.GetCacheContext(CacheGroupName);
            var cacheProvider = PresentationTypeLoader.CacheProvider(cacheContext);
            return cacheProvider;
        }
    }
}