﻿//---------------------------------------------------------------------
// <copyright file="Helpers.cs" company="commerceserver.net">
//     Copyright © commerceserver.net.  All rights reserved.
// </copyright>
// <summary>Helper for the site template.</summary>
//---------------------------------------------------------------------

namespace CommerceServerContrib.MVCSiteTemplate
{
    using System;
    using System.Collections.Generic;
    using System.Configuration;
    using System.Linq;
    using CommerceServer.Foundation;
    using CommerceServerContrib.MVCSiteTemplate.Models.Generated;
    using CommerceServerContrib.Web;

    /// <summary>
    /// Simple static methods for use in the MVC application.
    /// </summary>
    public static class Helpers
    {
        /// <summary>
        /// Items to skip when paging.
        /// </summary>
        /// <param name="page">Page requested.</param>
        /// <param name="itemsPerPage">Number of items per page.</param>
        /// <returns>Number of items to skip in subsequent query.</returns>
        public static int ItemsToSkip(string page, int itemsPerPage)
        {
            int pageAsInt;
            if (int.TryParse(page, out pageAsInt))
            {
                return (pageAsInt - 1) * itemsPerPage;
            }

            return 0;
        }

        /// <summary>
        /// Gets the image base URL and ensures that it has the correct protocol.
        /// </summary>
        /// <returns>Url as string for the base image path.</returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Usage", "CA2234:PassSystemUriObjectsInsteadOfStrings", Justification = "I want to use them as strings.")]
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1055:UriReturnValuesShouldNotBeStrings", Justification = "I want to use them as strings.")]
        public static string GetImageBaseUrl()
        {
            return CdnHelper.AdjustUrlScheme(ConfigurationManager.AppSettings["BaseImageUri"]);
        }

        /// <summary>
        /// Gets the anonymous user ID if the user is not logged in, and the proper user ID if the user is signed in.
        /// </summary>
        /// <returns>The user Id - a GUID as a string. </returns>
        public static string UserId()
        {
            var user = PresentationServiceLocator.Current.SiteContext.User;
            return string.IsNullOrWhiteSpace(user.Id) ? user.AnonymousId : user.Id;
        }

        /// <summary>
        /// Given a list of baskets this method will return the single default Commerce Server basket.
        /// </summary>
        /// <param name="baskets">A list of baskets.</param>
        /// <returns>Single Default Basket.</returns>
        public static Basket GetDefaultBasketFromCollectionOfBaskets(List<CommerceEntity> baskets)
        {
            Basket returnValue = null;

            if (baskets != null && baskets.Count > 0)
            {
                var basketCe = (baskets.FirstOrDefault(bk => bk.Properties.ContainsProperty(Basket.PropertyName.Name) &&
                                                          bk.GetPropertyValue(Basket.PropertyName.Name).ToString().Equals(
                                                            PresentationServiceLocator.Current.SiteContext.DefaultBasketName,
                                                            StringComparison.OrdinalIgnoreCase)));
                if (basketCe != null)
                {
                    returnValue = new Basket(basketCe);
                }
            }

            return returnValue;
        }

        /// <summary>
        /// Calculates the current pages assumuing 10 items per page.
        /// </summary>
        /// <param name="firstItemIndex">The first item index from the response.</param>
        /// <returns>The current page number.</returns>
        public static int CalculateCurrentPage(int firstItemIndex)
        {
            return (firstItemIndex / 10) + 1;
        }

        /// <summary>
        /// Calculates the total number of pages assuming 10 items per page. 
        /// </summary>
        /// <param name="totalItemCount">The total item count from the response.</param>
        /// <returns>The total number of pages.</returns>
        public static decimal CalculateTotalPages(int? totalItemCount)
        {
            if (totalItemCount != null && totalItemCount.HasValue && totalItemCount.Value > 0)
            {
                var returnValue = Math.Ceiling(Convert.ToDecimal(totalItemCount.Value) / 10m);
                return returnValue;
            }

            return 0;
        }
    }
}