﻿//-----------------------------------------------------------------------
// <copyright file="Helpers.cs" company="commerceserver.net">
//     Copyright © commerceserver.net.  All rights reserved.
// </copyright>
// <summary>A class to hold static helpers.  </summary>
//-----------------------------------------------------------------------
namespace CommerceServerContrib.SharePoint.Support
{
    using System;
    using System.Configuration;
    using System.Globalization;
    using CommerceServerContrib.Web;

    /// <summary>
    /// A class to hold static helpers.  
    /// </summary>
    public static class Helpers
    {
        /// <summary>
        /// Gets a value indicating whether if the site is in large basket mode.
        /// </summary>
        /// <value>
        ///   <c>true</c> if the site is in large basket mode; otherwise, <c>false</c>.
        /// </value>
        public static bool LargeBasketMode
        {
            get
            {
                bool returnValue = false;
                var basketModeAppSetting = ConfigurationManager.AppSettings["LargeBasketMode"];
                if (string.IsNullOrEmpty(basketModeAppSetting) && bool.TryParse(basketModeAppSetting, out returnValue))
                {
                    return returnValue;
                }

                return returnValue;
            }
        }

        /// <summary>
        /// Items to skip when paging.
        /// </summary>
        /// <param name="page">Page requested.</param>
        /// <param name="itemsPerPage">Number of items per page.</param>
        /// <returns>Number of items to skip in subsequent query.</returns>
        public static int ItemsToSkip(string page, int itemsPerPage)
        {
            int pageAsInt;
            if (int.TryParse(page, out pageAsInt))
            {
                return (pageAsInt - 1) * itemsPerPage;
            }

            return 0;
        }

        /// <summary>
        /// Gets the image base URL and ensures that it has the correct protocol.
        /// </summary>
        /// <returns>Url as string for the base image path.</returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Usage", "CA2234:PassSystemUriObjectsInsteadOfStrings", Justification = "I want to use them as strings.")]
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1055:UriReturnValuesShouldNotBeStrings", Justification = "I want to use them as strings.")]
        public static string GetImageBaseUrl()
        {
            return CdnHelper.AdjustUrlScheme(ConfigurationManager.AppSettings["BaseImageUri"]);
        }

        /// <summary>
        /// Turns a decimal value into a currency string
        /// </summary>
        /// <param name="currency">The decimal objet to act on</param>
        /// <returns>A decimal formatted as a string</returns>
        public static string ToCurrency(this decimal? currency)
        {
            return string.Format(CultureInfo.CurrentCulture, "{0:C}", currency.HasValue ? currency : 0);
        }

        /// <summary>
        /// Turns a decimal value into a currency string
        /// </summary>
        /// <param name="currency">The decimal objet to act on</param>
        /// <returns>A decimal formatted as a string</returns>
        public static string ToCurrency(this decimal currency)
        {
            return string.Format(CultureInfo.CurrentCulture, "{0:C}", currency);
        }

        /// <summary>
        /// Calculates the current pages assumuing 10 items per page.
        /// </summary>
        /// <param name="firstItemIndex">The first item index from the response.</param>
        /// <returns>The current page number.</returns>
        public static int CalculateCurrentPage(int firstItemIndex)
        {
            return (firstItemIndex / 10) + 1;
        }

        /// <summary>
        /// Calculates the total number of pages assuming 10 items per page. 
        /// </summary>
        /// <param name="totalItemCount">The total item count from the response.</param>
        /// <returns>The total number of pages.</returns>
        public static decimal CalculateTotalPages(int? totalItemCount)
        {
            if (totalItemCount != null && totalItemCount.HasValue && totalItemCount.Value > 0)
            {
                var returnValue = Math.Ceiling(Convert.ToDecimal(totalItemCount.Value) / 10m);
                return returnValue;
            }

            return 0;
        }
    }
}
