﻿//-----------------------------------------------------------------------
// <copyright file="AuthenticationControllerPage.cs" company="commerceserver.net">
//     Copyright © commerceserver.net.  All rights reserved.
// </copyright>
// <summary>A page to act as a controller for Authentication pages.  </summary>
//-----------------------------------------------------------------------
namespace CommerceServerContrib.SharePoint.Support
{
    using System;
    using System.Collections.Generic;
    using System.Web;
    using System.Web.Security;
    using System.Web.Services;
    using CommerceServer.Foundation.RequestTemplates;
    using CommerceServerContrib.CrossTier.Interfaces;
    using CommerceServerContrib.SharePoint.Support.Pages;
    using CommerceServerContrib.Web;

    /// <summary>
    /// A page to act as a controller for Authentication pages.  
    /// </summary>
    public partial class AuthenticationControllerPage : CommercePage
    {
        /// <summary>
        /// Handles the sign-in action.
        /// </summary>
        /// <param name="userName">Name of the user.</param>
        /// <param name="password">The password.</param>
        /// <returns>Simplified WebMethodResponse.</returns>
        [WebMethod]
        public static WebMethodResponse SignIn(string userName, string password)
        {
            #region Decode Input 
            var userNameDecoded = HttpUtility.UrlDecode(userName);
            var passwordDecoded = HttpUtility.UrlDecode(password);
            #endregion 

            string userId;
            var authenticationProvider = PresentationServiceLocator.Current.GetImplementation<IAuthenticationProvider>();
            var validUser = authenticationProvider.ValidateUser(out userId, userNameDecoded, passwordDecoded);
            bool signedIn = false;
            var returnValue = new WebMethodResponse();

            if (validUser)
            {
                // Call the service to get the correct username etc.?
                signedIn = authenticationProvider.SignIn(userId, userNameDecoded, userNameDecoded, passwordDecoded, false);
            }

            if (signedIn)
            {
                // Merge Baskets?
                returnValue.Success = true;
                returnValue.RedirectUrl = "/pages/default.aspx";
            }
            else
            {
                returnValue.Success = false;
            }

            return returnValue;
        }

        /// <summary>
        /// The first step in the forgot password action.
        /// </summary>
        /// <param name="email">The email.</param>
        /// <returns>Simplified WebMethodResponse.</returns>
        [WebMethod]
        public static WebMethodResponse ForgotPassword1(string email)
        {
            #region Decode Input
            var emailDecoded = HttpUtility.UrlDecode(email);
            #endregion 

            var authenticationProvider = PresentationServiceLocator.Current.GetImplementation<IAuthenticationProvider>();
            MembershipUser user = authenticationProvider.GetUser(emailDecoded);
            var returnValue = new WebMethodResponse();

            if (user != null && user.IsApproved)
            {
                var responseData = new List<object>();
                var adapterResponse = new ServiceAdapterResponse<object>();
                var questionAndEmail = new List<string>();
                questionAndEmail.Add(user.PasswordQuestion);
                questionAndEmail.Add(email);

                adapterResponse.Value = questionAndEmail;
                responseData.Add(adapterResponse);
                returnValue.HtmlPayload = RenderControl("/_layouts/CommerceServerContrib.Farm/Authentication/ForgotPassword2.ascx", responseData);
                returnValue.Success = true;
            }
            else
            {
                returnValue.Success = false;
            }
            
            return returnValue;
        }

        /// <summary>
        /// Handles the first password change action.
        /// </summary>
        /// <param name="answer">The answer.</param>
        /// <param name="email">The email.</param>
        /// <returns>Simplified WebMethodResponse.</returns>
        [WebMethod]
        public static WebMethodResponse ForgotPassword2(string answer, string email)
        {
            #region Decode Input 
            var answerDecoded = HttpUtility.UrlDecode(answer);
            var emailDecoded = HttpUtility.UrlDecode(email);
            #endregion 

            var authenticationProvider = PresentationServiceLocator.Current.GetImplementation<IAuthenticationProvider>();
            MembershipUser user = authenticationProvider.GetUser(emailDecoded);
            var webMethodResponse = new WebMethodResponse();

            try
            {
                string password = user.ResetPassword(answerDecoded);
                var responseData = new List<object>();
                var adapterResponse = new ServiceAdapterResponse<object>();
                adapterResponse.Value = password;
                responseData.Add(adapterResponse);
                webMethodResponse.HtmlPayload = RenderControl("/_layouts/CommerceServerContrib.Farm/Authentication/ForgotPassword3.ascx", responseData);
                webMethodResponse.Success = true;
            }
            catch (MembershipPasswordException)
            {
                // Report error
                webMethodResponse.Success = false;
            }

            return webMethodResponse;
        }

        /// <summary>
        /// Handles the change password action.
        /// </summary>
        /// <param name="currentPassword">The current password.</param>
        /// <param name="newPassword">The new password.</param>
        /// <param name="confirmPassword">The confirm password.</param>
        /// <returns>Simplified WebMethodResponse.</returns>
        [WebMethod]
        public static WebMethodResponse ChangePassword(string currentPassword, string newPassword, string confirmPassword)
        {
            #region Decode Input 
            var currentPasswordDecode = HttpUtility.UrlDecode(currentPassword);
            var newPasswordDecoded = HttpUtility.UrlDecode(newPassword);
            var confirmPasswordDecoded = HttpUtility.UrlDecode(confirmPassword);
            #endregion

            var authenticationProvider = PresentationServiceLocator.Current.GetImplementation<IAuthenticationProvider>();
            var webMethodResponse = new WebMethodResponse();
            MembershipUser user = authenticationProvider.GetUser(PresentationServiceLocator.Current.SiteContext.User.Email); 
            var passwordChanged = user.ChangePassword(currentPasswordDecode, newPasswordDecoded);

            if (passwordChanged)
            {
                // dump the person to the login page and make them sign in
                webMethodResponse.Success = true;
                webMethodResponse.RedirectUrl = "/pages/signin.aspx";
            }
            else
            {
                webMethodResponse.Success = false;
            }

            return webMethodResponse;
        }

        /// <summary>
        /// Handles the event to register a new user.
        /// </summary>
        /// <param name="firstName">The first name.</param>
        /// <param name="lastName">The last name.</param>
        /// <param name="email">The email.</param>
        /// <param name="confirmEmail">The confirm email.</param>
        /// <param name="password">The password.</param>
        /// <param name="confirmPassword">The confirm password.</param>
        /// <param name="securityQuestion">The security question.</param>
        /// <param name="securityAnswer">The security answer.</param>
        /// <returns>Simplified WebMethodResponse.</returns>
        [WebMethod]
        public static WebMethodResponse Register(
            string firstName,
            string lastName,
            string email,
            string confirmEmail,
            string password,
            string confirmPassword,
            string securityQuestion,
            string securityAnswer)
        {
            #region Decode Input
            var firstNameDecoded = HttpUtility.UrlDecode(firstName);
            var lastNameDecoded = HttpUtility.UrlDecode(lastName);
            var emailDecoded = HttpUtility.UrlDecode(email);
            var confirmEmailDecoded = HttpUtility.UrlDecode(confirmEmail);
            var passwordDecoded = HttpUtility.UrlDecode(password);
            var confirmPasswordDecoded = HttpUtility.UrlDecode(confirmPassword);
            var securityQuestionDecoded = HttpUtility.UrlDecode(securityQuestion);
            var securityAnswerDecoded = HttpUtility.UrlDecode(securityAnswer);
            #endregion

            string username = string.Concat(firstNameDecoded, " ", lastNameDecoded);
            MembershipCreateStatus status;
            var webMethodResponse = new WebMethodResponse();
            var user = Membership.CreateUser(
                emailDecoded,
                passwordDecoded,
                emailDecoded,
                securityQuestionDecoded,
                securityAnswerDecoded, 
                true, 
                out status);
            var userKey = (user == null) ? string.Empty : ((Guid)user.ProviderUserKey).ToString("B");

            if (user != null)
            {
                var authenticationProvider = PresentationServiceLocator.Current.GetImplementation<IAuthenticationProvider>();
                bool signedIn = authenticationProvider.SignIn(userKey, emailDecoded, username, passwordDecoded, false);
                if (signedIn)
                {
                    // Merge baskets
                    // redirect somewhere smart
                    webMethodResponse.Success = true;
                    webMethodResponse.RedirectUrl = "/pages/default.aspx";
                }
                else
                {
                    webMethodResponse.Success = false;
                }
            }
            else
            {
                webMethodResponse.Success = false;
            }

            return webMethodResponse;
        }
    }
}
