﻿//-----------------------------------------------------------------------
// <copyright file="CartControllerPage.cs" company="commerceserver.net">
//     Copyright © commerceserver.net.  All rights reserved.
// </copyright>
// <summary>A page to act as a controller for AJAX Cart operations. </summary>
//-----------------------------------------------------------------------
namespace CommerceServerContrib.SharePoint.Support
{
    using System;
    using System.Collections.Generic;
    using System.Configuration;
    using System.Globalization;
    using System.Linq;
    using System.Web;
    using System.Web.Services;
    using CommerceServer.Foundation;
    using CommerceServerContrib.SharePoint.Support.Entities;
    using CommerceServerContrib.Web;
    using RT = CommerceServerContrib.SharePoint.Support.RequestTemplates;

    /// <summary>
    /// A page to act as a controller for AJAX Cart operations.  
    /// </summary>
    public partial class CartControllerPage : CommercePage
    {
        private const string DoRefreshBasket = "true";

        #region Web Methods
        /// <summary>
        /// Used to refresh the mini cart via AJAX
        /// </summary>
        /// <returns>A rendered version of the mini cart</returns>
        [WebMethod]
        public static string MiniCart()
        {
            try
            {
                /*
                 * Since we don't have the option to change the sort order of the line items I think we should have a perf
                 * mode in the config.  If it's enable than the mini-cart doesn't display the line items, 
                 * if it's disabled it displays all of the line items.
                 * */
                List<CommerceEntity> baskets = null;
                if (Helpers.LargeBasketMode)
                {
                    baskets = RT.Orders.GetBasketPagedLineItems(
                        PresentationServiceLocator.Current.SiteContext.DefaultBasketName, 
                        PresentationServiceLocator.Current.SiteContext.UserId, 
                        "0", 
                        "false", 
                        "0", 
                        "10", 
                        "true");
                }
                else
                {
                    baskets = RT.Orders.GetBasket(
                        PresentationServiceLocator.Current.SiteContext.DefaultBasketName,
                        PresentationServiceLocator.Current.SiteContext.UserId,
                        PresentationServiceLocator.Current.SiteContext.DefaultBasketType.ToString(CultureInfo.InvariantCulture),
                        DoRefreshBasket, 
                        "true");
                }

                var responseData = BasketListToResponseData(baskets);

                return RenderControl("/_layouts/CommerceServerContrib.Farm/Cart/MiniCart.ascx", responseData);
            }
            catch (Exception ex)
            {
                CommerceServerContrib.Web.CommerceServerContribExceptionManager.HandleException(ex);
                return string.Empty;
            }
        }

        /// <summary>
        /// Used to add an item to cart via AJAX
        /// </summary>
        /// <param name="productId">The id of the product to add</param>
        /// <param name="variantId">The id of the products variant</param>
        /// <param name="quantity">The quantity of items to add</param>
        /// <returns>A boolean value indicating whether or not the add to cart succeeded</returns>
        [WebMethod]
        public static bool AddToCart(string productId, string variantId, int quantity)
        {
            #region Decode Input
            var productDecoded = HttpUtility.UrlDecode(productId);
            var variantDecoded = HttpUtility.UrlDecode(variantId);
            #endregion

            try
            {
                var basket = (Basket)RT.Orders.CreateLineItemWithPagedLineItems(
                    PresentationServiceLocator.Current.SiteContext.DefaultBasketName,
                    PresentationServiceLocator.Current.SiteContext.UserId,
                    PresentationServiceLocator.Current.SiteContext.DefaultBasketType.ToString(CultureInfo.InvariantCulture),
                    DoRefreshBasket,
                    ConfigurationManager.AppSettings["DefaultCommerceCatalog"],
                    productDecoded,
                    variantDecoded,
                    quantity.ToString(CultureInfo.InvariantCulture),
                    "0",
                    "10",
                    "true").FirstOrDefault<CommerceEntity>();

                return true;
            }
            catch (Exception ex)
            {
                CommerceServerContrib.Web.CommerceServerContribExceptionManager.HandleException(ex);
                return false;
            }
        }

        /// <summary>
        /// Removes an item from the basket
        /// </summary>
        /// <param name="lineItemId">The id of the line item to remove from the basket</param>
        /// <returns>A rendered cart control</returns>
        [WebMethod]
        public static string DeleteLineItem(string lineItemId)
        {
            #region Decode Input
            var lineItemIdDecoded = HttpUtility.UrlDecode(lineItemId);
            #endregion

            try
            {
                var basketList = RT.Orders.DeleteLineItem(
                    PresentationServiceLocator.Current.SiteContext.DefaultBasketName,
                    PresentationServiceLocator.Current.SiteContext.UserId,
                    PresentationServiceLocator.Current.SiteContext.DefaultBasketType.ToString(CultureInfo.InvariantCulture),
                    DoRefreshBasket,
                    lineItemIdDecoded);

                var responseData = BasketListToResponseData(basketList);

                return RenderControl("/_layouts/CommerceServerContrib.Farm/Cart/Cart.ascx", responseData);
            }
            catch (Exception ex)
            {
                CommerceServerContribExceptionManager.HandleException(ex);
            }

            return null;
        }

        /// <summary>
        /// Updates a line item in the basket
        /// </summary>
        /// <param name="lineItemId">The id of the line item to update</param>
        /// <param name="quantity">The quantity to update the lineitem to</param>
        /// <returns>A rendered cart page</returns>
        [WebMethod]
        public static string UpdateLineItem(string lineItemId, int quantity)
        {
            #region Decode Input
            var lineItemIdDecoded = HttpUtility.UrlDecode(lineItemId);
            #endregion

            try
            {
                var basketList = RT.Orders.UpdateLineItem(
                    PresentationServiceLocator.Current.SiteContext.DefaultBasketName,
                    PresentationServiceLocator.Current.SiteContext.UserId,
                    PresentationServiceLocator.Current.SiteContext.DefaultBasketType.ToString(CultureInfo.InvariantCulture),
                    DoRefreshBasket,
                    lineItemIdDecoded,
                    quantity.ToString(CultureInfo.InvariantCulture));

                var responseData = BasketListToResponseData(basketList);

                return RenderControl("/_layouts/CommerceServerContrib.Farm/Cart/Cart.ascx", responseData);
            }
            catch (Exception ex)
            {
                CommerceServerContribExceptionManager.HandleException(ex);
            }

            return null;
        }
        #endregion
    }
}
