﻿//-----------------------------------------------------------------------
// <copyright file="BasketViewModel.cs" company="commerceserver.net">
//     Copyright © commerceserver.net.  All rights reserved.
// </copyright>
// <summary>A ViewModel for displaying baskets.</summary>
//-----------------------------------------------------------------------
namespace CommerceServerContrib.SharePoint.Support.ViewModels
{
    using System.Collections.Generic;
    using System.Linq;
    using CommerceServer.Foundation;
    using CommerceServerContrib.SharePoint.Support.Entities;

    /// <summary>
    /// A ViewModel for displaying baskets.  
    /// </summary>
    public class BasketViewModel
    {
        private readonly List<LineItemViewModel> lineItems = new List<LineItemViewModel>();
        private readonly List<string> basketErrors = new List<string>();
        private Basket basket;
        private string shippingMethodId;
        private string paymentId;
        private PaymentMethodViewModel creditCardViewModel = new PaymentMethodViewModel();
        private bool useBillingAddressForShippingAddress = false;

        /// <summary>
        /// Initializes a new instance of the <see cref="BasketViewModel" /> class.
        /// </summary>
        /// <param name="basket">The basket.</param>
        public BasketViewModel(Basket basket)
        {
            this.basket = basket;
            this.shippingMethodId = this.ShippingMethodIdFromLineItems;
            this.paymentId = this.Payment != null && this.Payment.PaymentAccount != null && this.Payment.PaymentAccount.PaymentMethod != null ? this.Payment.PaymentAccount.PaymentMethod.Id : string.Empty;
            this.useBillingAddressForShippingAddress = this.DetermineIfBillingAddressAndShippingAddressAreTheSame();
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="BasketViewModel" /> class.
        /// </summary>
        /// <param name="basket">The basket.</param>
        public BasketViewModel(CommerceEntity basket) : this(new Basket(basket))
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="BasketViewModel" /> class.
        /// </summary>
        public BasketViewModel()
        {
            this.basket = new Basket();
        }

        /// <summary>
        /// Gets or sets the Basket Id.  
        /// </summary>
        public string BasketId { get; set; }

        /// <summary>
        /// Gets a list of LineItems.
        /// </summary>
        public List<LineItemViewModel> LineItems
        {
            get
            {
                return this.lineItems;
            }
        }

        /// <summary>
        /// Gets the first item index of the return line items. 
        /// </summary>
        public int LineItemsFirstItemIndex
        {
            get
            {
                return this.basket.LineItemsFirstItemIndex;
            }
        }

        /// <summary>
        /// Gets the Total Line item count.
        /// </summary>
        public int? TotalLineItemCount
        {
            get
            {
                if (this.basket.LineItemsTotalItemCount.HasValue)
                {
                    return this.basket.LineItemsTotalItemCount;
                }

                return 0;
            }
        }

        /// <summary>
        /// Gets or sets the UserId.
        /// </summary>
        public string UserId { get; set; }

        /// <summary>
        /// Gets or sets the shipping address.
        /// </summary>
        /// <value>
        /// The shipping address.
        /// </value>
        public AddressOrders ShippingAddress
        {
            get
            {
                return this.GetAddress("Shipping");
            }

            set
            {
                var address = value;
                address.AddressName = "Shipping";
                this.AddressFix(address, address.AddressName);
            }
        }

        /// <summary>
        /// Gets or sets the billing address.
        /// </summary>
        /// <value>
        /// The billing address.
        /// </value>
        public AddressOrders BillingAddress
        {
            get
            {
                return this.GetAddress("Billing");
            }

            set
            {
                var address = value;
                address.AddressName = "Billing";
                this.AddressFix(address, address.AddressName);
            }
        }

        /// <summary>
        /// Gets or sets the payment id.
        /// </summary>
        /// <value>
        /// The payment id.
        /// </value>
        public string PaymentId
        {
            get
            {
                return this.paymentId;
            }

            set
            {
                this.paymentId = value;
            }
        }

        /// <summary>
        /// Gets or sets the shipping method id.
        /// </summary>
        /// <value>
        /// The shipping method id.
        /// </value>
        public string ShippingMethodId
        {
            get
            {
                return this.shippingMethodId;
            }

            set
            {
                this.shippingMethodId = value;
            }
        }

        /// <summary>
        /// Gets or sets the credit card payment.
        /// </summary>
        /// <value>
        /// The credit card payment.
        /// </value>
        public PaymentMethodViewModel CreditCardPayment
        {
            get
            {
                return this.creditCardViewModel;
            }

            set
            {
                this.creditCardViewModel = value;
            }
        }

        /// <summary>
        /// Gets the total from the basket and provides a UIHint.
        /// </summary>
        /// <value>
        /// The total.
        /// </value>
        public decimal? CurrencyTotal
        {
            get
            {
                return this.basket.Total;
            }
        }

        /// <summary>
        /// Gets the shipping total and provides a UIHint.
        /// </summary>
        /// <value>
        /// The shipping total.
        /// </value>
        public decimal? CurrencyShippingTotal
        {
            get
            {
                return this.basket.ShippingTotal;
            }
        }

        /// <summary>
        /// Gets the tax total and provides a UIHint.
        /// </summary>
        /// <value>
        /// The tax total.
        /// </value>
        public decimal? CurrencyTaxTotal
        {
            get
            {
                return this.basket.TaxTotal;
            }
        }

        /// <summary>
        /// Gets the sub total and provides a UIHint.
        /// </summary>
        /// <value>
        /// The sub total.
        /// </value>
        public decimal? CurrencySubtotal
        {
            get
            {
                return this.basket.SubTotal;
            }
        }

        /// <summary>
        /// Gets the order number.
        /// </summary>
        /// <value>
        /// The order number.
        /// </value>
        public string OrderNumber
        {
            get
            {
                return this.basket.OrderNumber;
            }
        }

        /// <summary>
        /// Gets or sets a value indicating whether to use the billing address for the shipping method.
        /// </summary>
        /// <value>
        /// <c>true</c> if the billing address is to be used for the shipping address then true; otherwise, <c>false</c>.
        /// </value>
        public bool UseBillingAddressForShippingAddress
        {
            get
            {
                return this.useBillingAddressForShippingAddress;
            }

            set
            {
                this.useBillingAddressForShippingAddress = value;
            }
        }

        /// <summary>
        /// Gets a value indicating whether the basket should be cleaned out before saving.
        /// </summary>
        /// <value>
        /// <c>true</c> if should be cleaned before saving; otherwise, <c>false</c>.
        /// </value>
        public bool BasketNeedsToBeCleanedBeforeSave
        {
            get
            {
                return (this.basket.Addresses != null && this.basket.Addresses.Count > 0) || (this.basket.Payments != null && this.basket.Payments.Count > 0);
            }
        }

        /// <summary>
        /// Gets a list of BasketErrors which may occur during operations on the Basket, i.e. a product is no longer available and has been removed from the basket.
        /// </summary>
        public List<string> BasketErrors
        {
            get
            {
                return this.basketErrors;
            }
        }

        /// <summary>
        /// Gets or sets the basket payments.
        /// </summary>
        /// <value>
        /// The basket payments.
        /// </value>
        protected Payment Payment
        {
            get
            {
                if (this.basket.Payments != null && this.basket.Payments.Count > 0)
                {
                    return this.basket.Payments[0];
                }

                return null;
            }

            set
            {
                var paymentRelationship = this.basket.Properties[Basket.RelationshipName.Payments] as CommerceRelationshipList;
                if (paymentRelationship == null)
                {
                    paymentRelationship = new CommerceRelationshipList();
                    this.basket.Properties[Basket.RelationshipName.Payments] = paymentRelationship;
                }
                else if (paymentRelationship.Count > 0)
                {
                    paymentRelationship.Clear();
                }

                if (value != null)
                {
                    paymentRelationship.Add(new CommerceRelationship(value.ToCommerceEntity()));
                }
            }
        }

        /// <summary>
        /// Gets or sets the shipping method.
        /// </summary>
        /// <value>
        /// The shipping method.
        /// </value>
        protected ShippingMethod ShippingMethod
        {
            get
            {
                if (this.basket.Shipments != null && this.basket.Shipments.Count > 0)
                {
                    return this.basket.Shipments[0].ShippingMethod;
                }

                return null;
            }

            set
            {
                var shipmentRelationshipList = this.basket.Properties[Basket.RelationshipName.Shipments] as CommerceRelationshipList;
                if (shipmentRelationshipList == null)
                {
                    shipmentRelationshipList = new CommerceRelationshipList();
                    this.basket.Properties[Basket.RelationshipName.Shipments] = shipmentRelationshipList;
                }

                if (shipmentRelationshipList.Count > 0)
                {
                    var shippment = shipmentRelationshipList[0];
                    shipmentRelationshipList.Remove(shippment);
                }

                if (value != null)
                {
                    shipmentRelationshipList.Add(new CommerceRelationship(value.ToCommerceEntity()));
                }
            }
        }

        private string ShippingMethodIdFromLineItems
        {
            get
            {
                if (this.basket.LineItems != null && this.basket.LineItems.Count > 0)
                {
                    return this.basket.LineItems[0].ShippingMethod;
                }

                return string.Empty;
            }
        }

        private bool DetermineIfBillingAddressAndShippingAddressAreTheSame()
        {
            if (this.basket.Addresses != null &&
                this.basket.Addresses.Count > 0 &&
                this.basket.Addresses.Where(a => a.AddressName == "Shipping").Count() == 1 &&
                this.basket.Addresses.Where(a => a.AddressName == "Billing").Count() == 1)
            {
                if (this.BillingAddress.Equals(this.ShippingAddress))
                {
                    return true;
                }
                else
                {
                    return false;
                }
            }

            return true;
        }

        private void AddressFix(AddressOrders address, string addressName)
        {
            #region Creating collection
            var addressRelationship = this.basket.Properties[Basket.RelationshipName.Addresses] as CommerceRelationshipList;
            if (addressRelationship == null)
            {
                addressRelationship = new CommerceRelationshipList();
                this.basket.Properties[Basket.RelationshipName.Addresses] = addressRelationship;
            }
            else
            {
                var addressToDelete = addressRelationship.Where(r => r.Target.Properties[AddressOrders.PropertyName.AddressName].Equals(addressName)).SingleOrDefault();
                if (addressToDelete != null)
                {
                    addressRelationship.Remove(addressToDelete);
                }
            }

            var relationship = new CommerceRelationship(address.ToCommerceEntity());
            addressRelationship.Add(relationship);
            #endregion
        }

        private AddressOrders GetAddress(string type)
        {
            if (this.basket.Addresses != null)
            {
                var address = this.basket.Addresses.Where(a => a.AddressName == type).SingleOrDefault();
                if (address != null)
                {
                    return address;
                }
            }

            return new AddressOrders();
        }
    }
}
