﻿//-----------------------------------------------------------------------
// <copyright file="Brand.cs" company="commerceserver.net">
//     Copyright © commerceserver.net.  All rights reserved.
// </copyright>
// <summary>Defines the Brand class</summary>
//-----------------------------------------------------------------------

namespace CommerceServerContrib.CrossTier
{
    using CommerceServer.Foundation.Configuration;
    using CommerceServerContrib.CrossTier.Configuration;

    /// <summary>
    /// Class that represents the <see cref="Brand"/> configuration element.
    /// </summary>
    public class Brand : ConfigurationMappedItem
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="Brand"/> class.
        /// </summary>
        /// <param name="element">The <see cref="BaseBrandElement"/> this class is mapped to.</param>
        public Brand(BaseBrandElement element)
            : base(element)
        {
            if (element != null)
            {
                this.Name = element.Name;
                this.OperationServiceName = element.OperationServiceName;
                this.ExceptionHandling = new ExceptionHandling(element.ExceptionHandling);
                this.UserSecurityManagement = new UserSecurityManagement(element.UserSecurityManagement);
                this.EncryptionProviders = new EncryptionProviderDictionary(element.EncryptionKeys);
                this.SecurePageSettings = new SecurePageSettings(element.SecurePageSettings);
            }
            else
            {
                this.ExceptionHandling = new ExceptionHandling();
                this.UserSecurityManagement = new UserSecurityManagement();
                this.EncryptionProviders = new EncryptionProviderDictionary();
                this.SecurePageSettings = new SecurePageSettings();
            }
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="Brand"/> class.
        /// </summary>
        public Brand()
            : this(null)
        {
        }

        /// <summary>
        /// Gets the name of this brand.
        /// </summary>
        public string Name
        {
            get;
            private set;
        }

        /// <summary>
        /// Gets the name of the Operation Service Name.
        /// </summary>
        public string OperationServiceName
        {
            get;
            private set;
        }

        /// <summary>
        /// Gets the exception handling configuration for this brand.
        /// </summary>
        public ExceptionHandling ExceptionHandling
        {
            get;
            private set;
        }

        /// <summary>
        /// Gets the user security management configuration for this brand.
        /// </summary>
        public UserSecurityManagement UserSecurityManagement
        {
            get;
            private set;
        }

        /// <summary>
        /// Gets the encryption provider configuration for this brand.
        /// </summary>
        public EncryptionProviderDictionary EncryptionProviders
        {
            get;
            private set;
        }

        /// <summary>
        /// Gets the secure pages settings configuration for this brand.
        /// </summary>
        public SecurePageSettings SecurePageSettings
        {
            get;
            private set;
        }

        /// <summary>
        /// Merges this brand with the default brand.
        /// </summary>
        /// <param name="defaultBrand">The default brand to merge with this brand.</param>
        public void MergeWith(Brand defaultBrand)
        {
            if (!this.EncryptionProviders.IsMappedToConfiguration)
            {
                this.EncryptionProviders = defaultBrand.EncryptionProviders;
            }

            if (!this.ExceptionHandling.IsMappedToConfiguration)
            {
                this.ExceptionHandling = defaultBrand.ExceptionHandling;
            }

            if (!this.UserSecurityManagement.IsMappedToConfiguration)
            {
                this.UserSecurityManagement = defaultBrand.UserSecurityManagement;
            }

            if (!this.SecurePageSettings.IsMappedToConfiguration)
            {
                this.SecurePageSettings = defaultBrand.SecurePageSettings;
            }
        }
    }
}
