﻿//-----------------------------------------------------------------------
// <copyright file="CsrfSettingsElement.cs" company="commerceserver.net">
//     Copyright © commerceserver.net.  All rights reserved.
// </copyright>
// <summary>Defines the CsrfSettingsElement class</summary>
//-----------------------------------------------------------------------

namespace CommerceServerContrib.CrossTier.Configuration
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Text;
    using System.Configuration;
    using System.Globalization;

    /// <summary>
    /// Defines the action to take when a potential CSRF attack is identified.
    /// </summary>
    [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1704:IdentifiersShouldBeSpelledCorrectly", MessageId = "Csrf", Justification = "CSRF is a valid acronym.")]
    public enum CsrfErrorHandlingBehavior
    {
        /// <summary>
        /// Indicates that an exception should be thrown when a potential CSRF attack is identified.
        /// </summary>
        ThrowException,

        /// <summary>
        /// Indicates that the client should be redirected to a different web page when a potential CSRF attack is identified.
        /// </summary>
        RedirectToPage
    }

    /// <summary>
    /// Represents configuration for the CommerceServerContribCsrfMitigationModule.
    /// </summary>
    [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1704:IdentifiersShouldBeSpelledCorrectly", MessageId = "Csrf", Justification = "CSRF is a valid acronym.")]
    public class CsrfSettingsElement : ConfigurationElement
    {
        /// <summary>
        /// The default value for the hashingKey attribute.
        /// </summary>
        public const string HashingKeyDefaultValue = "DefaultCsrfHashingKey";

        /// <summary>
        /// The default value for the formFieldName attribute.
        /// </summary>
        public const string FormFieldNameDefaultValue = "__CSRFTOKEN";

        /// <summary>
        /// The default value for the errorHandlingBehavior attribute.
        /// </summary>
        public const CsrfErrorHandlingBehavior ErrorHandlingBehaviorDefaultValue = CsrfErrorHandlingBehavior.ThrowException;

        private const string HashingKeyAttributeName = "hashingKey";
        private const string FormFieldNameAttributeName = "formFieldName";
        private const string ErrorHandlingBehaviorAttributeName = "errorHandlingBehavior";
        private const string ErrorPageAttributeName = "errorPage";

        /// <summary>
        /// Gets or sets the key used used to hash the CSRF mitigation token.
        /// </summary>
        [ConfigurationProperty(HashingKeyAttributeName, IsRequired = false, DefaultValue = HashingKeyDefaultValue)]
        public string HashingKey
        {
            get { return (this[HashingKeyAttributeName] as string) ?? HashingKeyAttributeName; }
            set { this[HashingKeyAttributeName] = value; }
        }

        /// <summary>
        /// Gets or sets the name of the form field that is used to hold the CSRF mitigation token.
        /// </summary>
        [ConfigurationProperty(FormFieldNameAttributeName, IsRequired = false, DefaultValue = FormFieldNameDefaultValue)]
        public string FormFieldName
        {
            get { return (this[FormFieldNameAttributeName] as string) ?? FormFieldNameDefaultValue; }
            set { this[FormFieldNameAttributeName] = value; }
        }

        /// <summary>
        /// Gets or sets the behavior the CommerceServerContribCsrfMitigationModule will take when a potential CSRF attack is detected.
        /// </summary>
        [ConfigurationProperty(ErrorHandlingBehaviorAttributeName, IsRequired = false, DefaultValue = ErrorHandlingBehaviorDefaultValue)]
        public CsrfErrorHandlingBehavior ErrorHandlingBehavior
        {
            get { return (this[ErrorHandlingBehaviorAttributeName] as CsrfErrorHandlingBehavior?) ?? ErrorHandlingBehaviorDefaultValue; }
            set { this[ErrorHandlingBehaviorAttributeName] = value; }
        }

        /// <summary>
        /// Gets or sets the error page that clients will be redirected to when a potential CSRF attack is detected.
        /// </summary>
        /// <remarks>
        /// This parameter is only used when <see cref="CsrfSettingsElement.ErrorHandlingBehavior"/> is <see cref="CsrfErrorHandlingBehavior.RedirectToPage"/>.
        /// </remarks>
        [ConfigurationProperty(ErrorPageAttributeName, IsRequired = false)]
        public string ErrorPage
        {
            get { return this[ErrorPageAttributeName] as string; }
            set { this[ErrorPageAttributeName] = value; }
        }

        /// <summary>
        /// Called after deserialization.
        /// </summary>
        protected override void PostDeserialize()
        {
            base.PostDeserialize();

            if ((this.ErrorHandlingBehavior == CsrfErrorHandlingBehavior.RedirectToPage) &&
                (string.IsNullOrEmpty(this.ErrorPage)))
            {
                string message = string.Format(
                    CultureInfo.CurrentCulture,
                    CrossTierResources.ErrorPageNotProvided_Value_Attrib1_Attrib2,
                    this.ErrorHandlingBehavior,
                    ErrorHandlingBehaviorAttributeName,
                    ErrorPageAttributeName);
                throw new ConfigurationErrorsException(message);
            }
        }
    }
}
