﻿//---------------------------------------------------------------------
// <copyright file="PresentationConfigurationSection.cs" company="commerceserver.net">
//     Copyright © commerceserver.net.  All rights reserved.
// </copyright>
// <summary></summary>
//---------------------------------------------------------------------
namespace CommerceServerContrib.CrossTier.Configuration
{
    using System.Configuration;
    using System.Diagnostics;
    using System;
    using System.ComponentModel;
    using System.Collections.Generic;

    /// <summary>
    /// Handles the "commerceServerContrib/presentation" configuration element
    /// </summary>
    /// DebuggerNonUserCode is added as a way to stop this object counting against code coverage,
    /// As this is just an entity class there is no reason or benifit to testing it
    [DebuggerNonUserCode]
    public class PresentationConfigurationSection : ConfigurationSection
    {
        private const string CsrfSettingsElementName = "csfrSettings";
        private const string DefaultBrandElementName = "defaultBrand";
        private const string BrandsElementName = "brands";
        private const string SiteSettingElementName = "sites";
        private const string EnableXsltDebuggingAttribute = "enableXsltDebugging";
        private const string RenderFileLocationAttribute = "renderFileLocation";
        private const string PreferredRenderTypesAttribute = "preferredRenderTypes";
        private const string DisableRedirectsAttribute = "disableRedirects";
        private const string CultureAttributeName = "culture";
        private const string UICultureAttributeName = "uiCulture";
        private const string XmlnsAttribute = "xmlns";

        #region Properties (public, non-static)

        /// <summary>
        /// Gets or sets the CSRF mitigation settings.
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1704:IdentifiersShouldBeSpelledCorrectly", MessageId = "Csrf", Justification = "CSRF is a valid acronym.")]
        [ConfigurationProperty(CsrfSettingsElementName, IsRequired = false)]
        public CsrfSettingsElement CsrfSettings
        {
            get
            {
                var temp = this[CsrfSettingsElementName] as CsrfSettingsElement;
                if (temp == null)
                {
                    temp = new CsrfSettingsElement();
                    this[CsrfSettingsElementName] = temp;
                }

                return temp;
            }

            set
            {
                this[CsrfSettingsElementName] = value;
            }
        }

        /// <summary>
        /// Gets or sets the "caching" element.
        /// </summary>
        /// <value>
        /// The "caching" element of the configuration file.
        /// </value>
        [ConfigurationProperty(DefaultBrandElementName, IsRequired = true)]
        public DefaultBrandElement DefaultBrand
        {
            get { return (DefaultBrandElement)this[DefaultBrandElementName]; }
            set { this[DefaultBrandElementName] = value; }
        }

        /// <summary>
        /// Gets the "brands" configuration element collection.
        /// </summary>
        [ConfigurationProperty(BrandsElementName, IsRequired = false)]
        public BrandElementCollection Brands
        {
            get { return (BrandElementCollection)this[BrandsElementName]; }
        }

        /// <summary>
        /// Gets the site settings.
        /// </summary>
        /// <value>
        /// The site settings.
        /// </value>
        [ConfigurationProperty(SiteSettingElementName, IsRequired = false)]
        public SiteSettingsElementCollection SiteSettings
        {
            get
            {
                return (SiteSettingsElementCollection)this[SiteSettingElementName];
            }
        }

        /// <summary>
        /// Gets or sets a value indicating whether XSLT debugging is enabled.
        /// </summary>
        [ConfigurationProperty(EnableXsltDebuggingAttribute, IsRequired = false, DefaultValue = false)]
        public bool EnableXsltDebugging
        {
            get { return (this[EnableXsltDebuggingAttribute] as bool?) ?? false; }
            set { this[EnableXsltDebuggingAttribute] = value; }
        }

        /// <summary>
        /// Gets or sets the base location to pick up render files.
        /// </summary>
        [ConfigurationProperty(RenderFileLocationAttribute, IsRequired = true)]
        public string RenderFileLocation
        {
            get { return (string)this[RenderFileLocationAttribute]; }
            set { this[RenderFileLocationAttribute] = value; }
        }

        /// <summary>
        /// Gets or sets the preferred types to render data with e.g. ascx;xslt
        /// </summary>
        [ConfigurationProperty(PreferredRenderTypesAttribute, IsRequired = true)]
        public string PreferredRenderTypes
        {
            get { return (string)this[PreferredRenderTypesAttribute]; }
            set { this[PreferredRenderTypesAttribute] = value; }
        }

        /// <summary>
        /// Gets or sets a value indicating whether or not all redirects should be disabled.
        /// </summary>
        /// <value>
        /// [True] to redirect [False] otherwise.
        /// </value>
        [ConfigurationProperty(DisableRedirectsAttribute, IsRequired = false)]
        [TypeConverter(typeof(BooleanConverter))]
        public bool DisableRedirects
        {
            get { return (bool)this[DisableRedirectsAttribute]; }
            set { this[DisableRedirectsAttribute] = value; }
        }	

        /// <summary>
        /// Gets or sets the XML namespace of the element.
        /// </summary>
        [ConfigurationProperty(XmlnsAttribute, IsRequired = false)]
        public string Xmlns
        {
            get 
            { 
                return (string)this[XmlnsAttribute]; 
            }

            set 
            { 
                this[XmlnsAttribute] = value; 
            }
        }
        #endregion

        /// <summary>
        /// Gets a value indicating whether an unknown attribute is encountered during deserialization.
        /// </summary>
        /// <param name="name">The name of the unrecognized attribute.</param>
        /// <param name="value">The value of the unrecognized attribute.</param>
        /// <returns>true when an unknown attribute is encountered while deserializing; otherwise, false.</returns>
        protected override bool OnDeserializeUnrecognizedAttribute(string name, string value)
        {
            if (string.Equals(name, "xmlns", StringComparison.Ordinal))
            {
                return true;
            }

            return base.OnDeserializeUnrecognizedAttribute(name, value);
        }
    }
}
