﻿//-----------------------------------------------------------------------
// <copyright file="SecurePageSettingsElement.cs" company="commerceserver.net">
//     Copyright © commerceserver.net.  All rights reserved.
// </copyright>
// <summary>Config section for SecurePageSettingsElement.</summary>
//-----------------------------------------------------------------------

namespace CommerceServerContrib.CrossTier.Configuration
{
    using System.Configuration;
    using System.Diagnostics;

    /// <summary>
    /// Secure Page Section class.
    /// </summary>
    /// DebuggerNonUserCode is added as a way to stop this object counting against code coverage,
    /// As this is just an entity class there is no reason or benifit to testing it
    [DebuggerNonUserCode]
    public class SecurePageSettingsElement : BasePresentationElement
    {
        private const string RequireSslAttribute = "requireSsl";
        private const string RequireSslAttributeDefaultValue = "false";
        private const string SecureHostNameAttribute = "secureHostName";
        private const string SecureHostNameAttributeDefaultValue = "";
        private const string SecurePortAttribute = "securePort";
        private const string SecurePortAttributeDefaultValue = "443";
        private const string NonSecureHostNameAttribute = "nonSecureHostName";
        private const string NonSecureHostNameAttributeDefaultValue = "";
        private const string NonSecurePortNameAttribute = "nonSecurePort";
        private const string NonSecurePortNameAttributeDefaultValue = "80";
        private const string PagesElementCollection = "pages";
        private const string SslOffloadPortAttribute = "sslOffloadPort";

        /// <summary>
        /// Gets or sets a value indicating whether [require SSL].
        /// </summary>
        /// <value><c>true</c> if [require SSL]; otherwise, <c>false</c>.</value>
        [ConfigurationProperty(RequireSslAttribute, DefaultValue = RequireSslAttributeDefaultValue, IsRequired = true)]
        public bool RequireSsl
        {
            get
            {
                return (bool)this[RequireSslAttribute]; 
            }

            set
            {
                this[RequireSslAttribute] = value; 
            }
        }

        /// <summary>
        /// Gets or sets the name of the secure host.
        /// </summary>
        /// <value>The name of the secure host.</value>
        [ConfigurationProperty(SecureHostNameAttribute, DefaultValue = SecureHostNameAttributeDefaultValue, IsRequired = true)]
        public string SecureHostName
        {
            get
            {
                return (string)this[SecureHostNameAttribute]; 
            }

            set
            {
                this[SecureHostNameAttribute] = value; 
            }
        }

        /// <summary>
        /// Gets or sets the secure port.
        /// </summary>
        /// <value>The secure port.</value>
        [ConfigurationProperty(SecurePortAttribute, DefaultValue = SecurePortAttributeDefaultValue, IsRequired = true)]
        public int SecurePort
        {
            get
            {
                return (int)this[SecurePortAttribute];
            }

            set
            {
                this[SecurePortAttribute] = value;
            }
        }

        /// <summary>
        /// Gets or sets the name of the non secure host.
        /// </summary>
        /// <value>The name of the non secure host.</value>
        [ConfigurationProperty(NonSecureHostNameAttribute, DefaultValue = NonSecureHostNameAttributeDefaultValue, IsRequired = true)]
        public string NonSecureHostName
        {
            get
            {
                return (string)this[NonSecureHostNameAttribute];
            }

            set
            {
                this[NonSecureHostNameAttribute] = value;
            }
        }

        /// <summary>
        /// Gets or sets the non secure port.
        /// </summary>
        /// <value>The non secure port.</value>
        [ConfigurationProperty(NonSecurePortNameAttribute, DefaultValue = NonSecurePortNameAttributeDefaultValue, IsRequired = true)]
        public int NonSecurePort
        {
            get
            {
                return (int)this[NonSecurePortNameAttribute];
            }

            set
            {
                this[NonSecurePortNameAttribute] = value;
            }
        }

        /// <summary>
        /// Gets or sets the SSL offload port.
        /// </summary>
        /// <value>
        /// The SSL offload port.
        /// </value>
        [ConfigurationProperty(SslOffloadPortAttribute, IsRequired = false)]
        public int SslOffloadPort
        {
            get
            {
                return (int)this[SslOffloadPortAttribute];
            }

            set
            {
                this[SslOffloadPortAttribute] = value;
            }
        }

        /// <summary>
        /// Gets or sets the pages.
        /// </summary>
        /// <value>
        /// The pages.
        /// </value>
        [ConfigurationProperty(PagesElementCollection, IsRequired = false, IsKey = false, IsDefaultCollection = true)]
        public SecurePageElementCollection Pages
        {
            get
            {
                return this[PagesElementCollection] as SecurePageElementCollection;
            }

            set
            {
                base[PagesElementCollection] = value;
            }
        }
    }
}