﻿//---------------------------------------------------------------------
// <copyright file="ServiceEndpointElementCollection.cs" company="commerceserver.net">
//     Copyright © commerceserver.net.  All rights reserved.
// </copyright>
// <summary></summary>
//---------------------------------------------------------------------
namespace CommerceServerContrib.CrossTier.Configuration
{
    using System.Configuration;
    using System.Diagnostics;

    /// <summary>
    /// Represents the service endpoint element collection in the Request Template framework.
    /// </summary>
    /// DebuggerNonUserCode is added as a way to stop this object counting against code coverage,
    /// As this is just an entity class there is no reason or benifit to testing it
    [DebuggerNonUserCode]
    public class ServiceEndpointElementCollection : BasePresentationElementCollection
    {
        /// <summary>
        /// Gets the type of the <see cref="T:System.Configuration.ConfigurationElementCollection"/>.
        /// </summary>
        /// <returns>
        /// The <see cref="T:System.Configuration.ConfigurationElementCollectionType"/> of this collection.
        /// </returns>
        public override ConfigurationElementCollectionType CollectionType
        {
            get
            {
                return ConfigurationElementCollectionType.AddRemoveClearMap;
            }
        }

        /// <summary>
        /// Gets or sets a property, attribute, or child element of this configuration element.
        /// </summary>
        /// <param name="name">Name of the element requested.</param>
        /// <returns>
        /// The specified property, attribute, or child element
        /// </returns>
        ///   
        /// <exception cref="T:System.Configuration.ConfigurationErrorsException">
        ///   <paramref name="name"/> is read-only or locked.
        /// </exception>
        public new ServiceEndpointElement this[string name]
        {
            get
            {
                return (ServiceEndpointElement)BaseGet(name);
            }
        }

        /// <summary>
        /// Gets or sets a property, attribute, or child element of this configuration element.
        /// </summary>
        /// <param name="index">The index of the item requested.</param>
        /// <returns>
        /// The specified property, attribute, or child element
        ///   </returns>
        ///   
        /// <exception cref="T:System.Configuration.ConfigurationErrorsException">
        ///   <paramref name="index"/> is read-only or locked.
        /// </exception>
        public ServiceEndpointElement this[int index]
        {
            get
            {
                return (ServiceEndpointElement)BaseGet(index);
            }

            set
            {
                if (BaseGet(index) != null)
                {
                    this.BaseRemoveAt(index);
                }

                this.BaseAdd(index, value);
            }
        }

        /// <summary>
        /// Indexes the of.
        /// </summary>
        /// <param name="connectionString">The connection string.</param>
        /// <returns>The index of the specified ServiceEndpointElement.</returns>
        public int IndexOf(ServiceEndpointElement connectionString)
        {
            return BaseIndexOf(connectionString);
        }

        /// <summary>
        /// Adds the specified connection string.
        /// </summary>
        /// <param name="connectionString">The connection string.</param>
        public void Add(ServiceEndpointElement connectionString)
        {
            this.BaseAdd(connectionString);
        }

        /// <summary>
        /// Removes the specified connection string.
        /// </summary>
        /// <param name="connectionString">The connection string.</param>
        public void Remove(ServiceEndpointElement connectionString)
        {
            if (BaseIndexOf(connectionString) >= 0)
            {
                BaseRemove(connectionString.Name);
            }
        }

        /// <summary>
        /// Removes at the ServiceEndpointElement at the specified index.
        /// </summary>
        /// <param name="index">The index.</param>
        public void RemoveAt(int index)
        {
            BaseRemoveAt(index);
        }

        /// <summary>
        /// Removes the ServiceEndpointElement specified by the name.
        /// </summary>
        /// <param name="name">The name of the item to remove.</param>
        public void Remove(string name)
        {
            BaseRemove(name);
        }

        /// <summary>
        /// Clears this instance.
        /// </summary>
        public void Clear()
        {
            BaseClear();
        }

        /// <summary>
        /// Gets the element key for a specified configuration element when overridden in a derived class.
        /// </summary>
        /// <param name="element">The <see cref="T:System.Configuration.ConfigurationElement"/> to return the key for.</param>
        /// <returns>
        /// An <see cref="T:System.Object"/> that acts as the key for the specified <see cref="T:System.Configuration.ConfigurationElement"/>.
        /// </returns>
        protected override object GetElementKey(ConfigurationElement element)
        {
            return ((ServiceEndpointElement)element).Name;
        }

        /// <summary>
        /// When overridden in a derived class, creates a new <see cref="T:System.Configuration.ConfigurationElement"/>.
        /// </summary>
        /// <returns>
        /// A new <see cref="T:System.Configuration.ConfigurationElement"/>.
        /// </returns>
        protected override ConfigurationElement CreateNewElement()
        {
            return new ServiceEndpointElement();
        }

        /// <summary>
        /// Adds a configuration element to the <see cref="T:System.Configuration.ConfigurationElementCollection"/>.
        /// </summary>
        /// <param name="element">The <see cref="T:System.Configuration.ConfigurationElement"/> to add.</param>
        protected override void BaseAdd(ConfigurationElement element)
        {
            this.BaseAdd(element, false);
        }
    }
}
