﻿//---------------------------------------------------------------------
// <copyright file="QueryStringParameter.cs" company="commerceserver.net">
//     Copyright © commerceserver.net.  All rights reserved.
// </copyright>
// <summary></summary>
//---------------------------------------------------------------------

namespace CommerceServerContrib.CrossTier.Entities
{
    using System.Collections.Generic;
    using System.Diagnostics;

    /// <summary>
    /// Representas a single query string parameter with culture aware descriptions.
    /// </summary>
    /// DebuggerNonUserCode is added as a way to stop this object counting against code coverage,
    /// As this is just an entity class there is no reason or benifit to testing it
    [DebuggerNonUserCode]
    public class QueryStringParameter
    {
        private string _key;
        private string _queryParameterKey;
        private Dictionary<string, string> _localizedDescriptions;

        #region Constructors (public)

        /// <summary>
        /// Initializes a new instance of the QueryStringParameter class.
        /// </summary>
        /// <param name="key">The key used to identity the query parameter.</param>
        /// <param name="queryParameterKey">The query parameter key.</param>
        public QueryStringParameter(string key, string queryParameterKey)
        {
            this._key = key;
            this._queryParameterKey = queryParameterKey;
            this._localizedDescriptions = new Dictionary<string, string>();
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="QueryStringParameter"/> class.
        /// </summary>
        /// <param name="key">The key used to identity the query parameter.</param>
        /// <param name="queryParameterKey">The query parameter key.</param>
        /// <param name="localizedDescriptions">The localized descriptions where the dictionary key is the culture and the value is the description.</param>
        public QueryStringParameter(string key, string queryParameterKey, Dictionary<string, string> localizedDescriptions)
        {
            this._queryParameterKey = queryParameterKey;
            this._localizedDescriptions = localizedDescriptions;
        }

        #endregion

        #region Properties (public, non-static)

        /// <summary>
        /// Gets the internal key that is used to retrieve information on the query parameter.
        /// </summary>
        public string Key
        {
            get { return this._key; }
        }

        /// <summary>
        /// Gets the query string parameter key.
        /// </summary>
        public string QueryParameterKey
        {
            get { return this._queryParameterKey; }
        }

        /// <summary>
        /// Gets or sets culture aware the descriptions.
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Usage", "CA2227:CollectionPropertiesShouldBeReadOnly", Justification = "Needs to be reset.")]
        public Dictionary<string, string> LocalizedDescriptions
        {
            get { return this._localizedDescriptions; }
            set { this._localizedDescriptions = value; }
        }

        /// <summary>
        /// Gets the localized description based on the passed locale.
        /// </summary>
        /// <param name="locale">The locale.</param>
        /// <returns>Returns string display name for locale.</returns>
        public string GetLocalizedDescription(string locale)
        {
            if (this._localizedDescriptions.ContainsKey(locale))
            {
                return this._localizedDescriptions[locale];
            }

            if (this._localizedDescriptions.ContainsKey("*"))
            {
                return this._localizedDescriptions["*"];
            }

            return string.Empty;
        }

        #endregion
    }
}
