﻿//---------------------------------------------------------------------
// <copyright file="RenderExtensionObjects.cs" company="commerceserver.net">
//     Copyright © commerceserver.net.  All rights reserved.
// </copyright>
// <summary>RenderExtensionObjects Objects.</summary>
//---------------------------------------------------------------------
namespace CommerceServerContrib.CrossTier.Entities
{
    using System;
    using System.Collections.Generic;
    using CommerceServerContrib.CrossTier.Exceptions;
    using System.Diagnostics;

    /// <summary>
    /// Holds an manages a collection of RenderExtensionObject
    /// </summary>
    /// DebuggerNonUserCode is added as a way to stop this object counting against code coverage,
    /// As this is just an entity class there is no reason or benifit to testing it
    [DebuggerNonUserCode]
    [Serializable]
    public class RenderExtensionObjects
    {
        private List<RenderExtensionObject> _extensionObjects;

        #region Public Properties
        /// <summary>
        /// Gets the number of elements contained in the <see cref="T:System.Collections.Generic.ICollection`1"/>.
        /// </summary>
        /// <returns>
        /// The number of elements contained in the <see cref="T:System.Collections.Generic.ICollection`1"/>.
        /// </returns>
        public int Count
        {
            get
            {
                return this.ExtensionObjects.Count;
            }
        }

        /// <summary>
        /// Gets a value indicating whether the <see cref="T:System.Collections.Generic.ICollection`1"/> is read-only.
        /// </summary>
        /// <returns>
        /// true if the <see cref="T:System.Collections.Generic.ICollection`1"/> is read-only; otherwise, false.
        /// </returns>
        public bool IsReadOnly
        {
            get
            {
                return false;
            }
        }
        #endregion

        #region Private Properties
        /// <summary>
        /// Gets the ExtensionObjects.
        /// </summary>
        /// <value>
        /// The ExtensionObjects.
        /// </value>
        private List<RenderExtensionObject> ExtensionObjects
        {
            get
            {
                if (this._extensionObjects == null)
                {
                    this._extensionObjects = new List<RenderExtensionObject>();
                }

                return this._extensionObjects;
            }
        }
        #endregion

        /// <summary>
        /// Gets or sets the element at the specified index.
        /// </summary>
        /// <returns>
        /// The element at the specified index.
        /// </returns>
        /// <param name="index">The zero-based index of the element to get or set.</param><exception cref="T:System.ArgumentOutOfRangeException"><paramref name="index"/> is not a valid index in the <see cref="T:System.Collections.Generic.IList`1"/>.</exception><exception cref="T:System.NotSupportedException">The property is set and the <see cref="T:System.Collections.Generic.IList`1"/> is read-only.</exception>
        public RenderExtensionObject this[int index]
        {
            get { return this.ExtensionObjects[index]; }
            set { this.ExtensionObjects[index] = value; }
        }

        #region Public Methods

        /// <summary>
        /// Returns an enumerator that iterates through the collection.
        /// </summary>
        /// <returns>
        /// A <see cref="T:System.Collections.Generic.IEnumerator`1"/> that can be used to iterate through the collection.
        /// </returns>
        /// <filterpriority>1</filterpriority>
        public IEnumerator<RenderExtensionObject> GetEnumerator()
        {
            return this.ExtensionObjects.GetEnumerator();
        }

        /// <summary>
        /// Adds an item to the <see cref="T:System.Collections.Generic.ICollection`1"/>.
        /// </summary>
        /// <param name="item">The object to add to the <see cref="T:System.Collections.Generic.ICollection`1"/>.</param><exception cref="T:System.NotSupportedException">The <see cref="T:System.Collections.Generic.ICollection`1"/> is read-only.</exception>
        public void Add(RenderExtensionObject item)
        {
            this.Add(item, false);
        }

        /// <summary>
        /// Adds the specified item, if the override value exists the existing value for a colliding key is replaced otherwise an excepiton of type <see cref="DuplicateKeyException" /> is thrown.
        /// </summary>
        /// <param name="item">The item that needs to be added in.</param>
        /// <param name="overrideExistingValue">if set to <c>true</c> [override existing value].</param>
        public void Add(RenderExtensionObject item, bool overrideExistingValue)
        {
            int index = this.ExtensionObjects.FindIndex(x => x.Equals(item));
            if (index < 0)
            {
                this.ExtensionObjects.Add(item);
            }
            else
            {
                if (overrideExistingValue)
                {
                    this.ExtensionObjects[index] = item;
                }
                else
                {
                    throw new DuplicateKeyException(string.Empty, item.RenderExtensionObjectNamespace);
                }
            }
        }

        /// <summary>
        /// Removes all items from the <see cref="T:System.Collections.Generic.ICollection`1"/>.
        /// </summary>
        /// <exception cref="T:System.NotSupportedException">The <see cref="T:System.Collections.Generic.ICollection`1"/> is read-only. </exception>
        public void Clear()
        {
            this.ExtensionObjects.Clear();
        }

        /// <summary>
        /// Determines whether the <see cref="T:System.Collections.Generic.ICollection`1"/> contains a specific value.
        /// </summary>
        /// <returns>
        /// true if <paramref name="item"/> is found in the <see cref="T:System.Collections.Generic.ICollection`1"/>; otherwise, false.
        /// </returns>
        /// <param name="item">The object to locate in the <see cref="T:System.Collections.Generic.ICollection`1"/>.</param>
        public bool Contains(RenderExtensionObject item)
        {
            return this.ExtensionObjects.Find(x => x.Equals(item)) != null;
        }

        /// <summary>
        /// Copies to incoming collection of RenderExtensionObjects
        /// </summary>
        /// <param name="array">The array.</param>
        /// <param name="overrideExistingValue">if set to <c>true</c> [override existing value].</param>
        public void CopyTo(RenderExtensionObjects array, bool overrideExistingValue)
        {
            foreach (RenderExtensionObject renderExtensionObject in this.ExtensionObjects)
            {
                array.Add(renderExtensionObject, overrideExistingValue);
            }
        }

        /// <summary>
        /// Copies to incoming collection of RenderExtensionObjects
        /// </summary>
        /// <param name="array">The array.</param>
        /// <param name="overrideExistingValue">if set to <c>true</c> [override existing value].</param>
        public void CopyTo(Dictionary<string, object> array, bool overrideExistingValue)
        {
            foreach (RenderExtensionObject renderExtensionObject in this.ExtensionObjects)
            {
                array.Add(renderExtensionObject.RenderExtensionObjectNamespace, renderExtensionObject.RenderExtensionObjectClass);
            }
        }

        /// <summary>
        /// Removes the first occurrence of a specific object from the <see cref="T:System.Collections.Generic.ICollection`1"/>.
        /// </summary>
        /// <returns>
        /// true if <paramref name="item"/> was successfully removed from the <see cref="T:System.Collections.Generic.ICollection`1"/>; otherwise, false. This method also returns false if <paramref name="item"/> is not found in the original <see cref="T:System.Collections.Generic.ICollection`1"/>.
        /// </returns>
        /// <param name="item">The object to remove from the <see cref="T:System.Collections.Generic.ICollection`1"/>.</param><exception cref="T:System.NotSupportedException">The <see cref="T:System.Collections.Generic.ICollection`1"/> is read-only.</exception>
        public bool Remove(RenderExtensionObject item)
        {
            int originalLength = this.ExtensionObjects.Count;
            int index = this.ExtensionObjects.FindIndex(x => x.Equals(item));
            if (index >= 0)
            {
                this.RemoveAt(index);
                return (originalLength - 1) == this.ExtensionObjects.Count;
            }

            return false;
        }

        /// <summary>
        /// Determines the index of a specific item in the <see cref="T:System.Collections.Generic.IList`1"/>.
        /// </summary>
        /// <returns>
        /// The index of <paramref name="item"/> if found in the list; otherwise, -1.
        /// </returns>
        /// <param name="item">The object to locate in the <see cref="T:System.Collections.Generic.IList`1"/>.</param>
        public int IndexOf(RenderExtensionObject item)
        {
            return this.ExtensionObjects.IndexOf(this.ExtensionObjects.Find(x => x.Equals(item)));
        }

        /// <summary>
        /// Inserts an item to the <see cref="T:System.Collections.Generic.IList`1"/> at the specified index.
        /// </summary>
        /// <param name="index">The zero-based index at which <paramref name="item"/> should be inserted.</param><param name="item">The object to insert into the <see cref="T:System.Collections.Generic.IList`1"/>.</param><exception cref="T:System.ArgumentOutOfRangeException"><paramref name="index"/> is not a valid index in the <see cref="T:System.Collections.Generic.IList`1"/>.</exception><exception cref="T:System.NotSupportedException">The <see cref="T:System.Collections.Generic.IList`1"/> is read-only.</exception>
        public void Insert(int index, RenderExtensionObject item)
        {
            this.ExtensionObjects.Insert(index, item);
        }

        /// <summary>
        /// Removes the <see cref="T:System.Collections.Generic.IList`1"/> item at the specified index.
        /// </summary>
        /// <param name="index">The zero-based index of the item to remove.</param><exception cref="T:System.ArgumentOutOfRangeException"><paramref name="index"/> is not a valid index in the <see cref="T:System.Collections.Generic.IList`1"/>.</exception><exception cref="T:System.NotSupportedException">The <see cref="T:System.Collections.Generic.IList`1"/> is read-only.</exception>
        public void RemoveAt(int index)
        {
            this.ExtensionObjects.RemoveAt(index);
        }
        #endregion
    }
}
