﻿//---------------------------------------------------------------------
// <copyright file="RenderParameters.cs" company="commerceserver.net">
//     Copyright © commerceserver.net.  All rights reserved.
// </copyright>
// <summary>RenderParameters object</summary>
//---------------------------------------------------------------------
namespace CommerceServerContrib.CrossTier.Entities
{
    using System;
    using System.Collections.Generic;
    using System.Diagnostics;
    using CommerceServerContrib.CrossTier.Exceptions;

    /// <summary>
    /// Represents the Render Parameter Collection object
    /// </summary>
    /// DebuggerNonUserCode is added as a way to stop this object counting against code coverage,
    /// As this is just an entity class there is no reason or benifit to testing it
    [DebuggerNonUserCode]
    [Serializable]
    public class RenderParameters
    {
        private List<RenderParameter> _parameters;

        #region Public Properties
        /// <summary>
        /// Gets the number of elements contained in the <see cref="T:System.Collections.Generic.ICollection`1"/>.
        /// </summary>
        /// <returns>
        /// The number of elements contained in the <see cref="T:System.Collections.Generic.ICollection`1"/>.
        /// </returns>
        public int Count
        {
            get
            {
                return this.Parameters.Count;
            }
        }

        /// <summary>
        /// Gets a value indicating whether the <see cref="T:System.Collections.Generic.ICollection`1"/> is read-only.
        /// </summary>
        /// <returns>
        /// true if the <see cref="T:System.Collections.Generic.ICollection`1"/> is read-only; otherwise, false.
        /// </returns>
        public bool IsReadOnly
        {
            get
            {
                return false;
            }
        }

        #endregion

        #region Private Properties

        /// <summary>
        /// Gets the Render parameters.
        /// </summary>
        /// <value>
        /// The Render parameters.
        /// </value>
        private List<RenderParameter> Parameters
        {
            get
            {
                if (this._parameters == null)
                {
                    this._parameters = new List<RenderParameter>();
                }

                return this._parameters;
            }
        }
        #endregion

        /// <summary>
        /// Gets or sets the element at the specified index.
        /// </summary>
        /// <returns>
        /// The element at the specified index.
        /// </returns>
        /// <param name="index">The zero-based index of the element to get or set.</param><exception cref="T:System.ArgumentOutOfRangeException"><paramref name="index"/> is not a valid index in the <see cref="T:System.Collections.Generic.IList`1"/>.</exception><exception cref="T:System.NotSupportedException">The property is set and the <see cref="T:System.Collections.Generic.IList`1"/> is read-only.</exception>
        public RenderParameter this[int index]
        {
            get { return this.Parameters[index]; }
            set { this.Parameters[index] = value; }
        }

        #region Methods

        /// <summary>
        /// Returns an enumerator that iterates through the collection.
        /// </summary>
        /// <returns>
        /// A <see cref="T:System.Collections.Generic.IEnumerator`1"/> that can be used to iterate through the collection.
        /// </returns>
        /// <filterpriority>1</filterpriority>
        public IEnumerator<RenderParameter> GetEnumerator()
        {
            return this.Parameters.GetEnumerator();
        }

        /// <summary>
        /// Adds an item to the <see cref="T:System.Collections.Generic.ICollection`1"/>.
        /// </summary>
        /// <param name="item">The object to add to the <see cref="T:System.Collections.Generic.ICollection`1"/>.</param><exception cref="T:System.NotSupportedException">The <see cref="T:System.Collections.Generic.ICollection`1"/> is read-only.</exception>
        public void Add(RenderParameter item)
        {
            this.Add(item, false);
        }

        /// <summary>
        /// Adds the specified item into the collection of parameters, if allowed to overwrite then will replace the existing value otherwise will throw an exception.
        /// </summary>
        /// <param name="item">The item that needs to be added into the collection.</param>
        /// <param name="overrideExistingValue">if set to <c>true</c> will replace the existing value for a given RenderParameter.RenderParamKey.</param>
        public void Add(RenderParameter item, bool overrideExistingValue)
        {
            int index = this.Parameters.FindIndex(x => x.Equals(item));
            if (index < 0)
            {
                this.Parameters.Add(item);
            }
            else
            {
                if (overrideExistingValue)
                {
                    this.Parameters[index] = item;
                }
                else
                {
                    throw new DuplicateKeyException(string.Empty, item.RenderParameterKey);
                }
            }
        }

        /// <summary>
        /// Removes all items from the <see cref="T:System.Collections.Generic.ICollection`1"/>.
        /// </summary>
        /// <exception cref="T:System.NotSupportedException">The <see cref="T:System.Collections.Generic.ICollection`1"/> is read-only. </exception>
        public void Clear()
        {
            this.Parameters.Clear();
        }

        /// <summary>
        /// Determines whether the <see cref="T:System.Collections.Generic.ICollection`1"/> contains a specific value.
        /// </summary>
        /// <returns>
        /// true if <paramref name="item"/> is found in the <see cref="T:System.Collections.Generic.ICollection`1"/>; otherwise, false.
        /// </returns>
        /// <param name="item">The object to locate in the <see cref="T:System.Collections.Generic.ICollection`1"/>.</param>
        public bool Contains(RenderParameter item)
        {
            return this.Parameters.Find(x => x.Equals(item)) != null;
        }

        /// <summary>
        /// Copies to incoming collection of RenderParameters, also takes in directions informing whether to over-ride the existing  value or throw an exception.
        /// </summary>
        /// <param name="array">The array.</param>
        /// <param name="overrideExistingValue">if set to <c>true</c> [override existing value].</param>
        public void CopyTo(RenderParameters array, bool overrideExistingValue)
        {
            foreach (RenderParameter renderParameter in this.Parameters)
            {
                array.Add(renderParameter, overrideExistingValue);
            }
        }

        /// <summary>
        /// Copies to incoming collection of Dictionary[string, object], also takes in directions informing whether to over-ride the existing  value or throw an exception.
        /// </summary>
        /// <param name="array">The array.</param>
        /// <param name="overrideExistingValue">if set to <c>true</c> [override existing value].</param>
        public void CopyTo(Dictionary<string, object> array, bool overrideExistingValue)
        {
            foreach (RenderParameter renderParameter in this.Parameters)
            {
                array.Add(renderParameter.RenderParameterKey, renderParameter.RenderParameterValue);
            }
        }

        /// <summary>
        /// Removes the first occurrence of a specific object from the <see cref="T:System.Collections.Generic.ICollection`1"/>.
        /// </summary>
        /// <returns>
        /// true if <paramref name="item"/> was successfully removed from the <see cref="T:System.Collections.Generic.ICollection`1"/>; otherwise, false. This method also returns false if <paramref name="item"/> is not found in the original <see cref="T:System.Collections.Generic.ICollection`1"/>.
        /// </returns>
        /// <param name="item">The object to remove from the <see cref="T:System.Collections.Generic.ICollection`1"/>.</param><exception cref="T:System.NotSupportedException">The <see cref="T:System.Collections.Generic.ICollection`1"/> is read-only.</exception>
        public bool Remove(RenderParameter item)
        {
            int originalLength = this.Parameters.Count;
            int index = this.Parameters.FindIndex(x => x.Equals(item));
            if (index >= 0)
            {
                this.RemoveAt(index);
                return (originalLength - 1) == this.Parameters.Count;
            }

            return false;
        }

        /// <summary>
        /// Determines the index of a specific item in the <see cref="T:System.Collections.Generic.IList`1"/>.
        /// </summary>
        /// <returns>
        /// The index of <paramref name="item"/> if found in the list; otherwise, -1.
        /// </returns>
        /// <param name="item">The object to locate in the <see cref="T:System.Collections.Generic.IList`1"/>.</param>
        public int IndexOf(RenderParameter item)
        {
            return this.Parameters.IndexOf(this.Parameters.Find(x => x.Equals(item)));
        }

        /// <summary>
        /// Inserts an item to the <see cref="T:System.Collections.Generic.IList`1"/> at the specified index.
        /// </summary>
        /// <param name="index">The zero-based index at which <paramref name="item"/> should be inserted.</param><param name="item">The object to insert into the <see cref="T:System.Collections.Generic.IList`1"/>.</param><exception cref="T:System.ArgumentOutOfRangeException"><paramref name="index"/> is not a valid index in the <see cref="T:System.Collections.Generic.IList`1"/>.</exception><exception cref="T:System.NotSupportedException">The <see cref="T:System.Collections.Generic.IList`1"/> is read-only.</exception>
        public void Insert(int index, RenderParameter item)
        {
            this.Parameters.Insert(index, item);
        }

        /// <summary>
        /// Removes the <see cref="T:System.Collections.Generic.IList`1"/> item at the specified index.
        /// </summary>
        /// <param name="index">The zero-based index of the item to remove.</param><exception cref="T:System.ArgumentOutOfRangeException"><paramref name="index"/> is not a valid index in the <see cref="T:System.Collections.Generic.IList`1"/>.</exception><exception cref="T:System.NotSupportedException">The <see cref="T:System.Collections.Generic.IList`1"/> is read-only.</exception>
        public void RemoveAt(int index)
        {
            this.Parameters.RemoveAt(index);
        }
        #endregion
    }
}
