﻿//---------------------------------------------------------------------
// <copyright file="CommerceServerContribBaseException.cs" company="commerceserver.net">
//     Copyright © commerceserver.net.  All rights reserved.
// </copyright>
// <summary></summary>
//---------------------------------------------------------------------

namespace CommerceServerContrib.CrossTier.Exceptions
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Text;
    using System.Runtime.Serialization;
    using System.Security.Permissions;
    using CommerceServerContrib.CrossTier.Helpers;
    using CommerceServer.Foundation;

    /// <summary>
    /// Base exception used in Commerce Server Contrib.  All exceptions that we throw must be derived from this class.
    /// </summary>
    [Serializable]
    public class CommerceServerContribBaseException : Exception
    {
        private string _channel;

        #region Constructors (public)

        /// <summary>
        /// Initializes a new instance of the <see cref="CommerceServerContribBaseException"/> class.
        /// </summary>
        /// <param name="channel">The current channel.</param>
        /// <param name="message">The message of the exception.</param>
        public CommerceServerContribBaseException(string channel, string message)
            : base(message)
        {
            this._channel = channel;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="CommerceServerContribBaseException"/> class.
        /// </summary>
        /// <param name="channel">The current channel.</param>
        /// <param name="message">The message.</param>
        /// <param name="innerException">The inner exception.</param>
        public CommerceServerContribBaseException(string channel, string message, Exception innerException)
            : base(message, innerException)
        {
            this._channel = channel;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="CommerceServerContribBaseException"/> class.
        /// </summary>
        /// <param name="info">The <see cref="T:System.Runtime.Serialization.SerializationInfo"/> that holds the serialized object data about the exception being thrown.</param>
        /// <param name="context">The <see cref="T:System.Runtime.Serialization.StreamingContext"/> that contains contextual information about the source or destination.</param>
        /// <exception cref="T:System.ArgumentNullException">
        /// The <paramref name="info"/> parameter is null.
        /// </exception>
        ///   
        /// <exception cref="T:System.Runtime.Serialization.SerializationException">
        /// The class name is null or <see cref="P:System.Exception.HResult"/> is zero (0).
        /// </exception>
        protected CommerceServerContribBaseException(SerializationInfo info, StreamingContext context)
            : base(info, context)
        {
            ParameterChecker.CheckForNull(info, "info");
            this.EventId = info.GetInt32("EventId");
            this.Channel = info.GetString("Channel");
        }

        #endregion

        #region Constructor (protected)

        /// <summary>
        /// Initializes a new instance of the <see cref="CommerceServerContribBaseException"/> class.
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1032:ImplementStandardExceptionConstructors", Justification = "We do not want devs to use these")]
        protected CommerceServerContribBaseException()
            : this(String.Empty, String.Empty)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="CommerceServerContribBaseException"/> class.
        /// </summary>
        /// <param name="message">The message of the exception.</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1032:ImplementStandardExceptionConstructors", Justification = "We do not want devs to use these")]
        protected CommerceServerContribBaseException(string message)
            : this(String.Empty, message)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="CommerceServerContribBaseException"/> class.
        /// </summary>
        /// <param name="message">The message.</param>
        /// <param name="innerException">The inner exception.</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1032:ImplementStandardExceptionConstructors", Justification = "We do not want devs to use these")]
        protected CommerceServerContribBaseException(string message, Exception innerException)
            : this(String.Empty, message, innerException)
        {
        }

        #endregion

        #region Properties (public, non-satatic)

        /// <summary>
        /// Gets or sets the channel.
        /// </summary>
        /// <value>
        /// The channel.
        /// </value>
        public string Channel
        {
            get { return this._channel; }
            set { this._channel = value; }
        }

        /// <summary>
        /// Gets or sets the event ID associated with this exception.
        /// </summary>
        public int EventId
        {
            get;
            set;
        }

        #endregion

        #region Overrides

        /// <summary>
        /// When overridden in a derived class, sets the <see cref="T:System.Runtime.Serialization.SerializationInfo"/> with information about the exception.
        /// </summary>
        /// <param name="info">The <see cref="T:System.Runtime.Serialization.SerializationInfo"/> that holds the serialized object data about the exception being thrown.</param>
        /// <param name="context">The <see cref="T:System.Runtime.Serialization.StreamingContext"/> that contains contextual information about the source or destination.</param>
        /// <exception cref="T:System.ArgumentNullException">
        /// The <paramref name="info"/> parameter is a null reference (Nothing in Visual Basic).
        ///   </exception>
        ///   
        /// <PermissionSet>
        ///   <IPermission class="System.Security.Permissions.FileIOPermission, mscorlib, Version=2.0.3600.0, Culture=neutral, PublicKeyToken=b77a5c561934e089" version="1" Read="*AllFiles*" PathDiscovery="*AllFiles*"/>
        ///   <IPermission class="System.Security.Permissions.SecurityPermission, mscorlib, Version=2.0.3600.0, Culture=neutral, PublicKeyToken=b77a5c561934e089" version="1" Flags="SerializationFormatter"/>
        /// </PermissionSet>
        [SecurityPermission(SecurityAction.LinkDemand, Flags = SecurityPermissionFlag.SerializationFormatter)]
        public override void GetObjectData(SerializationInfo info, StreamingContext context)
        {
            ParameterChecker.CheckForNull(info, "info");
            base.GetObjectData(info, context);
            info.AddValue("EventId", this.EventId, typeof(int));
            info.AddValue("Channel", this.Channel, typeof(string));
        }

        #endregion
    }
}
