﻿//---------------------------------------------------------------------
// <copyright file="BaseServiceLocator.cs" company="commerceserver.net">
//     Copyright © commerceserver.net.  All rights reserved.
// </copyright>
// <summary></summary>
//---------------------------------------------------------------------
namespace CommerceServerContrib.CrossTier.Helpers
{
    using System.Collections;
    using System.Collections.Generic;
    using System;
    using Microsoft.Practices.ServiceLocation;
    using Microsoft.Practices.Unity;
    using CommerceServerContrib.CrossTier.Exceptions;
    using System.Globalization;
    using CommerceServer.Foundation;

    /// <summary>
    /// Delegate used to return the Unity container name.
    /// </summary>
    /// <returns>The Unity container name.</returns>
    public delegate string GetUnityContainerName();

    /// <summary>
    /// Base class encapsulating common operations functionality used accross the presentation and application tiers.
    /// </summary>
    public abstract class BaseServiceLocator
    {
        private GetUnityContainerName _getUnityContainerNameDelegate;
        private object _delegateLock = new object();

        /// <summary>
        /// Sets the unity container name delegate that returns the unit container to be used..
        /// </summary>
        /// <param name="newDelegate">The delegate method.</param>
        /// <returns>The Unity container name delegate.</returns>
        public GetUnityContainerName SetUnityContainerNameDelegate(GetUnityContainerName newDelegate)
        {
            ParameterChecker.CheckForNull(newDelegate, "newDelegate");

            GetUnityContainerName previousDelegate;

            lock (this._delegateLock)
            {
                previousDelegate = this._getUnityContainerNameDelegate;
                this._getUnityContainerNameDelegate = newDelegate;
            }

            return previousDelegate;
        }

        /// <summary>
        /// Gets the implementation.
        /// </summary>
        /// <typeparam name="T">The type to get an instance of</typeparam>
        /// <returns>The type instance</returns>
        public virtual T GetImplementation<T>()
        {
            return this.GetImplementation<T>(false);
        }

        /// <summary>
        /// Gets the implementation.
        /// </summary>
        /// <typeparam name="T">The type to get an instance of</typeparam>
        /// <param name="throwExceptionIfNotFound">if set to <c>true</c> an exception will be thrown if the implementation is not found for the given interface.</param>
        /// <returns>
        /// The type instance
        /// </returns>
        public virtual T GetImplementation<T>(bool throwExceptionIfNotFound)
        {
            try
            {
                IUnityContainer container = this.GetUnityContainer();
                return container.Resolve<T>();
            }
            catch (ResolutionFailedException e)
            {
                if (throwExceptionIfNotFound)
                {
                    throw new ProviderImplementationNotFoundException(String.Empty, typeof(T), e);
                }
                else
                {
                    return default(T);
                }
            }
        }

        /// <summary>
        /// Gets the implementation by type and configured name.
        /// </summary>
        /// <typeparam name="T">Type of the desired interface implementation.</typeparam>
        /// <param name="name">The configured name.</param>
        /// <param name="throwExceptionIfNotFound">if set to <c>true</c> will throw an exception if the implementation is not found.</param>
        /// <returns>The desired implementation instance or null if nto found.</returns>
        public virtual T GetImplementation<T>(string name, bool throwExceptionIfNotFound)
        {
            try
            {
                IUnityContainer container = this.GetUnityContainer();
                return container.Resolve<T>(name);
            }
            catch (ResolutionFailedException e)
            {
                if (throwExceptionIfNotFound)
                {
                    throw new ProviderImplementationNotFoundException(String.Empty, typeof(T), e);
                }
                else
                {
                    return default(T);
                }
            }
        }

        /// <summary>
        /// Gets the default implementation from the default unity container.
        /// </summary>
        /// <param name="aliasName">Name of the alias.</param>
        /// <param name="throwExceptionIfNotFound">if set to <c>true</c> it will throw a detailed exception if the implementation is not found.</param>
        /// <returns>Returns the implementation or null if throwExceptionIfnotFound is false and no implementation was found.</returns>
        public virtual object GetDefaultImplementation(string aliasName, bool throwExceptionIfNotFound)
        {
            Type aliasType = null;

            try
            {
                // Default implementation always are defined at the root of the Unity file.
                aliasType = UnityHelper.GetTypeForAlias(StringConstants.DefaultUnityContainerName, aliasName);

                IUnityContainer container;
                if (aliasType != null && UnityHelper.TryGetContainer(StringConstants.DefaultUnityContainerName, out container))
                {
                    return container.Resolve(aliasType);
                }
                else
                {
                    throw new ProviderImplementationNotFoundException(String.Empty, aliasType, null);
                }
            }
            catch (ResolutionFailedException e)
            {
                if (throwExceptionIfNotFound)
                {
                    throw new ProviderImplementationNotFoundException(String.Empty, aliasType, e);
                }
                else
                {
                    return null;
                }
            }
        }

        /// <summary>
        /// Gets the implementation.
        /// </summary>
        /// <param name="typeInterface">The type interface.</param>
        /// <param name="throwExceptionIfNotFound">if set to <c>true</c> will throw and exception if the typeInterface cannot be found.</param>
        /// <returns>
        /// The type instance
        /// </returns>
        public virtual object GetImplementation(Type typeInterface, bool throwExceptionIfNotFound)
        {
            try
            {
                IUnityContainer container = this.GetUnityContainer();
                return container.Resolve(typeInterface);
            }
            catch (ResolutionFailedException e)
            {
                if (throwExceptionIfNotFound)
                {
                    throw new ProviderImplementationNotFoundException(String.Empty, typeInterface, e);
                }
                else
                {
                    return null;
                }
            }
        }

        /// <summary>
        /// Gets the implementation.
        /// </summary>
        /// <param name="typeInterface">The type interface.</param>
        /// <param name="name">The unity name.</param>
        /// <param name="throwExceptionIfNotFound">if set to <c>true</c> [throw exception if not found].</param>
        /// <returns>The desired implementation instance or null if not found.</returns>
        public virtual object GetImplementation(Type typeInterface, string name, bool throwExceptionIfNotFound)
        {
            try
            {
                IUnityContainer container = this.GetUnityContainer();
                return container.Resolve(typeInterface, name);
            }
            catch (ResolutionFailedException e)
            {
                if (throwExceptionIfNotFound)
                {
                    throw new ProviderImplementationNotFoundException(String.Empty, typeInterface, e);
                }
                else
                {
                    return null;
                }
            }
        }

        /// <summary>
        /// Gets the implementation.
        /// </summary>
        /// <param name="typeInterface">The type interface.</param>
        /// <returns>The implementation or null if not found.</returns>
        public virtual object GetImplementation(Type typeInterface)
        {
            return this.GetImplementation(typeInterface, false);
        }

        /// <summary>
        /// Gets the implementation.
        /// </summary>
        /// <param name="typeInterface">The type interface.</param>
        /// <param name="name">The unity name.</param>
        /// <returns>The desired implementation instance or null if not found.</returns>
        public virtual object GetImplementation(Type typeInterface, string name)
        {
            return this.GetImplementation(typeInterface, name, false);
        }

        /// <summary>
        /// Gets the implementation.
        /// </summary>
        /// <param name="aliasName">The alias for a type.</param>
        /// <returns>The alias type instance</returns>
        public virtual object GetImplementation(string aliasName)
        {
            var aliasType = UnityHelper.GetTypeForAlias(this.GetContainerName(), aliasName);

            return this.GetImplementation(aliasType, false);
        }

        /// <summary>
        /// Gets the implementation.
        /// </summary>
        /// <param name="aliasName">The alias for a type.</param>
        /// <param name="throwExceptionIfNotFound">if set to <c>true</c> will throw a detailed exception if not found.</param>
        /// <returns>
        /// The alias type instance
        /// </returns>
        public virtual object GetImplementation(string aliasName, bool throwExceptionIfNotFound)
        {
            var aliasType = UnityHelper.GetTypeForAlias(this.GetContainerName(), aliasName);
            if (aliasType != null)
            {
                return this.GetImplementation(aliasType, throwExceptionIfNotFound);
            }
            else if (throwExceptionIfNotFound)
            {
                throw new ProviderImplementationNotFoundException(String.Empty, null);
            }

            return null;
        }

        /// <summary>
        /// Gets the concrete types defined for a particular implementations.
        /// </summary>
        /// <typeparam name="T">The interface defined as defined in the configuration file.</typeparam>
        /// <param name="throwExceptionIfNotFound">if set to <c>true</c> throws an exception when no implementation is defined in the configuration file.</param>
        /// <returns>List of mapped typed as defined in the unity container.</returns>
        /// <remarks>In order for this method to work ensure that the name attribute is defined at the container level.</remarks>
        public virtual List<T> GetImplementations<T>(bool throwExceptionIfNotFound)
        {
            try
            {
                IUnityContainer container = this.GetUnityContainer();

                IEnumerator<T> enumerator = container.ResolveAll<T>().GetEnumerator();
                List<T> objectList = new List<T>();
                while (enumerator.MoveNext())
                {
                    objectList.Add(enumerator.Current);
                }

                return objectList;
            }
            catch (ResolutionFailedException e)
            {
                if (throwExceptionIfNotFound)
                {
                    throw new ProviderImplementationNotFoundException(String.Empty, typeof(T), e);
                }

                return null;
            }
        }

        #region Methods (private, non-static)

        private IUnityContainer GetUnityContainer()
        {
            IUnityContainer container;
            if (this._getUnityContainerNameDelegate == null || !UnityHelper.TryGetContainer(this._getUnityContainerNameDelegate(), out container))
            {
                if (!UnityHelper.TryGetContainer(StringConstants.DefaultUnityContainerName, out container))
                {
                    throw new UnityContainerNotFoundException(String.Empty, String.Format(CultureInfo.InvariantCulture, "{0} or {1}", (this._getUnityContainerNameDelegate == null) ? "<null> Unity Container" : this._getUnityContainerNameDelegate(), StringConstants.DefaultUnityContainerName));
                }
            }

            return container;
        }

        private string GetContainerName()
        {
            string containerName = (this._getUnityContainerNameDelegate != null) ? this._getUnityContainerNameDelegate() : StringConstants.DefaultUnityContainerName;
            return containerName;
        }

        #endregion
    }
}
