﻿//---------------------------------------------------------------------
// <copyright file="Money.cs" company="commerceserver.net">
//     Copyright © commerceserver.net.  All rights reserved.
// </copyright>
// <summary></summary>
//---------------------------------------------------------------------
namespace CommerceServerContrib.CrossTier
{
    using System;
    using System.Globalization;

    /// <summary>
    /// Money: Deals with Multi-Currency
    /// </summary>
    public class Money
    {
        private readonly CultureInfo _culture;
        private long _value;

        /// <summary>
        /// Initializes a new instance of the <see cref="Money"/> class.
        /// </summary>
        public Money()
        {
            this._value = 0L;
            this._culture = CultureInfo.CurrentCulture;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="Money"/> class.
        /// </summary>
        /// <param name="value">The value.</param>
        public Money(long value)
            : this()
        {
            this._value = (long)(value * Math.Pow(10, this._culture.NumberFormat.CurrencyDecimalDigits));
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="Money"/> class.
        /// </summary>
        /// <param name="value">The value.</param>
        public Money(decimal value)
            : this()
        {
            this._value = (long)(value * (decimal)Math.Pow(10, this._culture.NumberFormat.CurrencyDecimalDigits));
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="Money"/> class.
        /// </summary>
        /// <param name="value">The value.</param>
        /// <param name="culture">The culture.</param>
        public Money(decimal value, CultureInfo culture)
            : this()
        {
            this._value = (long)(value * (decimal)Math.Pow(10, culture.NumberFormat.CurrencyDecimalDigits));
            this._culture = culture;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="Money"/> class.
        /// </summary>
        /// <param name="value">The value.</param>
        /// <param name="culture">The culture.</param>
        public Money(double value, CultureInfo culture)
            : this()
        {
            this._value = (long)(value * Math.Pow(10, culture.NumberFormat.CurrencyDecimalDigits));
            this._culture = culture;
        }

        /// <summary>
        /// Gets the culture info.
        /// </summary>
        /// <value>The culture info.</value>
        public CultureInfo CultureInfo
        {
            get
            {
                return this._culture;
            }
        }

        /// <summary>
        /// Gets the minor value.
        /// </summary>
        /// <value>The minor value.</value>
        public long MinorValue
        {
            get { return this._value; }
        }

        /// <summary>
        /// Performs an implicit conversion from <see cref="Money"/> to <see cref="System.String"/>.
        /// </summary>
        /// <param name="money">The money.</param>
        /// <returns>The result of the conversion.</returns>
        public static implicit operator string(Money money)
        {
            return money.ToString();
        }

        /// <summary>
        /// Implements the operator +.
        /// </summary>
        /// <param name="money1">The money1.</param>
        /// <param name="money2">The money2.</param>
        /// <returns>The result of the operator.</returns>
        public static Money operator +(Money money1, Money money2)
        {
            // guard statement
            if (!SameCurrency(money1, money2))
            {
                throw new ArgumentException("The culture of money1 and money2 are different");
            }

            return new Money { _value = money1.MinorValue + money2.MinorValue };
        }

        /// <summary>
        /// Implements the operator *.
        /// </summary>
        /// <param name="money1">The money1.</param>
        /// <param name="multiplier">The multiplier.</param>
        /// <returns>The result of the operator.</returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Usage", "CA2225:OperatorOverloadsHaveNamedAlternates", Justification = "Preferred Convention.")]
        public static Money operator *(Money money1, int multiplier)
        {
            return new Money { _value = money1.MinorValue * multiplier };
        }

        /// <summary>
        /// Implements the operator -.
        /// </summary>
        /// <param name="money1">The money1.</param>
        /// <param name="money2">The money2.</param>
        /// <returns>The result of the operator.</returns>
        public static Money operator -(Money money1, Money money2)
        {
            // guard statement
            string currency1 = new RegionInfo(money1.CultureInfo.LCID).CurrencySymbol;
            string currency2 = new RegionInfo(money2.CultureInfo.LCID).CurrencySymbol;
            if (currency1 != currency2)
            {
                throw new ArgumentException("The culture of money1 and money2 are different");
            }

            return new Money { _value = money1.MinorValue - money2.MinorValue };
        }

        /// <summary>
        /// Implements the operator ==.
        /// </summary>
        /// <param name="money1">The money1.</param>
        /// <param name="money2">The money2.</param>
        /// <returns>The result of the operator.</returns>
        public static bool operator ==(Money money1, Money money2)
        {
            return money1.Equals(money2);
        }

        /// <summary>
        /// Implements the operator !=.
        /// </summary>
        /// <param name="money1">The money1.</param>
        /// <param name="money2">The money2.</param>
        /// <returns>The result of the operator.</returns>
        public static bool operator !=(Money money1, Money money2)
        {
            return !money1.Equals(money2);
        }

        /// <summary>
        /// Implements the operator &lt;.
        /// </summary>
        /// <param name="money1">The money1.</param>
        /// <param name="money2">The money2.</param>
        /// <returns>The result of the operator.</returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Usage", "CA2225:OperatorOverloadsHaveNamedAlternates", Justification = "Preferred Convention.")]
        public static bool operator <(Money money1, Money money2)
        {
            // guard statement
            string currency1 = new RegionInfo(money1.CultureInfo.LCID).CurrencySymbol;
            string currency2 = new RegionInfo(money2.CultureInfo.LCID).CurrencySymbol;
            if (currency1 != currency2)
            {
                throw new ArgumentException("The culture of money1 and money2 are different");
            }

            return money1._value < money2._value;
        }

        /// <summary>
        /// Implements the operator <![CDATA[<=]]>. 
        /// </summary>
        /// <param name="money1">The money1.</param>
        /// <param name="money2">The money2.</param>
        /// <returns>The result of the operator.</returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Usage", "CA2225:OperatorOverloadsHaveNamedAlternates", Justification = "Preferred Convention.")]
        public static bool operator <=(Money money1, Money money2)
        {
            return ((money1 < money2) || (money1 == money2)) ? true : false;
        }

        /// <summary>
        /// Implements the operator >.
        /// </summary>
        /// <param name="money1">The money1.</param>
        /// <param name="money2">The money2.</param>
        /// <returns>The result of the operator.</returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Usage", "CA2225:OperatorOverloadsHaveNamedAlternates", Justification = "Preferred Convention.")]
        public static bool operator >(Money money1, Money money2)
        {
            // guard statement
            string currency1 = new RegionInfo(money1.CultureInfo.LCID).CurrencySymbol;
            string currency2 = new RegionInfo(money2.CultureInfo.LCID).CurrencySymbol;
            if (currency1 != currency2)
            {
                throw new ArgumentException("The culture of money1 and money2 are different");
            }

            return money1._value > money2._value;
        }

        /// <summary>
        /// Implements the operator >=.
        /// </summary>
        /// <param name="money1">The money1.</param>
        /// <param name="money2">The money2.</param>
        /// <returns>The result of the operator.</returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Usage", "CA2225:OperatorOverloadsHaveNamedAlternates", Justification = "Preferred Convention.")]
        public static bool operator >=(Money money1, Money money2)
        {
            return ((money1 > money2) || (money1 == money2)) ? true : false;
        }

        /// <summary>
        /// Prints the money.
        /// </summary>
        /// <param name="value">The value.</param>
        /// <returns>ToString form of money.</returns>
        public static string PrintMoney(string value)
        {
            decimal output;
            if (decimal.TryParse(value, NumberStyles.Currency, CultureInfo.CurrentCulture.NumberFormat, out output))
            {
                return new Money(output).ToString();
            }

            return "-";
        }

        /// <summary>
        /// Prints the money.
        /// </summary>
        /// <param name="value">The value.</param>
        /// <param name="cultureFormat">The culture format.</param>
        /// <returns>ToString form of money.</returns>
        public static string PrintMoney(string value, string cultureFormat)
        {
            decimal output;
            if (decimal.TryParse(value, NumberStyles.Currency, CultureInfo.CreateSpecificCulture(cultureFormat).NumberFormat, out output))
            {
                return new Money(output, CultureInfo.CreateSpecificCulture(cultureFormat)).ToString();
            }

            return "-";
        }

        /// <summary>
        /// Adds the money.
        /// </summary>
        /// <param name="value1">The value1 of first money string - this is culture sensitive.</param>
        /// <param name="value2">The value2 of second moeny string - this is culture sensitive.</param>
        /// <returns>The added culture senstive amount.</returns>
        public static string AddMoney(string value1, string value2)
        {
            decimal output1, output2;
            if (decimal.TryParse(value1, NumberStyles.Currency, CultureInfo.CurrentCulture.NumberFormat, out output1)
                && decimal.TryParse(value2, NumberStyles.Currency, CultureInfo.CurrentCulture.NumberFormat, out output2))
            {
                return new Money(output1) + new Money(output2);
            }

            return "-";
        }

        /// <summary>
        /// Adds the specified money.
        /// </summary>
        /// <param name="money">The money.</param>
        /// <returns>The added money.</returns>
        public Money Add(Money money)
        {
            return this + money;
        }

        /// <summary>
        /// Returns the fully qualified type name of this instance.
        /// </summary>
        /// <returns>
        /// A <see cref="T:System.String"/> containing a fully qualified type name.
        /// </returns>
        public override string ToString()
        {
            return (this.MinorValue / (Math.Pow(10, this.CultureInfo.NumberFormat.CurrencyDecimalDigits))).ToString("c", this.CultureInfo);
        }

        /// <summary>
        /// Returns the Decimal value of this instance.
        /// </summary>
        /// <returns>decimal value for the money object.</returns>
        public decimal ToDecimal()
        {
            return Convert.ToDecimal(this.MinorValue / (Math.Pow(10, this.CultureInfo.NumberFormat.CurrencyDecimalDigits)));
        }

        /// <summary>
        /// Indicates whether this instance and a specified object are equal.
        /// </summary>
        /// <param name="obj">Another object to compare to.</param>
        /// <returns>
        /// true if <paramref name="obj"/> and this instance are the same type and represent the same value; otherwise, false.
        /// </returns>
        public override bool Equals(object obj)
        {
            if (obj is Money && SameCurrency(this, (Money)obj))
            {
                return ((Money)obj).MinorValue == this.MinorValue;
            }

            return false;
        }

        /// <summary>
        /// Returns the hash code for this instance.
        /// </summary>
        /// <returns>
        /// A 32-bit signed integer that is the hash code for this instance.
        /// </returns>
        public override int GetHashCode()
        {
            return this.MinorValue.GetHashCode();
        }

        /// <summary>
        /// Subtracts the specified money.
        /// </summary>
        /// <param name="money">The money.</param>
        /// <returns>Subtraced money.</returns>
        public Money Subtract(Money money)
        {
            return this - money;
        }

        /// <summary>
        /// Sames the currency.
        /// </summary>
        /// <param name="money1">The money1.</param>
        /// <param name="money2">The money2.</param>
        /// <returns>true if the both money is same currency.</returns>
        private static bool SameCurrency(Money money1, Money money2)
        {
            string currency1 = new RegionInfo(money1.CultureInfo.LCID).CurrencySymbol;
            string currency2 = new RegionInfo(money2.CultureInfo.LCID).CurrencySymbol;
            return currency1 == currency2;
        }
    }
}
