﻿//---------------------------------------------------------------------
// <copyright file="UnityHelper.cs" company="commerceserver.net">
//     Copyright © commerceserver.net.  All rights reserved.
// </copyright>
// <summary></summary>
//---------------------------------------------------------------------
namespace CommerceServerContrib.CrossTier.Helpers
{
    using System.Globalization;
    using System;
    using System.Collections.Generic;
    using System.Configuration;
    using Microsoft.Practices.ServiceLocation;
    using Microsoft.Practices.Unity;
    using Microsoft.Practices.Unity.Configuration;
    using CommerceServer.Foundation;

    /// <summary>
    /// A helper class for Unity - creating containers, etc.
    /// </summary>
    public static class UnityHelper
    {
        private static Dictionary<string, Dictionary<string, Type>> _aliasMappings = null;
        private static object _syncRoot = new object();
        private static Dictionary<string, IUnityContainer> _unityContaners;
        private static bool _unityContainersInitialized = false;

        /// <summary>
        /// Setups the unity container.
        /// </summary>
        public static void SetupUnityContainer()
        {
            if (!_unityContainersInitialized)
            {
                lock (_syncRoot)
                {
                    if (!_unityContainersInitialized)    
                    {
                        _unityContaners = new Dictionary<string, IUnityContainer>();
                        _aliasMappings = new Dictionary<string, Dictionary<string, Type>>();

                        var section = UnityHelper.GetUnityConfigurationSection();

                        Dictionary<string, Type> aliasMap = new Dictionary<string, Type>();

                        foreach (var alias in section.TypeAliases)
                        {
                            aliasMap.Add(alias.Alias.ToLower(CultureInfo.InvariantCulture), Type.GetType(alias.TypeName));
                        }

                        // Top level aliases are mapped to and "" Empty string entry.
                        _aliasMappings[String.Empty] = aliasMap;

                        foreach (ContainerElement containerElement in section.Containers)
                        {
                            ValidateContainerUnityTypes(containerElement);

                            IUnityContainer unityContainer = new UnityContainer();
                            section.Configure(unityContainer, containerElement.Name);

                            _unityContaners[containerElement.Name.ToLower(CultureInfo.InvariantCulture)] = unityContainer;

                            aliasMap = new Dictionary<string, Type>();

                            foreach (RegisterElement registerElement in containerElement.Registrations)
                            {
                                if (!string.IsNullOrEmpty(registerElement.Name))
                                {
                                    aliasMap.Add(registerElement.Name.ToLower(CultureInfo.InvariantCulture), Type.GetType(registerElement.TypeName));
                                }
                            }

                            _aliasMappings[containerElement.Name.ToLower(CultureInfo.InvariantCulture)] = aliasMap;
                        }

                        _unityContainersInitialized = true;
                    }
                }
            }
        }

        /// <summary>
        /// Gets the implementation.
        /// </summary>
        /// <param name="containerName">Name of the Unity container.</param>
        /// <param name="container">The Unity container.</param>
        /// <returns>true of the unity container has been found.</returns>
        public static bool TryGetContainer(string containerName, out IUnityContainer container)
        {
            SetupUnityContainer();
            return _unityContaners.TryGetValue(containerName.ToLower(CultureInfo.InvariantCulture), out container);
        }

        /// <summary>
        /// Gets the unity configuration section assuming the default naming convention.
        /// </summary>
        /// <returns>UnityConfigurationSection from the web.config or the app.config</returns>
        public static UnityConfigurationSection GetUnityConfigurationSection()
        {
            return (UnityConfigurationSection)ConfigurationManager.GetSection(@"unity");
        }

        /// <summary>
        /// Gets the type for alias.
        /// </summary>
        /// <param name="containerName">Name of the container.</param>
        /// <param name="alias">The alias of the type.</param>
        /// <returns>
        /// The type of the alias
        /// </returns>
        public static Type GetTypeForAlias(string containerName, string alias)
        {
            ParameterChecker.CheckForNull(containerName, "containerName");
            ParameterChecker.CheckForNullOrEmpty(alias, "alias");

            SetupUnityContainer();

            alias = alias.ToLower(CultureInfo.InvariantCulture);
            containerName = containerName.ToLower(CultureInfo.InvariantCulture);

            if (String.IsNullOrEmpty(containerName) || !_aliasMappings.ContainsKey(containerName))
            {
                containerName = StringConstants.DefaultUnityContainerName.ToLower(CultureInfo.InvariantCulture); // default
            }

            Dictionary<string, Type> aliasMap = _aliasMappings[containerName];
            if (aliasMap.ContainsKey(alias))
            {
                return aliasMap[alias];
            }
            else
            {
                aliasMap = _aliasMappings[String.Empty]; // Check the top level
                if (aliasMap.ContainsKey(alias))
                {
                    return aliasMap[alias];
                }
            }

            return null;
        }

        #region Methods (private, static)

        private static void ValidateContainerUnityTypes(ContainerElement containerElement)
        {
            foreach (RegisterElement registerElement in containerElement.Registrations)
            {
                Type type = Type.GetType(registerElement.TypeName);
                if (type == null)
                {
                    string message = string.Format(
                        CultureInfo.CurrentCulture,
                        CrossTierResources.ErrorUnitySuppliedTypeDoesNotExist_Type,
                        registerElement.TypeName);
                    throw new ArgumentException(message);
                }

                if (!String.IsNullOrEmpty(registerElement.MapToName))
                {
                    type = Type.GetType(registerElement.MapToName);
                    if (type == null)
                    {
                        string message = string.Format(
                            CultureInfo.CurrentCulture,
                            CrossTierResources.ErrorUnitySuppliedTypeDoesNotExist_Type,
                            registerElement.TypeName);
                        throw new ArgumentException(message);
                    }
                }
            }
        }
        
        #endregion
    }
}
