﻿//---------------------------------------------------------------------
// <copyright file="SharePointBrandProvider.cs" company="commerceserver.net">
//     Copyright © commerceserver.net.  All rights reserved.
// </copyright>
// <summary></summary>
//---------------------------------------------------------------------

namespace CommerceServerContrib.Web.SharePoint
{
    using System.Collections.Generic;
    using System.Web;
    using CommerceServer.Foundation;
    using CommerceServerContrib.CrossTier;
    using Microsoft.SharePoint;

    /// <summary>
    /// Used to provide an instance of the current brand from SharePoint
    /// </summary>
    public class SharePointBrandProvider : BrandProvider
    {
        private const string ChannelWebPropertiesKey = "CommerceServerContrib.Web.Settings.Channel";
        private const string CurrentBrandKey = "CurrentBrand";
        private const string GuidFormatter = "B";

        private static Dictionary<string, string> _webIdBrandName = new Dictionary<string, string>();
        private static object _syncRoot = new object();

        /// <summary>
        /// Gets the name of the current channel.
        /// </summary>
        /// <value>
        /// The name of the current channel.
        /// </value>
        public override Brand Current
        {
            get
            {
                this.Initialize();

                var brand = this.GetBrandForCurrentSite();

                if (brand != null)
                {
                    return brand;
                }

                return base.Current;
            }
        }

        /// <summary>
        /// Check the current web to see if has an associated brand, and if it does not then check its parent, 
        /// and so on up the tree
        /// </summary>
        /// <returns>A Brand if found an Null otherwise</returns>
        protected virtual Brand GetBrandForCurrentSite()
        {
            try
            {
                // make sure we have a HTTP Content & Request to work with
                if (HttpContext.Current != null && HttpContext.Current.Request != null)
                {
                    // this is an expensive request so the brand should be stored in the request the first time it is figured out
                    var requestBrand = HttpContext.Current.Items[CurrentBrandKey];

                    if (requestBrand != null)
                    {
                        return requestBrand as Brand;
                    }

                    // get the id of the web
                    var id = SPContext.Current.Web.ID.ToString(GuidFormatter);

                    // check and see if we already know the brand for this site id
                    if (!_webIdBrandName.ContainsKey(id))
                    {
                        lock (_syncRoot)
                        {
                            if (!_webIdBrandName.ContainsKey(id))
                            {
                                // we don't know the brand for this web id so lets figure it out
                                this.SetupWebs(SPContext.Current.Web, new List<string>());
                            }
                        }
                    }

                    // get the name of the brand associated to this site
                    var brandName = _webIdBrandName[id];

                    // get the actual brand object for this brand name
                    requestBrand = this.GetBrandByName(brandName);

                    // store the brand in the request so that the next person has it straight away
                    HttpContext.Current.Items[CurrentBrandKey] = requestBrand;

                    return requestBrand as Brand;
                }
            }
            catch (System.Exception ex)
            {
                CommerceEventLog.PresentationTierSource.WriteException(ex);

                // we should only end up in this mess if there is a problem with the HttpContext or Request
                // so deinitialize and allow it to run again, and one of the following times the context will be setup
                this.Deinitialize();
            }

            // if we got this far then we couldn't figure it out so return null
            return null;
        }

        /// <summary>
        /// Recurse up the web tree to try and find a brand. If no brand is found,
        /// the default brand in the config will be used
        /// </summary>
        /// <param name="spWeb">The SPWeb to investigate.</param>
        /// <param name="emptyWebs">The collection of webs that we need to set once we find a parent with a value.</param>
        private void SetupWebs(SPWeb spWeb, List<string> emptyWebs)
        {
            var id = spWeb.ID.ToString(GuidFormatter);

            // add the web to the list of unknowns
            emptyWebs.Add(id);

            // if the property is present in the web property bag then we have found our channel
            if (spWeb.Properties.ContainsKey(ChannelWebPropertiesKey))
            {
                var defaultBrand = spWeb.Properties[ChannelWebPropertiesKey];

                this.SetBrandOnWebList(defaultBrand, emptyWebs);
            }
            else if (spWeb.IsRootWeb)
            {
                // if we are in the root web then there is no point in looping upwards anymore
                var defaultBrand = this.GetDefaultBrand();

                this.SetBrandOnWebList(defaultBrand.Name, emptyWebs);
            }
            else
            {
                // keep looping up because we have found nothing
                this.SetupWebs(spWeb.ParentWeb, emptyWebs);
            }
        }

        /// <summary>
        /// Set the brand for all of the webs in the list
        /// </summary>
        /// <param name="defaultBrand">The brand to associated with the webs.</param>
        /// <param name="emptyWebs">The list of webs that need to be set.</param>
        private void SetBrandOnWebList(string defaultBrand, List<string> emptyWebs)
        {
            foreach (var web in emptyWebs)
            {
                _webIdBrandName.Add(web, defaultBrand);
            }
        }
    }
}
