﻿//---------------------------------------------------------------------
// <copyright file="SharePointXsltProvider.cs" company="commerceserver.net">
//     Copyright © commerceserver.net.  All rights reserved.
// </copyright>
// <summary></summary>
//---------------------------------------------------------------------

namespace CommerceServerContrib.Web.SharePoint
{
    using System;
    using System.Globalization;
    using System.IO;
    using System.Net;
    using System.Text;
    using System.Xml;
    using System.Xml.Xsl;
    using CommerceServer.Foundation;
    using CommerceServerContrib.CrossTier.Configuration;
    using CommerceServerContrib.CrossTier.Helpers;
    using CommerceServerContrib.CrossTier.Interfaces;
    using Microsoft.SharePoint;

    /// <summary>
    /// A provider for getting XSLTs from SharePoint libraries
    /// </summary>
    public class SharePointXsltProvider : XmlResolver, IXsltProvider
    {
        #region Fields

        private static object _xsltCacheLock = new object();
        private ICredentials _credentials;

        #endregion

        #region XmlResolver Members

        /// <summary>
        /// When overridden in a derived class, sets the credentials used to authenticate Web requests.
        /// </summary>
        /// <returns>An <see cref="T:System.Net.ICredentials"/> object. If this property is not set, the value defaults to null; that is, the XmlResolver has no user credentials.</returns>
        public override ICredentials Credentials
        {
            set
            {
                this._credentials = value;
            }
        }

        /// <summary>
        /// When overridden in a derived class, maps a URI to an object containing the actual resource.
        /// </summary>
        /// <param name="absoluteUri">The URI returned from <see cref="M:System.Xml.XmlResolver.ResolveUri(System.Uri,System.String)"/>.</param>
        /// <param name="role">The current version does not use this parameter when resolving URIs. This is provided for future extensibility purposes. For example, this can be mapped to the xlink:role and used as an implementation specific argument in other scenarios.</param>
        /// <param name="ofObjectToReturn">The type of object to return. The current version only returns System.IO.Stream objects.</param>
        /// <returns>
        /// A System.IO.Stream object or null if a type other than stream is specified.
        /// </returns>
        /// <exception cref="T:System.Xml.XmlException">
        ///   <paramref name="ofObjectToReturn"/> is neither null nor a Stream type. </exception>
        ///   
        /// <exception cref="T:System.UriFormatException">The specified URI is not an absolute URI. </exception>
        ///   
        /// <exception cref="T:System.NullReferenceException">
        ///   <paramref name="absoluteUri"/> is null. </exception>
        ///   
        /// <exception cref="T:System.Exception">There is a runtime error (for example, an interrupted server connection). </exception>
        public override object GetEntity(Uri absoluteUri, string role, Type ofObjectToReturn)
        {
            var contents = this.GetXsltFileContentsFromDocumentLibrary(absoluteUri.OriginalString);
            byte[] byteArray = Encoding.ASCII.GetBytes(contents.ToString());
            return new MemoryStream(byteArray);
        }

        /// <summary>
        /// When overridden in a derived class, resolves the absolute URI from the base and relative URIs.
        /// </summary>
        /// <param name="baseUri">The base URI used to resolve the relative URI</param>
        /// <param name="relativeUri">The URI to resolve. The URI can be absolute or relative. If absolute, this value effectively replaces the <paramref name="baseUri"/> value. If relative, it combines with the <paramref name="baseUri"/> to make an absolute URI.</param>
        /// <returns>
        /// A <see cref="T:System.Uri"/> representing the absolute URI or null if the relative URI cannot be resolved.
        /// </returns>
        /// <exception cref="T:System.ArgumentException">
        /// <paramref name="relativeUri"/> is null</exception>
        public override System.Uri ResolveUri(System.Uri baseUri, string relativeUri)
        {
            return new Uri(relativeUri, UriKind.RelativeOrAbsolute);
        }

        #endregion

        #region IXsltProvider members
        /// <summary>
        /// Gets the requested Xslt
        /// </summary>
        /// <param name="path">The path to the XSLT relative to the root of the site</param>
        /// <returns>
        /// The requested Xslt in XslCompiledTransform form
        /// </returns>
        public XslCompiledTransform GetXslt(string path)
        {
            XslCompiledTransform xslt = null;

            CacheContext cacheContext = PresentationCacheSettings.GetCacheContext(CommerceServerContrib.CrossTier.Helpers.CacheConstants.XslCompiledTransform);

            if (cacheContext != null && cacheContext.IsCachingEnabled)
            {
                ICacheProvider cacheProvider = PresentationTypeLoader.CacheProvider(cacheContext);
                var channel = PresentationServiceLocator.Current.SiteContext.Channel;
                xslt = cacheProvider.GetData<XslCompiledTransform>(channel, cacheContext.CacheName, path);

                if (xslt == null)
                {
                    lock (_xsltCacheLock)
                    {
                        xslt = cacheProvider.GetData<XslCompiledTransform>(channel, cacheContext.CacheName, path);

                        if (xslt == null)
                        {
                            xslt = this.RetrieveFromDocumentLibrary(path);

                            cacheProvider.AddData<XslCompiledTransform>(channel, cacheContext.CacheName, path, xslt);
                        }
                    }
                }
            }
            else
            {
                xslt = this.RetrieveFromDocumentLibrary(path);
            }

            return xslt;
        }

        /// <summary>
        /// Indicates if the Xslt file exists or not.
        /// </summary>
        /// <param name="path">The path tp the Xslt file.</param>
        /// <returns>
        /// true if the fil exists; otherwise false.
        /// </returns>
        public bool XsltFileExists(string path)
        {
            return SharePointHelper.FileExists(SPContext.Current.Web, path);
        }

        /// <summary>
        /// Retrieves from a SharePoint doc library.
        /// </summary>
        /// <param name="fullPath">The full path.</param>
        /// <returns>A compiled version of the xslt</returns>
        private XslCompiledTransform RetrieveFromDocumentLibrary(string fullPath)
        {
            var xslt = new XslCompiledTransform(CommerceServerContribConfiguration.Presentation.EnableXsltDebugging);

            if (!string.IsNullOrEmpty(fullPath))
            {
                string fileContent = this.GetXsltFileContentsFromDocumentLibrary(fullPath);
                var xmlReader = new XmlTextReader(fileContent, XmlNodeType.Document, null);
                var xsltDoc = new XmlDocument();

                xsltDoc.Load(xmlReader);

                var xsltStr = xsltDoc.OuterXml;
                var strReader = new System.IO.StringReader(xsltStr);

                using (var reader = XmlReader.Create(strReader))
                {
                    xslt.Load(reader, new XsltSettings(), this);
                }

                return xslt;
            }

            return null;
        }
        #endregion

        private string GetXsltFileContentsFromDocumentLibrary(string fullPath)
        {
            if (!string.IsNullOrEmpty(fullPath))
            {
                SPWeb currentWeb = SPContext.Current.Web;
                string fileContent = string.Empty;
                
                SPSecurity.RunWithElevatedPrivileges(delegate
                {
                    using (var site = new SPSite(currentWeb.Site.ID))
                    {
                        SPWeb web = site.OpenWeb(currentWeb.ID);
                        fileContent = SharePointHelper.LoadFile(web, fullPath);

                        if (string.IsNullOrEmpty(fileContent))
                        {
                            throw new FileNotFoundException(string.Format(CultureInfo.CurrentCulture, ClientFrameworkSharePointResources.XsltNotFound, fullPath));
                        }
                    }
                });

                return fileContent;
            }

            return null;
        }
    }
}