﻿//---------------------------------------------------------------------
// <copyright file="ClaimsAuthenticationProvider.cs" company="commerceserver.net">
//     Copyright © commerceserver.net.  All rights reserved.
// </copyright>
// <summary>ClaimsAuthenticationProvider class</summary>
//---------------------------------------------------------------------

namespace CommerceServerContrib.Web.SharePoint
{
    using System;
    using System.Collections.Generic;
    using System.Globalization;
    using System.Linq;
    using System.Threading;
    using System.Web;
    using System.Web.Configuration;
    using System.Web.Security;
    using CommerceServer.Foundation;
    using CommerceServerContrib.CrossTier.Entities;
    using CommerceServerContrib.CrossTier.Interfaces;
    using Microsoft.IdentityModel.Claims;
    using Microsoft.IdentityModel.Web;
    using Microsoft.SharePoint;
    using Microsoft.SharePoint.Administration;
    using Microsoft.SharePoint.IdentityModel;
    using Microsoft.SharePoint.Utilities;

    /// <summary>
    /// ClaimsAuthenticationProvider class
    /// </summary>
    public class ClaimsAuthenticationProvider : IAuthenticationProvider
    {
        #region Private Fields

        private const string UserLogOnNameClaimType = "http://schemas.microsoft.com/sharepoint/2009/08/claims/userlogonname";
        private const string SharePointClaimIssuer = "SharePoint";
        private const string AuthenticationType = "Federation";

        #endregion

        #region Properties

        /// <summary>
        /// Gets a value indicating whether SSL is required for cookies issued by the authentication provider.
        /// </summary>
        public bool RequireSsl
        {
            get
            {
                // The SharePoint SPChunkedCookieHandler does not honor FederatedAuthentication.SessionAuthenticationModule.CookieHandler.RequireSsl.
                // It issues secure cookies over HTTPS and non-secure cookies over HTTP, which is the same as RequireSsl == false
                return false;
            }
        }

        /// <summary>
        /// Gets the commerce membership provider.
        /// </summary>
        private static MembershipProvider CommerceMembershipProvider
        {
            get
            {
                string webId = SPContext.Current.Site.WebApplication.Id.ToString("D");
                if (Membership.Providers["CS_" + webId + "_Internet"] == null)
                {
                    return Membership.Provider;
                }

                return Membership.Providers["CS_" + webId + "_Internet"];
            }
        }

        #endregion

        #region Implementation of IAuthenticationProvider

        /// <summary>
        /// Validates the user.
        /// </summary>
        /// <param name="userId">The user id.</param>
        /// <param name="userEmail">Name of the user.</param>
        /// <param name="password">The password.</param>
        /// <returns>
        /// True if the user is valid, false otherwise
        /// </returns>
        public bool ValidateUser(out string userId, string userEmail, string password)
        {
            userId = string.Empty;
            bool userAuthenticated = SPClaimsUtility.AuthenticateFormsUser(HttpContext.Current.Request.Url, userEmail, password);

            if (userAuthenticated)
            {
                MembershipUser user = CommerceMembershipProvider.GetUser(userEmail, false);
                if (user == null)
                {
                    return false;
                }

                if (user.ProviderUserKey is Guid)
                {
                    userId = ((Guid)user.ProviderUserKey).ToString("B");
                    return true;
                }
            }

            return false;
        }

        /// <summary>
        /// Signs in the user into the site.
        /// </summary>
        /// <param name="userId">The user id.</param>
        /// <param name="userEmail">The user email.</param>
        /// <param name="userName">Name of the user.</param>
        /// <param name="password">Password of the user.</param>
        /// <param name="rememberMe">if set to <c>true</c> [remember me].</param>
        /// <returns>
        /// True is the user was sucessfully signed in, false otherwise
        /// </returns>
        public bool SignIn(string userId, string userEmail, string userName, string password, bool rememberMe)
        {
            if (string.IsNullOrEmpty(userId) || string.IsNullOrEmpty(userEmail) || string.IsNullOrEmpty(userName) || string.IsNullOrEmpty(password))
            {
                return false;
            }

            bool isAuthenticated = SPClaimsUtility.AuthenticateFormsUser(HttpContext.Current.Request.Url, userEmail, password);

            if (isAuthenticated)
            {
                if (HttpContext.Current.Response.Cookies["FedAuth"] != null)
                {
                    HttpContext.Current.Response.Cookies["FedAuth"].Secure = false;
                    HttpContext.Current.Response.Cookies["FedAuth"].Expires = DateTime.MinValue;
                }

                UserCookieHelper.IssueRegisteredCookie(userId, userEmail, userName, AuthenticationType);
                if (rememberMe)
                {
                    UserCookieHelper.IssueRememberMeCookie(userEmail);
                }
                else
                {
                    UserCookieHelper.RemoveRememberMeCookie();
                }
            }

            return isAuthenticated;
        }

        /// <summary>
        /// Determines whether [is user identity valid].
        /// </summary>
        /// <param name="userCookie">The user cookie.</param>
        /// <returns>
        ///   <c>true</c> if [is user identity valid]; otherwise, <c>false</c>.
        /// </returns>
        public bool IsUserIdentityValid(UserCookie userCookie)
        {
            string email = RetrieveValueFromClaims(UserLogOnNameClaimType, SharePointClaimIssuer);
            MembershipUser user = CommerceMembershipProvider.GetUser(email, false);

            if (user != null && user.ProviderUserKey != null)
            {
                string providerUserKey = ((Guid)user.ProviderUserKey).ToString("B");
                if (providerUserKey.Equals(userCookie.User.Id, StringComparison.OrdinalIgnoreCase))
                {
                    if (userCookie.User.Email.Equals(user.Email, StringComparison.OrdinalIgnoreCase))
                    {
                        return true;
                    }
                }
            }

            return false;
        }

        /// <summary>
        /// Issues the registered cookie.
        /// </summary>
        /// <param name="userId">The user id.</param>
        /// <param name="userEmail">The user email.</param>
        /// <param name="userName">Name of the user.</param>
        public void IssueRegisteredCookie(string userId, string userEmail, string userName)
        {
            UserCookieHelper.RemoveRegisteredCookie();
            UserCookieHelper.IssueRegisteredCookie(userId, userEmail, userName, AuthenticationType);
        }

        /// <summary>
        /// Terminateds the session.
        /// </summary>
        public void TerminateSession()
        {
            PresentationServiceLocator.Current.SiteContext.User = new User();
            UserCookieHelper.RemoveRegisteredCookie();

            string redirectUrl = (HttpContext.Current.Request.Url.ToString().ToLower(CultureInfo.InvariantCulture).Contains("returnurl")) ?
                HttpContext.Current.Request.QueryString["returnurl"] : Microsoft.Security.Application.Encoder.UrlEncode(HttpContext.Current.Request.Url.PathAndQuery);

            if (HttpContext.Current.Session != null)
            {
                HttpContext.Current.Session.Clear();
            }

            string str = string.Empty;
            if (HttpContext.Current.Request.Browser["supportsEmptyStringInCookieValue"] == "false")
            {
                str = "NoCookie";
            }

            HttpCookie cookie = HttpContext.Current.Request.Cookies["WSS_KeepSessionAuthenticated"];
            if (cookie != null)
            {
                cookie.Value = str;
                HttpContext.Current.Response.Cookies.Remove("WSS_KeepSessionAuthenticated");
                HttpContext.Current.Response.Cookies.Add(cookie);
            }

            HttpCookie cookie2 = HttpContext.Current.Request.Cookies["WSS_KeepSessionAuthenticated"];
            if (cookie2 != null)
            {
                cookie2.Value = str;
                cookie2.Expires = new DateTime(0x7b2, 1, 1);
                HttpContext.Current.Response.Cookies.Remove("WSS_KeepSessionAuthenticated");
                HttpContext.Current.Response.Cookies.Add(cookie2);
            }

            SPIisSettings iisSettingsWithFallback = SPContext.Current.Site.WebApplication.GetIisSettingsWithFallback(SPContext.Current.Site.Zone);
            if (iisSettingsWithFallback.UseClaimsAuthentication)
            {
                FederatedAuthentication.SessionAuthenticationModule.SignOut();
                int num = 0;
                using (IEnumerator<SPAuthenticationProvider> enumerator = iisSettingsWithFallback.ClaimsAuthenticationProviders.GetEnumerator())
                {
                    while (enumerator.MoveNext())
                    {
                        num++;
                    }
                }

                if ((num != 1) || !iisSettingsWithFallback.UseWindowsIntegratedAuthentication)
                {
                    SPUtility.Redirect((string.IsNullOrEmpty(redirectUrl) ? redirectUrl : SharePointHelper.FindTopVariantSite(SPContext.Current.Web)), SPRedirectFlags.Default, HttpContext.Current);
                }
            }
            else if (AuthenticationMode.Windows != SPSecurity.AuthenticationMode)
            {
                throw new SPException();
            }
        }

        /// <summary>
        /// Gets a user by email.
        /// </summary>
        /// <param name="userEmail">The user email.</param>
        /// <returns>A MembershipUser.</returns>
        public MembershipUser GetUser(string userEmail)
        {
            ParameterChecker.CheckForNullOrEmpty(userEmail, "operationBuilder");
            return CommerceMembershipProvider.GetUser(userEmail, false);
        }

        #endregion

        #region Private Methods

        private static string RetrieveValueFromClaims(string claimType, string issuer)
        {
            ClaimsIdentity identity = (ClaimsIdentity)Thread.CurrentPrincipal.Identity;
            return (from claim in identity.Claims
                    where claim.ClaimType.Equals(claimType, StringComparison.OrdinalIgnoreCase) && claim.Issuer.Equals(issuer, StringComparison.OrdinalIgnoreCase)
                    select claim.Value).FirstOrDefault();
        }

        #endregion
    }
}
