﻿namespace CommerceServerContrib.Web.SharePoint
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Text;
    using System.Web.Security;
    using System.Web.ApplicationServices;

    /// <summary>
    /// A simple RoleProvider for SharePoint. 
    /// </summary>
    public class SingleRoleProvider : RoleProvider
    {
        /// <summary>
        /// Default role to assign if none is specified in configuration
        /// </summary>
        public const string DefaultRole = "member";

        private string _applicationName;

        /// <summary>
        /// Only one role is supported. If this is changed, please adjust method implementations
        /// </summary>
        private string[] _allRoles = null;

        /// <summary>
        /// Gets or sets the application name obtained from configuration
        /// </summary>
        public override string ApplicationName
        {
            get
            {
                return this._applicationName;
            }

            set
            {
                this._applicationName = value;
            }
        }

        /// <summary>
        /// Initializes the provider.
        /// </summary>
        /// <param name="name">The friendly name of the provider.</param>
        /// <param name="config">A collection of the name/value pairs representing the provider-specific attributes specified in the configuration for this provider.</param>
        public override void Initialize(string name, System.Collections.Specialized.NameValueCollection config)
        {
            base.Initialize(name, config);

            string roleToUse = null;
            string roleFromConfig = config["role"];

            // Use the one from configuration if it's not null and the trimmed version isn't empty
            if (roleFromConfig != null && (roleFromConfig = roleFromConfig.Trim()).Length > 0)
            {
                roleToUse = roleFromConfig;
            }
            else
            {
                // Otherwise, use the built-in default
                roleToUse = DefaultRole;
            }

            this._allRoles = new string[] { roleToUse };
        }

        /// <summary>
        /// Simply returns one role: "member"
        /// </summary>
        /// <returns>All roles supported</returns>
        public override string[] GetAllRoles()
        {
            return this._allRoles;
        }

        /// <summary>
        /// Gets a list of the roles that a specified user is in for the configured applicationName.
        /// </summary>
        /// <param name="username">The user to return a list of roles for.</param>
        /// <returns>
        /// A string array containing the names of all the roles that the specified user is in for the configured applicationName.
        /// </returns>
        public override string[] GetRolesForUser(string username)
        {
            return this._allRoles;
        }

        /// <summary>
        /// Gets a value indicating whether the specified user is in the specified role for the configured applicationName.
        /// </summary>
        /// <param name="username">The user name to search for.</param>
        /// <param name="roleName">The role to search in.</param>
        /// <returns>
        /// true if the specified user is in the specified role for the configured applicationName; otherwise, false.
        /// </returns>
        public override bool IsUserInRole(string username, string roleName)
        {
            return true;
        }

        /// <summary>
        /// Gets a value indicating whether the specified role name already exists in the role data source for the configured applicationName.
        /// </summary>
        /// <param name="roleName">The name of the role to search for in the data source.</param>
        /// <returns>
        /// true if the role name already exists in the data source for the configured applicationName; otherwise, false.
        /// </returns>
        public override bool RoleExists(string roleName)
        {
            return this._allRoles[0].Equals(roleName, StringComparison.OrdinalIgnoreCase);
        }

        #region Not implemented

        /// <summary>
        /// Adds the specified user names to the specified roles for the configured applicationName.
        /// </summary>
        /// <param name="usernames">A string array of user names to be added to the specified roles.</param>
        /// <param name="roleNames">A string array of the role names to add the specified user names to.</param>
        /// <exception cref="System.NotImplementedException">Not required by the existing SharePoint Application so it is not implemented.  </exception>
        public override void AddUsersToRoles(string[] usernames, string[] roleNames)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Adds a new role to the data source for the configured applicationName.
        /// </summary>
        /// <param name="roleName">The name of the role to create.</param>
        /// <exception cref="System.NotImplementedException">Not required by the existing SharePoint Application so it is not implemented.</exception>
        public override void CreateRole(string roleName)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Removes a role from the data source for the configured applicationName.
        /// </summary>
        /// <param name="roleName">The name of the role to delete.</param>
        /// <param name="throwOnPopulatedRole">If true, throw an exception if <paramref name="roleName" /> has one or more members and do not delete <paramref name="roleName" />.</param>
        /// <returns>
        /// true if the role was successfully deleted; otherwise, false.
        /// </returns>
        /// <exception cref="System.NotImplementedException">Not required by the existing SharePoint Application so it is not implemented.</exception>
        public override bool DeleteRole(string roleName, bool throwOnPopulatedRole)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Gets an array of user names in a role where the user name contains the specified user name to match.
        /// </summary>
        /// <param name="roleName">The role to search in.</param>
        /// <param name="usernameToMatch">The user name to search for.</param>
        /// <returns>
        /// A string array containing the names of all the users where the user name matches <paramref name="usernameToMatch" /> and the user is a member of the specified role.
        /// </returns>
        /// <exception cref="System.NotImplementedException">Not required by the existing SharePoint Application so it is not implemented.</exception>
        public override string[] FindUsersInRole(string roleName, string usernameToMatch)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Gets a list of users in the specified role for the configured applicationName.
        /// </summary>
        /// <param name="roleName">The name of the role to get the list of users for.</param>
        /// <returns>
        /// A string array containing the names of all the users who are members of the specified role for the configured applicationName.
        /// </returns>
        /// <exception cref="System.NotImplementedException">Not required by the existing SharePoint Application so it is not implemented.</exception>
        public override string[] GetUsersInRole(string roleName)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Removes the specified user names from the specified roles for the configured applicationName.
        /// </summary>
        /// <param name="usernames">A string array of user names to be removed from the specified roles.</param>
        /// <param name="roleNames">A string array of role names to remove the specified user names from.</param>
        /// <exception cref="System.NotImplementedException">Not required by the existing SharePoint Application so it is not implemented.</exception>
        public override void RemoveUsersFromRoles(string[] usernames, string[] roleNames)
        {
            throw new NotImplementedException();
        }

        #endregion
    }
}
