﻿//---------------------------------------------------------------------
// <copyright file="BaseFeatureReceiver.cs" company="commerceserver.net">
//     Copyright © commerceserver.net.  All rights reserved.
// </copyright>
// <summary></summary>
//---------------------------------------------------------------------
namespace CommerceServerContrib.Web.SharePoint
{
    using System;
    using System.Collections.ObjectModel;
    using System.Globalization;
    using System.IO;
    using System.Security.Permissions;
    using System.Xml;
    using Microsoft.SharePoint;
    using Microsoft.SharePoint.Administration;
    using Microsoft.SharePoint.Navigation;
    using Microsoft.SharePoint.Publishing;
    using Microsoft.SharePoint.Publishing.Navigation;
    using Microsoft.SharePoint.Security;

    /// <summary>
    /// The base feature receiver for Commerce Server features
    /// </summary>
    public class BaseFeatureReceiver : SPFeatureReceiver
    {
        #region Protected Fields

        /// <summary>
        /// Tracks the feature property used to remove files when a feature is being deactivated
        /// </summary>
        protected const string FeatureDeactivatingPipelineDelimitedElementsManifestPropertyKey = "FeatureDeactivatingPipelineDelimitedElementsManifest";

        /// <summary>
        /// Tracks the feature property used to remove files when a feature cann't be activated
        /// </summary>
        protected const string FeatureActivationDependencyPipelineDelimitedElementsManifestPropertyKey = "FeatureActivationDependencyPipelineDelimitedElementsManifest";

        /// <summary>
        /// Tracks the feature property used to remove libraries
        /// </summary>
        protected const string FeaturePipelineDelimitedLibrariesPropertyKey = "FeaturePipelineDelimitedLibraries";

        /// <summary>
        /// Tracks the feature property used to define feature dependencies on feature activation
        /// </summary>
        protected const string FeatureActivationDependencyPropertyKey = "FeatureActivationDependency";

        /// <summary>
        /// Tracks the feature property used to define feature dependencies on feature deactivation
        /// </summary>
        protected const string FeatureDeactivationDependencyPropertyKey = "FeatureDeactivationDependency";

        /// <summary>
        /// Suffix for the job name
        /// </summary>
        protected const string FeatureActivation = "_FeatureActivation";

        /// <summary>
        /// Suffix for the job name
        /// </summary>
        protected const string FeatureDeactivation = "_FeatureDeactivation";

        /// <summary>
        /// Tracks the feature property used to modify the site navigation
        /// </summary>
        protected const string NavigationFileNamePropertyKey = "NavigationFileName";

        /// <summary>
        /// Tracks the feature property used to overwrite files that already exists in a library
        /// </summary>
        protected const string FilesToOverwriteIfExistsPipelineDelimitedFolderAndPathPropertyKey = "FilesToOverwriteIfExistsPipelineDelimitedFolderAndPath";

        /// <summary>
        /// Tracks the feature property used to add/delete items from existing lists
        /// </summary>
        protected const string PipelineDelimitedListItemsFileNamesPropertyKey = "PipelineDelimitedListItemsFileNames";

        /// <summary>
        /// Tracks the feature property used to setup the master page for a site
        /// </summary>
        protected const string MasterPageNamePropertyKey = "MasterPageName";

        #endregion

        #region Private Fields

        private const string NameAttributeName = "Name";
        private const string UrlAttributeName = "Url";
        private const string ValueAttributeName = "Value";
        private const string RowsXPath = "//Rows/Row";
        private const string NavigationNodesXPath = "//Nodes/Node";
        private const string NavigationNodeTypeAttributeName = "NodeType";
        private const string ListNodeName = "List";
        private const string ListNameAttributeName = "ListName";
        private const string ListFieldNameToDeleteItemsAttributeName = "ListFieldNameToDeleteItems";

        #endregion

        #region Events

        /// <summary>
        /// Occurs after a Feature is activated.
        /// </summary>
        /// <param name="properties">An <see cref="T:Microsoft.SharePoint.SPFeatureReceiverProperties"></see> object that represents the properties of the event.</param>
        [SharePointPermission(SecurityAction.LinkDemand, ObjectModel = true)]
        public override void FeatureActivated(SPFeatureReceiverProperties properties)
        {
            var webApplication = properties.Feature.Parent as SPWebApplication;
            var service = properties.Feature.Parent as SPService;

            if (webApplication != null)
            {
                UpdateWebConfig(properties);

                if (Directory.Exists(Path.Combine(properties.Definition.RootDirectory, "webfiles")) || Directory.Exists(Path.Combine(properties.Definition.RootDirectory, "WebConfigEntries-NonSP")))
                {
                    string jobName = properties.Definition.DisplayName + FeatureActivation;
                    FeatureHelper.DeleteJobInstance(webApplication, jobName);
                    var job = new ActivationJobDefinition(jobName, webApplication)
                    {
                        Schedule = new SPOneTimeSchedule(DateTime.Now),
                        Title = jobName
                    };

                    job.Properties.Add("FeatureDefinitionId", properties.Feature.DefinitionId);
                    job.Properties.Add("SPWebAppId", webApplication.Id.ToString());
                    job.Update();

                    TimerJobHelper.WaitForOnetimeJobToFinish(SPWebService.ContentService.WebApplications[webApplication.Id].Farm, jobName, 60); 
                }

                return;
            }

            if (service != null)
            {
                if (Directory.Exists(Path.Combine(properties.Definition.RootDirectory, "WebConfigEntries-NonSP")))
                {
                    string jobName = properties.Definition.DisplayName + FeatureActivation;
                    FeatureHelper.DeleteJobInstance(service, jobName);
                    var job = new ActivationJobDefinition(jobName, service)
                    {
                        Schedule = new SPOneTimeSchedule(DateTime.Now),
                        Title = jobName
                    };

                    job.Properties.Add("FeatureDefinitionId", properties.Feature.DefinitionId);
                    job.Update();

                    TimerJobHelper.WaitForOnetimeJobToFinish(service.Farm, jobName, 60);
                }

                return;
            }

            SPSiteCollection siteCollection = properties.Feature.Parent as SPSiteCollection;
            if (siteCollection != null)
            {
                return;
            }

            SPSite site = properties.Feature.Parent as SPSite;
            if (site != null)
            {
                return;
            }
        }

        /// <summary>
        /// Occurs when a Feature is deactivated.
        /// </summary>
        /// <param name="properties">An <see cref="T:Microsoft.SharePoint.SPFeatureReceiverProperties"></see> object that represents the properties of the event.</param>
        [SharePointPermission(SecurityAction.LinkDemand, ObjectModel = true)]
        public override void FeatureDeactivating(SPFeatureReceiverProperties properties)
        {
            SPWebApplication webApplication = properties.Feature.Parent as SPWebApplication;
            var service = properties.Feature.Parent as SPService;

            if (webApplication != null)
            {
                Collection<SPWebConfigModification> webconfigModifications = SPWebService.ContentService.WebApplications[webApplication.Id].WebConfigModifications;

                int startCount = webconfigModifications.Count;

                for (int i = startCount - 1; i >= 0; i--)
                {
                    SPWebConfigModification webConfigModification = webconfigModifications[i];
                    if (webConfigModification.Owner == properties.Definition.DisplayName)
                    {
                        webconfigModifications.Remove(webConfigModification);
                    }
                }

                if (startCount > webconfigModifications.Count)
                {
                    SPWebService.ContentService.WebApplications[webApplication.Id].Update();
                    SPWebService.ContentService.WebApplications[webApplication.Id].WebService.ApplyWebConfigModifications();

                    if (SPWebService.ContentService.WebApplications[webApplication.Id].Farm.TimerService.Instances.Count > 1)
                    {
                        // When there are multiple front-end Web servers in the SharePoint farm, we need to wait for the timer job that performs the Web.config modifications to complete before continuing.
                        // Otherwise, we may encounter the following error
                        // (e.g. when applying Web.config changes from two different features in rapid succession):
                        // 
                        // "A web configuration modification operation is already running."
                        TimerJobHelper.WaitForOnetimeJobToFinish(SPWebService.ContentService.WebApplications[webApplication.Id].Farm, "Windows SharePoint Services Web.Config Update", 60);
                    }
                }

                if (Directory.Exists(Path.Combine(properties.Definition.RootDirectory, "webfiles")) || Directory.Exists(Path.Combine(properties.Definition.RootDirectory, "WebConfigEntries-NonSP")))
                {
                    string jobName = properties.Definition.DisplayName + FeatureDeactivation;
                    FeatureHelper.DeleteJobInstance(webApplication, jobName);
                    var job = new DeactivationJobDefinition(jobName, webApplication)
                                  {
                                      Schedule = new SPOneTimeSchedule(DateTime.Now),
                                      Title = jobName
                                  };
                    job.Properties.Add("FeatureRootDirectory", properties.Feature.Definition.RootDirectory);
                    job.Properties.Add("SPWebAppId", webApplication.Id.ToString());
                    job.Update();

                    TimerJobHelper.WaitForOnetimeJobToFinish(
                        SPWebService.ContentService.WebApplications[webApplication.Id].Farm, jobName, 60);
                }

                return;
            }

            if (service != null)
            {
                if (Directory.Exists(Path.Combine(properties.Definition.RootDirectory, "WebConfigEntries-NonSP")))
                {
                    string jobName = properties.Definition.DisplayName + FeatureDeactivation;
                    FeatureHelper.DeleteJobInstance(service, jobName);
                    var job = new DeactivationJobDefinition(jobName, service)
                    {
                        Schedule = new SPOneTimeSchedule(DateTime.Now),
                        Title = jobName
                    };
                    job.Properties.Add("FeatureRootDirectory", properties.Feature.Definition.RootDirectory);
                    job.Update();

                    TimerJobHelper.WaitForOnetimeJobToFinish(service.Farm, jobName, 60);
                }                
            }

            SPSiteCollection siteCollection = properties.Feature.Parent as SPSiteCollection;

            if (siteCollection != null)
            {
                return;
            }
        }

        /// <summary>
        /// Occurs after a Feature is installed.
        /// </summary>
        /// <param name="properties">An <see cref="T:Microsoft.SharePoint.SPFeatureReceiverProperties"></see> object that represents the properties of the event.</param>
        [SharePointPermission(SecurityAction.LinkDemand, ObjectModel = true)]
        public override void FeatureInstalled(SPFeatureReceiverProperties properties)
        {
        }

        /// <summary>
        /// Occurs when a Feature is uninstalled.
        /// </summary>
        /// <param name="properties">An <see cref="T:Microsoft.SharePoint.SPFeatureReceiverProperties"></see> object that represents the properties of the event.</param>
        [SharePointPermission(SecurityAction.LinkDemand, ObjectModel = true)]
        public override void FeatureUninstalling(SPFeatureReceiverProperties properties)
        {
        }

        #endregion

        #region Protected Methods

        /// <summary>
        /// Gets the major version of SharePoint as a string. 
        /// </summary>
        /// <returns>String for the major version of SharePoint.</returns>
        protected static string SPMajorVersion()
        {
            SPFarm farm = SPFarm.Local;
            System.Version ver = farm.BuildVersion;
            return ver.Major.ToString(CultureInfo.InvariantCulture);
        }

        /// <summary>
        /// Add nodes to site the navigation.
        /// </summary>
        /// <param name="publishingWeb">The publishing web.</param>
        /// <param name="xmlDoc">The XML doc containing the navigation nodes.</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1059:MembersShouldNotExposeCertainConcreteTypes", MessageId = "System.Xml.XmlNode", Justification = "XmlDocument required.")]
        protected void AddNodesToNavigation(PublishingWeb publishingWeb, XmlDocument xmlDoc)
        {
            XmlNodeList newCurrentNodes = xmlDoc.SelectNodes(NavigationNodesXPath);
            AddNodes(publishingWeb.Navigation.CurrentNavigationNodes, newCurrentNodes);
            publishingWeb.Update();
        }

        /// <summary>
        /// Hides the nodes from navigation.
        /// </summary>
        /// <param name="web">The web to modify.</param>
        /// <param name="xmlDoc">The XML doc containing the navigation nodes..</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1059:MembersShouldNotExposeCertainConcreteTypes", MessageId = "System.Xml.XmlNode", Justification = "XmlDocument Required")]
        protected void HideNodesFromNavigation(SPWeb web, XmlDocument xmlDoc)
        {
            PublishingWeb publishingWeb = PublishingWeb.GetPublishingWeb(web);
            XmlNodeList nodesToHide = xmlDoc.SelectNodes(NavigationNodesXPath);
            if (nodesToHide == null || nodesToHide.Count == 0)
            {
                return;
            }

            for (int i = 0; i < nodesToHide.Count; i++)
            {
                var nodeXml = (XmlElement)nodesToHide[i];
                string nodeUrl = nodeXml.GetAttribute(UrlAttributeName).StartsWith("/", StringComparison.OrdinalIgnoreCase) ? nodeXml.GetAttribute(UrlAttributeName) : string.Concat(@"\", nodeXml.GetAttribute(UrlAttributeName));
                string url = web.ServerRelativeUrl == "/" ? nodeUrl : string.Concat(web.ServerRelativeUrl, nodeUrl);
                NodeTypes type = NodeTypes.None;
                if (nodeXml.GetAttribute(NavigationNodeTypeAttributeName) != null && !string.IsNullOrEmpty(nodeXml.GetAttribute(NavigationNodeTypeAttributeName)))
                {
                    type = (NodeTypes)Enum.Parse(typeof(NodeTypes), nodeXml.GetAttribute(NavigationNodeTypeAttributeName));
                }

                if (type == NodeTypes.Page)
                {
                    PublishingPage page = publishingWeb.GetPublishingPage(url);
                    SPFile pagefile = web.GetFile(url);

                    if (page != null && pagefile != null)
                    {
                        if (pagefile.CheckOutType == SPFile.SPCheckOutType.None)
                        {
                            pagefile.CheckOut();
                        }

                        page.IncludeInCurrentNavigation = false;
                        page.IncludeInGlobalNavigation = false;
                        page.Update();

                        pagefile.Update();
                        pagefile.CheckIn(string.Empty);
                        pagefile.Publish(string.Empty);
                        pagefile.Approve(string.Empty);
                    }
                }
            }

            publishingWeb.Update();
            web.Update();
        }

        /// <summary>
        /// Deletes the files.
        /// </summary>
        /// <param name="properties">The feature's properties.</param>
        /// <param name="pipelineDelimitedPropertyName">Name of the pipeline delimited property.</param>
        protected void DeleteFiles(SPFeatureReceiverProperties properties, string pipelineDelimitedPropertyName)
        {
            SPWeb parent = GetFeatureWebParent(properties);
            string pipelineProperty = GetPipelinePropertyToUse(pipelineDelimitedPropertyName);

            if (!string.IsNullOrEmpty(pipelineProperty))
            {
                if (properties.Feature.Properties[pipelineProperty] != null && parent != null)
                {
                    string[] elements = properties.Feature.Properties[pipelineProperty].Value.Split(new[] { '|' });
                    var elementsFile = new XmlDocument();
                    foreach (string elementFileName in elements)
                    {
                        string elementFilePath = properties.Feature.Definition.RootDirectory + @"\" + elementFileName.Trim();
                        if (File.Exists(elementFilePath))
                        {
                            elementsFile.Load(elementFilePath);

                            if (elementsFile.DocumentElement != null)
                            {
                                foreach (XmlNode module in elementsFile.DocumentElement.ChildNodes)
                                {
                                    if (module.NodeType == XmlNodeType.Element && module.Name == "Module")
                                    {
                                        string fileName = string.Empty;
                                        string libraryUrl = module.Attributes[UrlAttributeName].Value + "/";
                                        foreach (XmlNode file in module.ChildNodes)
                                        {
                                            if (file.NodeType == XmlNodeType.Element)
                                            {
                                                if (file.Attributes[NameAttributeName] != null)
                                                {
                                                    fileName = file.Attributes[NameAttributeName].Value;
                                                }
                                                else if (file.Attributes[UrlAttributeName] != null)
                                                {
                                                    fileName = file.Attributes[UrlAttributeName].Value;
                                                }

                                                if (!string.IsNullOrEmpty(fileName))
                                                {
                                                    try
                                                    {
                                                        string fileUrl = parent.ServerRelativeUrl + libraryUrl + fileName;
                                                        SPFile fileToDelete = parent.GetFile(fileUrl);
                                                        if (fileToDelete.Exists)
                                                        {
                                                            try
                                                            {
                                                                fileToDelete.Delete();
                                                            }
                                                            catch
                                                            {
                                                                string failureMessage = "File: {0} cannot be deleted because it is still referenced by other pages";
                                                                string fileToDeleteName = ((SPFile)fileToDelete).Name;
                                                                throw new SPException(string.Format(CultureInfo.InvariantCulture, failureMessage, fileToDeleteName));
                                                            }
                                                        }
                                                    }
                                                    catch
                                                    {
                                                        string failureMessage = "File: {0} cannot be deleted because it wasn't found";
                                                        string fileToDeleteName = fileName;
                                                        throw new SPException(string.Format(CultureInfo.InvariantCulture, failureMessage, fileToDeleteName));
                                                    }
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }

                    parent.Update();
                }
            }
        }

        /// <summary>
        /// Deletes the folders.
        /// </summary>
        /// <param name="properties">The properties.</param>
        /// <param name="pipelineDelimitedPropertyName">Name of the pipeline delimited property.</param>
        protected void DeleteFolders(SPFeatureReceiverProperties properties, string pipelineDelimitedPropertyName)
        {
            SPWeb parent = GetFeatureWebParent(properties);
            string pipelineProperty = GetPipelinePropertyToUse(pipelineDelimitedPropertyName);

            if (parent != null)
            {
                if (!string.IsNullOrEmpty(pipelineProperty))
                {
                    if (properties.Feature.Properties[pipelineProperty] != null)
                    {
                        string[] elements = properties.Feature.Properties[pipelineProperty].Value.Split(new[] { '|' });
                        var elementsFile = new XmlDocument();
                        foreach (string elementFileName in elements)
                        {
                            string elementFilePath = properties.Feature.Definition.RootDirectory + @"\" + elementFileName.Trim();
                            if (File.Exists(elementFilePath))
                            {
                                elementsFile.Load(elementFilePath);

                                if (elementsFile.DocumentElement != null)
                                {
                                    foreach (XmlNode module in elementsFile.DocumentElement.ChildNodes)
                                    {
                                        if (module.NodeType == XmlNodeType.Element && module.Name == "Module")
                                        {
                                            string libraryUrl = module.Attributes[UrlAttributeName].Value + "/";
                                            try
                                            {
                                                string folderUrl = parent.ServerRelativeUrl + libraryUrl;
                                                SPFolder folderToDelete = parent.GetFolder(folderUrl);
                                                if (folderToDelete.Exists)
                                                {
                                                    SPFolder parentFolderToDelete = folderToDelete.ParentFolder;
                                                    try
                                                    {
                                                        folderToDelete.Delete();
                                                    }
                                                    catch
                                                    {
                                                        string failureMessage = "File: {0} cannot be deleted.";
                                                        string folderToDeleteName = folderToDelete.Name;
                                                        throw new SPException(string.Format(CultureInfo.InvariantCulture, failureMessage, folderToDeleteName));
                                                    }

                                                    DeleteParentFolders(parentFolderToDelete);
                                                }
                                            }
                                            catch
                                            {
                                                string failureMessage = "File: {0} cannot be deleted because it wasn't found";
                                                string folderToDeleteName = libraryUrl;
                                                throw new SPException(string.Format(CultureInfo.InvariantCulture, failureMessage, folderToDeleteName));
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }

                    parent.Update();
                }
            }
        }

        /// <summary>
        /// Determines whether this instance [can feature be activated] the specified feature activation dependency.
        /// </summary>
        /// <param name="properties">The feature's properties.</param>
        /// <returns>
        /// 	<c>true</c> if this instance [can feature be activated] the specified properties; otherwise, <c>false</c>.
        /// </returns>
        protected bool CanFeatureBeActivated(SPFeatureReceiverProperties properties)
        {
            var featureParent = properties.Feature.Parent as SPSite;
            if (featureParent != null)
            {
                if (properties.Feature.Properties[FeatureActivationDependencyPropertyKey] != null)
                {
                    string[] features = properties.Feature.Properties[FeatureActivationDependencyPropertyKey].Value.Split(new[] { ',' });
                    foreach (string feature in features)
                    {
                        string[] featureProperties = feature.Split(new[] { '|' });
                        string featureScope;
                        Guid featureId;

                        try
                        {
                            featureId = new Guid(featureProperties[0].Trim());
                            featureScope = featureProperties[1].Trim();
                        }
                        catch
                        {
                            featureId = new Guid(featureProperties[1].Trim());
                            featureScope = featureProperties[0].Trim();
                        }

                        if (featureScope.Equals(Enum.GetName(typeof(SPFeatureScope), SPFeatureScope.Web)))
                        {
                            return featureParent.OpenWeb().Features[featureId] != null;
                        }
                    }
                }
            }

            return true;
        }

        /// <summary>
        /// Determines whether this instance [can feature be deactivated] the specified properties.
        /// </summary>
        /// <param name="properties">The properties.</param>
        /// <returns>
        /// 	<c>true</c> if this instance [can feature be deactivated] the specified properties; otherwise, <c>false</c>.
        /// </returns>
        protected bool CanFeatureBeDeactivated(SPFeatureReceiverProperties properties)
        {
            var featureParent = properties.Feature.Parent as SPWeb;
            if (featureParent != null)
            {
                if (properties.Feature.Properties[FeatureDeactivationDependencyPropertyKey] != null)
                {
                    string[] features = properties.Feature.Properties[FeatureDeactivationDependencyPropertyKey].Value.Split(new[] { ',' });
                    foreach (string feature in features)
                    {
                        string[] featureProperties = feature.Split(new[] { '|' });
                        string featureScope;
                        Guid featureId;

                        try
                        {
                            featureId = new Guid(featureProperties[0].Trim());
                            featureScope = featureProperties[1].Trim();
                        }
                        catch
                        {
                            featureId = new Guid(featureProperties[1].Trim());
                            featureScope = featureProperties[0].Trim();
                        }

                        if (featureScope.Equals(Enum.GetName(typeof(SPFeatureScope), SPFeatureScope.Site)))
                        {
                            return featureParent.Site.Features[featureId] == null;
                        }
                    }
                }
            }

            return true;
        }

        /// <summary>
        /// Deletes the libraries.
        /// </summary>
        /// <param name="properties">The properties.</param>
        protected void DeleteLibraries(SPFeatureReceiverProperties properties)
        {
            SPWeb parent = GetFeatureWebParent(properties);
            if (properties.Feature.Properties[FeaturePipelineDelimitedLibrariesPropertyKey] != null)
            {
                string[] listNames = properties.Feature.Properties[FeaturePipelineDelimitedLibrariesPropertyKey].Value.Split(new[] { '|' });
                foreach (string listName in listNames)
                {
                    try
                    {
                        SPList list = parent.Lists[listName.TrimEnd().TrimStart()];
                        if (list != null)
                        {
                            list.Delete();
                        }
                    }
                    catch
                    {
                    }
                }

                parent.Update();
            }
        }

        /// <summary>
        /// Overwrites the files if exists.
        /// </summary>
        /// <param name="properties">The properties.</param>
        protected void OverwriteFilesIfExists(SPFeatureReceiverProperties properties)
        {
            SPWeb featureParent = GetFeatureWebParent(properties);
            if (featureParent != null)
            {
                if (properties.Feature.Properties[FilesToOverwriteIfExistsPipelineDelimitedFolderAndPathPropertyKey] != null)
                {
                    string[] foldersAndPaths = properties.Feature.Properties[FilesToOverwriteIfExistsPipelineDelimitedFolderAndPathPropertyKey].Value.Split(new[] { '|' });
                    foreach (string folderAndPath in foldersAndPaths)
                    {
                        string[] fp = folderAndPath.Split(new[] { ',' });
                        string folder = fp[0];
                        string path = fp[1];

                        var directory = new DirectoryInfo(properties.Feature.Definition.RootDirectory + @"\" + folder);
                        if (directory.Exists)
                        {
                            foreach (FileInfo file in directory.GetFiles("*.*", SearchOption.AllDirectories))
                            {
                                byte[] contents = File.ReadAllBytes(file.DirectoryName + @"\" + file.Name);
                                string fileUrl = path + @"/" + file.Name;
                                SPFile spFile = featureParent.GetFile(fileUrl);
                                if (spFile.Exists)
                                {
                                    if (spFile.CheckOutType == SPFile.SPCheckOutType.None)
                                    {
                                        spFile.CheckOut();
                                    }

                                    spFile = featureParent.Files.Add(fileUrl, contents, true);
                                    spFile.CheckIn(string.Empty, SPCheckinType.MajorCheckIn);
                                    spFile.Publish(string.Empty);
                                    spFile.Update();
                                }
                            }
                        }
                    }

                    featureParent.Update();
                }
            }
        }

        /// <summary>
        /// Adds the items to list.
        /// </summary>
        /// <param name="properties">The properties.</param>
        protected void AddItemsToList(SPFeatureReceiverProperties properties)
        {
            SPWeb featureParent = GetFeatureWebParent(properties);
            if (properties.Feature.Properties[PipelineDelimitedListItemsFileNamesPropertyKey] != null)
            {
                var fileNames = properties.Feature.Properties[PipelineDelimitedListItemsFileNamesPropertyKey].Value.Split(new[] { '|' });
                foreach (string fileName in fileNames)
                {
                    string filePath = properties.Feature.Definition.RootDirectory + @"\" + fileName.Trim();
                    if (File.Exists(filePath))
                    {
                        var xmlDocument = new XmlDocument();
                        xmlDocument.Load(filePath);
                        if (xmlDocument.DocumentElement != null)
                        {
                            XmlNodeList listNodes = xmlDocument.SelectNodes("List");
                            if (listNodes == null || listNodes.Count == 0)
                            {
                                return;
                            }

                            foreach (XmlNode listNode in listNodes)
                            {
                                string listName = listNode.Attributes[ListNameAttributeName].Value ?? string.Empty;
                                if (!string.IsNullOrEmpty((listName)))
                                {
                                    try
                                    {
                                        SPList list = featureParent.Lists[listName];
                                        XmlNodeList rows = listNode.SelectNodes(RowsXPath);
                                        if (rows == null || rows.Count == 0)
                                        {
                                            return;
                                        }

                                        foreach (XmlElement row in rows)
                                        {
                                            SPListItem listItem = list.Items.Add();
                                            foreach (XmlElement field in row.ChildNodes)
                                            {
                                                string fieldName = field.GetAttribute(NameAttributeName);
                                                string fieldValue = field.GetAttribute(ValueAttributeName);
                                                listItem[fieldName] = fieldValue;
                                            }

                                            listItem.Update();
                                        }
                                    }
                                    catch
                                    {
                                        string failureMessage = "Items can't be added to the list '{0}'";
                                        throw new SPException(string.Format(CultureInfo.InvariantCulture, failureMessage, listName));
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }

        /// <summary>
        /// Deletes the items from list.
        /// </summary>
        /// <param name="properties">The properties.</param>
        protected void DeleteItemsFromList(SPFeatureReceiverProperties properties)
        {
            SPWeb featureParent = GetFeatureWebParent(properties);
            if (properties.Feature.Properties[PipelineDelimitedListItemsFileNamesPropertyKey] != null)
            {
                var fileNames = properties.Feature.Properties[PipelineDelimitedListItemsFileNamesPropertyKey].Value.Split(new[] { '|' });
                foreach (string fileName in fileNames)
                {
                    string filePath = properties.Feature.Definition.RootDirectory + @"\" + fileName.Trim();
                    if (File.Exists(filePath))
                    {
                        var xmlDocument = new XmlDocument();
                        xmlDocument.Load(filePath);
                        if (xmlDocument.DocumentElement != null)
                        {
                            XmlNodeList listNodes = xmlDocument.SelectNodes("List");
                            if (listNodes == null || listNodes.Count == 0)
                            {
                                return;
                            }

                            foreach (XmlNode listNode in listNodes)
                            {
                                string listName = listNode.Attributes[ListNameAttributeName].Value ?? string.Empty;
                                string fieldNameToDeleteItems = listNode.Attributes[ListFieldNameToDeleteItemsAttributeName].Value ?? string.Empty;
                                if (!string.IsNullOrEmpty(listName) && !string.IsNullOrEmpty(fieldNameToDeleteItems))
                                {
                                    try
                                    {
                                        SPList list = featureParent.Lists[listName];
                                        SPListItemCollection listItems = list.Items;
                                        XmlNodeList rows = listNode.SelectNodes(RowsXPath);
                                        if (rows == null || rows.Count == 0)
                                        {
                                            return;
                                        }

                                        foreach (XmlElement row in rows)
                                        {
                                            string itemTitle = string.Empty;
                                            foreach (XmlElement field in row.ChildNodes)
                                            {
                                                string fieldName = field.GetAttribute(NameAttributeName);
                                                if (fieldName == fieldNameToDeleteItems)
                                                {
                                                    itemTitle = field.GetAttribute(ValueAttributeName);
                                                    break;
                                                }
                                            }

                                            if (!string.IsNullOrEmpty(itemTitle))
                                            {
                                                for (int i = 0; i < listItems.Count; i++)
                                                {
                                                    SPListItem listItem = listItems[i];
                                                    if (itemTitle == listItem[fieldNameToDeleteItems].ToString())
                                                    {
                                                        listItems.Delete(i);
                                                        break;
                                                    }
                                                }
                                            }
                                        }

                                        list.Update();
                                    }
                                    catch
                                    {
                                        string failureMessage = "Items can't be deleted from the list '{0}'";
                                        throw new SPException(string.Format(CultureInfo.InvariantCulture, failureMessage, listName));
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }

        #endregion

        #region Private Methods

        /// <summary>
        /// Update the web.config of each website.
        /// </summary>
        /// <remarks>See http://blogs.msdn.com/b/malag/archive/2009/05/22/spwebconfigmodification-does-not-work-on-farms-with-multiple-wfes.aspx for information on SPWebConfigUpdates on WFEs</remarks>
        /// <param name="properties">The properties of the calling SPFeatureReceiver</param>
        private static void UpdateWebConfig(SPFeatureReceiverProperties properties)
        {
            SPWebApplication webApplication = properties.Feature.Parent as SPWebApplication;

            if (webApplication != null)
            {
                string featureWebconfigPhysicalPath = Path.Combine(properties.Feature.Definition.RootDirectory, "webconfigentries");

                if (Directory.Exists(featureWebconfigPhysicalPath))
                {
                    foreach (string file in Directory.GetFiles(featureWebconfigPhysicalPath, "*.*", SearchOption.TopDirectoryOnly))
                    {
                        XmlDocument xml = new XmlDocument();
                        xml.Load(file);

                        XmlNodeList nodelistWebConfigUpdates = xml.SelectNodes("/webconfigUpdates/webconfigUpdate");

                        if (nodelistWebConfigUpdates != null)
                        {
                            foreach (XmlNode nodeWebConfigUpdate in nodelistWebConfigUpdates)
                            {
                                // Potentially check version here...
                                if (nodeWebConfigUpdate.Attributes["version"] == null || nodeWebConfigUpdate.Attributes["version"].Value.Equals(SPMajorVersion()))
                                {
                                    SPWebConfigModification webConfigModification = new SPWebConfigModification
                                    {
                                        Path = FeatureHelper.ReplaceTokens(nodeWebConfigUpdate.Attributes["path"].Value.Trim(), properties.Feature.Properties),
                                        Type = (SPWebConfigModification.SPWebConfigModificationType)Enum.Parse(typeof(SPWebConfigModification.SPWebConfigModificationType), nodeWebConfigUpdate.Attributes["type"].Value),
                                        Name = FeatureHelper.ReplaceTokens(nodeWebConfigUpdate.Attributes["name"].Value.Trim(), properties.Feature.Properties),
                                        Owner = properties.Definition.DisplayName,
                                        Sequence = uint.Parse(nodeWebConfigUpdate.Attributes["sequence"].Value, CultureInfo.InvariantCulture)
                                    };

                                    if (String.IsNullOrEmpty(nodeWebConfigUpdate.InnerXml) && String.IsNullOrEmpty(nodeWebConfigUpdate.InnerText))
                                    {
                                        webConfigModification.Value = "<" + FeatureHelper.ReplaceTokens(nodeWebConfigUpdate.Attributes["name"].Value.Trim(), properties.Feature.Properties) + "/>";
                                    }
                                    else if (!String.IsNullOrEmpty(nodeWebConfigUpdate.InnerXml))
                                    {
                                        webConfigModification.Value = FeatureHelper.ReplaceTokens(nodeWebConfigUpdate.InnerXml.Trim(), properties.Feature.Properties);
                                    }
                                    else if (!String.IsNullOrEmpty(nodeWebConfigUpdate.InnerText))
                                    {
                                        webConfigModification.Value = FeatureHelper.ReplaceTokens(nodeWebConfigUpdate.InnerText.Trim(), properties.Feature.Properties);
                                    }

                                    SPWebService.ContentService.WebApplications[webApplication.Id].WebConfigModifications.Add(webConfigModification);
                                }
                            }
                        }
                    }

                    SPWebService.ContentService.WebApplications[webApplication.Id].Update();
                    SPWebService.ContentService.WebApplications[webApplication.Id].WebService.ApplyWebConfigModifications();

                    if (SPWebService.ContentService.WebApplications[webApplication.Id].Farm.TimerService.Instances.Count > 1)
                    {
                        // When there are multiple front-end Web servers in the SharePoint farm, we need to wait for the timer job that performs the Web.config modifications to complete before continuing.
                        // Otherwise, we may encounter the following error
                        // (e.g. when applying Web.config changes from two different features in rapid succession):
                        // 
                        // "A web configuration modification operation is already running."
                        TimerJobHelper.WaitForOnetimeJobToFinish(SPWebService.ContentService.WebApplications[webApplication.Id].Farm, "Windows SharePoint Services Web.Config Update", 60);
                    }
                }
            }
        }

        /// <summary>
        /// Gets the feature parent.
        /// </summary>
        /// <param name="properties">The properties.</param>
        /// <returns>The SPWeb feature's parent</returns>
        private static SPWeb GetFeatureWebParent(SPFeatureReceiverProperties properties)
        {
            var featureParent = properties.Feature.Parent;
            if (featureParent.GetType().Equals(typeof(SPSite)))
            {
                var parentSite = properties.Feature.Parent as SPSite;
                if (parentSite != null)
                {
                    return parentSite.RootWeb;
                }
            }
            else if (featureParent.GetType().Equals(typeof(SPWeb)))
            {
                return properties.Feature.Parent as SPWeb;
            }

            return null;
        }

        /// <summary>
        /// Gets the pipeline property to use.
        /// </summary>
        /// <param name="pipelineDelimitedPropetyName">Name of the pipeline delimited propety.</param>
        /// <returns>The name of the property to be use</returns>
        private static string GetPipelinePropertyToUse(string pipelineDelimitedPropetyName)
        {
            if (pipelineDelimitedPropetyName.Equals(FeatureDeactivatingPipelineDelimitedElementsManifestPropertyKey))
            {
                return FeatureDeactivatingPipelineDelimitedElementsManifestPropertyKey;
            }

            if (pipelineDelimitedPropetyName.Equals(FeatureActivationDependencyPipelineDelimitedElementsManifestPropertyKey))
            {
                return FeatureActivationDependencyPipelineDelimitedElementsManifestPropertyKey;
            }

            return string.Empty;
        }

        /// <summary>
        /// Adds the nodes.
        /// </summary>
        /// <param name="existingNodes">The existing nodes.</param>
        /// <param name="newNodes">The new nodes.</param>
        private static void AddNodes(SPNavigationNodeCollection existingNodes, XmlNodeList newNodes)
        {
            if (newNodes.Count == 0)
            {
                return;
            }

            for (int i = 0; i < newNodes.Count; i++)
            {
                var newNodeXml = (XmlElement)newNodes[i];
                string url = newNodeXml.GetAttribute("Url");
                string title = newNodeXml.GetAttribute("Title");
                NodeTypes type = NodeTypes.None;
                if (newNodeXml.GetAttribute("NodeType") != null && !string.IsNullOrEmpty(newNodeXml.GetAttribute("NodeType")))
                {
                    type = (NodeTypes)Enum.Parse(typeof(NodeTypes), newNodeXml.GetAttribute("NodeType"));
                }

                SPNavigationNode node = SPNavigationSiteMapNode.CreateSPNavigationNode(title, url, type, existingNodes);
                if (node.Properties["CreatedDate"] == null)
                {
                    node.Properties["CreatedDate"] = DateTime.Now;
                }

                if (node.Properties["LastModifiedDate"] == null)
                {
                    node.Properties["LastModifiedDate"] = DateTime.Now;
                }

                node.Update();
                node.MoveToLast(existingNodes);

                XmlNodeList childNodes = newNodeXml.SelectNodes("Node");
                if (childNodes != null && childNodes.Count > 0)
                {
                    AddNodes(node.Children, childNodes);
                }
            }
        }

        /// <summary>
        /// Deletes the parent folders.
        /// </summary>
        /// <param name="parentFolder">The folder.</param>
        private static void DeleteParentFolders(SPFolder parentFolder)
        {
            SPFolder grandParentFolder = parentFolder.ParentFolder;
            if (parentFolder.SubFolders.Count == 0)
            {
                parentFolder.Delete();
            }

            if (grandParentFolder.SubFolders.Count == 0)
            {
                DeleteParentFolders(grandParentFolder);
            }
        }

        #endregion
    }
}
