﻿//----------------------------------------------------------------------------------------------------------------------------
// <copyright file="SharePointHelper.cs" company="commerceserver.net">
//     Copyright © commerceserver.net.  All rights reserved.
// </copyright>
// <summary>SharePointHelper class</summary>
//-----------------------------------------------------------------------------------------------------------------------------

namespace CommerceServerContrib.Web.SharePoint
{
    using System;
    using System.Globalization;
    using System.IO;
    using System.Web;
    using System.Security.Principal;
    using System.Threading;
    using CommerceServer.Foundation;
    using Microsoft.IdentityModel.Claims;
    using Microsoft.SharePoint;
    using Microsoft.SharePoint.Administration;
    using Microsoft.SharePoint.Publishing;
    using Microsoft.SharePoint.WebControls;

    /// <summary>
    /// A helper class for sharepoint related operations
    /// </summary>
    public static class SharePointHelper
    {
        private const string SharePointHelperCache = "SharePointHelperCache"; 
        private const string FilePathKey = "FilePath_";
        private const string ClaimsProviderType = "http://schemas.microsoft.com/sharepoint/2009/08/claims/identityprovider";
        private const string WindowsClaimsType = "windows";
        private static readonly object SpListLock = new object();
        private static readonly object TopLevelSiteCacheLock = new object();
        private static readonly object LibraryDocumentCacheLock = new object();
        private static readonly object FilePathCacheLock = new object();

        #region Properties

        /// <summary>
        /// Gets a value indicating whether this instance is WSS installed.
        /// </summary>
        /// <value><c>true</c> if this instance is WSS installed; otherwise, <c>false</c>.</value>
        public static bool IsWssInstalled
        {
            get
            {
                return (!IsMossInstalled);
            }
        }

        /// <summary>
        /// Gets a value indicating whether this instance is MOSS installed.
        /// </summary>
        /// <value><c>true</c> if this instance is MOSS installed; otherwise, <c>false</c>.</value>
        public static bool IsMossInstalled
        {
            get
            {                
                return (SPFarm.Local.FeatureDefinitions[new Guid("F6924D36-2FA8-4f0b-B16D-06B7250180FA")] != null);
            }
        }

        /// <summary>
        /// Gets a value indicating whether [should run elevated].
        /// </summary>
        /// <value>
        ///   <c>true</c> if [should run elevated]; otherwise, <c>false</c>.
        /// </value>
        public static bool ShouldRunElevated
        {
            get
            {
                //// Returns true when current identity is a
                //// WindowsIdentity (NTLM zone)

                IIdentity currentIdentity = null;
                if (HttpContext.Current != null &&
                    HttpContext.Current.User != null &&
                    HttpContext.Current.User.Identity != null)
                {
                    currentIdentity = HttpContext.Current.User.Identity;
                }
                else if (Thread.CurrentPrincipal != null &&
                         Thread.CurrentPrincipal.Identity != null)
                {
                    currentIdentity = Thread.CurrentPrincipal.Identity;
                }

                if (currentIdentity is WindowsIdentity)
                {
                    return true;
                }

                if (currentIdentity is ClaimsIdentity)
                {
                    foreach (Claim claim in ((ClaimsIdentity)currentIdentity).Claims)
                    {
                        if (claim.ClaimType == ClaimsProviderType)
                        {
                            if (claim.Value == WindowsClaimsType)
                            {
                                return true;
                            }

                            break;
                        }
                    }
                }

                return false;
            }
        }

        /// <summary>
        /// Gets the cache context.
        /// </summary>
        private static CacheContext CacheContext
        {
            get
            {
                return PresentationCacheSettings.GetCacheContext(SharePointHelperCache);
            }
        }

        /// <summary>
        /// Gets the cache provider.
        /// </summary>
        private static ICacheProvider CacheProvider
        {
            get
            {
                return PresentationTypeLoader.CacheProvider(CacheContext);
            }
        }

        /// <summary>
        /// Gets the brand.
        /// </summary>
        private static string Brand
        {
            get
            {
                return PresentationServiceLocator.Current.BrandContext.Name;
            }
        }

        #endregion

        #region Public Methods
        /// <summary>
        /// Loads the file from a SharePoint library based on the passed path.
        /// </summary>
        /// <param name="web">The SPWeb to use in order to get the library.</param>
        /// <param name="path">The path to the requested file.</param>
        /// <returns>String representation of the file.</returns>
        public static string LoadFile(SPWeb web, string path)
        {
            ParameterChecker.CheckForNull(web, "SPWeb");
            ParameterChecker.CheckForNullOrEmpty(path, "Path");

            bool fileWasNotFound;
            try
            {
                SPFile file = web.GetFile(path);
                fileWasNotFound = !file.Exists;
                if (file.Exists)
                {
                    string content = web.GetFileAsString(path);
                    return content;
                }
            }
            catch (Exception)
            {
                fileWasNotFound = true;
            }

            if (!web.IsRootWeb && fileWasNotFound)
            {
                return LoadFile(web.ParentWeb, path);
            }

            return null;
        }

        /// <summary>
        /// Indicates if the files the exists.
        /// </summary>
        /// <param name="web">The SharePoint web.</param>
        /// <param name="path">The path to the file.</param>
        /// <returns>true if the file exists, otherwise false.</returns>
        public static bool FileExists(SPWeb web, string path)
        {
            bool fileExists;
            try
            {
                SPFile file = web.GetFile(path);
                fileExists = file.Exists;
            }
            catch (Exception)
            {
                fileExists = false;
            }

            return fileExists;
        }

        /// <summary>
        /// Gets a value indicating whether this site does not have MOSS features activated
        /// </summary>
        /// <param name="site">The SharePoint current Site, in which the feature is being activated.</param>
        /// <returns>
        /// 	<c>true</c> if [is WSS activated] [the specified site]; otherwise, <c>false</c>.
        /// </returns>
        /// <value><c>true</c> if this instance does not have MOSS features activated; otherwise, <c>false</c>.</value>
        public static bool IsWssActivated(SPSite site)
        {
            return !IsMossActivated(site);
        }

        /// <summary>
        /// Gets a value indicating whether this site has MOSS features activated
        /// </summary>
        /// <param name="site">The current SharePoint Site, in which this feature is being activated.</param>
        /// <returns>
        /// 	<c>true</c> if [is moss activated] [the specified site]; otherwise, <c>false</c>.
        /// </returns>
        /// <value><c>true</c> if this instance is MOSS activated; otherwise, <c>false</c>.</value>
        public static bool IsMossActivated(SPSite site)
        {
            try
            {
                SPFeature feature = site.Features[new Guid("F6924D36-2FA8-4f0b-B16D-06B7250180FA")];
                return feature != null;
            }
            catch
            {
                return false;
            }
        }

        /// <summary>
        /// Gets the SharePoint list.
        /// </summary>
        /// <param name="web">The current web.</param>
        /// <param name="listPath">The list path.</param>
        /// <returns>The SharePoint List.</returns>
        public static SPList GetSPList(SPWeb web, string listPath)
        {
            string cacheKey = string.Concat("GetSpList", listPath);
            SPList list = null;

            if (CacheContext != null && CacheContext.IsCachingEnabled)
            {
                list = CacheProvider.GetData<SPList>(Brand, CacheContext.CacheName, cacheKey);

                if (list == null)
                {
                    lock (SpListLock)
                    {
                        list = CacheProvider.GetData<SPList>(Brand, CacheContext.CacheName, cacheKey);
                        if (list == null)
                        {
                            SPSecurity.RunWithElevatedPrivileges(delegate
                            {
                                list = web.GetListFromUrl(listPath);
                            });

                            CacheProvider.AddData(Brand, CacheContext.CacheName, cacheKey, list);
                        }
                    }
                } 
            }
            else
            {
                SPSecurity.RunWithElevatedPrivileges(delegate
                {
                    list = web.GetListFromUrl(listPath);
                });                    
            }

            return list;
        }

        /// <summary>
        /// Gets the file in string format.
        /// </summary>
        /// <param name="path">The path to the file</param>
        /// <returns>
        /// The requested file in string form
        /// </returns>
        public static string GetFileAsString(string path)
        {
            var fileContents = string.Empty;
            CacheContext cacheContext = PresentationCacheSettings.GetCacheContext(CommerceServerContrib.CrossTier.Helpers.CacheConstants.DiskFileCache);

            if (cacheContext != null && cacheContext.IsCachingEnabled)
            {
                ICacheProvider cacheProvider = PresentationTypeLoader.CacheProvider(cacheContext);

                fileContents = cacheProvider.GetData<string>(Brand, cacheContext.CacheName, path);

                if (fileContents == null)
                {
                    lock (LibraryDocumentCacheLock)
                    {
                        fileContents = cacheProvider.GetData<string>(Brand, cacheContext.CacheName, path);

                        if (fileContents == null)
                        {
                            fileContents = RetrieveFromDocumentLibrary(path);

                            cacheProvider.AddData<string>(Brand, cacheContext.CacheName, path, fileContents);
                        }
                    }
                }
            }
            else
            {
                fileContents = RetrieveFromDocumentLibrary(path);
            }

            if (string.IsNullOrEmpty(fileContents))
            {
                throw new FileNotFoundException(string.Format(CultureInfo.CurrentCulture, ClientFrameworkSharePointResources.FileNotFound, path));
            }

            return fileContents;
        }

        /// <summary>
        /// Finds the top variant site.
        /// </summary>
        /// <param name="web">The web who top level variant or root site needs to be traversed. Not using OpenWeb to avoid performance hit.</param>
        /// <returns>The top level Variant site.</returns>
        public static string FindTopVariantSite(SPWeb web)
        {
            string cacheKey = string.Concat(web.ID, "-", CultureInfo.CurrentCulture.Name);

            var returnPath = string.Empty;

            if (CacheContext != null && CacheContext.IsCachingEnabled)
            {
                returnPath = CacheProvider.GetData<string>(Brand, CacheContext.CacheName, cacheKey);

                if (returnPath == null)
                {
                    lock (TopLevelSiteCacheLock)
                    {
                        returnPath = CacheProvider.GetData<string>(Brand, CacheContext.CacheName, cacheKey);

                        if (returnPath == null)
                        {
                            returnPath = FindTopLevelVariantSiteRecursive(web);

                            CacheProvider.AddData(Brand, CacheContext.CacheName, cacheKey, returnPath);
                        }
                    }
                } 
            }
            else
            {
                returnPath = FindTopLevelVariantSiteRecursive(web);
            }

            return returnPath;
        }

        /// <summary>
        /// Retrieves from a SharePoint doc library.
        /// </summary>
        /// <param name="fullPath">The full path.</param>
        /// <returns>A compiled version of the xslt</returns>
        private static string RetrieveFromDocumentLibrary(string fullPath)
        {
            if (!string.IsNullOrEmpty(fullPath))
            {
                SPWeb currentWeb = SPContext.Current.Web;

                var fileContents = string.Empty;

                SPSecurity.RunWithElevatedPrivileges(delegate
                {
                    using (var site = new SPSite(currentWeb.Site.ID))
                    {
                        SPWeb web = site.OpenWeb(currentWeb.ID);
                        fileContents = SharePointHelper.LoadFile(web, fullPath);
                    }
                });

                return fileContents;
            }

            return null;
        }

        /// <summary>
        /// Finds the top level variant site recursively.
        /// </summary>
        /// <param name="web">The web to recurse up from.</param>
        /// <returns>The path to the top level site</returns>
        private static string FindTopLevelVariantSiteRecursive(SPWeb web)
        {
            if (web.ParentWeb == null)
            {
                return web.Url;
            }

            if (web.ParentWeb.ID == web.Site.RootWeb.ID)
            {
                if (IsMossActivated(web.Site))
                {
                    if (PublishingWeb.GetPublishingWeb(web).VariationPublishingWebUrls.Count > 0)
                    {
                        return web.Url;
                    }
                }

                return web.ParentWeb.Url;
            }

            return FindTopLevelVariantSiteRecursive(web.ParentWeb);
        }

        #endregion
    }
}
