﻿//-----------------------------------------------------------------------
// <copyright file="SharePointPageProperties.cs" company="commerceserver.net">
//     Copyright © commerceserver.net.  All rights reserved.
// </copyright>
// <summary>The MasterPages class implementation.</summary>
//-----------------------------------------------------------------------

namespace CommerceServerContrib.Web.SharePoint
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Text;
    using Microsoft.SharePoint;
    using System.Globalization;
    using CommerceServer.Foundation;

    /// <summary>
    /// Class used to hold the SP page properties that require caching
    /// </summary>
    public class SharePointPageProperties
    {
        private static object _cacheLock = new object();

        private bool _isAllowedToIndex = false;
        private bool _isAllowedToFollow = false;
        private DateTime? _lastModifiedDate = null;
        private string _changeFrequency;
        private decimal? _crawlPriority = null;

        #region Constructor

        /// <summary>
        /// Initializes a new instance of the <see cref="SharePointPageProperties"/> class.
        /// </summary>
        /// <param name="webPath">The web path.</param>
        private SharePointPageProperties(string webPath)
        {
        }

        #endregion

        #region Properties (public, non-static)

        /// <summary>
        /// Gets the meta keywords.
        /// </summary>
        public string MetaKeywords { get; private set; }

        /// <summary>
        /// Gets the meta description.
        /// </summary>
        public string MetaDescription { get; private set; }

        /// <summary>
        /// Gets the page title.
        /// </summary>
        public string PageTitle { get; private set; }

        /// <summary>
        /// Gets a value indicating whether can be indexed by search engines.
        /// </summary>
        public bool IsAllowedToIndex
        {
            get { return this._isAllowedToIndex; }
        }

        /// <summary>
        /// Gets a value indicating whether search engines can follow this page.
        /// </summary>
        public bool IsAllowedToFollow
        {
            get { return this._isAllowedToFollow; }
        }

        /// <summary>
        /// Gets the last modified date.
        /// </summary>
        public DateTime? LastModifiedDate
        {
            get { return this._lastModifiedDate; }
        }

        /// <summary>
        /// Gets the page change frequency.
        /// </summary>
        public string ChangeFrequency
        {
            get { return this._changeFrequency; }
        }

        /// <summary>
        /// Gets the craw priority.
        /// </summary>
        public decimal? CrawPriority
        {
            get { return this._crawlPriority; }
        }

        #endregion

        #region Methods (static, public)

        /// <summary>
        /// Gets the SharePointPageProperties instance.
        /// </summary>
        /// <param name="webPath">The web path.</param>
        /// <returns>The SharePointPageProperties instance.</returns>
        public static SharePointPageProperties Get(string webPath)
        {
            SharePointPageProperties returnValue = null;

            CacheContext cacheContext = PresentationCacheSettings.GetCacheContext(CommerceServerContrib.CrossTier.Helpers.CacheConstants.SharePointPagePropertiesCache);
            if (cacheContext != null && cacheContext.IsCachingEnabled)
            {
                ICacheProvider cacheProvider = PresentationTypeLoader.CacheProvider(cacheContext);
                var brandName = PresentationServiceLocator.Current.BrandContext.Name;

                returnValue = cacheProvider.GetData<SharePointPageProperties>(brandName, cacheContext.CacheName, webPath);
                if (returnValue == null)
                {
                    lock (_cacheLock)
                    {
                        returnValue = cacheProvider.GetData<SharePointPageProperties>(brandName, cacheContext.CacheName, webPath);
                        if (returnValue == null)
                        {
                            returnValue = new SharePointPageProperties(webPath);
                            returnValue.RetrieveSharePointFileProperties(webPath);

                            cacheProvider.AddData<SharePointPageProperties>(brandName, cacheContext.CacheName, webPath, returnValue);
                        }
                    }
                }
            }
            else
            {
                returnValue = new SharePointPageProperties(webPath);
                returnValue.RetrieveSharePointFileProperties(webPath);
            }

            return returnValue;
        }

        #endregion

        #region Methods (protected, virtual)

        /// <summary>
        /// Retrieves the sharepoint file properties.
        /// </summary>
        /// <param name="webPath">The web path.</param>
        protected virtual void RetrieveSharePointFileProperties(string webPath)
        {
            SPFile file = SPContext.Current.Web.GetFile(webPath);

            if (file.Properties.Contains(SharePointConstants.ExtendedPageProperties.AllowMetaRobotsToIndex))
            {
                this._isAllowedToIndex = Convert.ToBoolean(file.Properties[SharePointConstants.ExtendedPageProperties.AllowMetaRobotsToIndex], CultureInfo.InvariantCulture);
            }

            if (file.Properties.Contains(SharePointConstants.ExtendedPageProperties.AllowMetaRobotsToFollow))
            {
                this._isAllowedToFollow = Convert.ToBoolean(file.Properties[SharePointConstants.ExtendedPageProperties.AllowMetaRobotsToFollow], CultureInfo.InvariantCulture);
            }

            if (file.Properties.Contains(SharePointConstants.ExtendedPageProperties.LastDateModified))
            {
                this._lastModifiedDate = Convert.ToDateTime(file.Properties[SharePointConstants.ExtendedPageProperties.LastDateModified], CultureInfo.InvariantCulture);
            }

            if (file.Properties.Contains(SharePointConstants.ExtendedPageProperties.ChangeFrequency))
            {
                this._changeFrequency = file.Properties[SharePointConstants.ExtendedPageProperties.ChangeFrequency].ToString();
            }

            if (file.Properties.Contains(SharePointConstants.ExtendedPageProperties.CrawlPriority) && !String.IsNullOrEmpty(file.Properties[SharePointConstants.ExtendedPageProperties.CrawlPriority].ToString()))
            {
                this._crawlPriority = Convert.ToDecimal(file.Properties[SharePointConstants.ExtendedPageProperties.CrawlPriority], CultureInfo.InvariantCulture);
            }

            if (file.Properties.Contains(SharePointConstants.ExtendedPageProperties.MetaDescription))
            {
                this.MetaDescription = file.Properties[SharePointConstants.ExtendedPageProperties.MetaDescription] as string;
            }

            if (file.Properties.Contains(SharePointConstants.ExtendedPageProperties.MetaKeywords))
            {
                this.MetaKeywords = file.Properties[SharePointConstants.ExtendedPageProperties.MetaKeywords] as string;
            }

            if (file.Properties.Contains(SharePointConstants.ExtendedPageProperties.PageTitle))
            {
                this.PageTitle = file.Properties[SharePointConstants.ExtendedPageProperties.PageTitle] as string;
            }
        }

        #endregion
    }
}
