﻿//---------------------------------------------------------------------
// <copyright file="TimerJobHelper.cs" company="commerceserver.net">
//     Copyright © commerceserver.net.  All rights reserved.
// </copyright>
// <summary></summary>
//---------------------------------------------------------------------
namespace CommerceServerContrib.Web.SharePoint
{
    using System;
    using System.Threading;
    using Microsoft.SharePoint.Administration;
    using System.Globalization;

    /// <summary>
    /// Exposes static methods for commonly used helper functions for SharePoint timer jobs. This class cannot be inherited.
    /// </summary>
    /// <remarks>
    /// All methods of the <b>SharePointTimerJobHelper</b> class are static and can therefore be called without creating an instance of the class.
    /// <br>http://blogs.msdn.com/b/jjameson/archive/2010/03/31/waiting-for-sharepoint-web-config-modifications-to-finish.aspx</br></remarks>
    public static class TimerJobHelper
    {
        /// <summary>
        /// Determines whether the specified timer job is currently running (or
        /// scheduled to run).
        /// </summary>
        /// <param name="farm">The farm to check if the job is running on.</param>
        /// <param name="jobTitle">The title of the timer job.</param>
        /// <returns><c>true</c> if the specified timer job is currently running
        /// (or scheduled to run); otherwise <c>false</c>.</returns>
        public static bool IsJobRunning(SPFarm farm, string jobTitle)
        {
            SPServiceCollection services = farm.Services;

            foreach (SPService service in services)
            {
                foreach (SPRunningJob job in service.RunningJobs)
                {
                    if (string.Compare(
                        job.JobDefinitionTitle,
                        jobTitle,
                        StringComparison.OrdinalIgnoreCase) == 0)
                    {
                        return true;
                    }
                }
            }

            return false;
        }

        /// <summary>
        /// Waits for a one-time SharePoint timer job to finish.
        /// </summary>
        /// <param name="farm">The farm on which the timer job runs.</param>
        /// <param name="jobTitle">The title of the timer job (e.g. "Windows
        /// SharePoint Services Web.Config Update").</param>
        /// <param name="maximumWaitTime">The maximum time (in seconds) to wait
        /// for the timer job to finish.</param>
        public static void WaitForOnetimeJobToFinish(SPFarm farm, string jobTitle, byte maximumWaitTime)
        {
            if (farm == null)
            {
                throw new ArgumentNullException("farm");
            }

            if (jobTitle == null)
            {
                throw new ArgumentNullException("jobTitle");
            }

            if (string.IsNullOrEmpty(jobTitle) == true)
            {
                throw new ArgumentException("The job title must be specified.", "jobTitle");
            }

            float waitTime = 0;

            do
            {
                bool isJobDefined = IsJobDefined(farm, jobTitle);

                if (isJobDefined == false)
                {
                    UlsTraceProvider.LogMessage(string.Format(CultureInfo.CurrentCulture, "The timer job ({0}) is not defined. It may have been removed because the job completed.", jobTitle), UlsTraceProvider.TraceSeverity.InformationEvent);
                    break;
                }

                bool isJobRunning = IsJobRunning(farm, jobTitle);

                UlsTraceProvider.LogMessage(string.Format(CultureInfo.CurrentCulture, "The timer job ({0}) is currently {1}. Waiting for the job to finish...", jobTitle, isJobRunning == true ? "running" : "idle"), UlsTraceProvider.TraceSeverity.InformationEvent);

                const int SleepTime = 500; // milliseconds

                Thread.Sleep(SleepTime);
                waitTime += (SleepTime / 1000.0F); // seconds
            }
            while (waitTime < maximumWaitTime);

            if (waitTime >= maximumWaitTime)
            {
                UlsTraceProvider.LogMessage(string.Format(CultureInfo.CurrentCulture, "Waited the maximum amount of time ({0} {1}) for the one-time job ({2}) to finish.", maximumWaitTime, maximumWaitTime == 1 ? "second" : "seconds", jobTitle), UlsTraceProvider.TraceSeverity.Unexpected);
            }
            else
            {
                UlsTraceProvider.LogMessage(string.Format(CultureInfo.CurrentCulture, "Waited {0} {1} for the one-time job ({2}) to finish.", waitTime, waitTime == 1 ? "second" : "seconds", jobTitle), UlsTraceProvider.TraceSeverity.InformationEvent);
            }
        }

        private static bool IsJobDefined(SPFarm farm, string jobTitle)
        {
            SPServiceCollection services = farm.Services;

            foreach (SPService service in services)
            {
                foreach (SPJobDefinition job in service.JobDefinitions)
                {
                    if (string.Compare(
                        job.Title,
                        jobTitle,
                        StringComparison.OrdinalIgnoreCase) == 0)
                    {
                        return true;
                    }
                }
            }

            return false;
        }
    }
}
