﻿//-------------------------------------------------------------------------------------------------
// <copyright file="UlsTraceProvider.cs" company="commerceserver.net">
// Copyright © commerceserver.net.  All rights reserved.
// </copyright>
// <summary>A ULS Trace Provider which can be used in both SharePoint Foundation and SharePoint Server</summary>
//-------------------------------------------------------------------------------------------------
namespace CommerceServerContrib.Web.SharePoint
{
    using System;
    using System.Reflection;
    using System.Runtime.InteropServices;
    using Microsoft.SharePoint.Administration;

    /// <summary>
    /// A trace provider for writing to the ULS log in both SharePoint Foundation and SharePoint Server.  
    /// Adapted from the WSS SDK and Community Kit for SharePoint.
    /// </summary>
    [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1053:StaticHolderTypesShouldNotHaveConstructors", Justification = "Adapted from the WSS SDK and the CKS."),
    System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1704:IdentifiersShouldBeSpelledCorrectly", MessageId = "Uls", Justification = "Adapted from the WSS SDK and the CKS.")]
    public class UlsTraceProvider
    {
        #region
        private static ulong traceLog;
        private static ulong traceReg;

        // Following counter is to prevent errors from  multiple registrations followed by
        // multiple un-registrations.  Only the first registration and last unregistration
        // are performed.  Any intermediate (un)registrations are just counted, for this purpose.
        // This simple implementation assumes identical registrations (by controlGuid parameter).
        // If this assumption is wrong, this would need a stack of GUIDs instead of a counter.
        private static int numRegistrations = 0;
        #endregion

        /// <summary>
        /// An Enumeration of possible serverity levels for a message when logging to the ULS.
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1034:NestedTypesShouldNotBeVisible", Justification = "Adapted from the WSS SDK and CKS.")]
        public enum TraceSeverity
        {
            /// <summary>
            /// Unassigned value
            /// </summary>
            Unassigned = 0,

            /// <summary>
            /// CriticalEvent value
            /// </summary>
            CriticalEvent = 1,

            /// <summary>
            /// WarningEvent value
            /// </summary>
            WarningEvent = 2,

            /// <summary>
            /// InformationEvent value
            /// </summary>
            InformationEvent = 3,

            /// <summary>
            /// Exception value
            /// </summary>
            Exception = 4,

            /// <summary>
            /// Assert value
            /// </summary>
            Assert = 7,

            /// <summary>
            /// Unexpected value
            /// </summary>
            Unexpected = 10,

            /// <summary>
            /// Monitorable value
            /// </summary>
            Monitorable = 15,

            /// <summary>
            /// High value
            /// </summary>
            High = 20,

            /// <summary>
            /// Medium value
            /// </summary>
            Medium = 50,

            /// <summary>
            /// Verbose value
            /// </summary>
            Verbose = 100,
        }

        /// <summary>
        /// Writes the trace event to the ULS.
        /// </summary>
        /// <param name="tag">The tag used to mark the trace event.</param>
        /// <param name="level">The level of the trace event.</param>
        /// <param name="correlationValue">The correlation GUID.</param>
        /// <param name="exeName">Name of the exe.</param>
        /// <param name="productName">Name of the product.</param>
        /// <param name="categoryName">Name of the category.</param>
        /// <param name="message">The message for the trace event.</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Usage", "CA1806:DoNotIgnoreMethodResults", MessageId = "CommerceServerContrib.Web.SharePoint.UlsTraceProvider+NativeMethods.TraceEvent(System.UInt64,CommerceServerContrib.Web.SharePoint.UlsTraceProvider+NativeMethods+ULSTrace@)", Justification = "The result of the method is not required.")]
        public static void WriteTrace(uint tag, TraceSeverity level, Guid correlationValue, string exeName, string productName, string categoryName, string message)
        {
            const ushort SizeOfWCHAR = 2;
            NativeMethods.ULSTrace ulsTrace = new NativeMethods.ULSTrace();

            // Pretty standard code needed to make things work
            ulsTrace.Header.Size = (ushort)Marshal.SizeOf(typeof(NativeMethods.ULSTrace));
            ulsTrace.Header.Flags = NativeMethods.WNODE_FLAG_TRACED_GUID;
            ulsTrace.ULSHeader.dwVersion = NativeMethods.TRACE_VERSION_CURRENT;
            ulsTrace.ULSHeader.dwFlags = NativeMethods.TraceFlags.TRACE_FLAG_ID_AS_ASCII;
            ulsTrace.ULSHeader.Size = (ushort)Marshal.SizeOf(typeof(NativeMethods.ULSTraceHeader));

            // Variables communicated to SPTrace
            ulsTrace.ULSHeader.Id = tag;
            ulsTrace.Header.Class.Level = (byte)level;
            ulsTrace.ULSHeader.wzExeName = exeName;
            ulsTrace.ULSHeader.wzProduct = productName;
            ulsTrace.ULSHeader.wzCategory = categoryName;
            ulsTrace.ULSHeader.wzMessage = message;
            ulsTrace.ULSHeader.correlationID = correlationValue;

            // Optionally, to improve performance by reducing the amount of data copied around,
            // the Size parameters can be reduced by the amount of unused buffer in the Message
            if (message.Length < 800)
            {
                ushort unusedBuffer = (ushort)((800 - (message.Length + 1)) * SizeOfWCHAR);
                ulsTrace.Header.Size -= unusedBuffer;
                ulsTrace.ULSHeader.Size -= unusedBuffer;
            }

            if (traceLog != 0)
            {
                NativeMethods.TraceEvent(traceLog, ref ulsTrace);
            }
        }

        /// <summary>
        /// Registers the trace provider.
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Usage", "CA1806:DoNotIgnoreMethodResults", MessageId = "CommerceServerContrib.Web.SharePoint.UlsTraceProvider+NativeMethods.RegisterTraceGuids(CommerceServerContrib.Web.SharePoint.UlsTraceProvider+NativeMethods+EtwProc,System.Void*,System.Guid@,System.UInt32,System.IntPtr,System.String,System.String,System.UInt64@)", Justification = "Adapted from the WSS SDK")]
        public static unsafe void RegisterTraceProvider()
        {
            if (numRegistrations == 0)
            {
                SPFarm farm = SPFarm.Local;
                Guid traceGuid = farm.TraceSessionGuid;
                uint result = NativeMethods.RegisterTraceGuids(ControlCallback, null, ref traceGuid, 0, IntPtr.Zero, null, null, out traceReg);
                System.Diagnostics.Debug.Assert(result == NativeMethods.ERROR_SUCCESS, "Can't register a trace provider; click Ignore.  Result code " + result + ".  hTraceReg=" + traceReg);
            }

            numRegistrations++;
        }

        /// <summary>
        /// Unregisters the trace provider.
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Usage", "CA1806:DoNotIgnoreMethodResults", MessageId = "CommerceServerContrib.Web.SharePoint.UlsTraceProvider+NativeMethods.UnregisterTraceGuids(System.UInt64)", Justification = "Adapted from the WSS SDK")]
        public static void UnregisterTraceProvider()
        {
            if (numRegistrations == 1)
            {
                uint result = NativeMethods.UnregisterTraceGuids(traceReg);
                System.Diagnostics.Debug.Assert(result == NativeMethods.ERROR_SUCCESS, "Can't unregister the trace provider; click Ignore.  Result code " + result + ".  hTraceReg=" + traceReg);
            }

            if (numRegistrations > 0)
            {
                numRegistrations--;
            }
        }

        /// <summary>
        /// Tags from string.
        /// </summary>
        /// <param name="tag">The wz tag.</param>
        /// <returns>An unsigned int denoting tag size.</returns>
        public static uint TagFromString(string tag)
        {
            System.Diagnostics.Debug.Assert(tag.Length == 4, "Minimum tag length required.");
            return (uint)(tag[0] << 24 | tag[1] << 16 | tag[2] << 8 | tag[3]);
        }

        /// <summary>
        /// Logs the message.
        /// </summary>
        /// <param name="message">The message.</param>
        /// <param name="severity">The severity.</param>
        public static void LogMessage(string message, UlsTraceProvider.TraceSeverity severity)
        {
            Assembly thisAssembly = Assembly.GetExecutingAssembly();
            string exeName = thisAssembly.FullName;
            object[] productNames = thisAssembly.GetCustomAttributes(typeof(AssemblyProductAttribute), false);
            string productName = string.Empty;
            if (productNames != null && productNames.Length > 0)
            {
                productName = ((AssemblyProductAttribute)productNames[0]).Product;
            }

            Guid correlationGuid = new Guid();
            WriteTrace(0, severity, correlationGuid, exeName, productName, CategoryBySeverity(severity), message);
        }

        /// <summary>
        /// Controls the callback.
        /// </summary>
        /// <param name="requestCode">The request code.</param>
        /// <param name="context">The context.</param>
        /// <param name="inOutBufferSize">Size of the in out buffer.</param>
        /// <param name="buffer">The buffer.</param>
        /// <returns>An unsigned int denoting the status of the call.</returns>
        internal static unsafe uint ControlCallback(NativeMethods.WMIDPREQUESTCODE requestCode, IntPtr context, uint* inOutBufferSize, IntPtr buffer)
        {
            uint status;
            switch (requestCode)
            {
                case NativeMethods.WMIDPREQUESTCODE.WMI_ENABLE_EVENTS:
                    traceLog = NativeMethods.GetTraceLoggerHandle(buffer);
                    status = NativeMethods.ERROR_SUCCESS;
                    break;
                case NativeMethods.WMIDPREQUESTCODE.WMI_DISABLE_EVENTS:
                    traceLog = 0;
                    status = NativeMethods.ERROR_SUCCESS;
                    break;
                default:
                    status = NativeMethods.ERROR_INVALID_PARAMETER;
                    break;
            }

            *inOutBufferSize = 0;
            return status;
        }

        private static string CategoryBySeverity(UlsTraceProvider.TraceSeverity severity)
        {
            string category = "General";

            switch (severity)
            {
                case UlsTraceProvider.TraceSeverity.CriticalEvent:
                    category = "Error";
                    break;
                case UlsTraceProvider.TraceSeverity.Exception:
                    category = "Error";
                    break;
                case UlsTraceProvider.TraceSeverity.InformationEvent:
                    category = "Debug";
                    break;
                case UlsTraceProvider.TraceSeverity.WarningEvent:
                    category = "Debug";
                    break;
            }

            return category;
        }

        internal static class NativeMethods
        {
            internal const int TRACE_VERSION_CURRENT = 1;
            internal const int ERROR_SUCCESS = 0;
            internal const int ERROR_INVALID_PARAMETER = 87;
            internal const int WNODE_FLAG_TRACED_GUID = 0x00020000;

            // Copied from Win32 APIs
            internal unsafe delegate uint EtwProc(NativeMethods.WMIDPREQUESTCODE requestCode, IntPtr requestContext, uint* bufferSize, IntPtr buffer);

            // Copied from Win32 APIs
            internal enum WMIDPREQUESTCODE
            {
                /// <summary>
                /// WMI_GET_ALL_DATA value.
                /// </summary>
                WMI_GET_ALL_DATA = 0,

                /// <summary>
                /// WMI_GET_SINGLE_INSTANCE value.
                /// </summary>
                WMI_GET_SINGLE_INSTANCE = 1,

                /// <summary>
                /// WMI_SET_SINGLE_INSTANCE value.
                /// </summary>
                WMI_SET_SINGLE_INSTANCE = 2,

                /// <summary>
                /// WMI_SET_SINGLE_ITEM value.
                /// </summary>
                WMI_SET_SINGLE_ITEM = 3,

                /// <summary>
                /// WMI_ENABLE_EVENTS value.
                /// </summary>
                WMI_ENABLE_EVENTS = 4,

                /// <summary>
                /// WMI_DISABLE_EVENTS value.
                /// </summary>
                WMI_DISABLE_EVENTS = 5,

                /// <summary>
                /// WMI_ENABLE_COLLECTION value.
                /// </summary>
                WMI_ENABLE_COLLECTION = 6,

                /// <summary>
                /// WMI_DISABLE_COLLECTION value.
                /// </summary>
                WMI_DISABLE_COLLECTION = 7,

                /// <summary>
                /// WMI_REGINFO value.
                /// </summary>
                WMI_REGINFO = 8,

                /// <summary>
                /// WMI_EXECUTE_METHOD value.
                /// </summary>
                WMI_EXECUTE_METHOD = 9
            }

            internal enum TraceFlags
            {
                /// <summary>
                /// TRACE_FLAG_START value 
                /// </summary>
                TRACE_FLAG_START = 1,

                /// <summary>
                /// TRACE_FLAG_END value
                /// </summary>
                TRACE_FLAG_END = 2,

                /// <summary>
                /// TRACE_FLAG_MIDDLE value
                /// </summary>
                TRACE_FLAG_MIDDLE = 3,

                /// <summary>
                /// TRACE_FLAG_ID_AS_ASCII value
                /// </summary>
                TRACE_FLAG_ID_AS_ASCII = 4
            }

            // Copied from Win32 APIs
            [DllImport("advapi32.dll", CharSet = CharSet.Unicode)]
            internal static extern unsafe uint RegisterTraceGuids([In] EtwProc cbFunc, [In] void* context, [In] ref Guid controlGuid, [In] uint guidCount, IntPtr guidReg, [In] string mofImagePath, [In] string mofResourceName, out ulong regHandle);

            // Copied from Win32 APIs
            [DllImport("advapi32.dll", CharSet = CharSet.Unicode)]
            internal static extern uint UnregisterTraceGuids([In]ulong regHandle);

            // Copied from Win32 APIs
            [DllImport("advapi32.dll", CharSet = CharSet.Unicode)]
            internal static extern ulong GetTraceLoggerHandle([In]IntPtr Buffer);

            // Copied from Win32 APIs
            [DllImport("advapi32.dll", SetLastError = true)]
            internal static extern uint TraceEvent([In]ulong traceHandle, [In]ref ULSTrace evnt);

            // Copied from Win32 APIs
            [StructLayout(LayoutKind.Sequential)]
            internal struct EVENT_TRACE_HEADER_CLASS
            {
                internal byte Type;
                internal byte Level;
                internal ushort Version;
            }

            // Copied from Win32 APIs
            [StructLayout(LayoutKind.Sequential)]
            internal struct EVENT_TRACE_HEADER
            {
                internal ushort Size;
                internal ushort FieldTypeFlags;
                internal EVENT_TRACE_HEADER_CLASS Class;
                internal uint ThreadId;
                internal uint ProcessId;
                internal ulong TimeStamp;
                internal Guid Guid;
                internal uint ClientContext;
                internal uint Flags;
            }

            [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Unicode)]
            internal struct ULSTraceHeader
            {
                /// <summary>
                /// Size value
                /// </summary>
                internal ushort Size;

                /// <summary>
                /// dwVersion value
                /// </summary>
                internal uint dwVersion;

                /// <summary>
                /// Identity value
                /// </summary>
                internal uint Id;

                /// <summary>
                /// correlationID value
                /// </summary>
                internal Guid correlationID;

                /// <summary>
                /// dwFlags value
                /// </summary>
                internal TraceFlags dwFlags;

                /// <summary>
                /// wzExeName value
                /// </summary>
                [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 32)]
                internal string wzExeName;

                /// <summary>
                /// wzProduct value
                /// </summary>
                [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 32)]
                internal string wzProduct;

                /// <summary>
                /// wzCategory value
                /// </summary>
                [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 32)]
                internal string wzCategory;

                /// <summary>
                /// wzMessage value
                /// </summary>
                [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 800)]
                internal string wzMessage;
            }

            [StructLayout(LayoutKind.Sequential)]
            internal struct ULSTrace
            {
                /// <summary>
                /// Header value
                /// </summary>
                internal EVENT_TRACE_HEADER Header;

                /// <summary>
                /// ULSHeader value
                /// </summary>
                internal ULSTraceHeader ULSHeader;
            }
        }
    }
}
