﻿//-------------------------------------------------------------------------------------------------
// <copyright file="ActivationJobDefinition.cs" company="commerceserver.net">
// Copyright © commerceserver.net.  All rights reserved.
// </copyright>
// <summary>Base commerce job activation definition</summary>
//-------------------------------------------------------------------------------------------------
namespace CommerceServerContrib.Web.SharePoint
{
    using System;
    using System.Globalization;
    using System.IO;
    using System.Security.Permissions;
    using System.Xml;
    using System.Xml.XPath;
    using Microsoft.SharePoint.Administration;
    using Microsoft.SharePoint.Security;
    using Microsoft.SharePoint.Utilities;

    /// <summary>
    /// Base SharePoint activation job definition
    /// </summary>
    public class ActivationJobDefinition : BaseJobDefinition
    {
        /// <summary>
        /// Initializes a new instance of the ActivationJobDefinition class.
        /// </summary>
        public ActivationJobDefinition()
        {
        }

        /// <summary>
        /// Initializes a new instance of the ActivationJobDefinition class.
        /// </summary>
        /// <param name="jobName">Name of the job.</param>
        /// <param name="webApp">The web app.</param>
        public ActivationJobDefinition(string jobName, SPWebApplication webApp)
            : base(jobName, webApp)
        {
        }

        /// <summary>
        /// Initializes a new instance of the ActivationJobDefinition class.
        /// </summary>
        /// <param name="jobName">Name of the job.</param>
        /// <param name="service">The service.</param>
        public ActivationJobDefinition(string jobName, SPService service)
            : base(jobName, service)
        {
        }

        /// <summary>
        /// Execution method of the current job definition
        /// </summary>
        /// <param name="targetInstanceId">Content database instance ID</param>
        [SharePointPermission(SecurityAction.LinkDemand, ObjectModel = true)]
        public override void Execute(Guid targetInstanceId)
        {
            base.Execute(targetInstanceId);

            var webApp = this.Parent as SPWebApplication;
            var service = this.Parent as SPService;

            if (webApp != null)
            {
                this.DeployWebSiteFiles(new Guid(this.Properties["FeatureDefinitionId"].ToString()));
                this.UpdateWebConfigZones(new Guid(this.Properties["FeatureDefinitionId"].ToString()));
            }

            if (service != null)
            {
                this.UpdateWebConfigZones(new Guid(this.Properties["FeatureDefinitionId"].ToString()));
            }
        }

        /// <summary>
        /// Deploys the web site files.
        /// </summary>
        /// <param name="featureDefinitionId">The guid of the feature that fired this job.</param>
        private void DeployWebSiteFiles(Guid featureDefinitionId)
        {
            UlsTraceProvider.LogMessage("Deploying website files.", UlsTraceProvider.TraceSeverity.InformationEvent);

            foreach (SPUrlZone zone in this.WebApplication.IisSettings.Keys)
            {
                SPIisSettings settings = this.WebApplication.IisSettings[zone];
                string zonePhysicalPath = settings.Path.ToString();
                string featureWebPhysicalPath = Path.Combine(Farm.FeatureDefinitions[featureDefinitionId].RootDirectory, "webfiles");

                if (Directory.Exists(featureWebPhysicalPath))
                {
                    foreach (string file in Directory.GetFiles(featureWebPhysicalPath, "*.*", SearchOption.AllDirectories))
                    {
                        UlsTraceProvider.LogMessage(string.Format(CultureInfo.CurrentCulture, "Copying file {0} to virtual directory {1}", file, zonePhysicalPath), UlsTraceProvider.TraceSeverity.InformationEvent);

                        try
                        {
                            string relativePath = file.Replace(featureWebPhysicalPath, String.Empty);

                            // If the path starts with a backslash after the previous line, then it will not combine properly to create a full path.
                            if (Path.IsPathRooted(relativePath))
                            {
                                relativePath = relativePath.Substring(1);
                            }

                            string zoneFilePath = Path.Combine(zonePhysicalPath, relativePath);

                            // Create the directory, if needed
                            if (!Directory.Exists(Path.GetDirectoryName(zoneFilePath)))
                            {
                                Directory.CreateDirectory(Path.GetDirectoryName(zoneFilePath));
                            }
                            else
                            {
                                if (File.Exists(zoneFilePath))
                                {
                                    var zoneFileInfo = new FileInfo(zoneFilePath) { IsReadOnly = false };
                                }
                            }

                            File.Copy(file, zoneFilePath, true);
                        }
                        catch (Exception exception)
                        {
                            UlsTraceProvider.LogMessage(string.Format(CultureInfo.CurrentCulture, "Error while copying file {0} to virtual directory {1}", file, zonePhysicalPath), UlsTraceProvider.TraceSeverity.Exception);
                            UlsTraceProvider.LogMessage(exception.Message, UlsTraceProvider.TraceSeverity.Exception);
                        }
                    }
                }
            }
        }

        /// <summary>
        /// Updates the web config in a specific zone.
        /// </summary>
        /// <param name="featureDefinitionId">The guid of the feature calling this method.</param>
        private void UpdateWebConfigZones(Guid featureDefinitionId)
        {
            var fullPath = Path.Combine(Farm.FeatureDefinitions[featureDefinitionId].RootDirectory, "webconfigentries-nonsp");

            if (Directory.Exists(fullPath))
            {
                foreach (string featureWebconfigZonePhysicalPath in Directory.GetDirectories(fullPath, "*"))
                {
                    // if the web config entries are coming from a folder other than the External folder, modify web.config; otherwise
                    // modify the external config based on the file name of the web config entry.
                    if (!featureWebconfigZonePhysicalPath.Contains("External"))
                    {
                        this.UpdateWebConfig(featureWebconfigZonePhysicalPath);
                    }
                    else
                    {
                        UlsTraceProvider.LogMessage("Updating external configuration files.", UlsTraceProvider.TraceSeverity.InformationEvent);
                        this.UpdateExternalConfig(featureWebconfigZonePhysicalPath);
                    }
                }
            }
        }

        private void UpdateWebConfig(string featureWebconfigZonePhysicalPath)
        {
            var webConfigParentFolder = string.Empty;

            if (featureWebconfigZonePhysicalPath.Contains("Zone_"))
            {
                string zoneName = Path.GetFileName(featureWebconfigZonePhysicalPath).Replace("Zone_", string.Empty);
                SPUrlZone zone = (SPUrlZone)Enum.Parse(typeof(SPUrlZone), zoneName);

                SPIisSettings zoneSettings = this.WebApplication.IisSettings[zone];

                webConfigParentFolder = zoneSettings.Path.FullName;
            }
            else if (featureWebconfigZonePhysicalPath.EndsWith("STS", StringComparison.OrdinalIgnoreCase))
            {
                #if SP2013
                    webConfigParentFolder = String.Format(CultureInfo.InvariantCulture, "{0}\\SecurityToken\\", SPUtility.GetVersionedGenericSetupPath("WebServices", 14));
                #else
                    webConfigParentFolder = String.Format(CultureInfo.InvariantCulture, "{0}\\SecurityToken\\", SPUtility.GetGenericSetupPath("WebServices"));
                #endif
            }
            else if (featureWebconfigZonePhysicalPath.EndsWith("CA", StringComparison.OrdinalIgnoreCase))
            {
                webConfigParentFolder = Microsoft.SharePoint.Administration.SPAdministrationWebApplication.Local.IisSettings[0].Path.FullName;
            }

            if (!string.IsNullOrEmpty(webConfigParentFolder))
            {
                var webConfigPath = Path.Combine(webConfigParentFolder, "web.config");

                if (File.Exists(webConfigPath))
                {
                    XmlDocument xmlWebConfig = new XmlDocument();
                    xmlWebConfig.Load(webConfigPath);

                    foreach (string file in Directory.GetFiles(featureWebconfigZonePhysicalPath, "*.*", SearchOption.TopDirectoryOnly))
                    {
                        XmlDocument xmlFile = new XmlDocument();
                        xmlFile.Load(file);

                        this.UpdateConfig(xmlWebConfig, xmlFile, webConfigPath);
                    }

                    xmlWebConfig.Save(webConfigPath);
                }
            }
        }

        private void UpdateExternalConfig(string externalConfigPhysicalPath)
        {
            // Update external configs for all zones
            foreach (string file in Directory.GetFiles(externalConfigPhysicalPath, "*.*", SearchOption.TopDirectoryOnly))
            {
                foreach (SPUrlZone zone in this.WebApplication.IisSettings.Keys)
                {
                    SPIisSettings zoneSettings = this.WebApplication.IisSettings[zone];
                    var zonePath = zoneSettings.Path.FullName;

                    var externalConfigPath = Path.Combine(zonePath, file.Replace(externalConfigPhysicalPath + "\\", string.Empty).Replace(".xml", ".config"));

                    if (File.Exists(externalConfigPath))
                    {
                        XmlDocument xmlWebConfig = new XmlDocument();
                        xmlWebConfig.Load(externalConfigPath);

                        XmlDocument xmlFile = new XmlDocument();
                        xmlFile.Load(file);

                        this.UpdateConfig(xmlWebConfig, xmlFile, externalConfigPath);

                        xmlWebConfig.Save(externalConfigPath);
                    }
                    else
                    {
                        UlsTraceProvider.LogMessage(string.Format(CultureInfo.CurrentCulture, "External configuration file '{0}' cannot be found.", externalConfigPath), UlsTraceProvider.TraceSeverity.Exception);
                    }
                }
            }

            // Update external configs for each specific zone
            foreach (string zoneSpecificConfigZonePhysicalPath in Directory.GetDirectories(externalConfigPhysicalPath, "*"))
            {
                var folderName = zoneSpecificConfigZonePhysicalPath.Replace(externalConfigPhysicalPath + "\\", string.Empty);
                if (folderName.StartsWith("Zone_", StringComparison.OrdinalIgnoreCase))
                {
                    string zoneName = Path.GetFileName(folderName).Replace("Zone_", string.Empty);
                    SPUrlZone zone = (SPUrlZone)Enum.Parse(typeof(SPUrlZone), zoneName);

                    SPIisSettings zoneSettings = this.WebApplication.IisSettings[zone];

                    var zonePath = zoneSettings.Path.FullName;

                    foreach (string file in Directory.GetFiles(zoneSpecificConfigZonePhysicalPath, "*.*", SearchOption.TopDirectoryOnly))
                    {
                        var externalConfigPath = Path.Combine(zonePath, file.Replace(zoneSpecificConfigZonePhysicalPath + "\\", string.Empty).Replace(".xml", ".config"));

                        if (File.Exists(externalConfigPath))
                        {
                            XmlDocument xmlWebConfig = new XmlDocument();
                            xmlWebConfig.Load(externalConfigPath);

                            XmlDocument xmlFile = new XmlDocument();
                            xmlFile.Load(file);

                            this.UpdateConfig(xmlWebConfig, xmlFile, externalConfigPath);

                            xmlWebConfig.Save(externalConfigPath);
                        }
                    }
                }
            }
        }

        private void UpdateConfig(XmlDocument webConfigDoc, XmlDocument webConfigModificationDoc, string webConfigPath)
        {
            XmlNodeList nodelistWebConfigUpdates = webConfigModificationDoc.SelectNodes("/webconfigUpdates/webconfigUpdate");

            if (nodelistWebConfigUpdates != null)
            {
                foreach (XmlNode nodeWebConfigUpdate in nodelistWebConfigUpdates)
                {
                    // Potentially check version here...
                    if (nodeWebConfigUpdate.Attributes["version"] == null || nodeWebConfigUpdate.Attributes["version"].Value.Equals(this.SPMajorVersion()))
                    {
                        var xmlNamespace = string.Empty;
                        var prefix = string.Empty;

                        if (nodeWebConfigUpdate.Attributes["namespace"] != null &&
                            nodeWebConfigUpdate.Attributes["prefix"] != null)
                        {
                            xmlNamespace = FeatureHelper.ReplaceTokens(nodeWebConfigUpdate.Attributes["namespace"].Value, this.Properties);
                            prefix = FeatureHelper.ReplaceTokens(nodeWebConfigUpdate.Attributes["prefix"].Value, this.Properties);
                        }

                        XmlNode nodeFound = null;

                        if (string.IsNullOrEmpty(xmlNamespace) &&
                            string.IsNullOrEmpty(prefix))
                        {
                            nodeFound = webConfigDoc.SelectSingleNode(FeatureHelper.ReplaceTokens(nodeWebConfigUpdate.Attributes["path"].Value, this.Properties));
                        }
                        else
                        {
                            XmlNamespaceManager nsm = new XmlNamespaceManager(webConfigDoc.NameTable);
                            nsm.AddNamespace(prefix, xmlNamespace);
                            nodeFound = webConfigDoc.SelectSingleNode(FeatureHelper.ReplaceTokens(nodeWebConfigUpdate.Attributes["path"].Value, this.Properties), nsm);
                        }

                        switch (
                            (SPWebConfigModification.SPWebConfigModificationType)Enum.Parse(typeof(SPWebConfigModification.SPWebConfigModificationType), nodeWebConfigUpdate.Attributes["type"].Value))
                        {
                            case SPWebConfigModification.SPWebConfigModificationType.EnsureAttribute:
                                {
                                    if (nodeFound != null)
                                    {
                                        XmlAttribute attribute =
                                           nodeFound.Attributes.GetNamedItem(
                                               FeatureHelper.ReplaceTokens(
                                                   nodeWebConfigUpdate.Attributes["name"].Value,
                                                   this.Properties)) as XmlAttribute;

                                        if (attribute != null)
                                        {
                                            attribute.Value =
                                                FeatureHelper.ReplaceTokens(
                                                    nodeWebConfigUpdate.InnerText.Trim(), this.Properties);
                                        }
                                        else
                                        {
                                            XmlAttribute attributeNew =
                                                webConfigDoc.CreateAttribute(
                                                    FeatureHelper.ReplaceTokens(
                                                        nodeWebConfigUpdate.Attributes["name"].Value,
                                                        this.Properties));
                                            attributeNew.Value =
                                                FeatureHelper.ReplaceTokens(
                                                    nodeWebConfigUpdate.InnerText.Trim(), this.Properties);
                                            nodeFound.Attributes.Append(attributeNew);
                                        }
                                    }
                                    else
                                    {
                                        throw new XPathException("Unable to locate node [" + nodeWebConfigUpdate.Attributes["path"].Value + "] in [" + webConfigPath + "].");
                                    }

                                    break;
                                }

                            case SPWebConfigModification.SPWebConfigModificationType.EnsureChildNode:
                            case SPWebConfigModification.SPWebConfigModificationType.EnsureSection:
                                {
                                    if (nodeFound != null)
                                    {
                                        if (nodeFound.HasChildNodes)
                                        {
                                            var nodeName = FeatureHelper.ReplaceTokens(nodeWebConfigUpdate.Attributes["name"].Value, this.Properties);

                                            if (nodeFound.SelectSingleNode(nodeName) != null)
                                            {
                                                break;
                                            }
                                        }

                                        for (int indexer = 0; indexer < nodeWebConfigUpdate.ChildNodes.Count; indexer++)
                                        {
                                            var nodeValue = FeatureHelper.ReplaceTokens(nodeWebConfigUpdate.ChildNodes[indexer].OuterXml, this.Properties);
                                            var addToTop = nodeWebConfigUpdate.Attributes["addToTop"];

                                            if (addToTop != null && !string.IsNullOrEmpty(addToTop.InnerText) && bool.Parse(addToTop.InnerText))
                                            {
                                                nodeFound.InnerXml = nodeValue + nodeFound.InnerXml;
                                            }
                                            else
                                            {
                                                nodeFound.InnerXml = nodeFound.InnerXml + nodeValue;
                                            }
                                        }
                                    }

                                    break;
                                }
                        }
                    }
                }
            }
        }
    }
}
