﻿//-------------------------------------------------------------------------------------------------
// <copyright file="DeactivationJobDefinition.cs" company="commerceserver.net">
// Copyright © commerceserver.net.  All rights reserved.
// </copyright>
// <summary>Base commerce job activation definition</summary>
//-------------------------------------------------------------------------------------------------
namespace CommerceServerContrib.Web.SharePoint
{
    using System;
    using System.Globalization;
    using System.IO;
    using System.Reflection;
    using System.Security.Permissions;
    using System.Xml;
    using System.Xml.XPath;
    using Microsoft.SharePoint.Administration;
    using Microsoft.SharePoint.Security;
    using Microsoft.SharePoint.Utilities;

    /// <summary>
    /// Base SharePoint activation job definition
    /// </summary>
    public class DeactivationJobDefinition : BaseJobDefinition
    {
        /// <summary>
        /// Initializes a new instance of the DeactivationJobDefinition class.
        /// </summary>
        public DeactivationJobDefinition()
        {
        }

        /// <summary>
        /// Initializes a new instance of the DeactivationJobDefinition class.
        /// </summary>
        /// <param name="jobName">Name of the job.</param>
        /// <param name="webApp">The web app.</param>
        public DeactivationJobDefinition(string jobName, SPWebApplication webApp)
            : base(jobName, webApp)
        {
        }

        /// <summary>
        /// Initializes a new instance of the DeactivationJobDefinition class.
        /// </summary>
        /// <param name="jobName">Name of the job.</param>
        /// <param name="service">The service.</param>
        public DeactivationJobDefinition(string jobName, SPService service)
            : base(jobName, service)
        {
        }

        /// <summary>
        /// Execution method of the current job definition
        /// </summary>
        /// <param name="targetInstanceId">Content database instance ID</param>
        [SharePointPermission(SecurityAction.LinkDemand, ObjectModel = true)]
        public override void Execute(Guid targetInstanceId)
        {
            base.Execute(targetInstanceId);

            var webApp = this.Parent as SPWebApplication;
            var service = this.Parent as SPService;

            if (webApp != null)
            {
                this.RetractWebSiteFiles(this.Properties["FeatureRootDirectory"].ToString());
                this.RetractWebConfigZones(this.Properties["FeatureRootDirectory"].ToString());
            }

            if (service != null)
            {
                this.RetractWebConfigZones(this.Properties["FeatureRootDirectory"].ToString());
            }
        }

        /// <summary>
        /// Remove any file from the SharePoint virtual directories that are found in the SharePointRoot/web directory.
        /// </summary>
        /// <param name="featureRootDirectory">The root directory of the feature being activated</param>
        private void RetractWebSiteFiles(string featureRootDirectory)
        {
            foreach (SPUrlZone zone in this.WebApplication.IisSettings.Keys)
            {
                SPIisSettings settings = this.WebApplication.IisSettings[zone];
                string zonePhysicalPath = settings.Path.ToString();

                string featureWebPhysicalPath = Path.Combine(featureRootDirectory, "webfiles");

                if (Directory.Exists(featureWebPhysicalPath))
                {
                    foreach (string file in Directory.GetFiles(featureWebPhysicalPath, "*.*", SearchOption.AllDirectories))
                    {
                        string relativePath = file.Replace(featureWebPhysicalPath, String.Empty);

                        // If the path starts with a backslash after the previous line, then it will not combine properly to create a full path.
                        if (Path.IsPathRooted(relativePath))
                        {
                            relativePath = relativePath.Substring(1);
                        }

                        string zoneFilePath = Path.Combine(zonePhysicalPath, relativePath);

                        if (File.Exists(zoneFilePath))
                        {
                            var zoneFileInfo = new FileInfo(zoneFilePath)
                                                   {
                                                       IsReadOnly = false
                                                   };

                            zoneFileInfo.Delete();

                            var directoryDelete = new DirectoryInfo(Path.GetDirectoryName(zoneFilePath));

                            // If there are no more files in the directory, delete the directory.
                            if (directoryDelete.GetFiles("*", SearchOption.AllDirectories).Length == 0)
                            {
                                directoryDelete.Delete(true);
                            }
                        }
                    }
                }
            }
        }

        private void RetractWebConfigZones(string featureRootDirectory)
        {
            var fullPath = Path.Combine(featureRootDirectory, "webconfigentries-nonsp");

            if (Directory.Exists(fullPath))
            {
                foreach (string featureWebconfigZonePhysicalPath in Directory.GetDirectories(fullPath, "*"))
                {
                    var webConfigParentFolder = string.Empty;

                    if (featureWebconfigZonePhysicalPath.Contains("Zone_"))
                    {
                        string zoneName = Path.GetFileName(featureWebconfigZonePhysicalPath).Replace("Zone_", string.Empty);
                        SPUrlZone zone = (SPUrlZone)Enum.Parse(typeof(SPUrlZone), zoneName);

                        SPIisSettings zoneSettings = this.WebApplication.IisSettings[zone];

                        webConfigParentFolder = zoneSettings.Path.FullName;
                    }
                    else if (featureWebconfigZonePhysicalPath.EndsWith("STS", StringComparison.OrdinalIgnoreCase))
                    {
#if SP2013
                        webConfigParentFolder = string.Format(CultureInfo.InvariantCulture, "{0}\\SecurityToken\\", SPUtility.GetVersionedGenericSetupPath("WebServices", 14));
#else 
                        webConfigParentFolder = string.Format(CultureInfo.InvariantCulture, "{0}\\SecurityToken\\", SPUtility.GetGenericSetupPath("WebServices"));
#endif
                    }
                    else if (featureWebconfigZonePhysicalPath.EndsWith("CA", StringComparison.OrdinalIgnoreCase))
                    {
                        webConfigParentFolder = Microsoft.SharePoint.Administration.SPAdministrationWebApplication.Local.IisSettings[0].Path.FullName;
                    }

                    if (!string.IsNullOrEmpty(webConfigParentFolder))
                    {
                        string webConfigPath = Path.Combine(webConfigParentFolder, "web.config");
                        XmlDocument xmlWebConfig = new XmlDocument();
                        xmlWebConfig.Load(webConfigPath);

                        foreach (string file in Directory.GetFiles(featureWebconfigZonePhysicalPath, "*.*", SearchOption.TopDirectoryOnly))
                        {
                            XmlDocument xml = new XmlDocument();
                            xml.Load(file);

                            XmlNodeList nodelistWebConfigUpdates = xml.SelectNodes("/webconfigUpdates/webconfigUpdate");

                            if (nodelistWebConfigUpdates != null)
                            {
                                foreach (XmlNode nodeWebConfigUpdate in nodelistWebConfigUpdates)
                                {
                                    // Potentially check version here...
                                    if (nodeWebConfigUpdate.Attributes["version"] == null || nodeWebConfigUpdate.Attributes["version"].Value.Equals(this.SPMajorVersion()))
                                    {
                                        if (nodeWebConfigUpdate.Attributes["cleanup"] != null &&
                                            bool.Parse(nodeWebConfigUpdate.Attributes["cleanup"].InnerText) == true)
                                        {
                                            XmlNode nodeFound =
                                                xmlWebConfig.SelectSingleNode(
                                                    FeatureHelper.ReplaceTokens(
                                                        nodeWebConfigUpdate.Attributes["path"].Value, this.Properties));

                                            switch ((SPWebConfigModification.SPWebConfigModificationType)Enum.Parse(typeof(SPWebConfigModification.SPWebConfigModificationType), nodeWebConfigUpdate.Attributes["type"].Value))
                                            {
                                                case SPWebConfigModification.SPWebConfigModificationType.EnsureAttribute:
                                                    {
                                                        if (nodeFound != null)
                                                        {
                                                            XmlAttribute attribute =
                                                                nodeFound.Attributes.GetNamedItem(
                                                                    FeatureHelper.ReplaceTokens(
                                                                        nodeWebConfigUpdate.Attributes["name"].Value,
                                                                        this.Properties)) as XmlAttribute;

                                                            if (attribute != null)
                                                            {
                                                                attribute.Value = string.Empty;
                                                            }
                                                        }

                                                        break;
                                                    }

                                                case SPWebConfigModification.SPWebConfigModificationType.EnsureChildNode:
                                                case SPWebConfigModification.SPWebConfigModificationType.EnsureSection:
                                                    {
                                                        if (nodeFound != null)
                                                        {
                                                            var childNode =
                                                                nodeFound.SelectSingleNode(
                                                                    FeatureHelper.ReplaceTokens(
                                                                        nodeWebConfigUpdate.Attributes["name"].Value,
                                                                        this.Properties));

                                                            if (childNode != null)
                                                            {
                                                                childNode.ParentNode.RemoveChild(childNode);
                                                            }
                                                        }

                                                        break;
                                                    }
                                            }
                                        }
                                    }
                                }
                            }
                        }

                        xmlWebConfig.Save(webConfigPath);
                    }
                }
            }
        }
    }
}
