﻿//---------------------------------------------------------------------
// <copyright file="BrandProvider.cs" company="commerceserver.net">
//     Copyright © commerceserver.net.  All rights reserved.
// </copyright>
// <summary></summary>
//---------------------------------------------------------------------
namespace CommerceServerContrib.Web
{
    using CommerceServerContrib.CrossTier.Configuration;
    using CommerceServerContrib.CrossTier.Interfaces;
    using System.Collections.Generic;
    using System;
    using CommerceServerContrib.CrossTier;
    using CommerceServer.Foundation;

    /// <summary>
    /// Used to provide an instance of the current brand
    /// </summary>
    public class BrandProvider : IBrandProvider
    {
        private object _syncRoot = new object();
        private bool _initialized = false;
        private Brand _defaultBrand = null;
        private Dictionary<string, Brand> _brandNameToBrand = null;

        /// <summary>
        /// Gets the name of the current channel.
        /// </summary>
        /// <value>
        /// The name of the current channel.
        /// </value>
        public virtual Brand Current 
        {
            get
            {
                this.Initialize();

                /*
                 * This should try to get the brand based on the PresentationTypeLoader.ConfigurationProvider.CurrentConfiguration.Name
                 * If there is not a brand by that name it should return the default
                 * Optionally log that it wasn't able to load it by name.
                 * */
                var brand = this.GetBrandByName(PresentationTypeLoader.ConfigurationProvider.CurrentConfiguration.Name);
                if (brand == null)
                {
                    CommerceTrace.Information(CommerceClientTraceSource.PresentationTier, "Could not load Brand {0}.  Loading the Default Brand instead.");
                    brand = this.GetDefaultBrand();
                }

                return brand;
            }
        }

        /// <summary>
        /// Gets the default brand defined in the presentation configuration.
        /// </summary>
        /// <returns>The default brand defined in the presentation configuration.</returns>
        public virtual Brand GetDefaultBrand()
        {
            this.Initialize();
            return this._defaultBrand;
        }

        /// <summary>
        /// Gets a brand defined in the presentation configuration file by its name.
        /// </summary>
        /// <param name="brandName">The name of the brand to retrieve.</param>
        /// <returns>The brand with the provided <paramref name="brandName"/>.</returns>
        public virtual Brand GetBrandByName(string brandName)
        {
            this.Initialize();

            Brand brand = null;
            this._brandNameToBrand.TryGetValue(brandName, out brand);

            return brand;
        }

        /// <summary>
        /// Gets all brands defined in the configuration file.
        /// </summary>
        /// <returns>A list containing all brands defined in the configuration file.</returns>
        public virtual List<Brand> GetAllBrands()
        {
            this.Initialize();

            return new List<Brand>(this._brandNameToBrand.Values);
        }

        /// <summary>
        /// Initializes this instance.
        /// </summary>
        protected virtual void Initialize()
        {
            if (!this._initialized)
            {
                lock (this._syncRoot)
                {
                    if (this._initialized)
                    {
                        return;
                    }

                    this._brandNameToBrand = new Dictionary<string, Brand>(StringComparer.OrdinalIgnoreCase);

                    var presentationConfig = CommerceServerContribConfiguration.Presentation;
                    if (presentationConfig == null)
                    {
                        this._initialized = true;
                        return;
                    }

                    this._defaultBrand = new Brand(presentationConfig.DefaultBrand);
                    this._brandNameToBrand[this._defaultBrand.Name] = this._defaultBrand;

                    if (presentationConfig.Brands == null)
                    {
                        this._initialized = true;
                        return;
                    }

                    foreach (BrandElement element in presentationConfig.Brands)
                    {
                        if (element != null)
                        {
                            var brand = new Brand(element);
                            brand.MergeWith(this._defaultBrand);
                            this._brandNameToBrand[brand.Name] = brand;
                        }
                    }

                    this._initialized = true;
                }
            }
        }

        /// <summary>
        /// Deinitializes the setup.
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1704:IdentifiersShouldBeSpelledCorrectly", MessageId = "Deinitialize", Justification = "Spelling is correct.")]
        protected virtual void Deinitialize()
        {
            this._initialized = false;
        }
    }
}
