﻿//---------------------------------------------------------------------
// <copyright file="SiteContext.cs" company="commerceserver.net">
//     Copyright © commerceserver.net.  All rights reserved.
// </copyright>
// <summary>SiteContext class</summary>
//---------------------------------------------------------------------
namespace CommerceServerContrib.Web
{
    using CommerceServer.Foundation;
    using CommerceServerContrib.CrossTier.Configuration;
    using CommerceServerContrib.CrossTier.Entities;
    using CommerceServerContrib.CrossTier.Interfaces;
    using Microsoft.Practices.Unity;
    using System;
    using System.Globalization;
    using System.IO;
    using System.Security.Principal;
    using System.Web;
    using System.Web.Configuration;
    using System.Configuration;

    /// <summary>
    /// Provides contextual information about the current site instance
    /// </summary>
    public class SiteContext : CommerceServer.Foundation.SiteContext, ICommerceContext
    {
        private const string UserIdKey = "UserID_93BFD04D-C039-43C1-A3F3-645246CF8AB4";

        #region ICommerceContext Implementation

        /// <summary>
        /// Gets the current http context
        /// </summary>
        public override HttpContext CurrentContext
        {
            get
            {
                return HttpContext.Current;
            }
        }

        /// <summary>
        /// Gets or sets the user.
        /// </summary>
        /// <value>
        /// The current user.
        /// </value>
        public virtual User User
        {
            get
            {
                return HttpContext.Current.Items[UserIdKey] as User;
            }

            set
            {
                HttpContext.Current.Items[UserIdKey] = value;
            }
        }

        /// <summary>
        /// Gets the default name of the catalog.
        /// </summary>
        /// <value>
        /// The default name of the catalog.
        /// </value>
        public string DefaultCatalogName
        {
            get
            {
                return ConfigurationManager.AppSettings["DefaultCommerceCatalog"];
            }
        }

        /// <summary>
        /// Gets the user ID
        /// </summary>
        public virtual string UserId
        {
            get
            {
                if (this.User != null && !this.User.IsWindowsIdentity && !this.IsAnonymous)
                {
                    return this.User.Id;
                }
                else if (this.User != null)
                {
                    return this.User.AnonymousId;
                }

                return string.Empty;
            }
        }

        /// <summary>
        /// Gets a value indicating whether or not the user is anonymous.
        /// </summary>
        public virtual bool IsUserAuthenticated
        {
            get
            {
                var authenticationProvider = PresentationServiceLocator.Current.GetImplementation<IAuthenticationProvider>();
                if (authenticationProvider.RequireSsl && !this.IsSecureConnection)
                {
                    // if the authentication provider only issues secure cookies and we
                    // are on a non-secure page, the UserReference will always be null.
                    // We will use the registered cookie information to determine if
                    // the user has been authenticated instead.
                    return this.User != null && !string.IsNullOrEmpty(this.User.Name);
                }
                else
                {
                    return this.UserReference != null && this.UserReference.IsAuthenticated;
                }
            }
        }

        /// <summary>
        /// Gets a value indicating whether this instance is anonymous.
        /// </summary>
        /// <value>
        /// 	<c>true</c> if this instance is anonymous; otherwise, <c>false</c>.
        /// </value>
        public virtual bool IsAnonymous
        {
            get
            {
                return !this.IsUserAuthenticated;
            }
        }

        /// <summary>
        /// Gets the current date and time.
        /// </summary>
        public virtual DateTime CurrentDateTime
        {
            get
            {
                return DateTime.Now;
            }
        }

        /// <summary>
        /// Gets the culture code.
        /// </summary>
        public virtual string CultureCode
        {
            get
            {
                return this.Culture.Name;
            }
        }

        /// <summary>
        /// Gets the culture UI code.
        /// </summary>
        public virtual string CultureUICode
        {
            get
            {
                return this.UICulture.Name;
            }
        }

        /// <summary>
        /// Gets the default basket name
        /// </summary>
        public virtual string DefaultBasketName
        {
            get
            {
                return "Default";
            }
        }

        /// <summary>
        /// Gets the default basket type
        /// </summary>
        public virtual int DefaultBasketType
        {
            get
            {
                return 0;
            }
        }

        /// <summary>
        /// Gets the current location path
        /// </summary>
        public virtual string FullLocationPath
        {
            get
            {
                return this.CurrentContext != null ? this.CurrentContext.Request.Url.PathAndQuery : null;
            }
        }

        /// <summary>
        /// Gets the browser.
        /// </summary>
        /// <value>The browser.</value>
        public virtual string Browser
        {
            get
            {
                return this.CurrentContext.Request.Browser.Browser;
            }
        }

        /// <summary>
        /// Gets a value indicating whether this instance is secure connection.
        /// </summary>
        /// <value>
        /// 	<c>true</c> if this instance is secure connection; otherwise, <c>false</c>.
        /// </value>
        public virtual bool IsSecureConnection
        {
            get
            {
                if (this.CurrentContext == null)
                {
                    return false;
                }

                if (this.CurrentContext.Request.IsSecureConnection)
                {
                    return true;
                }
                else
                {
                    if (this.IsFromSslOffload)
                    {
                        return true;
                    }
                }

                return false;
            }
        }

        /// <summary>
        /// Gets the Current SiteId
        /// </summary>
        public virtual string SiteId
        {
            get
            {
                // Only the root site ID is currently supported.
                return CommerceServerContrib.CrossTier.Helpers.StringConstants.RootSiteId;
            }
        }

        /// <summary>
        /// Gets a value indicating whether or not the current call is an AJAX call
        /// </summary>
        public bool IsAjaxCall
        {
            get
            {
                return this.FullLocationPath.StartsWith("/services/", StringComparison.OrdinalIgnoreCase);
            }
        }

        #endregion

        /// <summary>
        /// Gets the user reference.
        /// </summary>
        private IIdentity UserReference
        {
            get
            {
                return this.CurrentContext.User != null && this.CurrentContext.User.Identity.IsAuthenticated ? this.CurrentContext.User.Identity : null;
            }
        }

        /// <summary>
        /// Gets a value indicating whether this instance is from SSL offload.
        /// </summary>
        /// <value>
        /// 	<c>true</c> if this instance is from SSL offload; otherwise, <c>false</c>.
        /// </value>
        private bool IsFromSslOffload
        {
            get
            {
                return this.CurrentContext.Request.Url.Port ==
                       PresentationServiceLocator.Current.BrandContext.SecurePageSettings.SslOffloadPort;
            }
        }

        /// <summary>
        /// Redirects to.
        /// </summary>
        /// <param name="path">The path to redirect to.</param>
        /// <param name="endResponse">if set to <c>true</c> [end response].</param>
        public virtual void RedirectTo(string path, bool endResponse)
        {
            if (CommerceServerContribConfiguration.Presentation.DisableRedirects)
            {
                return;
            }

            bool isPathValid = true;
            if (!String.IsNullOrEmpty(path))
            {
                Uri uri = new Uri(path, UriKind.RelativeOrAbsolute);
                if (uri.IsAbsoluteUri)
                {
                    string domain = uri.Host.ToLowerInvariant();
                    string currentDomain = HttpContext.Current.Request.Url.Host.ToLowerInvariant();
                    isPathValid = domain.Equals(currentDomain) ? true : false;
                }
            }

            if (isPathValid)
            {
                this.CurrentContext.Response.Redirect(path, endResponse);
            }
            else
            {
                this.RedirectTo404();
            }
        }

        /// <summary>
        /// Redirects to the configured page not found web page.  If none are configured then only a standard 404 is returned.
        /// </summary>
        public void RedirectTo404()
        {
            if (CommerceServerContribConfiguration.Presentation.DisableRedirects)
            {
                return;
            }

            HttpContext.Current.Response.StatusCode = (int)System.Net.HttpStatusCode.NotFound;

            System.Configuration.Configuration configuration = WebConfigurationManager.OpenWebConfiguration("~");
            if (configuration != null)
            {
                CustomErrorsSection customErrorsSection = (CustomErrorsSection)configuration.GetSection("system.web/customErrors");
                if (customErrorsSection != null)
                {
                    if (customErrorsSection.Mode == CustomErrorsMode.On)
                    {
                        CustomError customError = customErrorsSection.Errors["404"];
                        if (customError != null)
                        {
                            this.CurrentContext.Response.Redirect(customError.Redirect, true);
                        }
                        else if (!String.IsNullOrEmpty(customErrorsSection.DefaultRedirect))
                        {
                            this.CurrentContext.Response.Redirect(customErrorsSection.DefaultRedirect, true);
                        }
                    }
                }
            }

            HttpContext.Current.Response.End();
        }

        /// <summary>
        /// Maps the supplied virtual path to a physical path on the server
        /// </summary>
        /// <param name="path">The relative path to the file.</param>
        /// <returns>The Absolute path to the file, if one found otherwise returns an empty string.</returns>
        /// <remarks>If you use MapPath to map a supplied virtual path to a physical path on the server, use the overloaded Request.MapPath method that accepts a bool parameter so that you can prevent cross-application mapping.</remarks>
        public virtual string MapFilePath(string path)
        {
            string filePath = string.Empty;
            if (string.IsNullOrEmpty(path))
            {
                return string.Empty;
            }

            try
            {
                if (this.CurrentContext != null)
                {
                    filePath = this.CurrentContext.Request.MapPath(path, this.CurrentContext.Request.ApplicationPath, false);
                }
                else
                {
                    filePath = new DirectoryInfo(path).FullName;
                }
            }
            catch (HttpException httpException)
            {
                CommerceServerContribExceptionManager.HandleException(httpException);
            }

            return filePath;
        }

        /// <summary>
        /// Gets the web part property validation exception that will be thrown.
        /// </summary>
        /// <param name="message">The message of the exception.</param>
        /// <returns>Returns the webpart property validation exception.</returns>
        public virtual Exception GetWebPartPropertyValidationException(string message)
        {
            return new ArgumentException(message);
        }
    }
}
