﻿//-----------------------------------------------------------------------
// <copyright file="CatalogImageManager.cs" company="commerceserver.net">
//     Copyright © commerceserver.net.  All rights reserved.
// </copyright>
// <summary>CatalogImageManager class.</summary>
//-----------------------------------------------------------------------
namespace CommerceServerContrib.Web
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Xml;
    using System.Xml.XPath;
    using CommerceServerContrib.CrossTier.Entities;
    using CommerceServerContrib.CrossTier.Interfaces;

    /// <summary>
    /// Handles emission of catalog image urls which reference an Azure Content Deployment Network URL
    /// </summary>
    public class CatalogImageManager : ICatalogImageManager, IRenderExtensionObjectProvider
    {
        #region Private Fields

        private const string SizeM = "m";
        private const string SizeL = "l";
        private const string SizeXl = "xl";
        private const string SizeS = "s";

        private const string SizeMQualifier = "_m";
        private const string SizeLQualifier = "_l";
        private const string SizeXlQualifier = "_xl";
        private const string SizeSQualifier = "_s";

        private const string FileExtensionSeperator = ".";

        private const string ImagesXmlRootNode = "images";
        private const string ImageXmlNodeName = "image";
        private const string NameAttributeName = "name";
        private const string PathAttributeName = "path";
        private const string ImageXPath = "/images/image";

        private string catalogDelimiter = "|";
        private Uri _baseUrl;

        #endregion

        #region Constructor

        /// <summary>
        /// Initializes a new instance of the <see cref="CatalogImageManager"/> class.
        /// </summary>
        /// <param name="baseUrl">The base URL.</param>
        /// <param name="catalogDelimited">The catalog delimited.</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1054:UriParametersShouldNotBeStrings", MessageId = "0#", Justification = "We use urls as strings in the solution")]
        public CatalogImageManager(string baseUrl, string catalogDelimited)
        {
            Uri baseUri;
            var isValid = Uri.TryCreate(baseUrl, UriKind.RelativeOrAbsolute, out baseUri);
            if (!isValid)
            {
                throw new UriFormatException("Invalid CatalogImageManager base url format");
            }

            this.BaseUrl = baseUri;
            this.CatalogImageDelimiter = catalogDelimited ?? this.CatalogImageDelimiter;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="CatalogImageManager"/> class.
        /// </summary>
        /// <param name="catalogDelimited">The catalog delimited.</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1054:UriParametersShouldNotBeStrings", MessageId = "0#", Justification = "By design")]
        public CatalogImageManager(string catalogDelimited)
            : this(CdnHelper.GetBaseUrlForArea(CdnArea.CatalogCDN), catalogDelimited)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="CatalogImageManager"/> class.
        /// </summary>
        public CatalogImageManager()
            : this(CdnHelper.GetBaseUrlForArea(CdnArea.CatalogCDN), PresentationServiceLocator.Current.SiteSettingProvider.GetFromWebSetting("CatalogImageDelimiter"))
        {
        }
        #endregion

        #region Public Properties

        /// <summary>
        /// Gets the name of the extension object.
        /// </summary>
        public string ExtensionObjectName
        {
            get { return "imageManager"; }
        }

        #endregion

        #region ICatalogImageManager Members

        /// <summary>
        /// Gets or sets the base URL.
        /// </summary>
        /// <value>
        /// The base URL.
        /// </value>
        public Uri BaseUrl
        {
            get
            {
                return CdnHelper.AdjustUrlScheme(this._baseUrl);
            }

            set
            {
                this._baseUrl = value;
            }
        }

        /// <summary>
        /// Gets or sets the catalog image delimiter.
        /// </summary>
        /// <value>The catalog image delimiter.</value>
        public string CatalogImageDelimiter
        {
            get
            {
                return this.catalogDelimiter;
            }

            set
            {
                this.catalogDelimiter = value;
            }
        }

        /// <summary>
        /// Gets the image with img extension.
        /// </summary>
        /// <param name="imageName">Name of the image.</param>
        /// <param name="imageSize">Size of the image.</param>
        /// <param name="baseUrl">The base URL.</param>
        /// <returns>Returns the relative image path.</returns>
        public string GetCatalogImagePath(string imageName, string imageSize, Uri baseUrl)
        {
            if (string.IsNullOrEmpty(imageName) || string.IsNullOrEmpty(imageSize) || baseUrl == null || !Uri.IsWellFormedUriString(baseUrl.ToString(), UriKind.RelativeOrAbsolute))
            {
                return string.Empty;
            }

            string imageSizeQualifier;
            string imageExtension = GetImageExtension(imageName);
            switch (imageSize.ToLowerInvariant())
            {
                case (SizeM):
                    imageSizeQualifier = SizeMQualifier;
                    break;
                case (SizeL):
                    imageSizeQualifier = SizeLQualifier;
                    break;
                case (SizeXl):
                    imageSizeQualifier = SizeXlQualifier;
                    break;
                case (SizeS):
                    imageSizeQualifier = SizeSQualifier;
                    break;
                default:
                    imageSizeQualifier = string.Empty;
                    break;
            }

            return string.Concat(baseUrl, imageName.Replace(imageExtension, string.Concat(imageSizeQualifier, imageExtension)));
        }

        /// <summary>
        /// Gets the image with img extension.
        /// </summary>
        /// <param name="imageName">Name of the image.</param>
        /// <param name="imageSize">Size of the image.</param>
        /// <returns>Returns the relative image path.</returns>
        public string GetCatalogImagePath(string imageName, string imageSize)
        {
            return this.GetCatalogImagePath(imageName, imageSize, this.BaseUrl);
        }

        /// <summary>
        /// Gets the first catalog image for XSL transform.
        /// </summary>
        /// <param name="delimitedCatalogImageName">Name of the delimited catalog image.</param>
        /// <param name="imageSize">Size of the image.</param>
        /// <param name="baseUrl">The base URL.</param>
        /// <returns>
        /// Image name for XSLT
        /// </returns>
        public string GetFirstCatalogImagePath(string delimitedCatalogImageName, string imageSize, Uri baseUrl)
        {
            var firstImageName = this.GetList(delimitedCatalogImageName);
            if (firstImageName.Count > 0 && !string.IsNullOrEmpty(firstImageName[0]))
            {
                return this.GetCatalogImagePath(firstImageName[0], imageSize, baseUrl);
            }

            return string.Empty;
        }

        /// <summary>
        /// Gets the first catalog image for XSL transform.
        /// </summary>
        /// <param name="delimitedCatalogImageName">Name of the delimited catalog image.</param>
        /// <param name="imageSize">Size of the image.</param>
        /// <returns>
        /// Image name for XSLT
        /// </returns>
        public string GetFirstCatalogImagePath(string delimitedCatalogImageName, string imageSize)
        {
            return this.GetFirstCatalogImagePath(delimitedCatalogImageName, imageSize, this.BaseUrl);
        }

        /// <summary>
        /// Ges the catalog images for XSL transform.
        /// </summary>
        /// <param name="delimitedCatalogImageName">Name of the delimited catalog image.</param>
        /// <param name="imageSize">Size of the image.</param>
        /// <param name="baseUrl">The base URL.</param>
        /// <returns>
        /// Returns image paths for XSLT
        /// </returns>
        public XPathNodeIterator GetAlternativeCatalogImagePaths(string delimitedCatalogImageName, string imageSize, Uri baseUrl)
        {
            var imagesXml = new XmlDocument();
            XmlElement rootNode = imagesXml.CreateElement(ImagesXmlRootNode);
            imagesXml.AppendChild(rootNode);
            XPathNavigator navigator = imagesXml.CreateNavigator();

            var imageNames = this.GetList(delimitedCatalogImageName);
            if (imageNames.Count <= 1)
            {
                return navigator.Select(ImageXPath);
            }

            imageNames.RemoveAt(0);
            foreach (string imageName in imageNames)
            {
                var imagePath = this.GetCatalogImagePath(imageName, imageSize, baseUrl);
                XmlElement imageXmlNode = imagesXml.CreateElement(ImageXmlNodeName);
                imageXmlNode.SetAttribute(NameAttributeName, imageName);
                imageXmlNode.SetAttribute(PathAttributeName, imagePath);
                rootNode.AppendChild(imageXmlNode);
            }

            return navigator.Select(ImageXPath);
        }

        /// <summary>
        /// Ges the catalog images for XSL transform.
        /// </summary>
        /// <param name="delimitedCatalogImageName">Name of the delimited catalog image.</param>
        /// <param name="imageSize">Size of the image.</param>
        /// <returns>
        /// Returns image paths for XSLT
        /// </returns>
        public XPathNodeIterator GetAlternativeCatalogImagePaths(string delimitedCatalogImageName, string imageSize)
        {
            return this.GetAlternativeCatalogImagePaths(delimitedCatalogImageName, imageSize, this.BaseUrl);
        }

        /// <summary>
        /// Ges the catalog images for XSL transform.
        /// </summary>
        /// <param name="delimitedCatalogImageName">Name of the delimited catalog image.</param>
        /// <param name="imageSize">Size of the image.</param>
        /// <param name="baseUrl">The base URL.</param>
        /// <returns>
        /// Returns image paths for XSLT
        /// </returns>
        public List<string> GetAlternativeCatalogImagePathsNoXml(string delimitedCatalogImageName, string imageSize, Uri baseUrl)
        {
            var imageNames = this.GetList(delimitedCatalogImageName);
            var imagesToReturn = new List<string>();

            if (imageNames != null && imageNames.Count > 0)
            {
                imageNames.RemoveAt(0); 
            }

            foreach (string imageName in imageNames)
            {
                var imagePath = this.GetCatalogImagePath(imageName, imageSize, baseUrl);

                imagesToReturn.Add(imagePath);
            }

            return imagesToReturn;
        }

        /// <summary>
        /// Ges the catalog images for XSL transform.
        /// </summary>
        /// <param name="delimitedCatalogImageName">Name of the delimited catalog image.</param>
        /// <param name="imageSize">Size of the image.</param>
        /// <returns>
        /// Returns image paths for XSLT
        /// </returns>
        public List<string> GetAlternativeCatalogImagePathsNoXml(string delimitedCatalogImageName, string imageSize)
        {
            return this.GetAlternativeCatalogImagePathsNoXml(delimitedCatalogImageName, imageSize, this.BaseUrl);
        }

        #endregion

        #region Implementation of IRenderExtensionObjectProvider

        /// <summary>
        /// Gets the XSLT extension objects.
        /// </summary>
        /// <returns>Returns an XsltExtensionObjects which is a collection of RenderExtensionObject.</returns>
        public RenderExtensionObjects GetRenderExtensionObjects()
        {
            var xsltExtensionObjects = new RenderExtensionObjects();
            xsltExtensionObjects.Add(new RenderExtensionObject(this.ExtensionObjectName, this));
            
            return xsltExtensionObjects;
        }

        #endregion

        #region Private Methods

        /// <summary>
        /// Gets the image extension.
        /// </summary>
        /// <param name="imageName">Name of the image.</param>
        /// <returns>returns the image extension from a image file name.</returns>
        private static string GetImageExtension(string imageName)
        {
            return string.IsNullOrEmpty(imageName) || !imageName.Contains(FileExtensionSeperator) ? ".txt" : imageName.Substring(imageName.IndexOf(FileExtensionSeperator, StringComparison.OrdinalIgnoreCase));
        }

        /// <summary>
        /// Gets the list.
        /// </summary>
        /// <param name="delimitedString">The delimited string.</param>
        /// <returns>A list for the delimited string</returns>
        private List<string> GetList(string delimitedString)
        {
            if (string.IsNullOrEmpty(delimitedString))
            {
                return new List<string>();
            }

            var itemsArray = delimitedString.Split(this.CatalogImageDelimiter.ToCharArray());
            var itemsList = new List<string>();
            if (itemsArray.Length > 0)
            {
                itemsList.AddRange(itemsArray.Select(item => item.Trim()));
            }

            return itemsList.Count > 0 ? itemsList : new List<string>();
        }

        #endregion
    }
}