﻿//---------------------------------------------------------------------
// <copyright file="DiskFileProvider.cs" company="commerceserver.net">
//     Copyright © commerceserver.net.  All rights reserved.
// </copyright>
// <summary></summary>
//---------------------------------------------------------------------
namespace CommerceServerContrib.Web
{
    using System.Collections.Generic;
    using System.Globalization;
    using System.IO;
    using CommerceServerContrib.CrossTier.Interfaces;
    using CommerceServerContrib.CrossTier.Providers;
    using CommerceServerContrib.CrossTier.Helpers;
    using CommerceServer.Foundation;

    /// <summary>
    /// Used to retrieve files from disk relative to the currently running website. 
    /// </summary>
    public class DiskFileProvider : IFileProvider
    {
        #region Static Fields

        private static object _fileContentsDictionaryLock = new object();

        #endregion

        #region IFileProvider Members

        /// <summary>
        /// Indicates if the file exists or not.
        /// </summary>
        /// <param name="path">The path to the file.</param>
        /// <returns>true if the file exists; otherwise false.</returns>
        public bool FileExists(string path)
        {
            return File.Exists(this.GetFullPath(path));
        }

        /// <summary>
        /// Gets the file in string format.
        /// </summary>
        /// <param name="path">The path to the file</param>
        /// <returns>The requested file in string form</returns>
        public string GetFileAsString(string path)
        {
            var fileContents = string.Empty;

            if (!string.IsNullOrEmpty(path))
            {                
                // get the full path of the file
                var fullPath = this.GetFullPath(path);                

                if (File.Exists(fullPath))
                {
                    var cacheContext = PresentationCacheSettings.GetCacheContext(CommerceServerContrib.CrossTier.Helpers.CacheConstants.DiskFileCache);

                    if (cacheContext != null && cacheContext.IsCachingEnabled)
                    {
                        var cacheProvider = PresentationServiceLocator.Current.CacheProvider(cacheContext.CacheType);
                        var brandName = PresentationServiceLocator.Current.BrandContext.Name;

                        fileContents = cacheProvider.GetData<string>(brandName, cacheContext.CacheName, fullPath);

                        if (fileContents == null)
                        {
                            lock (_fileContentsDictionaryLock)
                            {
                                fileContents = cacheProvider.GetData<string>(brandName, cacheContext.CacheName, fullPath);

                                if (fileContents == null)
                                {
                                    fileContents = this.RetrieveFromDisk(fullPath);

                                    cacheProvider.AddData<string>(brandName, cacheContext.CacheName, fullPath, fileContents);
                                }
                            }
                        } 
                    }
                    else
                    {
                        fileContents = this.RetrieveFromDisk(fullPath);
                    }
                }
                else
                {
                    throw new System.IO.FileNotFoundException(string.Format(CultureInfo.InvariantCulture, ResourceHelper.GetCommerceServerContrib("CannotFindFile"), fullPath), fullPath);
                }
            }

            return fileContents;
        }

        #endregion

        /// <summary>
        /// Retrieves from disk.
        /// </summary>
        /// <param name="fullPath">The full path.</param>
        /// <returns>A the contents of the requested file</returns>
        private string RetrieveFromDisk(string fullPath)
        {
            if (!string.IsNullOrEmpty(fullPath))
            {
                using (StreamReader reader = File.OpenText(fullPath))
                {
                    var fileContents = reader.ReadToEnd();

                    return fileContents;
                }
            }

            return null;
        }

        /// <summary>
        /// Gets the full path.
        /// </summary>
        /// <param name="path">The relative path.</param>
        /// <returns>Teh xslt full path</returns>
        private string GetFullPath(string path)
        {
            var siteContext = PresentationServiceLocator.Current.SiteContext;
            if (siteContext.CurrentContext != null && !path.StartsWith("~", System.StringComparison.OrdinalIgnoreCase))
            {
                path = @"~\" + path;
            }

            return siteContext.MapFilePath(path);
        }
    }
}
