﻿//---------------------------------------------------------------------
// <copyright file="DiskXsltProvider.cs" company="commerceserver.net">
//     Copyright © commerceserver.net.  All rights reserved.
// </copyright>
// <summary></summary>
//---------------------------------------------------------------------
namespace CommerceServerContrib.Web
{
    using System.Globalization;
    using System.IO;
    using System.Net;
    using System.Xml;
    using System.Xml.Xsl;
    using CommerceServerContrib.CrossTier.Helpers;
    using CommerceServerContrib.CrossTier.Interfaces;
    using CommerceServerContrib.CrossTier.Providers;
    using System;
    using CommerceServerContrib.CrossTier.Configuration;
    using CommerceServer.Foundation;

    /// <summary>
    /// Used to retrieve Xslts from disk relative to the currently running website. There
    /// is no caching at this time, but this should be added
    /// </summary>
    public class DiskXsltProvider : XmlResolver, IXsltProvider
    {
        #region Fields

        private static object _xsltDictionaryLock = new object();
        private ICredentials _credentials;

        #endregion

        #region XmlResolver Members

        /// <summary>
        /// When overridden in a derived class, sets the credentials used to authenticate Web requests.
        /// </summary>
        /// <returns>An <see cref="T:System.Net.ICredentials"/> object. If this property is not set, the value defaults to null; that is, the XmlResolver has no user credentials.</returns>
        public override ICredentials Credentials
        {
            set
            {
                this._credentials = value;
            }
        }

        /// <summary>
        /// When overridden in a derived class, maps a URI to an object containing the actual resource.
        /// </summary>
        /// <param name="absoluteUri">The URI returned from <see cref="M:System.Xml.XmlResolver.ResolveUri(System.Uri,System.String)"/>.</param>
        /// <param name="role">The current version does not use this parameter when resolving URIs. This is provided for future extensibility purposes. For example, this can be mapped to the xlink:role and used as an implementation specific argument in other scenarios.</param>
        /// <param name="ofObjectToReturn">The type of object to return. The current version only returns System.IO.Stream objects.</param>
        /// <returns>
        /// A System.IO.Stream object or null if a type other than stream is specified.
        /// </returns>
        /// <exception cref="T:System.Xml.XmlException">
        ///   <paramref name="ofObjectToReturn"/> is neither null nor a Stream type. </exception>
        ///   
        /// <exception cref="T:System.UriFormatException">The specified URI is not an absolute URI. </exception>
        ///   
        /// <exception cref="T:System.NullReferenceException">
        ///   <paramref name="absoluteUri"/> is null. </exception>
        ///   
        /// <exception cref="T:System.Exception">There is a runtime error (for example, an interrupted server connection). </exception>
        public override object GetEntity(Uri absoluteUri, string role, Type ofObjectToReturn)
        {
           return File.OpenRead(absoluteUri.OriginalString);
        }

        /// <summary>
        /// When overridden in a derived class, resolves the absolute URI from the base and relative URIs.
        /// </summary>
        /// <param name="baseUri">The base URI used to resolve the relative URI</param>
        /// <param name="relativeUri">The URI to resolve. The URI can be absolute or relative. If absolute, this value effectively replaces the <paramref name="baseUri"/> value. If relative, it combines with the <paramref name="baseUri"/> to make an absolute URI.</param>
        /// <returns>
        /// A <see cref="T:System.Uri"/> representing the absolute URI or null if the relative URI cannot be resolved.
        /// </returns>
        /// <exception cref="T:System.ArgumentException">
        /// <paramref name="relativeUri"/> is null</exception>
        public override System.Uri ResolveUri(System.Uri baseUri, string relativeUri)
        {
            var fullPath = this.GetFullPath(relativeUri);

            return new Uri(fullPath, UriKind.Absolute);
        }

        #endregion

        #region IXsltProvider Members

        /// <summary>
        /// Gets the requested Xslt
        /// </summary>
        /// <param name="path">The path to the XSLT relative to the root of the site</param>
        /// <returns>The requested Xslt in XslCompiledTransform form</returns>
        public System.Xml.Xsl.XslCompiledTransform GetXslt(string path)
        {
            XslCompiledTransform xslt = null;

            if (!string.IsNullOrEmpty(path))
            {
                // get the full path of the Xslt
                var fullPath = this.GetFullPath(path);

                if (File.Exists(fullPath))
                {
                    var cacheContext = PresentationCacheSettings.GetCacheContext(CommerceServerContrib.CrossTier.Helpers.CacheConstants.XslCompiledTransform);

                    if (cacheContext != null && cacheContext.IsCachingEnabled)
                    {
                        var cacheProvider = PresentationServiceLocator.Current.CacheProvider(cacheContext.CacheType);
                        var brandName = PresentationServiceLocator.Current.BrandContext.Name;

                        xslt = cacheProvider.GetData<XslCompiledTransform>(brandName, cacheContext.CacheName, fullPath);

                        if (xslt == null)
                        {
                            lock (_xsltDictionaryLock)
                            {
                                xslt = cacheProvider.GetData<XslCompiledTransform>(brandName, cacheContext.CacheName, fullPath);
                                if (xslt == null)
                                {
                                    xslt = this.RetrieveFromDisk(fullPath);

                                    cacheProvider.AddData<XslCompiledTransform>(brandName, cacheContext.CacheName, fullPath, xslt);
                                }
                            }
                        }
                    }
                    else
                    {
                        xslt = this.RetrieveFromDisk(fullPath);
                    }
                }
                else
                {
                    throw new System.IO.FileNotFoundException(string.Format(CultureInfo.InvariantCulture, ResourceHelper.GetCommerceServerContrib("CannotFindXslt"), fullPath), fullPath);
                }
            }

            return xslt;
        }

        /// <summary>
        /// Indicates if the Xslt file exists or not.
        /// </summary>
        /// <param name="path">The path to the xslt file.</param>
        /// <returns>true if the fil exists; otherwise false.</returns>
        public bool XsltFileExists(string path)
        {
            return File.Exists(this.GetFullPath(path));
        }

        #endregion

        #region Private Members

        /// <summary>
        /// Gets the full path.
        /// </summary>
        /// <param name="path">The relative path.</param>
        /// <returns>Teh xslt full path</returns>
        private string GetFullPath(string path)
        {
            var siteContext = PresentationServiceLocator.Current.SiteContext;
            if (siteContext.CurrentContext != null && !path.StartsWith("~", System.StringComparison.OrdinalIgnoreCase))
            {
                path = @"~\" + path;
            }

            return siteContext.MapFilePath(path);
        }

        /// <summary>
        /// Retrieves from disk.
        /// </summary>
        /// <param name="fullPath">The full path.</param>
        /// <returns>A compiled version of the xslt</returns>
        private XslCompiledTransform RetrieveFromDisk(string fullPath)
        {
            if (!string.IsNullOrEmpty(fullPath))
            {
                var xslt = new XslCompiledTransform(CommerceServerContribConfiguration.Presentation.EnableXsltDebugging);
                var xsltDoc = new XmlDocument();

                xsltDoc.Load(fullPath);

                var xsltStr = xsltDoc.OuterXml;
                var strReader = new System.IO.StringReader(xsltStr);

                using (var reader = XmlReader.Create(strReader))
                {
                    xslt.Load(reader, new XsltSettings(), this);
                }

                return xslt;
            }

            return null;
        }

        #endregion
    }
}
