﻿//---------------------------------------------------------------------
// <copyright file="QueryStringProvider.cs" company="commerceserver.net">
//     Copyright © commerceserver.net.  All rights reserved.
// </copyright>
// <summary>QueryStringProvider class</summary>
//---------------------------------------------------------------------
namespace CommerceServerContrib.Web
{
    using System;
    using CommerceServerContrib.CrossTier.Entities;
    using System.Collections.Generic;
    using CommerceServerContrib.CrossTier.Interfaces;
    using CommerceServerContrib.CrossTier.Exceptions;

    /// <summary>
    /// Interface implementation that returns the valid query string and site map query string parameters.
    /// </summary>
    public class QueryStringProvider : IQueryStringProvider
    {
        private static readonly Dictionary<string, QueryStringParameter> QueryStringParameterDictionary = new Dictionary<string, QueryStringParameter>();

        #region Static Constructor

        static QueryStringProvider()
        {
            var settingResponse = PresentationServiceLocator.Current.SiteSettingProvider.GetSiteSettings("QueryKeyList");
            if (settingResponse.Count <= 0)
            {
                return;
            }

            foreach (SiteSettingsResponse sr in settingResponse)
            {
                var parm = new QueryStringParameter(sr.Key, sr.Value);
                if (sr.LocalizedValues != null && sr.LocalizedValues.Count > 0)
                {
                    parm.LocalizedDescriptions = sr.LocalizedValues;
                }

                QueryStringParameterDictionary[parm.Key] = parm;
            }
        }

        #endregion

        #region IQueryStringProvider Implementation

        /// <summary>
        /// Gets the query strings that are supported by the site.
        /// </summary>
        /// <returns>
        /// A list of query string parameters.
        /// </returns>
        public Dictionary<string, QueryStringParameter> GetQueryStrings()
        {
            return QueryStringParameterDictionary;
        }

        /// <summary>
        /// Gets the query string.
        /// </summary>
        /// <param name="queryParameterKey">The query parameter key.</param>
        /// <returns>The QueryStringParameter instance for the given query string parameter key.</returns>
        public QueryStringParameter GetQueryString(string queryParameterKey)
        {
            if (!QueryStringParameterDictionary.ContainsKey(queryParameterKey))
            {
                throw new QueryParameterKeyNotFoundException(queryParameterKey);
            }

            return QueryStringParameterDictionary[queryParameterKey];
        }

        #endregion

        #region Implementation of IRenderParameterProvider

        /// <summary>
        /// Gets the XSLT parameters.
        /// </summary>
        /// <returns>Returns all the Query Keys in a list to be used by the base web part.</returns>
        public RenderParameters GetRenderParameters()
        {
            var parameters = new RenderParameters();
            foreach (var key in QueryStringParameterDictionary.Keys)
            {
                QueryStringParameter queryStringParameter = QueryStringParameterDictionary[key];
                string paramKey = (queryStringParameter.Key.StartsWith("_", StringComparison.OrdinalIgnoreCase)
                                       ? queryStringParameter.QueryParameterKey
                                       : string.Concat("_", queryStringParameter.Key));
                parameters.Add(new RenderParameter(paramKey, string.Empty, queryStringParameter.QueryParameterKey), false);
            }

            return parameters;
        }

        #endregion
    }
}
