﻿//-----------------------------------------------------------------------
// <copyright file="ResourceProvider.cs" company="commerceserver.net">
//     Copyright © commerceserver.net.  All rights reserved.
// </copyright>
// <summary>ResourceProvider class.</summary>
//-----------------------------------------------------------------------
namespace CommerceServerContrib.Web
{
    using System.Collections;
    using System.Collections.Generic;
    using System.Globalization;
    using System.IO;
    using System.Linq;
    using System.Resources;
    using CommerceServerContrib.CrossTier.Helpers;
    using CommerceServerContrib.CrossTier.Interfaces;
    using System;
    using CommerceServerContrib.CrossTier.Providers;
    using CommerceServer.Foundation;

    /// <summary>
    /// Used to retrive values from resource files
    /// </summary>
    public class ResourceProvider : IResourceProvider
    {
        #region Private Fields

        private const string DefaultCacheName = "ResourceProviderCache";
        private const string ResxFileExtension = ".resx";
        private const string ResourcesCachedKey = "ResourceProvider_{0}";
        private const string ResourceNamesCachedKey = "ResourceProvider_ResourceFileNames";
        private const string ResourceNotFound = "[{0}].{1}";

        private static object _resourceFilenameLock = new object();
        private static object _resourceDictionaryLock = new object();
        private static FileSystemWatcher _directoryWatcher;

        private readonly string _absolutePath;

        #endregion

        #region Constructor

        /// <summary>
        /// Initializes a new instance of the <see cref="ResourceProvider"/> class.
        /// </summary>
        /// <param name="directoryName">Name of the directory.</param>
        public ResourceProvider(string directoryName)
        {
            ParameterChecker.CheckForNullOrEmpty(directoryName, "directoryName");
            this._absolutePath = PresentationServiceLocator.Current.SiteContext.MapFilePath(directoryName);
            ParameterChecker.CheckForNullOrEmpty(this._absolutePath, "this._absolutePath");
            
            if (!Directory.Exists(this._absolutePath))
            {
                throw new DirectoryNotFoundException(directoryName + "not found");
            }

            _directoryWatcher = new FileSystemWatcher(this._absolutePath)
                                    {
                                        NotifyFilter = NotifyFilters.LastAccess | NotifyFilters.LastWrite | NotifyFilters.Attributes | NotifyFilters.Size,
                                        Filter = string.Concat("*", ResxFileExtension)
                                    };
            _directoryWatcher.Changed += OnFileChange;
            _directoryWatcher.EnableRaisingEvents = true;
        }

        #endregion

        #region IResourceProvider Members

        /// <summary>
        /// Gets the resource.
        /// </summary>
        /// <param name="location">The location.</param>
        /// <param name="key">The key to search for.</param>
        /// <returns>
        /// The specific resource by key and location
        /// </returns>
        public string GetResource(string location, string key)
        {
            return this.GetResource(location, key, PresentationServiceLocator.Current.SiteContext.UICulture);
        }

        /// <summary>
        /// Gets the resource.
        /// </summary>
        /// <param name="location">The location.</param>
        /// <param name="key">The key to search for.</param>
        /// <param name="cultureInfo">The culture info.</param>
        /// <returns>
        /// The specific resource by key and location
        /// </returns>
        public string GetResource(string location, string key, CultureInfo cultureInfo)
        {
            var resourceNotFound = string.Format(CultureInfo.InvariantCulture, ResourceNotFound, location, key);
            if (string.IsNullOrEmpty(location) || string.IsNullOrEmpty(key))
            {
                return resourceNotFound;
            }

            var cacheContext = PresentationCacheSettings.GetCacheContext(CommerceServerContrib.CrossTier.Helpers.CacheConstants.ResourceCache);
            var cacheProvider = PresentationTypeLoader.CacheProvider(cacheContext);

            var resourceFileName = this.GetResourceFileName(cacheProvider, cacheContext, location, cultureInfo);
            if (string.IsNullOrEmpty(resourceFileName))
            {
                return resourceNotFound;
            }

            var fullLocation = Path.Combine(this._absolutePath, resourceFileName);
            var cacheKey = string.Format(CultureInfo.InvariantCulture, ResourcesCachedKey, fullLocation);

            Dictionary<string, object> resourceDictionaryEntry;

            string brandName = PresentationServiceLocator.Current.BrandContext.Name;
            resourceDictionaryEntry = cacheProvider.GetData<Dictionary<string, object>>(brandName, cacheContext.CacheName, cacheKey);
            if (resourceDictionaryEntry == null)
            {
                lock (_resourceDictionaryLock)
                {
                    resourceDictionaryEntry = cacheProvider.GetData<Dictionary<string, object>>(brandName, cacheContext.CacheName, cacheKey);
                    if (resourceDictionaryEntry == null)
                    {
                        resourceDictionaryEntry = this.LoadResourceFile(fullLocation);
                        cacheProvider.AddData<Dictionary<string, object>>(brandName, cacheContext.CacheName, cacheKey, resourceDictionaryEntry);
                    }
                }
            }

            // If the key does not exist in this resource, let's check the parent culture (resource file) for it.
            if (!resourceDictionaryEntry.ContainsKey(key) && !string.IsNullOrEmpty(cultureInfo.Parent.Name))
            {
                return this.GetResource(location, key, cultureInfo.Parent);
            }

            return (resourceDictionaryEntry.ContainsKey(key) && resourceDictionaryEntry[key] != null) ? resourceDictionaryEntry[key].ToString() : resourceNotFound;
        }

        /// <summary>
        /// Gets the resources.
        /// </summary>
        /// <returns>
        /// All the resources defined for the current UICulture
        /// </returns>
        public List<string> GetResources()
        {
            var cacheContext = PresentationCacheSettings.GetCacheContext(CommerceServerContrib.CrossTier.Helpers.CacheConstants.ResourceCache);
            var cacheProvider = PresentationServiceLocator.Current.CacheProvider(cacheContext.CacheType);
            var brand = PresentationServiceLocator.Current.BrandContext.Name;

            var resources = new List<string>();
            var cultureInfo = PresentationServiceLocator.Current.SiteContext.UICulture;
            var directoryInfo = new DirectoryInfo(this._absolutePath);
            var searchPattern = string.Concat("*", ".", cultureInfo, ResxFileExtension);

            if (directoryInfo.Exists)
            {
                foreach (var file in directoryInfo.GetFiles(searchPattern))
                {
                    Dictionary<string, object> resourceDictionaryEntry;
                    var cacheKey = string.Format(CultureInfo.InvariantCulture, ResourcesCachedKey, file.FullName);

                    resourceDictionaryEntry = cacheProvider.GetData<Dictionary<string, object>>(brand, cacheContext.CacheName, cacheKey);
                    if (resourceDictionaryEntry == null)
                    {
                        lock (_resourceDictionaryLock)
                        {
                            resourceDictionaryEntry = cacheProvider.GetData<Dictionary<string, object>>(brand, cacheContext.CacheName, cacheKey);
                            if (resourceDictionaryEntry == null)
                            {
                                using (var reader = new ResXResourceReader(file.FullName))
                                {
                                    resourceDictionaryEntry = reader.Cast<DictionaryEntry>().Aggregate(new Dictionary<string, object>(), (d, e) => { d.Add(e.Key.ToString(), e.Value); return d; });
                                    cacheProvider.AddData<Dictionary<string, object>>(brand, cacheContext.CacheName, cacheKey, resourceDictionaryEntry);
                                }
                            }
                        }
                    }

                    resources.AddRange(resourceDictionaryEntry.Keys.Select(key => resourceDictionaryEntry[key].ToString()));
                }
            }

            return resources;
        }

        #endregion

        #region Methods (public, virtual)

        /// <summary>
        /// Gets the name of the resource file.
        /// </summary>
        /// <param name="cacheProvider">The cache provider.</param>
        /// <param name="cacheContext">The cache context.</param>
        /// <param name="location">The resource location.</param>
        /// <param name="cultureInfo">The culture info.</param>
        /// <returns>
        /// the resource file name
        /// </returns>
        public virtual string GetResourceFileName(ICacheProvider cacheProvider, CacheContext cacheContext, string location, CultureInfo cultureInfo)
        {
            ParameterChecker.CheckForNullOrEmpty(location, "location");
            ParameterChecker.CheckForNull(cultureInfo, "cultureInfo");

            string cacheKey = String.Concat(PresentationServiceLocator.Current.BrandContext.Name, "_", location, "_", cultureInfo.Name);
            string resourceFilename = cacheProvider.GetData<string>(PresentationServiceLocator.Current.BrandContext.Name, cacheContext.CacheName, cacheKey);
            if (string.IsNullOrEmpty(resourceFilename))
            {
                lock (_resourceFilenameLock)
                {
                    resourceFilename = cacheProvider.GetData<string>(PresentationServiceLocator.Current.BrandContext.Name, cacheContext.CacheName, cacheKey);
                    if (string.IsNullOrEmpty(resourceFilename))
                    {
                        resourceFilename = this.InternalGetResourceFilename(location, cultureInfo);
                        cacheProvider.AddData<string>(PresentationServiceLocator.Current.BrandContext.Name, cacheContext.CacheName, cacheKey, resourceFilename);
                    }
                }
            }

            return resourceFilename;
        }

        #endregion

        #region Methids (private, static)

        private static void OnFileChange(object sender, FileSystemEventArgs args)
        {
            var cacheContext = PresentationCacheSettings.GetCacheContext(CommerceServerContrib.CrossTier.Helpers.CacheConstants.ResourceCache);
            var cacheProvider = PresentationServiceLocator.Current.CacheProvider(cacheContext.CacheType);

            var cacheKey = string.Format(CultureInfo.InvariantCulture, ResourcesCachedKey, args.FullPath);
            Dictionary<string, object> resourceDictionaryEntry;

            string brandName = PresentationServiceLocator.Current.BrandContext.Name;
            resourceDictionaryEntry = cacheProvider.GetData<Dictionary<string, object>>(brandName, cacheContext.CacheName, cacheKey);
            if (resourceDictionaryEntry == null)
            {
                lock (_resourceDictionaryLock)
                {
                    resourceDictionaryEntry = cacheProvider.GetData<Dictionary<string, object>>(brandName, cacheContext.CacheName, cacheKey);
                    if (resourceDictionaryEntry == null)
                    {
                        cacheProvider.RemoveData(brandName, cacheContext.CacheName, cacheKey);
                    }
                }
            }
        }

        #endregion

        #region Private Methods

        private string InternalGetResourceFilename(string location, CultureInfo cultureInfo)
        {
            ParameterChecker.CheckForNullOrEmpty(location, "location");
            ParameterChecker.CheckForNull(cultureInfo, "cultureInfo"); 

            string resourceFileName = !string.IsNullOrEmpty(cultureInfo.Name) ? string.Concat(location, ".", cultureInfo, ResxFileExtension) : string.Concat(location, ResxFileExtension);
            var fullPath = Path.Combine(this._absolutePath, resourceFileName);

            if (!File.Exists(fullPath))
            {
                if (!string.IsNullOrEmpty(cultureInfo.Name))
                {
                    return this.InternalGetResourceFilename(location, cultureInfo.Parent);
                }

                resourceFileName = string.Empty;
            }

            return resourceFileName;
        }

        private Dictionary<string, object> LoadResourceFile(string absoluteFilename)
        {
            Dictionary<string, object> resourceDictionary;

            using (var reader = new ResXResourceReader(absoluteFilename))
            {
                resourceDictionary = reader.Cast<DictionaryEntry>().Aggregate(new Dictionary<string, object>(), (d, e) => { d.Add(e.Key.ToString(), e.Value); return d; });
            }

            return resourceDictionary;
        }

        #endregion
    }
}