﻿//---------------------------------------------------------------------
// <copyright file="SiteSettingsProvider.cs" company="commerceserver.net">
//     Copyright © commerceserver.net.  All rights reserved.
// </copyright>
// <summary>A helper that allows you to query site settings.</summary>
//---------------------------------------------------------------------

namespace CommerceServerContrib.Web
{
    using System.Collections.Generic;
    using System.Configuration;
    using System.Globalization;
    using CommerceServer.Foundation;
    using CommerceServerContrib.CrossTier.Configuration;
    using CommerceServerContrib.CrossTier.Entities;
    using CommerceServerContrib.CrossTier.Interfaces;
    using CSCC = CommerceServerContrib.CrossTier.Configuration;

    /// <summary>
    /// A SiteSettings provider that goes against the default configuration.  
    /// </summary>
    public class SiteSettingsProvider : ISiteSettingsProvider
    {
        /// <summary>
        /// Gets the web setting.
        /// </summary>
        /// <param name="key">The key to search for.</param>
        /// <param name="groupId">The group id.</param>
        /// <returns>
        /// A string value for the setting or string.Empty if the setting doesn't exist.
        /// </returns>
        /// <exception cref="System.Configuration.ConfigurationErrorsException">If an element has not been configured for one of the specified identifiers a ConfigurationErrorsException is thrown.</exception>
        public string GetSiteSetting(string key, string groupId)
        {
            ParameterChecker.CheckForNullOrEmpty(key, "key");
            ParameterChecker.CheckForNullOrEmpty(groupId, "groupId");

            var siteId = PresentationServiceLocator.Current.SiteContext.SiteId;
            var site = CommerceServerContribConfiguration.Presentation.SiteSettings[siteId];
            if (site == null)
            {
                throw new ConfigurationErrorsException(string.Format(CultureInfo.InvariantCulture, ResourceHelper.GetCommerceServerContrib("SiteElementDoesNotExistForID"), siteId));
            }

            return this.GetSiteSetting(key, siteId, groupId);
        }

        /// <summary>
        /// Gets the web setting.
        /// </summary>
        /// <param name="key">The key to search for.</param>
        /// <param name="siteId">The site id.</param>
        /// <param name="groupId">The group id.</param>
        /// <returns>
        /// A string value for the setting or string.Empty if the setting doesn't exist.
        /// </returns>
        /// <exception cref="System.Configuration.ConfigurationErrorsException">If an element has not been configured for one of the specified identifiers a ConfigurationErrorsException is thrown.</exception>
        public string GetSiteSetting(string key, string siteId, string groupId)
        {
            ParameterChecker.CheckForNullOrEmpty(key, "key");
            ParameterChecker.CheckForNullOrEmpty(groupId, "groupId");
            ParameterChecker.CheckForNullOrEmpty(siteId, "siteId");

            SiteElement site = CommerceServerContribConfiguration.Presentation.SiteSettings[siteId];
            if (site == null)
            {
                throw new ConfigurationErrorsException(string.Format(CultureInfo.InvariantCulture, ResourceHelper.GetCommerceServerContrib("SiteElementDoesNotExistForID"), siteId));
            }

            var group = site.Groups[groupId];
            if (group == null)
            {
                throw new ConfigurationErrorsException(string.Format(CultureInfo.InvariantCulture, ResourceHelper.GetCommerceServerContrib("GroupElementDoesNotExistForID"), groupId));
            }

            var setting = group.Settings[key];
            if (setting == null)
            {
                throw new ConfigurationErrorsException(string.Format(CultureInfo.InvariantCulture, ResourceHelper.GetCommerceServerContrib("SettingElementDoesNotExistForKey"), key));
            }
            
            return setting.Value;
        }

        /// <summary>
        /// Gets the site settings.
        /// </summary>
        /// <param name="groupId">The group id.</param>
        /// <returns>
        /// A list of simpleResponse values for the groupId or empty list of simpleResponse if the groupId doesn't exist.
        /// </returns>
        public List<SiteSettingsResponse> GetSiteSettings(string groupId)
        {
            ParameterChecker.CheckForNullOrEmpty(groupId, "groupId");

            var siteId = PresentationServiceLocator.Current.SiteContext.SiteId;
            return this.GetSiteSettings(siteId, groupId);
        }

        /// <summary>
        /// Gets the site settings.
        /// </summary>
        /// <param name="siteId">The site id.</param>
        /// <param name="groupId">The group id.</param>
        /// <returns>
        /// A list of simpleResponse values for the groupId or empty list of simpleResponse if the groupId doesn't exist.
        /// </returns>
        /// <exception cref="System.Configuration.ConfigurationErrorsException">If an element has not been configured for one of the specified identifiers a ConfigurationErrorsException is thrown.</exception>
        public List<SiteSettingsResponse> GetSiteSettings(string siteId, string groupId)
        {
            ParameterChecker.CheckForNullOrEmpty(groupId, "groupId");
            ParameterChecker.CheckForNullOrEmpty(siteId, "siteId");

            var site = CommerceServerContribConfiguration.Presentation.SiteSettings[siteId];
            if (site == null)
            {
                throw new ConfigurationErrorsException(string.Format(CultureInfo.InvariantCulture, ResourceHelper.GetCommerceServerContrib("SiteElementDoesNotExistForID"), siteId));
            }

            var group = site.Groups[groupId];
            if (group == null)
            {
                throw new ConfigurationErrorsException(string.Format(CultureInfo.InvariantCulture, ResourceHelper.GetCommerceServerContrib("GroupElementDoesNotExistForID"), groupId));
            }

            var valueToReturn = new List<SiteSettingsResponse>();

            foreach (CSCC.SettingElement item in group.Settings)
            {
                valueToReturn.Add(new SiteSettingsResponse(item));
            }

            return valueToReturn;
        }

        /// <summary>
        /// Gets the web setting.
        /// </summary>
        /// <param name="key">The key to search for.</param>
        /// <returns>
        /// A string value for the setting or string.Empty if the setting doesn't exist.
        /// </returns>
        /// <exception cref="System.NotImplementedException">Not currently needed.</exception>
        public string GetFromWebSetting(string key)
        {
            throw new System.NotImplementedException();
        }

        /// <summary>
        /// Gets the web setting.
        /// </summary>
        /// <param name="key">The key to search for.</param>
        /// <param name="siteId">The site id.</param>
        /// <returns>
        /// A string value for the setting or string.Empty if the setting doesn't exist.
        /// </returns>
        /// <exception cref="System.NotImplementedException">Not currently required.</exception>
        public string GetFromWebSetting(string key, string siteId)
        {
            throw new System.NotImplementedException();
        }

        /// <summary>
        /// Determines if the site setting is defined.
        /// </summary>
        /// <param name="key">The key to search for.</param>
        /// <param name="groupId">The group id to search for.</param>
        /// <param name="siteId">The site id to search for.</param>
        /// <returns>
        ///   <c>true</c> if the setting key has been found;  Otherwise <c>false</c> is returned.
        /// </returns>
        /// <exception cref="System.NotImplementedException">Not currently required.  </exception>
        public bool HasSiteSetting(string key, string groupId, string siteId)
        {
            throw new System.NotImplementedException();
        }

        /// <summary>
        /// Determines if the site setting is defined.
        /// </summary>
        /// <param name="key">The key to search for.</param>
        /// <param name="groupId">The group id to search for.</param>
        /// <returns>
        ///   <c>true</c> if the setting key has been found;  Otherwise <c>false</c> is returned.
        /// </returns>
        /// <exception cref="System.NotImplementedException">Not currently required.</exception>
        public bool HasSiteSetting(string key, string groupId)
        {
            throw new System.NotImplementedException();
        }

        /// <summary>
        /// Takes a friendly url and converts it to its physical mapping
        /// </summary>
        /// <param name="friendlyUrl">The friendly url to convert.</param>
        /// <returns>
        /// Returns the physical url as a string, if it cannot be found then null is returned
        /// </returns>
        /// <exception cref="System.NotImplementedException">Not currently required.</exception>
        public string ConvertFriendlyToPhysicalUrl(string friendlyUrl)
        {
            throw new System.NotImplementedException();
        }
    }
}
