﻿//---------------------------------------------------------------------
// <copyright file="LocalizedWebDisplayNameAttribute.cs" company="commerceserver.net">
//     Copyright © commerceserver.net.  All rights reserved.
// </copyright>
// <summary></summary>
//---------------------------------------------------------------------
namespace CommerceServerContrib.Web
{
    using System;
    using System.Globalization;
    using System.Web.UI.WebControls.WebParts;
    using CommerceServerContrib.CrossTier.Interfaces;

    /// <summary>
    /// Overrides WebDisplayNameAttribute to return a localized display name from web global resources.
    /// </summary>
    [AttributeUsage(AttributeTargets.Property)]
    public sealed class LocalizedWebDisplayNameAttribute : WebDisplayNameAttribute
    {
        #region Private fields

        private readonly string _resourceClassKey;
        private readonly string _resourceKey;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the LocalizedWebDisplayNameAttribute class
        /// by providing the resource "class key" and "resource key" used to obtain the localized display name.
        /// This class simply uses
        /// HttpContext.GetGlobalResourceObject
        /// to retrieve a resource string from a global resource file identified by a class key.
        /// </summary>
        /// <param name="resourceClassKey">The key of the global resource file to use (corresponds to file name, no extension)</param>
        /// <param name="resourceKey">The key of the localized string to retrieve from the resource file</param>
        public LocalizedWebDisplayNameAttribute(string resourceClassKey, string resourceKey)
        {
            this._resourceClassKey = resourceClassKey;
            this._resourceKey = resourceKey;
        }

        #endregion Constructors

        #region Public properties

        /// <summary>
        /// Gets a localized display name from resources.
        /// </summary>
        public override string DisplayName
        {
            get
            {
                return this.GetLocalizedString(this._resourceKey);
            }
        }

        /// <summary>
        /// Gets the key of the resource file used
        /// </summary>
        public string ResourceClassKey
        {
            get
            {
                return this._resourceClassKey;
            }
        }

        /// <summary>
        /// Gets the key used to retrieve the display name string
        /// </summary>
        public string ResourceKey
        {
            get
            {
                return this._resourceKey;
            }
        }

        #endregion Public properties

        #region Private Methods

        /// <summary>
        /// Retrieves a string using the specified resourceKey and the class key 
        /// obtained from the ResourceClassKey property
        /// </summary>
        /// <param name="resourceKey">the key of the resource to retrieve</param>
        /// <returns>the specified resource string</returns>
        private string GetLocalizedString(string resourceKey)
        {
            return ResourceHelper.Get(this.ResourceClassKey, resourceKey);
        }

        #endregion
    }
}
