﻿//---------------------------------------------------------------------
// <copyright file="FormsAuthenticationProvider.cs" company="commerceserver.net">
//     Copyright © commerceserver.net.  All rights reserved.
// </copyright>
// <summary>FormsAuthenticationProvider class</summary>
//---------------------------------------------------------------------

namespace CommerceServerContrib.Web
{
    using System;
    using System.Security.Principal;
    using System.Web;
    using System.Web.Security;
    using CommerceServerContrib.CrossTier.Entities;
    using CommerceServerContrib.CrossTier.Interfaces;
    using CommerceServerContrib.CrossTier.Helpers;
    using CommerceServer.Foundation;

    /// <summary>
    /// FormsAuthenticationProvider class
    /// </summary>
    public class FormsAuthenticationProvider : IAuthenticationProvider
    {        
        #region Implementation of IAuthenticationProvider

        #region Private Fields

        private const string AuthenticationType = "Forms";

        #endregion

        /// <summary>
        /// Gets a value indicating whether SSL is required for cookies issued by the authentication provider.
        /// </summary>
        public bool RequireSsl
        {
            get { return FormsAuthentication.RequireSSL; }
        }

        /// <summary>
        /// Validates the user.
        /// </summary>
        /// <param name="userId">The user id.</param>
        /// <param name="userEmail">Name of the user.</param>
        /// <param name="password">The password.</param>
        /// <returns>
        /// True if the user is valid, false otherwise
        /// </returns>
        public bool ValidateUser(out string userId, string userEmail, string password)
        {
            userId = string.Empty;
            if (Membership.ValidateUser(userEmail, password))
            {
                var user = Membership.GetUser(userEmail);
                if (user != null && user.ProviderUserKey != null)
                {
                    userId = ((Guid)user.ProviderUserKey).ToString("B");
                    return true;
                }
            }

            return false;
        }

        /// <summary>
        /// Signs in the user into the site.
        /// </summary>
        /// <param name="userId">The user id.</param>
        /// <param name="userEmail">The user email.</param>
        /// <param name="userName">Name of the user.</param>
        /// <param name="password">Password of the user.</param>
        /// <param name="rememberMe">if set to <c>true</c> [remember me].</param>
        /// <returns>
        /// True is the user was sucessfully signed in, false otherwise
        /// </returns>
        public bool SignIn(string userId, string userEmail, string userName, string password, bool rememberMe)
        {
            if (string.IsNullOrEmpty(userId) || string.IsNullOrEmpty(userEmail) || string.IsNullOrEmpty(userName))
            {
                return false;
            }

            FormsAuthentication.SetAuthCookie(userEmail, false);
            UserCookieHelper.IssueRegisteredCookie(userId, userEmail, userName, AuthenticationType);
            if (rememberMe)
            {
                UserCookieHelper.IssueRememberMeCookie(userEmail);
            }
            else
            {
                UserCookieHelper.RemoveRememberMeCookie();
            }

            return true;
        }

        /// <summary>
        /// Determines whether [is user identity valid].
        /// </summary>
        /// <param name="userCookie">The user cookie.</param>
        /// <returns>
        ///   <c>true</c> if [is user identity valid]; otherwise, <c>false</c>.
        /// </returns>
        public bool IsUserIdentityValid(UserCookie userCookie)
        {
            IPrincipal currentPrincipal = UserCookieHelper.CurrentPrincipal;

            if (currentPrincipal != null && !string.IsNullOrEmpty(currentPrincipal.Identity.Name))
            {
                IIdentity identity = currentPrincipal.Identity;
                return (string.Equals(userCookie.User.Email, identity.Name, StringComparison.OrdinalIgnoreCase) && string.Equals(userCookie.User.AuthenticationType, identity.AuthenticationType, StringComparison.OrdinalIgnoreCase));
            }

            if (userCookie != null && !string.IsNullOrEmpty(userCookie.User.Email))
            {
                HttpCookie authCookie = HttpContext.Current.Request.Cookies.Get(FormsAuthentication.FormsCookieName);
                if (authCookie != null && authCookie.Value != null)
                {
                    FormsAuthenticationTicket authTicket = FormsAuthentication.Decrypt(authCookie.Value);
                    if ((authTicket != null) && (!authTicket.Expired))
                    {
                        return string.Equals(userCookie.User.Email, authTicket.Name, StringComparison.OrdinalIgnoreCase);
                    }
                }
            }

            return false;
        }

        /// <summary>
        /// Issues the registered cookie.
        /// </summary>
        /// <param name="userId">The user id.</param>
        /// <param name="userEmail">The user email.</param>
        /// <param name="userName">Name of the user.</param>
        public void IssueRegisteredCookie(string userId, string userEmail, string userName)
        {
            UserCookieHelper.RemoveRegisteredCookie();
            UserCookieHelper.IssueRegisteredCookie(userId, userEmail, userName, AuthenticationType);
        }

        /// <summary>
        /// Terminateds the session.
        /// </summary>
        public void TerminateSession()
        {
            PresentationServiceLocator.Current.SiteContext.User = new User();
            UserCookieHelper.RemoveRegisteredCookie();

            if (HttpContext.Current.Session != null)
            {
                HttpContext.Current.Session.Clear();
            }

            FormsAuthentication.SignOut();
            var siteSettingsProvider = PresentationServiceLocator.Current.SiteSettingProvider;
            string signInPageUrl = siteSettingsProvider.GetFromWebSetting("SignInPageUrl");

            PresentationServiceLocator.Current.SiteContext.RedirectTo(signInPageUrl, true);
        }

        /// <summary>
        /// Gets a user by email.
        /// </summary>
        /// <param name="userEmail">The user email.</param>
        /// <returns>A MembershipUser.</returns>
        public MembershipUser GetUser(string userEmail)
        {
            ParameterChecker.CheckForNullOrEmpty(userEmail, "operationBuilder");
            return Membership.GetUser(userEmail);
        }

        #endregion
    }
}