﻿//---------------------------------------------------------------------
// <copyright file="UserCookieHelper.cs" company="commerceserver.net">
//     Copyright © commerceserver.net.  All rights reserved.
// </copyright>
// <summary>UserCookieHelper class</summary>
//---------------------------------------------------------------------

namespace CommerceServerContrib.Web
{
    using System;
    using System.Web;
    using System.Configuration;
    using System.Web.Hosting;
    using System.Collections.Generic;
    using CommerceServerContrib.CrossTier.Entities;
    using CommerceServerContrib.CrossTier.Configuration;
    using CommerceServerContrib.CrossTier.Helpers;
    using CommerceServerContrib.CrossTier.Interfaces;
    using CommerceServerContrib.Web.Helpers;
    using CommerceServer.Foundation;

    /// <summary>
    /// UserCookieHelperHelper class
    /// </summary>
    public static class UserCookieHelper
    {
        #region Private Fields

        private const string AnonymousCookieDefaultName = "__CLAnonymousUser";
        private const string RegisteredCookieDefaultName = "__CLRegisteredUser";
        private const string RegisteredNonSecuredCookieDefaultName = "__CLRegisteredUserHttp";
        private const string RememberMeCookieDefaultName = "__CLRememberMe";
        private const string CheckoutFlowIdCookieName = "__CLCheckoutFlowId";
        private static readonly object SyncRoot = new object();
        private static readonly List<string> WindowsAutenticationTypes = new List<string> { "ntlm", "negotiate", "kerberos", "wdigest" };
        private static bool _initialized;
        private static string _domain;
        private static string _path;
        private static string _anonymousCookieName;
        private static string _rememberMeCookieName;
        private static bool _anonymousSlidingExpiration;
        private static TimeSpan _anonymousTimeout;
        private static string _registeredCookieName;
        private static string _registeredNonSecuredCookieName;
        private static bool _registeredValidateIdentity;
        private static string _secureEncryptionKey;
        private static string _nonSecureEncryptionKey;
        private static UserCookieSslMode _sslMode;

        #endregion

        #region Properties

        /// <summary>
        /// Gets the current principal.
        /// </summary>
        internal static System.Security.Principal.IPrincipal CurrentPrincipal
        {
            get
            {
                if ((HostingEnvironment.IsHosted && (HttpContext.Current != null)) && (HttpContext.Current.User != null))
                {
                    return HttpContext.Current.User;
                }

                return System.Threading.Thread.CurrentPrincipal;
            }
        }

        #endregion

        #region Public Methods

        /// <summary>
        /// Sets the cookie.
        /// </summary>
        /// <param name="cookieName">Name of the cookie.</param>
        /// <param name="value">The value.</param>
        public static void SetCookie(string cookieName, string value)
        {
            if (PresentationServiceLocator.Current.SiteContext.CurrentContext == null)
            {
                return;
            }

            HttpCookie cookie = PresentationServiceLocator.Current.SiteContext.CurrentContext.Response.Cookies[cookieName];
            if (cookie == null)
            {
                PresentationServiceLocator.Current.SiteContext.CurrentContext.Response.Cookies.Add(new HttpCookie(cookieName, value));
            }
            else
            {
                cookie.Value = value;
            }
        }

        /// <summary>
        /// Issues the registered cookie.
        /// </summary>
        /// <param name="userId">The user id.</param>
        /// <param name="userEmail">The user email.</param>
        /// <param name="userName">Name of the user.</param>
        /// <param name="authenticationType">Type of the authentication.</param>
        public static void IssueRegisteredCookie(string userId, string userEmail, string userName, string authenticationType)
        {
            var isWindowsAuthenticationType = !string.IsNullOrEmpty(authenticationType) && WindowsAutenticationTypes.Contains(authenticationType.ToLowerInvariant());
            IssueRegisteredCookie(userId, userEmail, userName, authenticationType, isWindowsAuthenticationType);
        }

        /// <summary>
        /// Issues the registered cookie.
        /// </summary>
        /// <param name="userId">The user id.</param>
        /// <param name="userEmail">The user email.</param>
        /// <param name="userName">Name of the user.</param>
        /// <param name="authenticationType">Type of the authentication.</param>
        /// <param name="isWindowsIdentity">if set to <c>true</c> [is windows identity].</param>
        public static void IssueRegisteredCookie(string userId, string userEmail, string userName, string authenticationType, bool isWindowsIdentity)
        {
            Initialize();
            if (HttpContext.Current == null)
            {
                return;
            }

            bool requireSsl = (_sslMode != UserCookieSslMode.Disabled);
            HttpCookie cookie = CreateRegisteredCookie(_registeredCookieName, userId, userEmail, userName, authenticationType, isWindowsIdentity, requireSsl);
            HttpContext.Current.Response.Cookies.Set(cookie);

            if (_sslMode != UserCookieSslMode.Mixed)
            {
                return;
            }

            HttpCookie nonSecureCookie = CreateRegisteredCookie(_registeredNonSecuredCookieName, userId, userEmail, userName, authenticationType, isWindowsIdentity, false);
            HttpContext.Current.Response.Cookies.Set(nonSecureCookie);
        }

        /// <summary>
        /// Issues the anonymous cookie.
        /// </summary>
        /// <param name="userId">The user id.</param>
        public static void IssueAnonymousCookie(string userId)
        {
            Initialize();
            if (HttpContext.Current == null)
            {
                return;
            }

            HttpCookie cookie = CreateAnonymousCookie(userId);
            HttpContext.Current.Response.Cookies.Set(cookie);
        }

        /// <summary>
        /// Issues the remember me cookie.
        /// </summary>
        /// <param name="userEmail">The user email.</param>
        public static void IssueRememberMeCookie(string userEmail)
        {
            Initialize();
            if (HttpContext.Current == null)
            {
                return;
            }

            HttpCookie cookie = CreateRememberMeCookie(userEmail);
            HttpContext.Current.Response.Cookies.Set(cookie);
        }

        /// <summary>
        /// Issues a new checkout flow ID cookie.
        /// </summary>
        /// <param name="checkoutFlowId">The checkout flow ID value.</param>
        public static void IssueCheckoutFlowIdCookie(string checkoutFlowId)
        {
            ParameterChecker.CheckForNullOrEmpty(checkoutFlowId, "checkoutFlowId");

            Initialize();

            HttpCookie cookie = new HttpCookie(CheckoutFlowIdCookieName)
            {
                Value = checkoutFlowId,
                Path = _path,
                Expires = DateTime.MaxValue, // The cookie must exist as long as the basket.
            };

            if (PresentationServiceLocator.Current.BrandContext.SecurePageSettings.RequireSsl)
            {
                cookie.Secure = true;
            }

            if (!string.IsNullOrEmpty(_domain))
            {
                cookie.Domain = _domain;
            }

            HttpContext currentContext = PresentationServiceLocator.Current.SiteContext.CurrentContext;
            currentContext.Response.Cookies.Add(cookie);

            // Make this cookie accessible in the current request.
            currentContext.Request.Cookies.Add(cookie);
        }

        /// <summary>
        /// Removes the checkout flow ID cookie.
        /// </summary>
        public static void RemoveCheckoutFlowIdCookie()
        {
            Initialize();
            if (HttpContext.Current == null)
            {
                return;
            }

            RemoveCookie(CheckoutFlowIdCookieName);
        }

        /// <summary>
        /// Gets the checkout flow ID of the current user.
        /// </summary>
        /// <returns>The checkout flow ID of the current user.</returns>
        public static string GetCheckoutFlowId()
        {
            Initialize();
            var cookie = PresentationServiceLocator.Current.SiteContext.CurrentContext.Request.Cookies.Get(CheckoutFlowIdCookieName);
            if (cookie != null)
            {
                return cookie.Value;
            }

            return null;
        }

        /// <summary>
        /// Removes the registered cookie.
        /// </summary>
        public static void RemoveRegisteredCookie()
        {
            Initialize();
            if (HttpContext.Current == null)
            {
                return;
            }

            RemoveCookie(_registeredNonSecuredCookieName);
            RemoveCookie(_registeredCookieName);
        }

        /// <summary>
        /// Removes the anonymous cookie.
        /// </summary>
        public static void RemoveAnonymousCookie()
        {
            Initialize();
            if (HttpContext.Current != null)
            {
                RemoveCookie(_anonymousCookieName);
            }
        }

        /// <summary>
        /// Removes the remember me cookie.
        /// </summary>
        public static void RemoveRememberMeCookie()
        {
            Initialize();
            if (HttpContext.Current != null)
            {
                RemoveCookie(_rememberMeCookieName);
            }
        }

        /// <summary>
        /// Tries the get registered user.
        /// </summary>
        /// <param name="user">The current user.</param>
        /// <returns>
        /// True if user is registered, false otherwise
        /// </returns>
        public static bool TryGetRegisteredUser(out User user)
        {
            Initialize();
            user = new User();

            bool terminated = false;

            if (HttpContext.Current != null)
            {
                try
                {
                    UserCookie userCookie;
                    if (!PresentationServiceLocator.Current.SiteContext.IsSecureConnection && (_sslMode == UserCookieSslMode.Mixed))
                    {
                        userCookie = ExtractUserFromCookie(HttpContext.Current.Request.Cookies[_registeredNonSecuredCookieName], false);
                    }
                    else
                    {
                        HttpCookie cookie = HttpContext.Current.Request.Cookies[_registeredCookieName];
                        userCookie = ExtractUserFromCookie(cookie, _sslMode != UserCookieSslMode.Disabled);
                    }

                    if (userCookie != null)
                    {
                        if (IsRegisteredUserIdentityValid(userCookie))
                        {
                            user.Id = userCookie.User.Id;
                            user.Name = userCookie.User.Name;
                            user.Email = userCookie.User.Email;
                            user.AuthenticationType = userCookie.User.AuthenticationType;
                            user.IsWindowsIdentity = userCookie.User.IsWindowsIdentity;

                            return true;
                        }

                        RemoveRegisteredCookie();
                        terminated = true;
                        PresentationServiceLocator.Current.GetImplementation<IAuthenticationProvider>().TerminateSession();
                    }
                }
                catch (Exception exception)
                {
                    CommerceServerContribExceptionManager.HandleException(exception);
                    RemoveRegisteredCookie();

                    // If the exception was due to a terminating session, we must re-throw the exception for the JSON response to work properly.
                    if (!terminated)
                    {
                        PresentationServiceLocator.Current.GetImplementation<IAuthenticationProvider>().TerminateSession();
                    }
                    else
                    {
                        throw;
                    }
                }
            }

            return false;
        }

        /// <summary>
        /// Gets the anonymous user id.
        /// </summary>
        /// <returns>The anonymous user id</returns>
        public static string GetAnonymousUserId()
        {
            Initialize();
            if (HttpContext.Current != null)
            {
                try
                {
                    UserCookie userCookie = ExtractUserFromCookie(HttpContext.Current.Request.Cookies[_anonymousCookieName], false);
                    if (userCookie != null)
                    {
                        if (userCookie.CookieName != _anonymousCookieName)
                        {
                            return null;
                        }

                        if (IsAnonymousUserSlidingWindowExpired(userCookie))
                        {
                            IssueAnonymousCookie(userCookie.User.Id);
                        }

                        return userCookie.User.Id;
                    }
                }
                catch
                {
                    HttpContext.Current.Request.Cookies.Remove(_anonymousCookieName);
                    RemoveAnonymousCookie();
                }
            }
            
            return null;
        }

        /// <summary>
        /// Gets the remember me email.
        /// </summary>
        /// <returns>The user email</returns>
        public static string GetRememberMe()
        {
            Initialize();
            if (HttpContext.Current != null)
            {
                try
                {
                    UserCookie userCookie = ExtractUserFromCookie(HttpContext.Current.Request.Cookies[_rememberMeCookieName], false);
                    if (userCookie != null)
                    {
                        return userCookie.CookieName != _rememberMeCookieName ? null : userCookie.User.Email;
                    }
                }
                catch
                {
                    HttpContext.Current.Request.Cookies.Remove(_rememberMeCookieName);
                    RemoveRememberMeCookie();
                }
            }

            return null;
        }

        #endregion

        #region Private Methods

        private static HttpCookie CreateCookie(string cookieName, string userId, string userEmail, string userName, string authenticationType, bool isWindowsIdentity, TimeSpan? timeout, bool requireSsl)
        {
            DateTime now = DateTime.Now;
            var userCookie = new UserCookie
                                 {
                                     User = new User { Id = userId, Email = userEmail, Name = userName, AuthenticationType = authenticationType, IsWindowsIdentity = isWindowsIdentity },
                                     IssueDate = now,
                                     CookieName = cookieName,
                                     ExpirationDate = now.Add(timeout.HasValue ? timeout.GetValueOrDefault() : TimeSpan.Zero)
                                 };

            string encryptCookie = requireSsl ?
                EncryptionHelper.Encrypt((SerializationHelper.Serialize(userCookie, CommerceServerContrib.CrossTier.Helpers.StringConstants.OutputTypeXmlDataContract)), _secureEncryptionKey) :
                EncryptionHelper.Encrypt((SerializationHelper.Serialize(userCookie, CommerceServerContrib.CrossTier.Helpers.StringConstants.OutputTypeXmlDataContract)), _nonSecureEncryptionKey);

            var cookie = new HttpCookie(cookieName)
                            {
                                Value = encryptCookie,
                                HttpOnly = true,
                                Path = _path,
                                Secure = requireSsl
                            };

            if (!string.IsNullOrEmpty(_domain))
            {
                cookie.Domain = _domain;
            }

            if (timeout.HasValue && timeout.Value != TimeSpan.Zero)
            {
                cookie.Expires = userCookie.ExpirationDate;
            }

            return cookie;
        }

        private static HttpCookie CreateRegisteredCookie(string cookieName, string userId, string userEmail, string userName, string authenticationType, bool isWindowsIdentity, bool requireSsl)
        {
            return CreateCookie(cookieName, userId, userEmail, userName, authenticationType, isWindowsIdentity, null, requireSsl);
        }

        private static HttpCookie CreateAnonymousCookie(string userId)
        {
            return CreateCookie(_anonymousCookieName, userId, null, null, null, false, (_anonymousTimeout), false);
        }

        private static HttpCookie CreateRememberMeCookie(string userEmail)
        {
            return CreateCookie(_rememberMeCookieName, null, userEmail, null, null, false, TimeSpan.FromDays(30), false);
        }

        private static void RemoveCookie(string cookieName)
        {
            var cookie = new HttpCookie(cookieName, string.Empty)
                             {
                                 HttpOnly = true,
                                 Path = _path,
                                 Expires = DateTime.Now.AddDays(-1)
                             };

            if (!string.IsNullOrEmpty(_domain))
            {
                cookie.Domain = _domain;
            }

            HttpContext.Current.Response.Cookies.Set(cookie);
        }

        private static UserCookie ExtractUserFromCookie(HttpCookie cookie, bool isSecure)
        {
            UserCookie userCookie = null;
            if ((cookie != null) && !string.IsNullOrEmpty(cookie.Value))
            {
                string decryptCookie = isSecure ? EncryptionHelper.Decrypt(cookie.Value, _secureEncryptionKey) : EncryptionHelper.Decrypt(cookie.Value, _nonSecureEncryptionKey);
                userCookie = SerializationHelper.Deserialize<UserCookie>(decryptCookie, CommerceServerContrib.CrossTier.Helpers.StringConstants.OutputTypeXmlDataContract);
            }

            return userCookie;
        }

        private static bool IsRegisteredUserIdentityValid(UserCookie userCookie)
        {
            if (!_registeredValidateIdentity)
            {
                return true;
            }

            var authenticationProvider = PresentationServiceLocator.Current.GetImplementation<IAuthenticationProvider>();
            if (!authenticationProvider.RequireSsl && !UserCookieHelper.CurrentPrincipal.Identity.IsAuthenticated)
            {
                return false;
            }

            if (!PresentationServiceLocator.Current.SiteContext.IsSecureConnection && (_sslMode == UserCookieSslMode.Mixed))
            {
                return true;
            }

            return authenticationProvider.IsUserIdentityValid(userCookie);
        }

        private static bool IsAnonymousUserSlidingWindowExpired(UserCookie userCookie)
        {
            if (_anonymousSlidingExpiration)
            {
                DateTime now = DateTime.Now;
                TimeSpan spanIssue = (now - userCookie.IssueDate);
                TimeSpan spanExpiration = (userCookie.ExpirationDate - now);

                if (spanExpiration < spanIssue)
                {
                    return true;
                }
            }

            return false;
        }

        private static void Initialize()
        {
            if (_initialized)
            {
                return;
            }

            lock (SyncRoot)
            {
                if (_initialized)
                {
                    return;
                }

                var managementElement = PresentationServiceLocator.Current.BrandContext.UserSecurityManagement;
                CommerceServerContrib.CrossTier.RegisteredUser registeredUser = null; 
                CommerceServerContrib.CrossTier.AnonymousUser anonymousUser = null;

                if (managementElement == null)
                {
                    _path = "/";
                    _domain = null;
                }
                else
                {
                    _path = managementElement.Path;
                    _domain = managementElement.Domain;
                    registeredUser = managementElement.RegisteredUser;
                    anonymousUser = managementElement.AnonymousUser;
                }

                if (anonymousUser == null)
                {
                    _anonymousCookieName = AnonymousCookieDefaultName;
                    _anonymousSlidingExpiration = true;
                    _anonymousTimeout = AnonymousUserElement.TimeoutDefaultValue;
                }
                else
                {
                    _anonymousCookieName = anonymousUser.CookieName;
                    _anonymousSlidingExpiration = anonymousUser.SlidingExpiration;
                    _anonymousTimeout = anonymousUser.Timeout;
                }

                if (registeredUser == null)
                {
                    _registeredCookieName = RegisteredCookieDefaultName;
                    _registeredNonSecuredCookieName = RegisteredNonSecuredCookieDefaultName;
                    _rememberMeCookieName = RememberMeCookieDefaultName;
                    _sslMode = UserCookieSslMode.Disabled;
                    _registeredValidateIdentity = true;
                }
                else
                {
                    if ((registeredUser.SslMode == UserCookieSslMode.Disabled) || (registeredUser.SslMode == UserCookieSslMode.Enabled))
                    {
                        if (string.IsNullOrEmpty(registeredUser.SecureEncryptionKeyName))
                        {
                            CommerceServerContribExceptionManager.HandleException(new ConfigurationErrorsException(ClientFrameworkResources.UserSecurityMangementEncryptionKeyRequired));
                        }
                    }
                    else if ((registeredUser.SslMode == UserCookieSslMode.Mixed) && (string.IsNullOrEmpty(registeredUser.SecureEncryptionKeyName) || string.IsNullOrEmpty(registeredUser.NonSecureEncryptionKeyName)))
                    {
                        CommerceServerContribExceptionManager.HandleException(new ConfigurationErrorsException(ClientFrameworkResources.UserSecurityMangementEncryptionKeysRequired));
                    }

                    _registeredCookieName = registeredUser.SecureCookieName;
                    _registeredNonSecuredCookieName = registeredUser.NonSecuredCookieName;
                    _rememberMeCookieName = registeredUser.RememberMeCookieName;
                    _sslMode = registeredUser.SslMode;
                    _registeredValidateIdentity = registeredUser.ValidateIdentity;
                    _secureEncryptionKey = registeredUser.SecureEncryptionKeyName;
                    _nonSecureEncryptionKey = registeredUser.NonSecureEncryptionKeyName;
                }

                _initialized = true;
            }
        }

        #endregion
    }
}