﻿//---------------------------------------------------------------------
// <copyright file="CdnHelper.cs" company="commerceserver.net">
//     Copyright © commerceserver.net.  All rights reserved.
// </copyright>
// <summary>A class for generating CDN links and other items required for CDN support. </summary>
//---------------------------------------------------------------------

namespace CommerceServerContrib.Web
{
    using System;
    using System.Web;
    using CommerceServerContrib.Web.Helpers;
    using CommerceServerContrib.CrossTier.Helpers;
    using CommerceServer.Foundation;

    /// <summary>
    /// Out of the box areas for CDN support.
    /// </summary>
    public enum CdnArea
    {
        /// <summary>
        /// Images and other resources used generally throughout the site.
        /// </summary>
        SiteCDN,

        /// <summary>
        /// Catalog images CDN base url
        /// </summary>
        CatalogCDN, 

        /// <summary>
        /// Marketing images CDN base url 
        /// </summary>
        MarketingCDN,

        /// <summary>
        /// JavaScript files CDN base url.
        /// </summary>
        ScriptCDN,

        /// <summary>
        /// Style files CDN base url.
        /// </summary>
        CssCDN
    }

    /// <summary>
    /// This class contains methods that will generate CDNized link and provider reference to CDN properties.  Stubbed for CTP1.
    /// </summary>
    public static class CdnHelper
    {
        /// <summary>
        /// Gets a value indicating whether to hide scripts.    
        /// </summary>
        /// <value>
        /// 	<c>true</c> if scripts are to be hidden; otherwise, <c>false</c>.
        /// </value>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Usage", "CA1806:DoNotIgnoreMethodResults", MessageId = "System.Boolean.TryParse(System.String,System.Boolean@)", Justification = "The default of false is correct in this case.")]
        public static bool HideDependentCalls
        {
            get
            {
                string hideDependentString = PresentationServiceLocator.Current.SiteSettingProvider.GetFromWebSetting("HideDependentCalls");
                bool hideDependentCalls = false;
                bool.TryParse(hideDependentString, out hideDependentCalls);
                return hideDependentCalls;
            }
        }

        /// <summary>
        /// Creates a CDN URL with the specified area.
        /// </summary>
        /// <param name="relativeUrl">The relative URL.</param>
        /// <param name="area">The area of the site in the site settings.</param>
        /// <returns>String built with the base url from the site settings.</returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1055:UriReturnValuesShouldNotBeStrings", Justification = "Urls are used as strings"),
        System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1054:UriParametersShouldNotBeStrings", MessageId = "0#", Justification = "Url is going to be used as a string")]
        public static string CdnUrl(string relativeUrl, CdnArea area)
        {
            ParameterChecker.CheckForNullOrEmpty(relativeUrl, "relativeUrl");
            ParameterChecker.CheckForNull(area, "area");

            return CdnHelper.CdnUrl(relativeUrl, area.ToString());
        }

        /// <summary>
        /// Creates a CDN URL based on the area passed as a string.  If preformancing testing is turned on it will return the general dummy url.
        /// </summary>
        /// <param name="relativeUrl">The relative URL.</param>
        /// <param name="area">The area or the site the url is formed with.</param>
        /// <returns>The url built with the base url of the area.</returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1055:UriReturnValuesShouldNotBeStrings", Justification = "Url is going to be used as a string"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1054:UriParametersShouldNotBeStrings", MessageId = "0#", Justification = "Url is going to be used as a string.")]
        public static string CdnUrl(string relativeUrl, string area)
        {
            ParameterChecker.CheckForNullOrEmpty(relativeUrl, "relativeUrl");
            ParameterChecker.CheckForNullOrEmpty(area, "area");

            if (relativeUrl.StartsWith("http", StringComparison.OrdinalIgnoreCase))
            {
                return relativeUrl;
            }

            if (CdnHelper.HideDependentCalls)
            {
                return CdnHelper.BlankPath();
            }

            string baseUrl = CdnHelper.GetBaseUrlForArea(area);

            if (relativeUrl.IndexOf("/", StringComparison.Ordinal) == 0 &&
                relativeUrl.Length > 1 &&
                !string.IsNullOrEmpty(baseUrl))
            {
                relativeUrl = relativeUrl.Substring(1);
            } 
            else if (relativeUrl.Equals("/"))
            {
                relativeUrl = string.Empty;
            }

            return string.Join(string.Empty, new string[] { baseUrl, relativeUrl });
        }

        /// <summary>
        /// Creates the Site URL.
        /// </summary>
        /// <param name="relativeUrl">The relative URL.</param>
        /// <returns>A Site url with the CDN base url</returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1055:UriReturnValuesShouldNotBeStrings", Justification = "We use urls as strings throughout the product."),
        System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1054:UriParametersShouldNotBeStrings", MessageId = "0#", Justification = "WE use urls as strings throughout the solution."),
        System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "CSS", Justification = "Industry accepted name.")]
        public static string CreateSiteImageUrl(string relativeUrl)
        {
            if (CdnHelper.HideDependentCalls)
            {
                return CdnHelper.BlankSitePath();
            }

            return CdnHelper.CdnUrl(relativeUrl, CdnArea.SiteCDN);
        }

        /// <summary>
        /// Creates the CSS URL.
        /// </summary>
        /// <param name="relativeUrl">The relative URL.</param>
        /// <returns>A CSS url with the CDN base url</returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1055:UriReturnValuesShouldNotBeStrings", Justification = "We use urls as strings throughout the product."),
        System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1054:UriParametersShouldNotBeStrings", MessageId = "0#", Justification = "WE use urls as strings throughout the solution."),
        System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "CSS", Justification = "Industry accepted name.")]
        public static string CreateCSSUrl(string relativeUrl)
        {
            if (CdnHelper.HideDependentCalls)
            {
                return CdnHelper.BlankCssPath();
            }

            return CdnHelper.CdnUrl(relativeUrl, CdnArea.CssCDN);
        }

        /// <summary>
        /// Creates the script URL.
        /// </summary>
        /// <param name="relativeUrl">The relative URL.</param>
        /// <returns>A script url with the CDN base url</returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1055:UriReturnValuesShouldNotBeStrings", Justification = "We use urls as strings throughout the product."),
        System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1054:UriParametersShouldNotBeStrings", MessageId = "0#", Justification = "We use urls as strings throughout the product.")]
        public static string CreateScriptUrl(string relativeUrl)
        {
            if (CdnHelper.HideDependentCalls)
            {
                return CdnHelper.BlankScriptPath();
            }

            return CdnHelper.CdnUrl(relativeUrl, CdnArea.ScriptCDN);
        }

        /// <summary>
        /// Creates the catalog image URL.
        /// </summary>
        /// <param name="relativeUrl">The relative URL.</param>
        /// <returns>An image url with the catalog base url.</returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1055:UriReturnValuesShouldNotBeStrings", Justification = "We use strings for urls thoughout the product"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1054:UriParametersShouldNotBeStrings", MessageId = "0#", Justification = "We use urls as strings throughout the product.")]
        public static string CreateCatalogImageUrl(string relativeUrl)
        {
            if (CdnHelper.HideDependentCalls)
            {
                return CdnHelper.BlankCatalogImagePath();
            }

            return CdnHelper.CdnUrl(relativeUrl, CdnArea.CatalogCDN);
        }

        /// <summary>
        /// Creates the marketing image URL.
        /// </summary>
        /// <param name="relativeUrl">The relative URL.</param>
        /// <returns>An image url with the marketing base url.</returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1055:UriReturnValuesShouldNotBeStrings", Justification = "We use urls as strings throughout the product."),
        System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1054:UriParametersShouldNotBeStrings", MessageId = "0#", Justification = "We use urls as strings throughout the product.")]
        public static string CreateMarketingImageUrl(string relativeUrl)
        {
            if (CdnHelper.HideDependentCalls)
            {
                return CdnHelper.BlankMarketingImagePath();
            }

            return CdnHelper.CdnUrl(relativeUrl, CdnArea.MarketingCDN);
        }

        /// <summary>
        /// Trying to determine if the url is relative or absolute.  Converting it to a Uri and using IsAbsolute sometimes fails for relative urls. 
        /// </summary>
        /// <param name="url">The string to check.</param>
        /// <returns>
        ///   <c>true</c> if [is absolute URL] [the specified p]; otherwise, <c>false</c>.
        /// </returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1054:UriParametersShouldNotBeStrings", MessageId = "0#", Justification = "Urls used as strings throughout the system.")]
        public static bool IsAbsoluteUrl(string url)
        {
            if (url.Contains("http://") || url.Contains("https://"))
            {
                return true;
            }

            return false;
        }

        /// <summary>
        /// Adjusts the for Http or Https depending on the current page mode
        /// </summary>
        /// <param name="url">The URL to adjust.</param>
        /// <returns>The url adjusted based on the current scheme</returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Usage", "CA2234:PassSystemUriObjectsInsteadOfStrings", Justification = "Required no to break existing functionality.")]
        public static string AdjustUrlScheme(string url)
        {
            var uriUrl = new UrlBuilder(url);

            return AdjustUrlScheme(uriUrl.Uri).ToString();
        }

        /// <summary>
        /// Adjusts the for Http or Https depending on the current page mode
        /// </summary>
        /// <param name="url">The URL to adjust.</param>
        /// <returns>The url adjusted based on the current scheme</returns>
        public static Uri AdjustUrlScheme(Uri url)
        {
            if (url.IsAbsoluteUri)
            {
                var urlBuilder = new UrlBuilder(url);
                urlBuilder.ToggleScheme(PresentationServiceLocator.Current.SiteContext.IsSecureConnection);
                
                return urlBuilder.Uri;
            }

            return url;
        }

        internal static string BlankPath()
        {
            string baseUrl = PresentationServiceLocator.Current.SiteSettingProvider.GetFromWebSetting("EmptyLocation");

            if (!string.IsNullOrEmpty(baseUrl))
            {
                return AdjustUrlScheme(baseUrl);
            }

            return string.Empty;
        }

        /// <summary>
        /// Empty path for doing preformance testing.
        /// </summary>
        /// <returns>Dummy url for the script path</returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Usage", "CA2234:PassSystemUriObjectsInsteadOfStrings", Justification = "We use urls as strings thoughout the site")]
        internal static string BlankScriptPath()
        {
            string baseUrl = PresentationServiceLocator.Current.SiteSettingProvider.GetFromWebSetting("EmptyScriptLocation");

            if (!string.IsNullOrEmpty(baseUrl))
            {
                return AdjustUrlScheme(baseUrl);
            }

            return string.Empty;
        }

        /// <summary>
        /// Empty path for images when doing perfomance testing.
        /// </summary>
        /// <returns>String url for the dummy image path</returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Usage", "CA2234:PassSystemUriObjectsInsteadOfStrings", Justification = "We use urls as strings throughout the site")]
        internal static string BlankCatalogImagePath()
        {
            string baseUrl = PresentationServiceLocator.Current.SiteSettingProvider.GetFromWebSetting("EmptyCatalogImageLocation");
            if (!string.IsNullOrEmpty(baseUrl))
            {
                return AdjustUrlScheme(baseUrl);
            }

            return string.Empty;
        }

        /// <summary>
        /// Returns dummy image path for marketing images when doing preformance testing.
        /// </summary>
        /// <returns>image path for marketing images when doing preformance testing.</returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Usage", "CA2234:PassSystemUriObjectsInsteadOfStrings", Justification = "We use strings for urls throughout the site.")]
        internal static string BlankMarketingImagePath()
        {
            string baseUrl = PresentationServiceLocator.Current.SiteSettingProvider.GetFromWebSetting("EmptyMarketingImageLocation");
            if (!string.IsNullOrEmpty(baseUrl))
            {
                return AdjustUrlScheme(baseUrl);
            }

            return string.Empty;
        }

        /// <summary>
        /// Empty path for CSS files when doing peformance testing
        /// </summary>
        /// <returns>String url of the dummy CSS path.</returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Usage", "CA2234:PassSystemUriObjectsInsteadOfStrings", Justification = "We use urls as strings throughout the site")]
        internal static string BlankCssPath()
        {
            string baseUrl = PresentationServiceLocator.Current.SiteSettingProvider.GetFromWebSetting("EmptyStyleLocation");
            if (!string.IsNullOrEmpty(baseUrl))
            {
                return AdjustUrlScheme(baseUrl);
            }

            return string.Empty;
        }

        /// <summary>
        /// Empty path for Site files when doing peformance testing
        /// </summary>
        /// <returns>String url of the dummy Site path.</returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Usage", "CA2234:PassSystemUriObjectsInsteadOfStrings", Justification = "We use urls as strings throughout the site")]
        internal static string BlankSitePath()
        {
            string baseUrl = PresentationServiceLocator.Current.SiteSettingProvider.GetFromWebSetting("EmptyStyleLocation");
            if (!string.IsNullOrEmpty(baseUrl))
            {
                return AdjustUrlScheme(baseUrl);
            }

            return string.Empty;
        }

        internal static string GetBaseUrlForArea(CdnArea area)
        {
            return CdnHelper.GetBaseUrlForArea(area.ToString());
        }

        /// <summary>
        /// Gets the base URL for area will always ends with a /
        /// </summary>
        /// <param name="area">The area to get the base url for.</param>
        /// <returns>A base url ending in /</returns>
        internal static string GetBaseUrlForArea(string area)
        {
            string baseUrl;
            if (!CdnHelper.HideDependentCalls)
            {
                baseUrl = PresentationServiceLocator.Current.SiteSettingProvider.GetFromWebSetting(area);
            }
            else
            {
                baseUrl = PresentationServiceLocator.Current.SiteSettingProvider.GetFromWebSetting("EmptyLocation");
            }

            if (!string.IsNullOrEmpty(baseUrl))
            {
                if (CdnHelper.IsAbsoluteUrl(baseUrl))
                {
                    return AdjustUrlScheme(baseUrl);
                }
                else
                {
                    if (!baseUrl.Substring(baseUrl.Length - 1).Equals("/"))
                    {
                        baseUrl += "/";
                    }

                    return baseUrl;
                }
            }

            return string.Empty;
        }
    }
}