﻿//---------------------------------------------------------------------
// <copyright file="BaseController.cs" company="commerceserver.net">
//     Copyright © commerceserver.net.  All rights reserved.
// </copyright>
// <summary></summary>
//---------------------------------------------------------------------
namespace CommerceServerContrib.Web.Controllers
{
    using CommerceServerContrib.CrossTier.Helpers;
    using CommerceServerContrib.CrossTier.Interfaces;
    using CommerceServerContrib.CrossTier.Interfaces.Controllers;
    using System.Web;
    using System;
    using CommerceServer.Foundation.RequestTemplates;
    using CommerceServer.Foundation;
    using StringConstants = CommerceServerContrib.CrossTier.Helpers.StringConstants;

    /// <summary>
    /// A base controller class to help with some of the most common scenarios
    /// </summary>
    /// <typeparam name="TRequestTemplateController">The type of the request template controller.</typeparam>
    public abstract class BaseController<TRequestTemplateController> : IBaseController
        where TRequestTemplateController : IRequestTemplateController
    {
        /// <summary>
        /// The request template controller instannce
        /// </summary>
        private TRequestTemplateController _requestTemplateController;

        /// <summary>
        /// The display name associated to the controller
        /// </summary>
        private string _displayName;

        /// <summary>
        /// Gets the template controller responsible for providing available templates for the subsystem.
        /// </summary>
        public IRequestTemplateController TemplateController
        {
            get
            {
                if (this._requestTemplateController == null)
                {
                    this._requestTemplateController = PresentationServiceLocator.Current.GetImplementation<TRequestTemplateController>();
                }

                return this._requestTemplateController;
            }
        }

        /// <summary>
        /// Gets the group id associated to the controller.
        /// </summary>
        public abstract string GroupId
        {
            get;
        }

        /// <summary>
        /// Gets or sets the display name for the controller.
        /// </summary>
        public virtual string DisplayName
        {
            get
            {
                if (string.IsNullOrEmpty(this._displayName))
                {
                    this._displayName = ClientFrameworkResources.ResourceManager.GetString(this.GetType().Name + "_DisplayName");
                }

                return this._displayName;
            }

            set
            {
                this._displayName = value;
            }
        }

        /// <summary>
        /// Gets or sets the query id.
        /// </summary>
        /// <value>
        /// The query id.
        /// </value>
        protected string QueryId
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the output.
        /// </summary>
        /// <value>
        /// The output.
        /// </value>
        protected string Output
        {
            get;
            set;
        }

        /// <summary>
        /// Queues up the request to the backend.
        /// </summary>
        /// <param name="templateId">The template id.</param>
        /// <param name="parameters">The parameters required by the template.</param>
        public virtual void QueueRequest(string templateId, RequestTemplateParameterCollection parameters)
        {            
            this.QueueRequest(templateId, parameters, StringConstants.OutputTypeXmlDataContract);
        }

        /// <summary>
        /// Queues up the request to the backend.
        /// </summary>
        /// <param name="templateId">The template id.</param>
        /// <param name="parameters">The parameters required by the template.</param>
        /// <param name="outputType">Type of the output.</param>
        public virtual void QueueRequest(string templateId, RequestTemplateParameterCollection parameters, string outputType)
        {
            ParameterChecker.CheckForNullOrEmpty(templateId, "templateId");
            ParameterChecker.CheckForNullOrEmpty(this.GroupId, "GroupId");
            ParameterChecker.CheckForNull(outputType, "outputType");

            this.ValidateRequest(templateId, parameters);
            this.Output = outputType;

            var svcMgr = PresentationTypeLoader.ServiceAdapter;

            this.QueryId = svcMgr.QueueRequest(templateId, parameters, outputType);
        }

        /// <summary>
        /// Executes the template the request.
        /// </summary>
        /// <returns>
        /// The results from the request
        /// </returns>
        public virtual ServiceAdapterResponse<object> ProcessRequest()
        {
            CommerceParameterChecker.CheckForNullOrEmptyProperty(this.QueryId, this, "QueryId");

            var svcMgr = PresentationTypeLoader.ServiceAdapter;
            var response = svcMgr.RetrieveResponse(this.QueryId);

            this.QueryId = null;

            return response;
        }

        /// <summary>
        /// Immediately prcesses a request template request.
        /// </summary>
        /// <param name="templateId">The template id.</param>
        /// <param name="parameters">The parameters required by the template.</param>
        /// <param name="outputType">Type of the output.</param>
        /// <returns>The results from the RequestTemplate request</returns>
        public virtual ServiceAdapterResponse<object> ProcessRequest(string templateId, RequestTemplateParameterCollection parameters, string outputType)
        {
            ParameterChecker.CheckForNullOrEmpty(templateId, "templateId");
            ParameterChecker.CheckForNullOrEmpty(this.GroupId, "GroupId");
            ParameterChecker.CheckForNull(outputType, "outputType");

            this.ValidateRequest(templateId, parameters);

            // TODO: 
            var svcMgr = PresentationTypeLoader.ServiceAdapter;

            return svcMgr.ProcessRequest(templateId, parameters, outputType);
        }

        /// <summary>
        /// Adds the value to dictionary.
        /// </summary>
        /// <param name="key">The key of the item in the dictionary.</param>
        /// <param name="value">The value to add into the dictionary.</param>
        /// <param name="parameters">The parameters to operate against.</param>
        protected void AddValueToDictionary(string key, object value, RequestTemplateParameterCollection parameters)
        {
            ParameterChecker.CheckForNull(parameters, "parameters");

            if (!parameters.Contains(key))
            {
                var parameter = new RequestTemplateParameter(key, value);

                parameters.Add(parameter);
            }
            else
            {
                parameters[key].Value = value;
            }
        }

        private void ValidateRequest(string templateId, RequestTemplateParameterCollection parameters)
        {
            var siteContext = PresentationServiceLocator.Current.SiteContext;
            var requestTemplate = this.TemplateController.GetRequestTemplate(templateId);

            if (siteContext.IsAjaxCall)
            {
                if (requestTemplate.ContainsSensitiveData.GetValueOrDefault())
                {
                    throw new InvalidOperationException(ClientFrameworkResources.RTSensitiveDataException);
                }

                if (!requestTemplate.GroupName.Equals(this.GroupId, StringComparison.OrdinalIgnoreCase))
                {
                    throw new InvalidOperationException(ClientFrameworkResources.RTAssociatedControllerException);
                }
            }
        }
    }
}
