﻿//---------------------------------------------------------------------
// <copyright file="RequestTemplateController.cs" company="commerceserver.net">
//     Copyright © commerceserver.net.  All rights reserved.
// </copyright>
// <summary></summary>
//---------------------------------------------------------------------

namespace CommerceServerContrib.Web.Controllers
{
    using System;
    using System.Globalization;
    using System.Collections.Generic;
    using CommerceServerContrib.CrossTier.Entities;
    using CommerceServerContrib.CrossTier.Helpers;
    using CommerceServerContrib.CrossTier.Interfaces;
    using CommerceServerContrib.CrossTier.Interfaces.Controllers;
    using CommerceServer.Foundation.RequestTemplates;
    using CommerceServer.Foundation.Definitions;
    
    /// <summary>
    /// Provides request template access
    /// </summary>
    public class RequestTemplateController : IRequestTemplateController
    {
        /// <summary>
        /// Gets all of the available templates for the specified group.
        /// </summary>
        /// <param name="groupId">The group to look for.</param>
        /// <returns>
        /// A collection of all of the templates in that group
        /// </returns>
        public ServiceAdapterResponse<ClientRequestTemplateCollection> GetAvailableTemplates(string groupId)
        {
            var serviceAdapter = PresentationServiceLocator.Current.ServiceAdapter;

            return serviceAdapter.GetAvailableTemplates(groupId);
        }

        /// <summary>
        /// Gets all of the available templates arguments for the specified group and template id.
        /// </summary>
        /// <param name="templateId">The template id.</param>
        /// <returns>
        /// A collection of all of the templates arguments
        /// </returns>
        public ServiceAdapterResponse<RequestTemplateParameterCollection> GetTemplateArguments(string templateId)
        {
            var serviceAdapter = PresentationServiceLocator.Current.ServiceAdapter;

            return serviceAdapter.GetTemplateArguments(templateId);
        }

        /// <summary>
        /// Gets all of the validation rules for the template id
        /// </summary>
        /// <param name="templateId">The template id.</param>
        /// <returns>
        /// All of the rules for the template
        /// </returns>
        public ServiceAdapterResponse<object> GetTemplateValidationRules(string templateId)
        {
            var constraints = new Dictionary<string, Dictionary<string, Dictionary<string, object>>>();
            var rulesCollection = new Dictionary<string, Dictionary<string, object>>();
            var messagesCollection = new Dictionary<string, Dictionary<string, object>>();

            if (!string.IsNullOrEmpty(templateId))
            {
                var requestTemplate = this.GetRequestTemplate(templateId);
                foreach (Operation operation in requestTemplate.Operations)
                {
                    this.GetConstraints(operation, rulesCollection, messagesCollection);
                }
            }

            constraints.Add("rules", rulesCollection);
            constraints.Add("messages", messagesCollection);

            return new ServiceAdapterResponse<object>
            {
                Succeeded = true,
                Value = constraints
            };
        }

        /// <summary>
        /// Gets the request template.
        /// </summary>
        /// <param name="templateId">The template id.</param>
        /// <returns>
        /// The request template
        /// </returns>
        public RequestTemplate GetRequestTemplate(string templateId)
        {
            var serviceAdapter = PresentationServiceLocator.Current.ServiceAdapter;

            return serviceAdapter.GetRequestTemplate(templateId);
        }

        /// <summary>
        /// Gets the constraints.
        /// </summary>
        /// <param name="operation">The operation.</param>
        /// <param name="rulesCollection">The rules collection.</param>
        /// <param name="messagesCollection">The messages collection.</param>
        protected virtual void GetConstraints(Operation operation, Dictionary<string, Dictionary<string, object>> rulesCollection, Dictionary<string, Dictionary<string, object>> messagesCollection)
        {
            foreach (PropertyToSetArgument propertyToSetArgument in operation.PropertiesToSet)
            {
                Dictionary<string, object> propertyRules;
                Dictionary<string, object> propertyMessages;

                this.GetConstraints(propertyToSetArgument.PropertyMetadata, out propertyRules, out propertyMessages);

                if (propertyRules.Count > 0)
                {
                    rulesCollection.Add(propertyToSetArgument.ShortName, propertyRules);
                }

                if (propertyMessages.Count > 0)
                {
                    messagesCollection.Add(propertyToSetArgument.ShortName, propertyMessages);
                }
            }

            foreach (SearchArgument searchArgument in operation.Search)
            {
                Dictionary<string, object> propertyRules;
                Dictionary<string, object> propertyMessages;

                this.GetConstraints(searchArgument.PropertyMetadata, out propertyRules, out propertyMessages);

                if (propertyRules.Count > 0)
                {
                    rulesCollection.Add(searchArgument.ShortName, propertyRules);
                }

                if (propertyMessages.Count > 0)
                {
                    messagesCollection.Add(searchArgument.ShortName, propertyMessages);
                }
            }

            if (operation.RelatedOperations == null || operation.RelatedOperations.Count <= 0)
            {
                return;
            }

            foreach (Operation relatedOperation in operation.RelatedOperations)
            {
                this.GetConstraints(relatedOperation, rulesCollection, messagesCollection);
            }
        }

        /// <summary>
        /// Gets the constraints.
        /// </summary>
        /// <param name="propertyMetadata">The property metadata.</param>
        /// <param name="propertyRules">The property rules.</param>
        /// <param name="propertyMessages">The property messages.</param>
        protected virtual void GetConstraints(PropertyMetadata propertyMetadata, out Dictionary<string, object> propertyRules, out Dictionary<string, object> propertyMessages)
        {
            propertyRules = new Dictionary<string, object>();
            propertyMessages = new Dictionary<string, object>();

            if (propertyMetadata == null || propertyMetadata.Constraints == null)
            {
                return;
            }

            foreach (var constraint in propertyMetadata.Constraints)
            {
                string validatorName;
                object validatorValue;
                switch (constraint.ModelName)
                {
                    case Validators.RangeValidator.ModelName:
                        this.GetRangeValidatorValues(out validatorName, out validatorValue, constraint, propertyMetadata.DataType);
                        break;
                    case Validators.RequiredValidator.ModelName:
                        this.GetRequiredValidatorValues(out validatorName, out validatorValue);
                        break;
                    case Validators.RegularExpression.ModelName:
                        this.GetRegularExpressionValidatorValues(out validatorName, out validatorValue, constraint);
                        break;
                    default:
                        this.GetUnknownValidatorValues(out validatorName, out validatorValue);
                        break;
                }

                if (!string.IsNullOrEmpty(validatorName) && validatorValue != null)
                {
                    propertyRules.Add(validatorName, validatorValue);
                }

                string validatorMessage = string.Empty;
                if (constraint.ValidationMessage != null)
                {
                    foreach (CommerceServer.Foundation.Definitions.MultilingualValue<string>.LanguageValuePair validationMessage in constraint.ValidationMessage)
                    {
                        if (validationMessage.Language == PresentationServiceLocator.Current.SiteContext.UICulture.Name)
                        {
                            validatorMessage = validationMessage.Value;
                            break;
                        }

                        if (string.IsNullOrEmpty(validationMessage.Language))
                        {
                            // Store the default validation message for when the culture specific one isn't found
                            validatorMessage = validationMessage.Value;
                        }
                    }
                }

                if (!string.IsNullOrEmpty(validatorName))
                {
                    propertyMessages.Add(validatorName, validatorMessage);
                }
            }
        }

        /// <summary>
        /// Gets the required validator values.
        /// </summary>
        /// <param name="validatorName">Name of the validator.</param>
        /// <param name="validatorValue">The validator value.</param>
        protected virtual void GetRequiredValidatorValues(out string validatorName, out object validatorValue)
        {
            validatorName = Validators.RequiredValidator.ValidatorJsonName;
            validatorValue = true;
        }

        /// <summary>
        /// Gets the range validator values.
        /// </summary>
        /// <param name="validatorName">Name of the validator.</param>
        /// <param name="validatorValue">The validator value.</param>
        /// <param name="constraint">The constraint.</param>
        /// <param name="dataType">Type of the data.</param>
        protected virtual void GetRangeValidatorValues(out string validatorName, out object validatorValue, ConstraintBase constraint, PropertyDataType? dataType)
        {
            validatorName = string.Empty;

            switch (dataType)
            {
                case PropertyDataType.String:
                    validatorName = Validators.RangeValidator.ValidatorJsonName.RangeLength;
                    break;
                case PropertyDataType.Decimal:
                case PropertyDataType.Integer:
                case PropertyDataType.DateTime:
                    validatorName = Validators.RangeValidator.ValidatorJsonName.RangeValue;
                    break;
            }

            var values = new int[2];
            values[0] = Convert.ToInt32(constraint.Properties[Validators.RangeValidator.PropertyName.MinValue], CultureInfo.InvariantCulture);
            values[1] = Convert.ToInt32(constraint.Properties[Validators.RangeValidator.PropertyName.MaxValue], CultureInfo.InvariantCulture);
            validatorValue = values;
        }

        /// <summary>
        /// Gets the regular expresion validator values.
        /// </summary>
        /// <param name="validatorName">Name of the validator.</param>
        /// <param name="validatorValue">The validator value.</param>
        /// <param name="constraint">The constraint.</param>
        protected virtual void GetRegularExpressionValidatorValues(out string validatorName, out object validatorValue, ConstraintBase constraint)
        {
            validatorName = Validators.RegularExpression.ValidatorJsonName;
            validatorValue = string.Empty;

            var patterns = constraint.Properties[Validators.RegularExpression.PropertyName.Pattern] as RegularExpressionList;
            if (patterns == null)
            {
                return;
            }

            foreach (RegularExpression pattern in patterns)
            {
                if (pattern.Culture == PresentationServiceLocator.Current.SiteContext.UICulture.Name)
                {
                    validatorValue = pattern.Value;
                    break;
                }

                if (!string.IsNullOrEmpty(pattern.Culture))
                {
                    continue;
                }

                validatorValue = pattern.Value;
                break;
            }
        }

        /// <summary>
        /// Gets the unknown validator values.
        /// </summary>
        /// <param name="validatorName">Name of the validator.</param>
        /// <param name="validatorValue">The validator value.</param>
        protected virtual void GetUnknownValidatorValues(out string validatorName, out object validatorValue)
        {
            validatorName = string.Empty;
            validatorValue = string.Empty;
        }
    }
}
