﻿//-----------------------------------------------------------------------
// <copyright file="BaseControl.cs" company="commerceserver.net">
//     Copyright © commerceserver.net.  All rights reserved.
// </copyright>
// <summary>Defines the BaseControl class</summary>
//-----------------------------------------------------------------------

namespace CommerceServerContrib.Web
{
    using System;
    using System.Collections.Generic;
    using CommerceServer.Foundation;
    using CommerceServer.Foundation.RequestTemplates;
    using CommerceServerContrib.CrossTier.Interfaces;

    /// <summary>
    /// The base control used by Commerce Server Contrib control
    /// </summary>
    public class BaseControl : System.Web.UI.UserControl, IBaseControl
    {
        private Dictionary<string, object> _viewBag = null;
        private List<ServiceAdapterResponse<object>> _responses;
        private Func<CommerceEntity, string, bool> checkDataName = delegate(CommerceEntity entity, string modelName) { return entity.ModelName == modelName; };

        /// <summary>
        /// Gets or sets the responses that came in from the backend.
        /// </summary>
        /// <value>
        /// The responses from the backend.
        /// </value>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Usage", "CA2227:CollectionPropertiesShouldBeReadOnly", Justification = "Needed to allow external class to change.")]
        public List<object> ResponseData
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the control ID prefix used to denote unique fields in the same user control is used on a page multiple times.
        /// </summary>
        /// <value>
        /// The control ID prefix.
        /// </value>
        public string ControlIdPrefix
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the view bag.
        /// </summary>
        /// <value>
        /// The view bag.
        /// </value>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Usage", "CA2227:CollectionPropertiesShouldBeReadOnly", Justification = "Needs to be reset")]
        public Dictionary<string, object> ViewBag
        {
            get
            {
                return this._viewBag;
            }

            set
            {
                this._viewBag = value;
            }
        }

        /// <summary>
        /// Gets or sets the data.
        /// </summary>
        /// <value>
        /// The data.
        /// </value>
        public object Data
        {
            get;
            set;
        }

        /// <summary>
        /// Gets all of the responses passed to the control.
        /// </summary>
        protected List<ServiceAdapterResponse<object>> Responses
        {
            get
            {
                if (this._responses == null)
                {
                    this._responses = new List<ServiceAdapterResponse<object>>();

                    foreach (ServiceAdapterResponse<object> item in this.ResponseData)
                    {
                        this._responses.Add(item);
                    }
                }

                return this._responses;
            }
        }

        /// <summary>
        /// Models this instance.
        /// </summary>
        /// <typeparam name="T">The type to convert the response to</typeparam>
        /// <returns>The model as type T</returns>
        public virtual T Model<T>()
        {
            return GetModel<T>(0);
        }

        /// <summary>
        /// Models this instance.
        /// </summary>
        /// <typeparam name="T">The type to convert the response to</typeparam>
        /// <param name="index">The particular response to get the model from</param>
        /// <returns>The model as type T</returns>
        public virtual T Model<T>(int index)
        {
            return GetModel<T>(index);
        }

        /// <summary>
        /// Models this instance.
        /// </summary>
        /// <param name="modelName">Name of the model.</param>
        /// <returns>
        /// A list of Commerce Entities
        /// </returns>
        public virtual List<CommerceEntity> Model(string modelName)
        {
            return this.Model<CommerceEntity>(modelName, this.checkDataName);
        }

        /// <summary>
        /// Models this instance.
        /// </summary>
        /// <typeparam name="T">The type that the model should return</typeparam>
        /// <param name="modelName">Name of the model.</param>
        /// <param name="matches">The function for figuring out matches.</param>
        /// <returns>
        /// A list of Commerce Entities
        /// </returns>
        public virtual List<T> Model<T>(string modelName, Func<T, string, bool> matches)
        {
            ParameterChecker.CheckForNullOrEmpty(modelName, "modelName");
            ParameterChecker.CheckForNull(matches, "matches");

            if (this.Responses.Count > 0)
            {
                foreach (ServiceAdapterResponse<object> response in this.Responses)
                {
                    if (response != null && response.Value != null && ((List<T>)response.Value).Count > 0)
                    {
                        var value = (List<T>)response.Value;

                        foreach (T item in value)
                        {
                            if (matches(item, modelName))
                            {
                                return (List<T>)response.Value;
                            }
                        }
                    }
                }
            }

            return null;
        }

        /// <summary>
        /// Sets the view bag data.
        /// </summary>
        /// <param name="viewBag">The view bag data.</param>
        public void SetViewBag(Dictionary<string, object> viewBag)
        {
            this._viewBag = viewBag;
        }

        /// <summary>
        /// Returns a specific item from the ViewBag
        /// </summary>
        /// <param name="key">The key to look for in the bag.</param>
        /// <returns>
        /// If found the object will be returned otherwise null
        /// </returns>
        public object GetFromViewBag(string key)
        {
            ParameterChecker.CheckForNullOrEmpty(key, "key");

            if (this._viewBag != null && this._viewBag.ContainsKey(key))
            {
                return this._viewBag[key];
            }

            return null;
        }

        /// <summary>
        /// Returns a specific item from the ViewBag
        /// </summary>
        /// <typeparam name="T">The type to return the object as</typeparam>
        /// <param name="key">The key to look for in the bag.</param>
        /// <returns>If found the object will be returned otherwise null</returns>
        public T GetFromViewBag<T>(string key)
        {
            ParameterChecker.CheckForNullOrEmpty(key, "key");

            if (this._viewBag != null && this._viewBag.ContainsKey(key))
            {
                return (T)this._viewBag[key];
            }

            return default(T);
        }

        /// <summary>
        /// Resources the resource for the specified key.
        /// </summary>
        /// <param name="key">The key.</param>
        /// <returns>Value of the resource.</returns>
        public string Resource(string key)
        {
            return Resource("CSCSiteTemplate", key);
        }

        /// <summary>
        /// Gets a key from the General resource
        /// </summary>
        /// <param name="source">The file to pull the key from</param>
        /// <param name="key">The key to return</param>        
        /// <returns>The value associated to the key</returns>
        public string Resource(string source, string key)
        {
            return (string)GetGlobalResourceObject(source, key);
        }

        private T GetModel<T>(int index)
        {
            if (index < 0)
            {
                index = 0;
            }

            if (this.Responses != null && this.Responses.Count > 0)
            {
                var specificResponse = this.Responses[index];

                if (specificResponse != null && specificResponse.Value != null)
                {
                    return (T)specificResponse.Value;
                }
            }

            return default(T);
        }
    }
}
