﻿//---------------------------------------------------------------------
// <copyright file="BaseWebControl.cs" company="commerceserver.net">
//     Copyright © commerceserver.net.  All rights reserved.
// </copyright>
// <summary>BaseWebControl class</summary>
//---------------------------------------------------------------------
namespace CommerceServerContrib.Web
{
    using System.ComponentModel;
    using System.Security.Permissions;
    using System.Web;
    using System.Web.UI;
    using System.Web.UI.WebControls;

    /// <summary>
    /// Indicates which id to use
    /// </summary>
    public enum RenderIdMethodTypes
    {
        /// <summary>
        /// Do not render any id
        /// </summary>
        None,

        /// <summary>
        /// Render with the client id
        /// </summary>
        ClientId,

        /// <summary>
        /// Render with the server id
        /// </summary>
        ServerId
    }

    /// <summary>
    /// Indicates what tag should be used to render an item
    /// </summary>
    public enum LabelTag
    {
        /// <summary>
        /// Do not render with any tag
        /// </summary>
        None,

        /// <summary>
        /// Render with a H1 tag
        /// </summary>
        H1,

        /// <summary>
        /// Render with a H2 tag
        /// </summary>
        H2,

        /// <summary>
        /// Render with a H3 tag
        /// </summary>
        H3,

        /// <summary>
        /// Render with a H4 tag
        /// </summary>
        H4,

        /// <summary>
        /// Render with a H5 tag
        /// </summary>
        H5,

        /// <summary>
        /// Render with a H6 tag
        /// </summary>
        H6,

        /// <summary>
        /// Render with a Label tag
        /// </summary>
        Label,

        /// <summary>
        /// Render with a Span tag
        /// </summary>
        Span,

        /// <summary>
        /// Render with a P tag
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1704:IdentifiersShouldBeSpelledCorrectly", MessageId = "P", Justification = "This represents a paragraph tag so should not be changed.")]
        P,

        /// <summary>
        /// Render with a Li tag
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "Li", Justification = "This represents a LI tag so should not be changed.")]
        Li
    }

    /// <summary>
    /// Used as a base class for all web controls
    /// </summary>
    [ToolboxItem(false), AspNetHostingPermission(SecurityAction.InheritanceDemand, Level = AspNetHostingPermissionLevel.Minimal), AspNetHostingPermission(SecurityAction.LinkDemand, Level = AspNetHostingPermissionLevel.Minimal)]
    public class BaseWebControl : WebControl
    {
        private RenderIdMethodTypes _renderIdMethod;

        /// <summary>
        /// Initializes a new instance of the <see cref="BaseWebControl"/> class.
        /// </summary>
        public BaseWebControl()
        {
            this._renderIdMethod = RenderIdMethodTypes.None;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="BaseWebControl"/> class.
        /// </summary>
        /// <param name="tag">An HTML tag.</param>
        public BaseWebControl(string tag)
            : base(tag)
        {
            this._renderIdMethod = RenderIdMethodTypes.None;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="BaseWebControl"/> class.
        /// </summary>
        /// <param name="tag">The tag to create.</param>
        public BaseWebControl(HtmlTextWriterTag tag)
            : base(tag)
        {
            this._renderIdMethod = RenderIdMethodTypes.None;
        }

        /// <summary>
        /// Gets the automation client ID.
        /// </summary>
        [DefaultValue(""), Category("Behavior"), Bindable(false), Localizable(false)]
        public virtual string AutomationClientId
        {
            get
            {
                return base.ClientID;
            }
        }

        /// <summary>
        /// Gets the server control identifier generated by ASP.NET.
        /// </summary>
        /// <returns>The server control identifier generated by ASP.NET.</returns>
        public override string ClientID
        {
            get
            {
                switch (this._renderIdMethod)
                {
                    case RenderIdMethodTypes.ClientId:
                        return base.ClientID;

                    case RenderIdMethodTypes.ServerId:
                        return this.ID;
                }

                return string.Empty;
            }
        }

        /// <summary>
        /// Gets or sets the render id method.
        /// </summary>
        /// <value>
        /// The render id method.
        /// </value>
        [DefaultValue(0), Localizable(false), Bindable(false), Category("Behavior")]
        public virtual RenderIdMethodTypes RenderIdMethod
        {
            get
            {
                return this._renderIdMethod;
            }

            set
            {
                this._renderIdMethod = value;
            }
        }

        /// <summary>
        /// Adds HTML attributes and styles that need to be rendered to the specified <see cref="T:System.Web.UI.HtmlTextWriterTag"/>. This method is used primarily by control developers.
        /// </summary>
        /// <param name="writer">A <see cref="T:System.Web.UI.HtmlTextWriter"/> that represents the output stream to render HTML content on the client.</param>
        protected override void AddAttributesToRender(HtmlTextWriter writer)
        {
            string oldID = this.ID;

            if (this._renderIdMethod == RenderIdMethodTypes.None)
            {
                this.ID = null;
            }

            base.AddAttributesToRender(writer);

            if (this._renderIdMethod == RenderIdMethodTypes.None)
            {
                this.ID = oldID;
            }
        }
    }
}
