﻿//---------------------------------------------------------------------
// <copyright file="HyperLink.cs" company="commerceserver.net">
//     Copyright © commerceserver.net.  All rights reserved.
// </copyright>
// <summary>HyperLink class</summary>
//---------------------------------------------------------------------
namespace CommerceServerContrib.Web
{
    using System.Web.UI;
    using System.Web;
    using System.ComponentModel;
    using System.Web.UI.WebControls;
    using System.Security.Permissions;
    using System.Drawing.Design;

    /// <summary>
    /// Used to display hyperlinks
    /// </summary>
    [ToolboxData("<{0}:Hyperlink runat=\"server\"></{0}:Hyperlink>"), ToolboxItem(true), DefaultProperty("Text"), ParseChildren(false), ControlBuilder(typeof(HyperLinkControlBuilder)), AspNetHostingPermission(SecurityAction.LinkDemand, Level = AspNetHostingPermissionLevel.Minimal), AspNetHostingPermission(SecurityAction.InheritanceDemand, Level = AspNetHostingPermissionLevel.Minimal)]
    public class Hyperlink : BaseWebControl
    {
        private bool _encoded;
        private string _text;

        /// <summary>
        /// Initializes a new instance of the <see cref="Hyperlink"/> class.
        /// </summary>
        public Hyperlink()
            : base(HtmlTextWriterTag.A)
        {
            this._text = string.Empty;
            this._encoded = false;
        }

        /// <summary>
        /// Gets or sets the alternate text.
        /// </summary>
        /// <value>
        /// The alternate text.
        /// </value>
        [Category("Appearance"), DefaultValue(""), Bindable(true), Localizable(true)]
        public virtual string AlternateText
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the CSS class disabled.
        /// </summary>
        /// <value>
        /// The CSS class disabled.
        /// </value>
        [DefaultValue(""), Bindable(false), Localizable(false), Category("Appearance")]
        public string CssClassDisabled
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the image URL.
        /// </summary>
        /// <value>
        /// The image URL.
        /// </value>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1056:UriPropertiesShouldNotBeStrings", Justification = "This is a control property so it needs to be a string.")]
        [DefaultValue(""), Bindable(true), Localizable(false), Category("Appearance"), UrlProperty, Editor("System.Web.UI.Design.ImageUrlEditor, System.Design, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a", typeof(UITypeEditor))]
        public virtual string ImageUrl
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the image URL hover.
        /// </summary>
        /// <value>
        /// The image URL hover.
        /// </value>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1056:UriPropertiesShouldNotBeStrings", Justification = "This is a control property so it needs to be a string.")]
        [Bindable(true), Editor("System.Web.UI.Design.ImageUrlEditor, System.Design, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a", typeof(UITypeEditor)), Localizable(false), Category("Appearance"), UrlProperty, DefaultValue("")]
        public virtual string ImageUrlHover
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the inner span number.
        /// </summary>
        /// <value>
        /// The inner span number.
        /// </value>
        [Category("Appearance"), Bindable(true), Localizable(false), DefaultValue(0)]
        public int InnerSpanNumber
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the navigate URL.
        /// </summary>
        /// <value>
        /// The navigate URL.
        /// </value>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1056:UriPropertiesShouldNotBeStrings", Justification = "This is a control property so it needs to be a string.")]
        [UrlProperty, Bindable(true), DefaultValue(""), Editor("System.Web.UI.Design.UrlEditor, System.Design, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a", typeof(UITypeEditor)), Category("Navigation"), Localizable(false)]
        public virtual string NavigateUrl
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the on client click.
        /// </summary>
        /// <value>
        /// The on client click.
        /// </value>
        [DefaultValue(""), Bindable(true), Localizable(false)]
        public virtual string OnClientClick
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the target.
        /// </summary>
        /// <value>
        /// The target.
        /// </value>
        [Bindable(false), Category("Navigation"), Localizable(false), TypeConverter(typeof(TargetConverter)), DefaultValue("")]
        public virtual string Target
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets a value indicating whether the hyperlink requires SSL.
        /// </summary>
        [Bindable(false), Category("Navigation"), Localizable(false), DefaultValue(false)]
        public virtual bool RequireSsl
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the text.
        /// </summary>
        /// <value>
        /// The text to display.
        /// </value>
        [Localizable(true), Bindable(true), DefaultValue(""), Category("Appearance"), PersistenceMode(PersistenceMode.InnerDefaultProperty)]
        public virtual string Text
        {
            get
            {
                return this._text;
            }

            set
            {
                this._text = value;
                this._encoded = true;
            }
        }

        /// <summary>
        /// Adds the attributes to render.
        /// </summary>
        /// <param name="writer">The writer.</param>
        protected override void AddAttributesToRender(HtmlTextWriter writer)
        {
            if (!string.IsNullOrEmpty(this.Target))
            {
                writer.AddAttribute(HtmlTextWriterAttribute.Target, this.Target);
            }

            if (!string.IsNullOrEmpty(this.NavigateUrl))
            {
                writer.AddAttribute(HtmlTextWriterAttribute.Href, this.NamingContainer.ResolveClientUrl(this.NavigateUrl));
            }

            if (this.RequireSsl)
            {
                writer.AddAttribute("requireSSL", "true");
            }

            if (this.Enabled)
            {
                if (!string.IsNullOrEmpty(this.OnClientClick))
                {
                    if (this.Attributes["onclick"] != null)
                    {
                        writer.AddAttribute(HtmlTextWriterAttribute.Onclick, this.Attributes["onclick"] + this.OnClientClick);
                        this.Attributes.Remove("onclick");
                    }
                    else
                    {
                        writer.AddAttribute(HtmlTextWriterAttribute.Onclick, this.OnClientClick);
                    }
                }
            }
            else
            {
                writer.AddAttribute(HtmlTextWriterAttribute.Onclick, "return false;");
            }

            bool enabled = this.Enabled;
            this.Enabled = true;
            base.AddAttributesToRender(writer);
            this.Enabled = enabled;
        }

        /// <summary>
        /// Notifies the server control that an element, either XML or HTML, was parsed, and adds the element to the server control's <see cref="T:System.Web.UI.ControlCollection"/> object.
        /// </summary>
        /// <param name="obj">An <see cref="T:System.Object"/> that represents the parsed element.</param>
        protected override void AddParsedSubObject(object obj)
        {
            if (this.HasControls())
            {
                base.AddParsedSubObject(obj);
            }
            else if (obj is LiteralControl)
            {
                this.Text = ((LiteralControl)obj).Text;
            }
            else
            {
                string text = this.Text;

                if (!string.IsNullOrEmpty(text))
                {
                    this.Text = null;
                    base.AddParsedSubObject(new LiteralControl(text));
                }

                base.AddParsedSubObject(obj);
            }

            this._encoded = false;
        }

        /// <summary>
        /// Renders the contents of the control to the specified writer. This method is used primarily by control developers.
        /// </summary>
        /// <param name="writer">A <see cref="T:System.Web.UI.HtmlTextWriter"/> that represents the output stream to render HTML content on the client.</param>
        protected override void RenderContents(HtmlTextWriter writer)
        {
            for (int i = 0; i < this.InnerSpanNumber; i++)
            {
                writer.RenderBeginTag(HtmlTextWriterTag.Span);
            }

            if (!string.IsNullOrEmpty(this.ImageUrl))
            {
                string imageUrl = this.NamingContainer.ResolveClientUrl(this.ImageUrl);

                if (!string.IsNullOrEmpty(this.ImageUrlHover))
                {
                    writer.AddAttribute("onmouseover", "this.src='" + this.NamingContainer.ResolveClientUrl(this.ImageUrlHover) + "';");
                    writer.AddAttribute("onmouseout", "this.src='" + imageUrl + "';");
                }

                writer.AddAttribute("border", "0");
                writer.AddAttribute(HtmlTextWriterAttribute.Src, imageUrl);
                writer.AddAttribute(HtmlTextWriterAttribute.Alt, string.IsNullOrEmpty(this.AlternateText) ? string.Empty : this.AlternateText);

                if (!string.IsNullOrEmpty(this.ToolTip))
                {
                    writer.AddAttribute(HtmlTextWriterAttribute.Title, this.ToolTip);
                }

                writer.RenderBeginTag(HtmlTextWriterTag.Img);
                writer.RenderEndTag();
            }
            else if (!string.IsNullOrEmpty(this.Text))
            {
                writer.Write(this._encoded ? HttpContext.Current.Server.HtmlEncode(this.Text) : this.Text);
            }
            else if (this.DesignMode)
            {
                writer.Write(this.ID);
            }

            base.RenderContents(writer);

            for (int i = 0; i < this.InnerSpanNumber; i++)
            {
                writer.RenderEndTag();
            }
        }
    }
}
