﻿//---------------------------------------------------------------------
// <copyright file="Label.cs" company="commerceserver.net">
//     Copyright © commerceserver.net.  All rights reserved.
// </copyright>
// <summary>Label class</summary>
//---------------------------------------------------------------------
namespace CommerceServerContrib.Web
{
    using System.ComponentModel;
    using System.Web.UI;
    using System.Web.UI.WebControls;
    using System.Web;
    using System.Security.Permissions;
    using System.Globalization;

    /// <summary>
    /// Used as a label control
    /// </summary>
    [ToolboxData("<{0}:Label runat=\"server\"></{0}:Label>"), ControlBuilder(typeof(LabelControlBuilder)), DefaultProperty("Text"), ParseChildren(false), ToolboxItem(true), ControlValueProperty("Text"), AspNetHostingPermission(SecurityAction.LinkDemand, Level = AspNetHostingPermissionLevel.Minimal), AspNetHostingPermission(SecurityAction.InheritanceDemand, Level = AspNetHostingPermissionLevel.Minimal)]
    public class Label : BaseWebControl
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="Label"/> class.
        /// </summary>
        public Label()
        {
            this.RenderTag = LabelTag.None;
        }

        /// <summary>
        /// Gets or sets the associated control ID.
        /// </summary>
        /// <value>
        /// The associated control ID.
        /// </value>
        [Bindable(true), Localizable(false), DefaultValue(""), Category("Accessibility"), IDReferenceProperty, TypeConverter(typeof(AssociatedControlConverter))]
        public string AssociatedControlId
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the render tag.
        /// </summary>
        /// <value>
        /// The render tag.
        /// </value>
        [DefaultValue(0), Bindable(false), Category("Behavior"), Localizable(false)]
        public LabelTag RenderTag
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the text.
        /// </summary>
        /// <value>
        /// The text to display.
        /// </value>
        [Localizable(true), Bindable(true), Category("Appearance"), DefaultValue("")]
        public string Text
        {
            get;
            set;
        }

        /// <summary>
        /// Renders the HTML opening tag of the control to the specified writer. This method is used primarily by control developers.
        /// </summary>
        /// <param name="writer">A <see cref="T:System.Web.UI.HtmlTextWriter"/> that represents the output stream to render HTML content on the client.</param>
        public override void RenderBeginTag(HtmlTextWriter writer)
        {
            if (this.RenderTag != LabelTag.None)
            {
                this.AddAttributesToRender(writer);
                writer.RenderBeginTag(this.RenderTag.ToString().ToLower(CultureInfo.InvariantCulture));
            }
        }

        /// <summary>
        /// Renders the HTML closing tag of the control into the specified writer. This method is used primarily by control developers.
        /// </summary>
        /// <param name="writer">A <see cref="T:System.Web.UI.HtmlTextWriter"/> that represents the output stream to render HTML content on the client.</param>
        public override void RenderEndTag(HtmlTextWriter writer)
        {
            if (this.RenderTag != LabelTag.None)
            {
                writer.RenderEndTag();
            }
        }

        /// <summary>
        /// Adds the attributes to render.
        /// </summary>
        /// <param name="writer">The writer.</param>
        protected override void AddAttributesToRender(HtmlTextWriter writer)
        {
            base.AddAttributesToRender(writer);
            if ((this.RenderTag == LabelTag.Label) && !string.IsNullOrEmpty(this.AssociatedControlId))
            {
                Control control = this.FindControl(this.AssociatedControlId);
                if (control != null)
                {
                    writer.AddAttribute(HtmlTextWriterAttribute.For, control.ClientID);
                }
            }
        }

        /// <summary>
        /// Notifies the server control that an element, either XML or HTML, was parsed, and adds the element to the server control's <see cref="T:System.Web.UI.ControlCollection"/> object.
        /// </summary>
        /// <param name="obj">An <see cref="T:System.Object"/> that represents the parsed element.</param>
        protected override void AddParsedSubObject(object obj)
        {
            if (this.HasControls())
            {
                base.AddParsedSubObject(obj);
            }
            else if (obj is LiteralControl)
            {
                this.Text = ((LiteralControl)obj).Text;
            }
            else
            {
                string text = this.Text;

                if (!string.IsNullOrEmpty(text))
                {
                    this.Text = null;
                    base.AddParsedSubObject(new LiteralControl(text));
                }

                base.AddParsedSubObject(obj);
            }
        }

        /// <summary>
        /// Renders the control to the specified HTML writer.
        /// </summary>
        /// <param name="writer">The <see cref="T:System.Web.UI.HtmlTextWriter"/> object that receives the control content.</param>
        protected override void Render(HtmlTextWriter writer)
        {
            if (!string.IsNullOrEmpty(this.Text) && !string.IsNullOrEmpty(this.Text.Trim()))
            {
                base.Render(writer);
            }
        }

        /// <summary>
        /// Renders the contents of the control to the specified writer. This method is used primarily by control developers.
        /// </summary>
        /// <param name="writer">A <see cref="T:System.Web.UI.HtmlTextWriter"/> that represents the output stream to render HTML content on the client.</param>
        protected override void RenderContents(HtmlTextWriter writer)
        {
            if (!string.IsNullOrEmpty(this.Text))
            {
                writer.Write(this.Text);
            }
            else if (this.DesignMode)
            {
                writer.Write(this.ID);
            }

            base.RenderContents(writer);
        }
    }
}
