﻿//---------------------------------------------------------------------
// <copyright file="AuthorizationEditorPart.cs" company="commerceserver.net">
//     Copyright © commerceserver.net.  All rights reserved.
// </copyright>
// <summary></summary>
//---------------------------------------------------------------------

namespace CommerceServerContrib.Web
{
    using System.Web.UI;
    using System.Web.UI.HtmlControls;
    using System.Web.UI.WebControls;
    using System.Web.UI.WebControls.WebParts;
    using CommerceServerContrib.CrossTier.Interfaces;

    /// <summary>
    /// An editor part for configuring the request template.
    /// </summary>
    public class AuthorizationEditorPart : EditorPart
    {
        #region Private Fields

        private const string TitleKey = "AuthorizationEditorPartTitle";

        private HtmlTable _outerTable;
        private Label _authorizationRoleNameLabel;
        private DropDownList _authorizationRoleNameDropDown;

        #endregion

        /// <summary>
        /// Initializes a new instance of the <see cref="AuthorizationEditorPart"/> class.
        /// </summary>
        public AuthorizationEditorPart()
        {
            this.Title = ResourceHelper.GetCommerceServerContrib(TitleKey);
        }

        #region Public Properties

        /// <summary>
        /// Gets or sets the authorization name
        /// </summary>
        public string AuthorizationRoleName
        {
            get { return this.CurrentViewState["authorizationRoleName"] == null ? string.Empty : this.CurrentViewState["authorizationRoleName"].ToString(); }
            set { this.CurrentViewState["authorizationRoleName"] = value; }
        }

        /// <summary>
        /// Gets the webpart associated with this editor webpart
        /// </summary>
        protected virtual IGenericServiceWebPart AssociatedWebPart
        {
            get
            {
                return this.WebPartToEdit as IGenericServiceWebPart;
            }
        }

        /// <summary>
        /// Gets the current view state.
        /// </summary>
        /// <value>
        /// The current StateBag
        /// </value>
        protected virtual StateBag CurrentViewState
        {
            get
            {
                return this.ViewState;
            }
        }

        #endregion

        #region Overrides

        /// <summary>
        /// Saves the values in an <see cref="T:System.Web.UI.WebControls.WebParts.EditorPart"/> control to the corresponding properties in the associated <see cref="T:System.Web.UI.WebControls.WebParts.WebPart"/> control.
        /// </summary>
        /// <returns>
        /// True if the action of saving values from the <see cref="T:System.Web.UI.WebControls.WebParts.EditorPart"/> control to the <see cref="T:System.Web.UI.WebControls.WebParts.WebPart"/> control is successful; otherwise (if an error occurs), false.
        /// </returns>
        public override bool ApplyChanges()
        {
            var wp = this.AssociatedWebPart;

            if (wp != null)
            {
                wp.AuthorizationRoleName = this._authorizationRoleNameDropDown.SelectedValue;
            }

            return true;
        }

        /// <summary>
        /// Retrieves the property values from a <see cref="T:System.Web.UI.WebControls.WebParts.WebPart"/> control for its associated <see cref="T:System.Web.UI.WebControls.WebParts.EditorPart"/> control.
        /// </summary>
        public override void SyncChanges()
        {
            var wp = this.AssociatedWebPart;
            if (wp != null)
            {
                this.AuthorizationRoleName = wp.AuthorizationRoleName;
            }
        }

        /// <summary>
        /// Called by the ASP.NET page framework to notify server controls that use composition-based implementation to create any child controls they contain in preparation for posting back or rendering.
        /// </summary>
        protected override void CreateChildControls()
        {
            this._outerTable = new HtmlTable();

            // Setup the authorization name drop down controls
            this.SetupAuthorizationControls(this._outerTable);

            // if the user has any existing selections then set them up
            this.SetExistingUserSelections(this._outerTable);

            this.Controls.Add(this._outerTable);
        }

        #endregion

        #region Private Methods

        /// <summary>
        /// Setups the sort selection controls.
        /// </summary>
        /// <param name="controlTable">The control table.</param>
        private void SetupAuthorizationControls(HtmlTable controlTable)
        {
            // Authorization drop down:
            this._authorizationRoleNameLabel = new Label { Text = ResourceHelper.GetCommerceServerContrib("AuthorizationEditorPart_AuthRoleName") };
            this._authorizationRoleNameDropDown = new DropDownList();
            this.BindAuthorizationRoleNames(this._authorizationRoleNameDropDown);
            controlTable.Rows.Add(UIHelper.CreateControlRow(this._authorizationRoleNameLabel, ResourceHelper.GetCommerceServerContrib("AuthorizationEditorPart_AuthRoleNameDescription")));
            controlTable.Rows.Add(UIHelper.CreateControlRow(this._authorizationRoleNameDropDown, string.Empty));
        }

        /// <summary>
        /// Sets the existing user selections.
        /// </summary>
        /// <param name="controlTable">The control table.</param>
        private void SetExistingUserSelections(HtmlTable controlTable)
        {
            if (!string.IsNullOrEmpty(this.AuthorizationRoleName))
            {
                var selectedAuthName = this._authorizationRoleNameDropDown.Items.FindByValue(this.AuthorizationRoleName);
                this._authorizationRoleNameDropDown.SelectedIndex = this._authorizationRoleNameDropDown.Items.IndexOf(selectedAuthName);
            }
        }

        #endregion

        #region Binding Methods

        /// <summary>
        /// Binds the authorization names to the drop down list.
        /// </summary>
        /// <param name="ddl">The drop down list to bind to.</param>
        private void BindAuthorizationRoleNames(DropDownList ddl)
        {
            ddl.Items.Clear();
            ddl.Items.Add(new ListItem(string.Empty));

            // Load values from configured AuthorizationRoleNamesProvider
            var namesProvider = PresentationServiceLocator.Current.GetImplementation<CommerceServerContrib.CrossTier.Interfaces.IAuthorizationRoleNamesProvider>();

            if (namesProvider != null && namesProvider.GetAuthorizationRoleNames().Count > 0)
            {
                foreach (var name in namesProvider.GetAuthorizationRoleNames())
                {
                    ddl.Items.Add(new ListItem(name));
                }

                this.SetControlsVisibility(true);
            }
            else
            {
                this.SetControlsVisibility(false);
            }
        }

        /// <summary>
        /// This method sets the visibility of UI controls within this EditorPart
        /// </summary>
        /// <param name="visible">Boolean representing whether they should be visible or not</param>
        private void SetControlsVisibility(bool visible)
        {
            this._authorizationRoleNameDropDown.Visible = visible;
            this._authorizationRoleNameLabel.Visible = visible;
        }

        #endregion
    }
}
