﻿//---------------------------------------------------------------------
// <copyright file="ServiceEditorPart.cs" company="commerceserver.net">
//     Copyright © commerceserver.net.  All rights reserved.
// </copyright>
// <summary></summary>
//---------------------------------------------------------------------

namespace CommerceServerContrib.Web
{
    using System;
    using System.Collections.Generic;
    using System.Drawing;
    using System.Linq;
    using System.Web.UI;
    using System.Web.UI.HtmlControls;
    using System.Web.UI.WebControls;
    using System.Web.UI.WebControls.WebParts;
    using CommerceServerContrib.CrossTier.Entities;
    using CommerceServerContrib.CrossTier.Helpers;
    using CommerceServerContrib.CrossTier.Interfaces;
    using CommerceServerContrib.CrossTier.Interfaces.Controllers;
    using Microsoft.Security.Application;
    using CommerceServer.Foundation.RequestTemplates;
    using CommerceServer.Foundation;

    /// <summary>
    /// An editor part for configuring the request template.
    /// </summary>
    public class ServiceEditorPart : EditorPart
    {
        #region Private Fields

        private const string TitleKey = "ServiceEditorPartTitle";

        private DropDownList _serviceDropDown;
        private DropDownList _validationServiceDropDown;
        private DropDownList _requestTemplateDropDown;
        private DropDownList _validationRequestTemplateDropDown;
        private DropDownList _parameterDropDown;
        private DropDownList _parameterSettingType;
        private Button _saveParameterChange;
        private Button _resetSettings;
        private HtmlTable _outerTable;
        private int _parameterTypeRowStartIndex;
        private Label _parameterValueLabel;
        private Label _defaultValueLabel;
        private TextBox _defaultValue;
        private Label _existingParameterLabel;
        private HtmlTable _existingParameterTable;
        private Label _errorSummary;
        private DropDownList _sortTypeDropDown;
        private Label _sortParameterLabel;
        private DropDownList _sortParameterDropDown;
        private Label _sortOrderLabel;
        private DropDownList _sortOrderDropDown;
        private Label _sortParameterQueryStringLabel;
        private DropDownList _sortParameterQueryStringDropDown;
        private Label _sortOrderQueryStringLabel;
        private DropDownList _sortOrderQueryStringDropDown;

        #endregion

        /// <summary>
        /// Initializes a new instance of the <see cref="ServiceEditorPart"/> class.
        /// </summary>
        public ServiceEditorPart()
        {
            this.Title = ResourceHelper.GetCommerceServerContrib(TitleKey);
        }

        #region Public Properties

        /// <summary>
        /// Gets or sets the parameter settings for the request template.
        /// </summary>
        public string ParameterSettings
        {
            get
            {
                return this.CurrentViewState["parameterSettings"] == null ? string.Empty : this.CurrentViewState["parameterSettings"].ToString();
            }

            set
            {
                this.CurrentViewState["parameterSettings"] = value;
            }
        }

        /// <summary>
        /// Gets or sets the sort parameter settings.
        /// </summary>
        public string SortParameterSettings
        {
            get
            {
                return this.CurrentViewState["sortParameterSettings"] == null ? string.Empty : this.CurrentViewState["sortParameterSettings"].ToString();
            }

            set
            {
                this.CurrentViewState["sortParameterSettings"] = value;
            }
        }

        /// <summary>
        /// Gets or sets the controller id.
        /// </summary>
        public string ControllerId
        {
            get
            {
                return this.CurrentViewState["controllerId"] == null ? string.Empty : this.CurrentViewState["controllerId"].ToString();
            }

            set
            {
                this.CurrentViewState["controllerId"] = value;
            }
        }

        /// <summary>
        /// Gets or sets the request template.
        /// </summary>
        public string RequestTemplateId
        {
            get
            {
                return this.CurrentViewState["requestTemplateId"] == null ? string.Empty : this.CurrentViewState["requestTemplateId"].ToString();
            }

            set
            {
                this.CurrentViewState["requestTemplateId"] = value;
            }
        }

        /// <summary>
        /// Gets or sets the validation controller id.
        /// </summary>
        /// <value>
        /// The validation controller id.
        /// </value>
        public string ValidationControllerId
        {
            get
            {
                return this.CurrentViewState["validationControllerId"] == null ? string.Empty : this.CurrentViewState["validationControllerId"].ToString();
            }

            set
            {
                this.CurrentViewState["validationControllerId"] = value;
            }
        }

        /// <summary>
        /// Gets or sets the validation request template id.
        /// </summary>
        /// <value>
        /// The validation request template id.
        /// </value>
        public string ValidationRequestTemplateId
        {
            get
            {
                return this.CurrentViewState["validationRequestTemplateId"] == null ? string.Empty : this.CurrentViewState["validationRequestTemplateId"].ToString();
            }

            set
            {
                this.CurrentViewState["validationRequestTemplateId"] = value;
            }
        }

        /// <summary>
        /// Gets the webpart associated with this editor webpart
        /// </summary>
        protected virtual IGenericServiceWebPart AssociatedWebPart
        {
            get
            {
                return this.WebPartToEdit as IGenericServiceWebPart;
            }
        }

        /// <summary>
        /// Gets the current view state.
        /// </summary>
        /// <value>
        /// The current StateBag
        /// </value>
        protected virtual StateBag CurrentViewState
        {
            get
            {
                return this.ViewState;
            }
        }

        #endregion

        #region Overrides

        /// <summary>
        /// Saves the values in an <see cref="T:System.Web.UI.WebControls.WebParts.EditorPart"/> control to the corresponding properties in the associated <see cref="T:System.Web.UI.WebControls.WebParts.WebPart"/> control.
        /// </summary>
        /// <returns>
        /// true if the action of saving values from the <see cref="T:System.Web.UI.WebControls.WebParts.EditorPart"/> control to the <see cref="T:System.Web.UI.WebControls.WebParts.WebPart"/> control is successful; otherwise (if an error occurs), false.
        /// </returns>
        public override bool ApplyChanges()
        {
            var wp = this.AssociatedWebPart;

            if (wp != null)
            {
                wp.ControllerId = this._serviceDropDown.SelectedValue;
                wp.RequestTemplateId = this._requestTemplateDropDown.SelectedValue;
                wp.ValidationControllerId = this._validationServiceDropDown.SelectedValue;
                wp.ValidationRequestTemplateId = this._validationRequestTemplateDropDown.SelectedValue;
                wp.ParameterSettings = this.ParameterSettings;

                var sortParameterSelection = this.GetSortOrderParameterSettings();

                if (sortParameterSelection != null)
                {
                    switch (sortParameterSelection.ConfigurationType)
                    {
                        case SortConfiguration.Static:
                            if (string.IsNullOrEmpty(sortParameterSelection.ParameterId))
                            {
                                this.SetZoneErrorMessage(ResourceHelper.GetCommerceServerContrib("ServiceEditorPart_SortParameterRequired"));
                                return false;
                            }

                            if (sortParameterSelection.Order == null)
                            {
                                this.SetZoneErrorMessage(ResourceHelper.GetCommerceServerContrib("ServiceEditorPart_SortOrderRequired"));
                                return false;
                            }

                            break;
                        case SortConfiguration.Dynamic:
                            if (string.IsNullOrEmpty(sortParameterSelection.ParameterIdQueryString))
                            {
                                this.SetZoneErrorMessage(ResourceHelper.GetCommerceServerContrib("ServiceEditorPart_SortParameterQueryStringRequired"));
                                return false;
                            }

                            if (string.IsNullOrEmpty(sortParameterSelection.OrderQueryString))
                            {
                                this.SetZoneErrorMessage(ResourceHelper.GetCommerceServerContrib("ServiceEditorPart_SortOrderQueryStringRequired"));
                                return false;
                            }

                            break;
                    }

                    this.SortParameterSettings = SerializationHelper.Serialize(sortParameterSelection, CommerceServerContrib.CrossTier.Helpers.StringConstants.OutputTypeXmlDataContract);
                }
                else
                {
                    this.SortParameterSettings = string.Empty;
                }

                wp.SortParameterSettings = this.SortParameterSettings;
            }

            return true;
        }

        /// <summary>
        /// Retrieves the property values from a <see cref="T:System.Web.UI.WebControls.WebParts.WebPart"/> control for its associated <see cref="T:System.Web.UI.WebControls.WebParts.EditorPart"/> control.
        /// </summary>
        public override void SyncChanges()
        {
            var wp = this.AssociatedWebPart;
            if (wp == null)
            {
                return;
            }

            this.ControllerId = wp.ControllerId;
            this.RequestTemplateId = wp.RequestTemplateId;
            this.ValidationControllerId = wp.ValidationControllerId;
            this.ValidationRequestTemplateId = wp.ValidationRequestTemplateId;
            this.ParameterSettings = wp.ParameterSettings;
            this.SortParameterSettings = wp.SortParameterSettings;
        }

        /// <summary>
        /// Called by the ASP.NET page framework to notify server controls that use composition-based implementation to create any child controls they contain in preparation for posting back or rendering.
        /// </summary>
        protected override void CreateChildControls()
        {
            this._outerTable = new HtmlTable();

            // setup the main selection controls
            this.SetupMainSelectorControls(this._outerTable);

            // setup the controls for specifying service parameters
            this.SetupParameterSelectionControls(this._outerTable);

            // render the existing parameters 
            this.RenderExistingParametersTable(this._outerTable);

            // setup the controls for selecting a parameter to sort by
            this.SetupSortSelectionControls(this._outerTable);

            // if the user has any existing selections then set them up
            this.SetExistingUserSelections(this._outerTable);

            this.Controls.Add(this._outerTable);
        }

        #endregion

        #region Private Methods

        /// <summary>
        /// Setups the main selector controls.
        /// </summary>
        /// <param name="controlTable">The control table.</param>
        private void SetupMainSelectorControls(HtmlTable controlTable)
        {
            var culture = PresentationServiceLocator.Current.SiteContext.UICulture;

            // Validation Service:
            this._validationServiceDropDown = new DropDownList { ID = "validationServiceDropDown", AutoPostBack = true };
            this._validationServiceDropDown.SelectedIndexChanged += (this.ValidationServiceDropDownSelectedIndexChanged);
            this.BindServices(this._validationServiceDropDown);

            controlTable.Rows.Add(UIHelper.CreateControlRow(new Label { Text = ResourceHelper.GetCommerceServerContrib("ServiceEditorPart_ValidationService") }, ResourceHelper.GetCommerceServerContrib("ServiceEditorPart_ValidationServiceDescription")));
            controlTable.Rows.Add(UIHelper.CreateControlRow(this._validationServiceDropDown, string.Empty));

            // Validation Request Template:
            this._validationRequestTemplateDropDown = new DropDownList { ID = "validationRequestTemplateDropDown" };

            controlTable.Rows.Add(UIHelper.CreateControlRow(new Label { Text = ResourceHelper.GetCommerceServerContrib("ServiceEditorPart_ValidationRequestTemplate") }, ResourceHelper.GetCommerceServerContrib("ServiceEditorPart_ValidationRequestTemplateDescription")));
            controlTable.Rows.Add(UIHelper.CreateControlRow(this._validationRequestTemplateDropDown, string.Empty));

            // Service:
            this._serviceDropDown = new DropDownList { ID = "serviceDropDown", AutoPostBack = true };
            this._serviceDropDown.SelectedIndexChanged += (this.ServiceDropDownSelectedIndexChanged);
            this.BindServices(this._serviceDropDown);

            controlTable.Rows.Add(UIHelper.CreateControlRow(new Label { Text = ResourceHelper.GetCommerceServerContrib("ServiceEditorPart_Service") }, ResourceHelper.GetCommerceServerContrib("ServiceEditorPart_ServiceDescription")));
            controlTable.Rows.Add(UIHelper.CreateControlRow(this._serviceDropDown, string.Empty));

            // Request Template:
            this._requestTemplateDropDown = new DropDownList { ID = "requestTemplateDropDown", AutoPostBack = true };
            this._requestTemplateDropDown.SelectedIndexChanged += (this.RequestTemplateDropDownSelectedIndexChanged);

            controlTable.Rows.Add(UIHelper.CreateControlRow(new Label { Text = ResourceHelper.GetCommerceServerContrib("ServiceEditorPart_RequestTemplate") }, ResourceHelper.GetCommerceServerContrib("ServiceEditorPart_RequestTemplateDescription")));
            controlTable.Rows.Add(UIHelper.CreateControlRow(this._requestTemplateDropDown, string.Empty));

            // Parameter:
            this._parameterDropDown = new DropDownList { ID = "parameterDropDown", AutoPostBack = false };

            controlTable.Rows.Add(UIHelper.CreateControlRow(new Label { Text = ResourceHelper.GetCommerceServerContrib("ServiceEditorPart_Parameter") }, ResourceHelper.GetCommerceServerContrib("ServiceEditorPart_ParameterDescription")));
            controlTable.Rows.Add(UIHelper.CreateControlRow(this._parameterDropDown, string.Empty));
        }

        /// <summary>
        /// Setups the parameter selection controls.
        /// </summary>
        /// <param name="controlTable">The control table.</param>
        private void SetupParameterSelectionControls(HtmlTable controlTable)
        {
            var culture = PresentationServiceLocator.Current.SiteContext.UICulture;

            // Parameter Type:
            this._parameterSettingType = new DropDownList { ID = "parameterSettingType", AutoPostBack = true };
            this._parameterSettingType.SelectedIndexChanged += (this.ParameterSettingTypeSelectedIndexChanged);
            this.BindParameterTypes(this._parameterSettingType);
            controlTable.Rows.Add(UIHelper.CreateControlRow(new Label { Text = ResourceHelper.GetCommerceServerContrib("ServiceEditorPart_ParameterType") }, ResourceHelper.GetCommerceServerContrib("ServiceEditorPart_ParameterTypeDescription")));
            controlTable.Rows.Add(UIHelper.CreateControlRow(this._parameterSettingType, string.Empty));

            // Default Value:
            this._defaultValueLabel = new Label { Text = ResourceHelper.GetCommerceServerContrib("ServiceEditorPart_ParameterDefaultValue") };
            this._defaultValue = new TextBox();
            controlTable.Rows.Add(UIHelper.CreateControlRow(this._defaultValueLabel, ResourceHelper.GetCommerceServerContrib("ServiceEditorPart_ParameterDefaultValueDescription")));
            controlTable.Rows.Add(UIHelper.CreateControlRow(this._defaultValue, string.Empty));

            // Parameter Value:
            this._parameterValueLabel = new Label
                                            {
                                                Text = ResourceHelper.GetCommerceServerContrib("ServiceEditorPart_ParameterValue"),
                                                Visible = false
                                            };
            controlTable.Rows.Add(UIHelper.CreateControlRow(this._parameterValueLabel, ResourceHelper.GetCommerceServerContrib("ServiceEditorPart_ParameterValueDescription")));
            this._parameterTypeRowStartIndex = controlTable.Rows.Count;

            this.RenderParameterTypeControls(controlTable);

            // set up the save button to allow the user to save their XSLT parameter changes
            this._saveParameterChange = new Button { ID = "saveParameterChange", Text = ResourceHelper.GetCommerceServerContrib("ServiceEditorPart_Save"), Visible = false };
            this._saveParameterChange.Click += new EventHandler(this.SaveParameterChangeClick);
            controlTable.Rows.Add(UIHelper.CreateControlRow(this._saveParameterChange, string.Empty));

            // set up the label to store an error message in the event an error occurs
            this._errorSummary = new Label() { ID = "errorSummary", RenderTag = LabelTag.Span, Visible = false, ForeColor = Color.Red, Text = ResourceHelper.GetCommerceServerContrib("ServiceEditorPart_CannotSave") };

            controlTable.Rows.Add(UIHelper.CreateControlRow(this._errorSummary, string.Empty));
        }

        /// <summary>
        /// Setups the sort selection controls.
        /// </summary>
        /// <param name="controlTable">The control table.</param>
        private void SetupSortSelectionControls(HtmlTable controlTable)
        {
            var culture = PresentationServiceLocator.Current.SiteContext.UICulture;

            // Spacer
            controlTable.Rows.Add(UIHelper.CreateControlRow(new Label { Text = "&nbsp;" }, string.Empty));

            // Sort Type:
            this._sortTypeDropDown = new DropDownList { ID = "sortTypeDropDown", AutoPostBack = true };
            this._sortTypeDropDown.SelectedIndexChanged += (this.SortTypeDropDownSelectedIndexChanged);
            this.BindSortTypes(this._sortTypeDropDown);
            controlTable.Rows.Add(UIHelper.CreateControlRow(new Label { Text = ResourceHelper.GetCommerceServerContrib("ServiceEditorPart_SortType") }, ResourceHelper.GetCommerceServerContrib("ServiceEditorPart_SortTypeDescription")));
            controlTable.Rows.Add(UIHelper.CreateControlRow(this._sortTypeDropDown, string.Empty));

            // Sort Parameter:
            this._sortParameterLabel = new Label
                                           {
                                               ID = "sortParameterLabel",
                                               Text = ResourceHelper.GetCommerceServerContrib("ServiceEditorPart_SortParameter"),
                                               Visible = false
                                           };
            this._sortParameterDropDown = new DropDownList { Visible = false };
            controlTable.Rows.Add(UIHelper.CreateControlRow(this._sortParameterLabel, ResourceHelper.GetCommerceServerContrib("ServiceEditorPart_SortParameterDescription")));
            controlTable.Rows.Add(UIHelper.CreateControlRow(this._sortParameterDropDown, string.Empty));

            // Sort Order:
            this._sortOrderLabel = new Label
                                       {
                                           Text = ResourceHelper.GetCommerceServerContrib("ServiceEditorPart_SortOrder"),
                                           Visible = false
                                       };
            this._sortOrderDropDown = new DropDownList { Visible = false };
            this.BindSortOrders(this._sortOrderDropDown);
            controlTable.Rows.Add(UIHelper.CreateControlRow(this._sortOrderLabel, ResourceHelper.GetCommerceServerContrib("ServiceEditorPart_SortOrderDescription")));
            controlTable.Rows.Add(UIHelper.CreateControlRow(this._sortOrderDropDown, string.Empty));

            // Sort Parameter QueryString:
            this._sortParameterQueryStringLabel = new Label
                                                      {
                                                          Text = ResourceHelper.GetCommerceServerContrib("ServiceEditorPart_SortParameterQueryString"),
                                                          Visible = false
                                                      };
            this._sortParameterQueryStringDropDown = new DropDownList { Visible = false };
            this.BindSortQueryStrings(this._sortParameterQueryStringDropDown);
            controlTable.Rows.Add(UIHelper.CreateControlRow(this._sortParameterQueryStringLabel, ResourceHelper.GetCommerceServerContrib("ServiceEditorPart_SortParameterQueryStringDescription")));
            controlTable.Rows.Add(UIHelper.CreateControlRow(this._sortParameterQueryStringDropDown, string.Empty));

            // Sort Order QueryString:
            this._sortOrderQueryStringLabel = new Label
                                                  {
                                                      Text = ResourceHelper.GetCommerceServerContrib("ServiceEditorPart_SortOrderQueryString"),
                                                      Visible = false
                                                  };
            this._sortOrderQueryStringDropDown = new DropDownList { Visible = false };
            this.BindSortQueryStrings(this._sortOrderQueryStringDropDown);
            controlTable.Rows.Add(UIHelper.CreateControlRow(this._sortOrderQueryStringLabel, ResourceHelper.GetCommerceServerContrib("ServiceEditorPart_SortOrderQueryStringDescription")));
            controlTable.Rows.Add(UIHelper.CreateControlRow(this._sortOrderQueryStringDropDown, string.Empty));
        }

        /// <summary>
        /// Renders the parameter type controls.
        /// </summary>
        /// <param name="controlTable">The control table.</param>
        private void RenderParameterTypeControls(HtmlTable controlTable)
        {
            // Create the different type of parameter controls and add them to the control collection
            for (var i = this._parameterSettingType.Items.Count - 1; i > 0; i--)
            {
                var parameterType = Type.GetType(this._parameterSettingType.Items[i].Value);
                if (parameterType == null)
                {
                    continue;
                }

                var parameterImplementation = PresentationServiceLocator.Current.GetImplementation(parameterType) as IParameterType;
                if (parameterImplementation == null)
                {
                    continue;
                }

                var parameterControl = parameterImplementation.CreateEditorControl();
                HtmlTableRow newControlRow = UIHelper.CreateControlRow(parameterControl, string.Empty);
                newControlRow.Visible = false;
                controlTable.Rows.Insert(this._parameterTypeRowStartIndex, newControlRow);
            }
        }

        /// <summary>
        /// Sets the existing user selections.
        /// </summary>
        /// <param name="controlTable">The control table.</param>
        private void SetExistingUserSelections(HtmlTable controlTable)
        {
            if (!string.IsNullOrEmpty(this.ControllerId))
            {
                var selectedSvc = this._serviceDropDown.Items.FindByValue(this.ControllerId);
                this._serviceDropDown.SelectedIndex = this._serviceDropDown.Items.IndexOf(selectedSvc);

                this.BindRequestTemplates(this._requestTemplateDropDown, this.ControllerId);

                var selectedTemplate = this._requestTemplateDropDown.Items.FindByValue(this.RequestTemplateId);
                this._requestTemplateDropDown.SelectedIndex = this._requestTemplateDropDown.Items.IndexOf(selectedTemplate);
                this._requestTemplateDropDown.Visible = true;

                this.BindRequestTemplateParameters(this._parameterDropDown, this.ControllerId, this.RequestTemplateId);
                this.BindSortParameters(this._sortParameterDropDown, this.ControllerId, this.RequestTemplateId);
                this._parameterDropDown.Visible = true;

                if (!string.IsNullOrEmpty(this.ValidationControllerId))
                {
                    var selectedValidationSvc = this._validationServiceDropDown.Items.FindByValue(this.ValidationControllerId);
                    this._validationServiceDropDown.SelectedIndex = this._validationServiceDropDown.Items.IndexOf(selectedValidationSvc);

                    this.BindRequestTemplates(this._validationRequestTemplateDropDown, this.ValidationControllerId);

                    var selectedValidationTemplate = this._validationRequestTemplateDropDown.Items.FindByValue(this.ValidationRequestTemplateId);
                    this._validationRequestTemplateDropDown.SelectedIndex = this._validationRequestTemplateDropDown.Items.IndexOf(selectedValidationTemplate);
                    this._validationRequestTemplateDropDown.Visible = true;
                }
            }

            if (string.IsNullOrEmpty(this.SortParameterSettings))
            {
                return;
            }

            var sortParameterSelection = SerializationHelper.Deserialize<SortParameterSelection>(this.SortParameterSettings, CommerceServerContrib.CrossTier.Helpers.StringConstants.OutputTypeXmlDataContract);
            var selectedSortType = this._sortTypeDropDown.Items.FindByValue(sortParameterSelection.ConfigurationType.ToString());
            this._sortTypeDropDown.SelectedIndex = this._sortTypeDropDown.Items.IndexOf(selectedSortType);

            switch (this._sortTypeDropDown.SelectedIndex)
            {
                case (int)SortConfiguration.Static:
                    var selectedSortParameter = this._sortParameterDropDown.Items.FindByValue(sortParameterSelection.ParameterId);
                    this._sortParameterDropDown.SelectedIndex = this._sortParameterDropDown.Items.IndexOf(selectedSortParameter);

                    var selectedSortOder = this._sortOrderDropDown.Items.FindByValue(sortParameterSelection.Order.ToString());
                    this._sortOrderDropDown.SelectedIndex = this._sortOrderDropDown.Items.IndexOf(selectedSortOder);

                    this.DisplaySelectedSortType(SortConfiguration.Static);

                    break;

                case (int)SortConfiguration.Dynamic:
                    var selectedSortParameterQueryString = this._sortParameterQueryStringDropDown.Items.FindByValue(sortParameterSelection.ParameterIdQueryString);
                    this._sortParameterQueryStringDropDown.SelectedIndex = this._sortParameterQueryStringDropDown.Items.IndexOf(selectedSortParameterQueryString);

                    var selectedSortOrderQueryString = this._sortOrderQueryStringDropDown.Items.FindByValue(sortParameterSelection.OrderQueryString);
                    this._sortOrderQueryStringDropDown.SelectedIndex = this._sortOrderQueryStringDropDown.Items.IndexOf(selectedSortOrderQueryString);

                    this.DisplaySelectedSortType(SortConfiguration.Dynamic);

                    break;
            }
        }

        /// <summary>
        /// Renders the a table listing the existing parameters.
        /// </summary>
        /// <param name="controlTable">The control table.</param>
        private void RenderExistingParametersTable(HtmlTable controlTable)
        {
            var culture = PresentationServiceLocator.Current.SiteContext.UICulture;

            if (this._existingParameterTable == null)
            {
                this._existingParameterLabel = new Label { Text = ResourceHelper.GetCommerceServerContrib("ServiceEditorPart_ExistingParameter") };
                controlTable.Rows.Add(UIHelper.CreateControlRow(this._existingParameterLabel, ResourceHelper.GetCommerceServerContrib("ServiceEditorPart_ExistingParameterDescription")));

                controlTable.Rows.Add(UIHelper.CreateControlRow(new Literal { Text = "<style type=\"text/css\"> .existingParameters TD {padding-left:10px;min-width: 80px;padding-bottom: 10px;}</style>" }, string.Empty));

                this._existingParameterTable = this.GenerateExistingParametersTable(this._existingParameterTable, this.ParameterSettings);
                controlTable.Rows.Add(UIHelper.CreateControlRow(this._existingParameterTable, string.Empty));

                this._resetSettings = new Button { ID = "resetSettings", Text = ResourceHelper.GetCommerceServerContrib("ServiceEditorPart_Reset") };
                this._resetSettings.Click += (this.ResetSettingsClick);
                controlTable.Rows.Add(UIHelper.CreateControlRow(this._resetSettings, string.Empty));
            }
            else
            {
                this._existingParameterTable = this.GenerateExistingParametersTable(this._existingParameterTable, this.ParameterSettings);
            }

            this.ToggleExistingParametersTable(!string.IsNullOrEmpty(this.ParameterSettings));
        }

        /// <summary>
        /// Generates the table of existing parameters.
        /// </summary>
        /// <param name="existingParameterTable">The table of existing parameters.</param>
        /// <param name="parameterSettings">The list of parameter settings.</param>
        /// <returns>A table of existing parameters.</returns>
        private HtmlTable GenerateExistingParametersTable(HtmlTable existingParameterTable, string parameterSettings)
        {
            if (existingParameterTable == null)
            {
                existingParameterTable = new HtmlTable();
            }
            else
            {
                existingParameterTable.Rows.Clear();
            }

            existingParameterTable.Attributes.Add("class", "existingParameters");

            if (!string.IsNullOrEmpty(parameterSettings))
            {
                var culture = PresentationServiceLocator.Current.SiteContext.UICulture;
                var paramSettings = SerializationHelper.Deserialize<List<ParameterSelection>>(parameterSettings, CommerceServerContrib.CrossTier.Helpers.StringConstants.OutputTypeXmlDataContract);
                var headerRow = new HtmlTableRow();
                headerRow.Cells.Add(UIHelper.CreateCell(string.Empty));
                headerRow.Cells.Add(UIHelper.CreateCell(string.Concat("<b>", ResourceHelper.GetCommerceServerContrib("ServiceEditorPart_ParameterCell"), "</b>")));
                headerRow.Cells.Add(UIHelper.CreateCell(string.Concat("<b>", ResourceHelper.GetCommerceServerContrib("ServiceEditorPart_TypeCell"), "</b>")));
                headerRow.Cells.Add(UIHelper.CreateCell(string.Concat("<b>", ResourceHelper.GetCommerceServerContrib("ServiceEditorPart_ValueCell"), "</b>")));
                headerRow.Cells.Add(UIHelper.CreateCell(string.Concat("<b>", ResourceHelper.GetCommerceServerContrib("ServiceEditorPart_DefaultValueCell"), "</b>")));
                existingParameterTable.Rows.Add(headerRow);

                foreach (var item in paramSettings)
                {
                    var bodyRow = new HtmlTableRow();
                    var delete = new Button
                    {
                        ID = Encoder.HtmlAttributeEncode(item.Id),
                        CommandName = Encoder.HtmlAttributeEncode(item.Id),
                        Text = ResourceHelper.GetCommerceServerContrib("ServiceEditorPart_Delete")
                    };
                    delete.Click += (this.DeleteSettingClick);
                    bodyRow.Cells.Add(UIHelper.CreateControlCell(delete));
                    bodyRow.Cells.Add(UIHelper.CreateCell(Encoder.HtmlEncode(item.Id)));
                    var friendlyName = UIHelper.GetAttributeValue(Type.GetType(item.SetType), AttributeProperty.FriendlyName);
                    bodyRow.Cells.Add(UIHelper.CreateCell(Encoder.HtmlEncode(friendlyName)));
                    bodyRow.Cells.Add(UIHelper.CreateCell(Encoder.HtmlEncode(item.ContextValue)));
                    bodyRow.Cells.Add(UIHelper.CreateCell(Encoder.HtmlEncode(item.DefaultValue)));
                    existingParameterTable.Rows.Add(bodyRow);
                }
            }

            return existingParameterTable;
        }

        /// <summary>
        /// Toggles the visibility of the existing parameters table.
        /// </summary>
        /// <param name="visible">The visibility flag.</param>
        private void ToggleExistingParametersTable(bool visible)
        {
            this._existingParameterLabel.Visible = visible;
            this._existingParameterTable.Visible = visible;
            this._resetSettings.Visible = visible;
        }

        /// <summary>
        /// Toggles the visibility of the parameter selector controls.
        /// </summary>
        /// <param name="visible">The visibility flag.</param>
        private void ToggleParameterSelectorControls(bool visible)
        {
            this._parameterValueLabel.Visible = visible;
            this._saveParameterChange.Visible = visible;

            if (visible)
            {
                this.ClearErrors();
            }
        }

        /// <summary>
        /// Toggles the visibility of the sort selector controls.
        /// </summary>
        /// <param name="sortType">The sort type.</param>
        private void DisplaySelectedSortType(SortConfiguration sortType)
        {
            this._sortParameterLabel.Visible = sortType == SortConfiguration.Static;
            this._sortParameterDropDown.Visible = sortType == SortConfiguration.Static;
            this._sortOrderLabel.Visible = sortType == SortConfiguration.Static;
            this._sortOrderDropDown.Visible = sortType == SortConfiguration.Static;

            this._sortParameterQueryStringLabel.Visible = sortType == SortConfiguration.Dynamic;
            this._sortParameterQueryStringDropDown.Visible = sortType == SortConfiguration.Dynamic;
            this._sortOrderQueryStringLabel.Visible = sortType == SortConfiguration.Dynamic;
            this._sortOrderQueryStringDropDown.Visible = sortType == SortConfiguration.Dynamic;
        }

        /// <summary>
        /// Resets all parameter types.
        /// </summary>
        private void ResetAllParameterTypes()
        {
            for (var i = 1; i < this._parameterSettingType.Items.Count; i++)
            {
                var parameterTypeControl = this._outerTable.Rows[this._parameterTypeRowStartIndex + (i - 1)].Controls[0].Controls[0] as Control;
                var parameterType = Type.GetType(this._parameterSettingType.Items[i].Value);
                var parameterImplementation = PresentationServiceLocator.Current.GetImplementation(parameterType) as IParameterType;

                if (parameterImplementation != null)
                {
                    parameterImplementation.ResetEditorControl(parameterTypeControl);
                }
            }

            this.DisplaySelectedParameterType();
        }

        /// <summary>
        /// Displays the type of the selected parameter.
        /// </summary>
        private void DisplaySelectedParameterType()
        {
            for (var i = 1; i < this._parameterSettingType.Items.Count; i++)
            {
                this._outerTable.Rows[this._parameterTypeRowStartIndex + (i - 1)].Visible = i == this._parameterSettingType.SelectedIndex;
            }
        }

        /// <summary>
        /// Resets the settings click.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void ResetSettingsClick(object sender, EventArgs e)
        {
            this.ControllerId = string.Empty;
            this.RequestTemplateId = string.Empty;
            this.ValidationControllerId = string.Empty;
            this.ValidationRequestTemplateId = string.Empty;
            this.ParameterSettings = string.Empty;
            this.RenderExistingParametersTable(this._outerTable);
            this.ResetSelectorControls();
            this.ResetSortingControls();
        }

        /// <summary>
        /// Deletes the settings click.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void DeleteSettingClick(object sender, EventArgs e)
        {
            var paramenterId = ((Button)sender).CommandName;
            var workingParameterCollection = SerializationHelper.Deserialize<List<ParameterSelection>>(this.ParameterSettings, CommerceServerContrib.CrossTier.Helpers.StringConstants.OutputTypeXmlDataContract);
            var parameterSelected = workingParameterCollection.Where(s => s.Id == paramenterId).FirstOrDefault();
            workingParameterCollection.Remove(parameterSelected);
            this.ParameterSettings = SerializationHelper.Serialize(workingParameterCollection, CommerceServerContrib.CrossTier.Helpers.StringConstants.OutputTypeXmlDataContract);

            this.RenderExistingParametersTable(this._outerTable);
        }

        /// <summary>
        /// Resets the sorting controls.
        /// </summary>
        private void ResetSortingControls()
        {
            this._sortTypeDropDown.SelectedIndex = 0;
            this.DisplaySelectedSortType(SortConfiguration.None);
        }

        /// <summary>
        /// Resets the parameter selector controls.
        /// </summary>
        private void ResetParameterSelectorControls()
        {
            if (this._parameterDropDown.Items.Count > 0)
            {
                this._parameterDropDown.SelectedIndex = 0;
            }

            this._parameterSettingType.SelectedIndex = 0;
            this._defaultValue.Text = string.Empty;
            this.ToggleParameterSelectorControls(false);
            this.ResetAllParameterTypes();
        }

        /// <summary>
        /// Resets the selector controls.
        /// </summary>
        private void ResetSelectorControls()
        {
            this._serviceDropDown.SelectedIndex = 0;
            this._requestTemplateDropDown.Items.Clear();
            this._validationServiceDropDown.SelectedIndex = 0;
            this._validationRequestTemplateDropDown.Items.Clear();
            this._parameterDropDown.Items.Clear();

            this.ResetParameterSelectorControls();
        }

        /// <summary>
        /// Saves the parameter change click.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void SaveParameterChangeClick(object sender, EventArgs e)
        {
            var parameterControlIndex = this._parameterTypeRowStartIndex + (this._parameterSettingType.SelectedIndex - 1);
            var parameterControl = this._outerTable.Rows[parameterControlIndex].Controls[0].Controls[0];

            var parameterSettings = this.SaveParameter(
                this.ParameterSettings,
                this._parameterDropDown.SelectedValue,
                this._parameterSettingType.SelectedValue,
                this._defaultValue.Text,
                parameterControl);

            if (!string.IsNullOrEmpty(parameterSettings))
            {
                this.ParameterSettings = parameterSettings;
                this.ResetParameterSelectorControls();
            }

            this.RenderExistingParametersTable(this._outerTable);
        }

        /// <summary>
        /// Saves the parameter.
        /// </summary>
        /// <param name="parameterSettings">The parameter settings.</param>
        /// <param name="parameterId">The parameter id.</param>
        /// <param name="parameterTypeValue">The parameter type value.</param>
        /// <param name="defaultValue">The default value.</param>
        /// <param name="parameterControl">The parameter control.</param>
        /// <returns>The saved parameter</returns>
        private string SaveParameter(
            string parameterSettings,
            string parameterId,
            string parameterTypeValue,
            string defaultValue,
            Control parameterControl)
        {
            bool isValid = true;
            this.ClearErrors();
            var culture = PresentationServiceLocator.Current.SiteContext.UICulture;

            if (string.IsNullOrEmpty(parameterId))
            {
                this.SetError(ResourceHelper.GetCommerceServerContrib("ServiceEditorPart_ParameterNameRequired"));
                isValid = false;
            }

            if (string.IsNullOrEmpty(parameterTypeValue))
            {
                this.SetError(ResourceHelper.GetCommerceServerContrib("ServiceEditorPart_ParameterTypeRequired"));
                isValid = false;
            }

            if (isValid)
            {
                var workingParameterCollection = new List<ParameterSelection>();
                if (!string.IsNullOrEmpty(parameterSettings))
                {
                    workingParameterCollection = SerializationHelper.Deserialize<List<ParameterSelection>>(parameterSettings, CommerceServerContrib.CrossTier.Helpers.StringConstants.OutputTypeXmlDataContract);
                }

                var parameterSelection = workingParameterCollection.Where(s => s.Id == parameterId).FirstOrDefault();
                if (parameterSelection == null)
                {
                    parameterSelection = new ParameterSelection();
                    workingParameterCollection.Add(parameterSelection);
                }

                parameterSelection.Id = parameterId;
                parameterSelection.SetType = parameterTypeValue;
                parameterSelection.DefaultValue = defaultValue;

                var parameterType = Type.GetType(parameterTypeValue);
                var parameterImplementation = PresentationServiceLocator.Current.GetImplementation(parameterType) as IParameterType;

                if (parameterImplementation != null)
                {
                    parameterSelection.ContextValue =
                        parameterImplementation.GetContextValueFromEditorControl(parameterControl);

                    if (string.IsNullOrEmpty(parameterSelection.ContextValue))
                    {
                        this.SetError(ResourceHelper.GetCommerceServerContrib("ServiceEditorPart_ParameterValueRequired"));
                    }
                    else
                    {
                        return SerializationHelper.Serialize(workingParameterCollection, CommerceServerContrib.CrossTier.Helpers.StringConstants.OutputTypeXmlDataContract);
                    }
                }
            }

            return string.Empty;
        }

        /// <summary>
        /// Sets the zone error message.
        /// </summary>
        /// <param name="message">The message.</param>
        private void SetZoneErrorMessage(string message)
        {
            if (this.Zone != null)
            {
                this.Zone.ErrorText = message;
            }
        }

        #endregion

        #region Binding Methods

        /// <summary>
        /// Binds a list of all of the available parameter types to the drop down
        /// </summary>
        /// <param name="ddl">The drop down list to bind to.</param>
        private void BindParameterTypes(DropDownList ddl)
        {
            ddl.Items.Clear();

            foreach (var param in PresentationServiceLocator.Current.GetImplementations<IParameterType>(false))
            {
                var friendlyName = UIHelper.GetAttributeValue(param.GetType(), AttributeProperty.FriendlyName);
                ddl.Items.Add(new ListItem(friendlyName, param.GetType().AssemblyQualifiedName));
            }

            ddl.Items.Insert(0, new ListItem(string.Empty));
        }

        /// <summary>
        /// Binds a list of all of the available services to the drop down
        /// </summary>
        /// <param name="ddl">The drop down list to bind to.</param>
        private void BindServices(DropDownList ddl)
        {
            ddl.Items.Clear();

            foreach (var svc in PresentationServiceLocator.Current.AvailableServices)
            {
                ddl.Items.Add(new ListItem(svc.FriendlyName, svc.Id));
            }

            ddl.Items.Insert(0, new ListItem(string.Empty));
        }

        /// <summary>
        /// Binds the all of the request templates associated with the controller to the specified drop down
        /// </summary>
        /// <param name="ddl">The drop down list to bind to.</param>
        /// <param name="controllerId">The controller containing all of the methods.</param>
        private void BindRequestTemplates(DropDownList ddl, string controllerId)
        {
            var service = PresentationServiceLocator.Current.AvailableServices.Where(s => s.Id == controllerId).FirstOrDefault();

            ddl.Items.Clear();

            if (service != null)
            {
                foreach (var method in service.Methods)
                {
                    ddl.Items.Add(new ListItem(method.Description, method.Id));
                }
            }

            ddl.Items.Insert(0, new ListItem(string.Empty));
        }

        /// <summary>
        /// Binds the parameters in the method owned by the service to the drop down list.
        /// </summary>
        /// <param name="ddl">The drop down list to bind to.</param>
        /// <param name="controllerId">The id of the controller.</param>
        /// <param name="requestTemplateId">The id of the request template.</param>
        private void BindRequestTemplateParameters(DropDownList ddl, string controllerId, string requestTemplateId)
        {
            ddl.Items.Clear();

            var parameters = this.GetRequestTemplateParameters(controllerId, requestTemplateId);

            foreach (var param in parameters)
            {
                ddl.Items.Add(new ListItem(param.Key, param.Key));
            }

            ddl.Items.Insert(0, new ListItem(string.Empty));
        }

        /// <summary>
        /// Gets the list of requeste template parameters.
        /// </summary>
        /// <param name="controllerId">The id of the service containing the method.</param>
        /// <param name="requestTemplateId">The id of the method whose parameters we want to bind.</param>
        /// <returns>A collection of request template parameters.</returns>
        private RequestTemplateParameterCollection GetRequestTemplateParameters(string controllerId, string requestTemplateId)
        {
            ParameterChecker.CheckForNullOrEmpty(controllerId, "controllerId");
            ParameterChecker.CheckForNullOrEmpty(requestTemplateId, "requestTemplateId");

            var service = PresentationServiceLocator.Current.AvailableServices.Where(s => s.Id == controllerId).FirstOrDefault();

            if (service != null)
            {
                var method = service.Methods.Where(s => s.Id == requestTemplateId).FirstOrDefault();

                if (!string.IsNullOrEmpty(method.Id))
                {
                    var baseService =
                        PresentationServiceLocator.Current.GetImplementation(service.TypeReference) as IBaseController;

                    if (baseService != null)
                    {
                        var serviceResponse = baseService.TemplateController.GetTemplateArguments(requestTemplateId);
                        return serviceResponse.Value;
                    }
                }
            }

            return null;
        }

        /// <summary>
        /// Binds the sort types to the drop down list.
        /// </summary>
        /// <param name="ddl">The drop down list to bind to.</param>
        private void BindSortTypes(DropDownList ddl)
        {
            ddl.Items.Clear();

            ddl.Items.Add(new ListItem("Static"));
            ddl.Items.Add(new ListItem("Dynamic"));

            ddl.Items.Insert(0, new ListItem(string.Empty));
        }

        /// <summary>
        /// Binds the sort parameters to the drop down list.
        /// </summary>
        /// <param name="ddl">The drop down list to bind to.</param>        
        /// <param name="controllerId">The id of the controller.</param>
        /// <param name="requestTemplateId">The id of the request template.</param>
        private void BindSortParameters(DropDownList ddl, string controllerId, string requestTemplateId)
        {
            ddl.Items.Clear();

            var parameters = this.GetRequestTemplateParameters(controllerId, requestTemplateId);

            foreach (var parameter in parameters)
            {
                if (!string.IsNullOrEmpty(parameter.Key) && (parameter.GetRequestTemplateParameterType(parameter.Key) == RequestTemplateParameterType.Sort))
                {
                    ddl.Items.Add(new ListItem(parameter.Key));
                }
            }

            ddl.Items.Insert(0, new ListItem(string.Empty));
        }

        /// <summary>
        /// Binds the sort orders to the drop down list.
        /// </summary>
        /// <param name="ddl">The drop down list to bind to.</param>
        private void BindSortOrders(DropDownList ddl)
        {
            ddl.Items.Clear();

            ddl.Items.Add(new ListItem(string.Empty));
            ddl.Items.Add(new ListItem(SortOrder.Ascending.ToString(), SortOrder.Ascending.ToString()));
            ddl.Items.Add(new ListItem(SortOrder.Descending.ToString(), SortOrder.Descending.ToString()));
            ddl.Items.Add(new ListItem(SortOrder.Default.ToString(), SortOrder.Default.ToString()));
        }

        /// <summary>
        /// Binds the sort query strings to the drop down list. 
        /// </summary>
        /// <param name="ddl">The drop down list to bind to.</param>
        private void BindSortQueryStrings(DropDownList ddl)
        {
            var queryStringProvider = PresentationServiceLocator.Current.GetImplementation<IQueryStringProvider>();

            ddl.Items.Clear();
            var queryStringsDictionary = queryStringProvider.GetQueryStrings();
            foreach (var key in queryStringsDictionary.Keys)
            {
                var queryParameterKey = queryStringsDictionary[key].QueryParameterKey;
                var queryParameterDescription = queryStringsDictionary[key].LocalizedDescriptions.ContainsKey(PresentationServiceLocator.Current.SiteContext.CultureUICode)
                    ? queryStringsDictionary[key].LocalizedDescriptions[PresentationServiceLocator.Current.SiteContext.CultureUICode]
                    : queryStringsDictionary[key].LocalizedDescriptions[CommerceServerContrib.CrossTier.Helpers.StringConstants.SiteSettingDefaultCultureName];
                ddl.Items.Add(new ListItem(queryParameterDescription, queryParameterKey));
            }

            ddl.Items.Insert(0, new ListItem(string.Empty));
        }

        #endregion

        #region Control Events

        /// <summary>
        /// Handles the SelectedIndexChanged event of the serviceDropDown control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void ServiceDropDownSelectedIndexChanged(object sender, EventArgs e)
        {
            this.BindRequestTemplates(this._requestTemplateDropDown, this._serviceDropDown.SelectedValue);
            this._requestTemplateDropDown.Visible = true;
        }

        /// <summary>
        /// Validations the service drop down selected index changed.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void ValidationServiceDropDownSelectedIndexChanged(object sender, EventArgs e)
        {
            this.BindRequestTemplates(this._validationRequestTemplateDropDown, this._validationServiceDropDown.SelectedValue);
            this._validationRequestTemplateDropDown.Visible = true;
        }

        /// <summary>
        /// Handles the SelectedIndexChanged event of the methodDropDown control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void RequestTemplateDropDownSelectedIndexChanged(object sender, EventArgs e)
        {
            this.BindRequestTemplateParameters(this._parameterDropDown, this._serviceDropDown.SelectedValue, this._requestTemplateDropDown.SelectedValue);
            this.BindSortParameters(this._sortParameterDropDown, this._serviceDropDown.SelectedValue, this._requestTemplateDropDown.SelectedValue);
            this._parameterDropDown.Visible = true;
        }

        /// <summary>
        /// Parameters the setting type selected index changed.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void ParameterSettingTypeSelectedIndexChanged(object sender, EventArgs e)
        {
            this.ToggleParameterSelectorControls(this._parameterSettingType.SelectedIndex != 0);
            this.DisplaySelectedParameterType();
        }

        /// <summary>
        /// Sorts the type drop down selected index changed.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void SortTypeDropDownSelectedIndexChanged(object sender, EventArgs e)
        {
            this.DisplaySelectedSortType((SortConfiguration)this._sortTypeDropDown.SelectedIndex);
        }

        /// <summary>
        /// Set the error message label.
        /// </summary>
        /// <param name="errorText">The error text.</param>
        private void SetError(string errorText)
        {
            if (this._errorSummary != null)
            {
                this._errorSummary.Visible = true;
                this._errorSummary.Text = this._errorSummary.Text + @"<br>" + errorText;
                this._errorSummary.Visible = true;
            }
        }

        /// <summary>
        /// Clears the error messages.
        /// </summary>
        private void ClearErrors()
        {
            if (this._errorSummary != null)
            {
                this._errorSummary.Visible = false;
                this._errorSummary.Text = ResourceHelper.GetCommerceServerContrib("ServiceEditorPart_CannotSave");
            }
        }

        /// <summary>
        /// Get the sort order parameter settings from the editor part.
        /// </summary>
        /// <returns>The sort order parameter settings.</returns>
        private SortParameterSelection GetSortOrderParameterSettings()
        {
            var selection = new SortParameterSelection();

            if (this._sortTypeDropDown.SelectedIndex == 0)
            {
                return null;
            }

            switch (this._sortTypeDropDown.SelectedIndex)
            {
                case (int)SortConfiguration.Static:

                    selection.ConfigurationType = SortConfiguration.Static;
                    selection.ParameterId = this._sortParameterDropDown.SelectedValue;
                    selection.Order = (SortOrder)Enum.Parse(typeof(SortOrder), this._sortOrderDropDown.SelectedValue);

                    break;

                case (int)SortConfiguration.Dynamic:

                    selection.ConfigurationType = SortConfiguration.Dynamic;
                    selection.ParameterIdQueryString = this._sortParameterQueryStringDropDown.SelectedValue;
                    selection.OrderQueryString = this._sortOrderQueryStringDropDown.SelectedValue;

                    break;
            }

            return selection;
        }

        #endregion
    }
}