﻿//---------------------------------------------------------------------
// <copyright file="XsltParameterEditorPart.cs" company="commerceserver.net">
//     Copyright © commerceserver.net.  All rights reserved.
// </copyright>
// <summary></summary>
//---------------------------------------------------------------------

namespace CommerceServerContrib.Web
{
    using System;
    using System.Collections.Generic;
    using System.Drawing;
    using System.Linq;
    using System.Web.UI;
    using System.Web.UI.HtmlControls;
    using System.Web.UI.WebControls;
    using System.Web.UI.WebControls.WebParts;
    using CommerceServerContrib.CrossTier.Entities;
    using CommerceServerContrib.CrossTier.Helpers;
    using CommerceServerContrib.CrossTier.Interfaces;
    using Microsoft.Security.Application;

    /// <summary>
    /// An editor part for configuring xslt parameters.
    /// </summary>
    public class XsltParameterEditorPart : EditorPart
    {
        #region Private Fields

        private const string TitleKey = "XsltParameterEditorPartTitle";

        private TextBox _xsltParameterName;
        private DropDownList _xsltParameterSettingType;
        private Button _saveParameterChange;
        private Button _resetSettings;
        private HtmlTable _outerTable;
        private int _parameterTypeRowStartIndex;
        private Label _parameterValueLabel;
        private int _existingParameterTableIndex;
        private Label _defaultParameterValueLabel;
        private TextBox _xsltDefaultParameterValue;
        private Label _existingParameterLabel;
        private HtmlTable _existingParameterTable;
        private Label _errorSummary;

        #endregion

        /// <summary>
        /// Initializes a new instance of the <see cref="XsltParameterEditorPart"/> class.
        /// </summary>
        public XsltParameterEditorPart()
        {
            this.Title = ResourceHelper.GetCommerceServerContrib(TitleKey);
        }

        #region Public Properties

        /// <summary>
        /// Gets or sets the parameter settings for the request template.
        /// </summary>
        public string XsltParameterSettings
        {
            get
            {
                return this.CurrentViewState["xsltParameterSettings"] == null ? string.Empty : this.CurrentViewState["xsltParameterSettings"].ToString();
            }

            set
            {
                this.CurrentViewState["xsltParameterSettings"] = value;
            }
        }

        /// <summary>
        /// Gets the webpart associated with this editor webpart
        /// </summary>
        protected virtual BaseWebPart AssociatedWebPart
        {
            get
            {
                return this.WebPartToEdit as BaseWebPart;
            }
        }

        /// <summary>
        /// Gets the current view state.
        /// </summary>
        /// <value>
        /// The current StateBag
        /// </value>
        protected virtual StateBag CurrentViewState
        {
            get
            {
                return this.ViewState;
            }
        }

        #endregion

        #region Overrides

        /// <summary>
        /// Saves the values in an <see cref="T:System.Web.UI.WebControls.WebParts.EditorPart"/> control to the corresponding properties in the associated <see cref="T:System.Web.UI.WebControls.WebParts.WebPart"/> control.
        /// </summary>
        /// <returns>
        /// true if the action of saving values from the <see cref="T:System.Web.UI.WebControls.WebParts.EditorPart"/> control to the <see cref="T:System.Web.UI.WebControls.WebParts.WebPart"/> control is successful; otherwise (if an error occurs), false.
        /// </returns>
        public override bool ApplyChanges()
        {
            var wp = this.AssociatedWebPart;

            if (wp != null)
            {
                wp.XsltParameterSettings = this.XsltParameterSettings;
            }

            return true;
        }

        /// <summary>
        /// Retrieves the property values from a <see cref="T:System.Web.UI.WebControls.WebParts.WebPart"/> control for its associated <see cref="T:System.Web.UI.WebControls.WebParts.EditorPart"/> control.
        /// </summary>
        public override void SyncChanges()
        {
            var wp = this.AssociatedWebPart;

            if (wp != null)
            {
                this.XsltParameterSettings = wp.XsltParameterSettings;
            }
        }

        /// <summary>
        /// Called by the ASP.NET page framework to notify server controls that use composition-based implementation to create any child controls they contain in preparation for posting back or rendering.
        /// </summary>
        protected override void CreateChildControls()
        {
            this._outerTable = new HtmlTable();

            // setup the controls for specifying an XSLT parameter
            this.SetupParameterSelectionControls(this._outerTable);

            // render the existing XSLT parameters 
            this.RenderExistingParametersTable(this._outerTable);

            this.Controls.Add(this._outerTable);
        }

        #endregion

        #region Bind Methods

        /// <summary>
        /// Binds a list of all of the available parameter types to the drop down
        /// </summary>
        /// <param name="ddl">The drop down list to bind to.</param>
        private void BindParameterTypes(DropDownList ddl)
        {
            ddl.Items.Clear();

            foreach (var param in PresentationServiceLocator.Current.GetImplementations<IParameterType>(false))
            {
                var friendlyName = UIHelper.GetAttributeValue(param.GetType(), AttributeProperty.FriendlyName);
                ddl.Items.Add(new ListItem(friendlyName, param.GetType().AssemblyQualifiedName));
            }

            ddl.Items.Insert(0, new ListItem(string.Empty));
        }

        #endregion

        #region Control Events

        /// <summary>
        /// Parameters the setting type selected index changed.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void ParameterSettingTypeSelectedIndexChanged(object sender, EventArgs e)
        {
            this.ToggleParameterSelectorControls(this._xsltParameterSettingType.SelectedIndex != 0);
            this.DisplaySelectedParameterType();
        }

        /// <summary>
        /// Resets the settings click.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void ResetSettingsClick(object sender, EventArgs e)
        {
            this.XsltParameterSettings = string.Empty;
            this.RenderExistingParametersTable(this._outerTable);
            this.ResetSelectorControls();
        }

        /// <summary>
        /// Saves the parameter change click.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void SaveParameterChangeClick(object sender, EventArgs e)
        {
            var parameterControlIndex = this._parameterTypeRowStartIndex + (this._xsltParameterSettingType.SelectedIndex - 1);
            var parameterControl = this._outerTable.Rows[parameterControlIndex].Controls[0].Controls[0];

            var xsltParameterSettings = this.SaveXsltParameter(
                this.XsltParameterSettings,
                this._xsltParameterName.Text,
                this._xsltParameterSettingType.SelectedValue,
                this._xsltDefaultParameterValue.Text,
                parameterControl);

            if (!string.IsNullOrEmpty(xsltParameterSettings))
            {
                this.XsltParameterSettings = xsltParameterSettings;
                this.ResetSelectorControls();
            }

            this.RenderExistingParametersTable(this._outerTable);
        }

        #endregion

        #region Private Methods

        /// <summary>
        /// Setups the parameter selection controls.
        /// </summary>
        /// <param name="controlTable">The control table.</param>
        private void SetupParameterSelectionControls(HtmlTable controlTable)
        {
            // Xslt Parameter Name:
            this._xsltParameterName = new TextBox();
            controlTable.Rows.Add(UIHelper.CreateControlRow(new Label { Text = ResourceHelper.GetCommerceServerContrib("XsltParameterEditorPart_XsltParameterName") }, ResourceHelper.GetCommerceServerContrib("XsltParameterEditorPart_XsltParameterNameDescription")));
            controlTable.Rows.Add(UIHelper.CreateControlRow(this._xsltParameterName, string.Empty));

            // Xslt Parameter Type:
            this._xsltParameterSettingType = new DropDownList { ID = "xsltParameterSettingType", AutoPostBack = true };
            this._xsltParameterSettingType.SelectedIndexChanged += new EventHandler(this.ParameterSettingTypeSelectedIndexChanged);
            this.BindParameterTypes(this._xsltParameterSettingType);
            controlTable.Rows.Add(UIHelper.CreateControlRow(new Label { Text = ResourceHelper.GetCommerceServerContrib("XsltParameterEditorPart_XsltParameterType") }, ResourceHelper.GetCommerceServerContrib("XsltParameterEditorPart_XsltParameterTypeDescription")));
            controlTable.Rows.Add(UIHelper.CreateControlRow(this._xsltParameterSettingType, string.Empty));

            // XSLT Default Parameter Value:
            this._defaultParameterValueLabel = new Label { Text = ResourceHelper.GetCommerceServerContrib("XsltParameterEditorPart_XsltParameterDefaultValue") };
            this._xsltDefaultParameterValue = new TextBox();
            controlTable.Rows.Add(UIHelper.CreateControlRow(this._defaultParameterValueLabel, ResourceHelper.GetCommerceServerContrib("XsltParameterEditorPart_XsltParameterDefaultValueDescription")));
            controlTable.Rows.Add(UIHelper.CreateControlRow(this._xsltDefaultParameterValue, string.Empty));

            // XSLT Parameter Value:
            this._parameterValueLabel = new Label
                                            {
                                                Text = ResourceHelper.GetCommerceServerContrib("XsltParameterEditorPart_XsltParameterValue"),
                                                Visible = false
                                            };
            controlTable.Rows.Add(UIHelper.CreateControlRow(this._parameterValueLabel, ResourceHelper.GetCommerceServerContrib("XsltParameterEditorPart_XsltParameterValueDescription")));
            this._parameterTypeRowStartIndex = controlTable.Rows.Count;

            this.RenderParameterTypeControls(controlTable);

            // set up the save button to allow the user to save their XSLT parameter changes
            this._saveParameterChange = new Button { ID = "saveXsltParameterChange", Text = ResourceHelper.GetCommerceServerContrib("XsltParameterEditorPart_Save"), Visible = false };
            this._saveParameterChange.Click += (this.SaveParameterChangeClick);
            controlTable.Rows.Add(UIHelper.CreateControlRow(this._saveParameterChange, string.Empty));

            // set up the label to store an error message in the event an error occurs
            this._errorSummary = new Label { ID = "errorSummary", Visible = false, RenderTag = LabelTag.Span, ForeColor = Color.Red, Text = ResourceHelper.GetCommerceServerContrib("XsltParameterEditorPart_CannotSave") };
            controlTable.Rows.Add(UIHelper.CreateControlRow(this._errorSummary, string.Empty));
        }

        /// <summary>
        /// Renders the parameter type controls.
        /// </summary>
        /// <param name="controlTable">The control table.</param>
        private void RenderParameterTypeControls(HtmlTable controlTable)
        {
            // Create the different type of parameter controls and add them to the control collection
            for (var i = this._xsltParameterSettingType.Items.Count - 1; i > 0; i--)
            {
                var parameterType = Type.GetType(this._xsltParameterSettingType.Items[i].Value);
                if (parameterType == null)
                {
                    continue;
                }

                var parameterImplementation = PresentationServiceLocator.Current.GetImplementation(parameterType) as IParameterType;

                if (parameterImplementation == null)
                {
                    continue;
                }

                var parameterControl = parameterImplementation.CreateEditorControl();
                HtmlTableRow newControlRow = UIHelper.CreateControlRow(parameterControl, string.Empty);
                newControlRow.Visible = false;
                controlTable.Rows.Insert(this._parameterTypeRowStartIndex, newControlRow);
            }
        }

        /// <summary>
        /// Displays the type of the selected parameter.
        /// </summary>
        private void DisplaySelectedParameterType()
        {
            for (var i = 1; i < this._xsltParameterSettingType.Items.Count; i++)
            {
                this._outerTable.Rows[this._parameterTypeRowStartIndex + (i - 1)].Visible = i == this._xsltParameterSettingType.SelectedIndex;
            }
        }

        /// <summary>
        /// Saves the XSLT parameter.
        /// </summary>
        /// <param name="xsltParameterSettings">The XSLT parameter settings.</param>
        /// <param name="xlstParameterId">The XLST parameter id.</param>
        /// <param name="xlstParameterValue">The XLST parameter value.</param>
        /// <param name="xsltDefaultValue">The XSLT default value.</param>
        /// <param name="parameterControl">The parameter control.</param>
        /// <returns>The xslt parameter</returns>
        private string SaveXsltParameter(
            string xsltParameterSettings,
            string xlstParameterId,
            string xlstParameterValue,
            string xsltDefaultValue,
            Control parameterControl)
        {
            bool isValid = true;
            this.ClearErrors();

            if (string.IsNullOrEmpty(xlstParameterId))
            {
                this.SetError(ResourceHelper.GetCommerceServerContrib("XsltParameterEditorPart_ParameterNameRequired"));
                isValid = false;
            }

            if (string.IsNullOrEmpty(xlstParameterValue))
            {
                this.SetError(ResourceHelper.GetCommerceServerContrib("XsltParameterEditorPart_ParameterTypeRequired"));
                isValid = false;
            }

            if (isValid)
            {
                var workingParameterCollection = new List<ParameterSelection>();

                if (!string.IsNullOrEmpty(xsltParameterSettings))
                {
                    workingParameterCollection = SerializationHelper.Deserialize<List<ParameterSelection>>(xsltParameterSettings, StringConstants.OutputTypeXmlDataContract);
                }

                while (xlstParameterId.Contains("."))
                {
                    xlstParameterId = xlstParameterId.Substring(xlstParameterId.IndexOf(".", StringComparison.OrdinalIgnoreCase) + 1);
                }

                var parameterSelection = workingParameterCollection.Where(s => s.Id == xlstParameterId).FirstOrDefault();

                if (parameterSelection == null)
                {
                    parameterSelection = new ParameterSelection();
                    workingParameterCollection.Add(parameterSelection);
                }

                parameterSelection.Id = xlstParameterId;
                parameterSelection.SetType = xlstParameterValue;
                parameterSelection.DefaultValue = xsltDefaultValue;

                var parameterType = Type.GetType(xlstParameterValue);
                var parameterImplementation = PresentationServiceLocator.Current.GetImplementation(parameterType) as IParameterType;

                if (parameterImplementation != null)
                {
                    parameterSelection.ContextValue =
                        parameterImplementation.GetContextValueFromEditorControl(parameterControl);

                    if (string.IsNullOrEmpty(parameterSelection.ContextValue))
                    {
                        this.SetError(ResourceHelper.GetCommerceServerContrib("XsltParameterEditorPart_ParameterValueRequired"));
                    }
                    else
                    {
                        return SerializationHelper.Serialize(workingParameterCollection, StringConstants.OutputTypeXmlDataContract);
                    }
                }
            }

            return string.Empty;
        }

        /// <summary>
        /// Renders the existing parameters table.
        /// </summary>
        /// <param name="controlTable">The control table.</param>
        private void RenderExistingParametersTable(HtmlTable controlTable)
        {
            if (this._existingParameterTable == null)
            {
                this._existingParameterLabel = new Label { Text = ResourceHelper.GetCommerceServerContrib("XsltParameterEditorPart_ExistingParameter") };
                controlTable.Rows.Add(UIHelper.CreateControlRow(this._existingParameterLabel, string.Empty));
                this._existingParameterTableIndex = this._outerTable.Rows.Count;

                controlTable.Rows.Add(UIHelper.CreateControlRow(new Literal { Text = "<style type=\"text/css\"> .existingParameters TD {padding-left:10px;min-width: 80px;padding-bottom: 10px;}</style>" }, string.Empty));

                this._existingParameterTable = this.GenerateExistingParametersTable(this._existingParameterTable, this.XsltParameterSettings);
                controlTable.Rows.Add(UIHelper.CreateControlRow(this._existingParameterTable, string.Empty));

                this._resetSettings = new Button { ID = "resetSettings", Text = ResourceHelper.GetCommerceServerContrib("XsltParameterEditorPart_Reset") };
                this._resetSettings.Click += (this.ResetSettingsClick);
                controlTable.Rows.Add(UIHelper.CreateControlRow(this._resetSettings, string.Empty));
            }
            else
            {
                this._existingParameterTable = this.GenerateExistingParametersTable(this._existingParameterTable, this.XsltParameterSettings);
            }

            this.ToggleExistingParametersTable(!string.IsNullOrEmpty(this.XsltParameterSettings));
        }

        /// <summary>
        /// Generates the existing parameters table.
        /// </summary>
        /// <param name="existingParameterTable">The existing parameter table.</param>
        /// <param name="xsltParameterSettings">The XSLT parameter settings.</param>
        /// <returns>The result html table</returns>
        private HtmlTable GenerateExistingParametersTable(HtmlTable existingParameterTable, string xsltParameterSettings)
        {
            if (existingParameterTable == null)
            {
                existingParameterTable = new HtmlTable();
            }
            else
            {
                existingParameterTable.Rows.Clear();
            }

            existingParameterTable.Attributes.Add("class", "existingParameters");

            if (!string.IsNullOrEmpty(xsltParameterSettings))
            {
                var paramSettings = SerializationHelper.Deserialize<List<ParameterSelection>>(xsltParameterSettings, StringConstants.OutputTypeXmlDataContract);
                var headerRow = new HtmlTableRow();
                headerRow.Cells.Add(UIHelper.CreateCell(string.Empty));
                headerRow.Cells.Add(UIHelper.CreateCell(string.Concat("<b>", ResourceHelper.GetCommerceServerContrib("XsltParameterEditorPart_ParameterCell"), "</b>")));
                headerRow.Cells.Add(UIHelper.CreateCell(string.Concat("<b>", ResourceHelper.GetCommerceServerContrib("XsltParameterEditorPart_TypeCell"), "</b>")));
                headerRow.Cells.Add(UIHelper.CreateCell(string.Concat("<b>", ResourceHelper.GetCommerceServerContrib("XsltParameterEditorPart_ValueCell"), "</b>")));
                headerRow.Cells.Add(UIHelper.CreateCell(string.Concat("<b>", ResourceHelper.GetCommerceServerContrib("XsltParameterEditorPart_DefaultValueCell"), "</b>")));
                existingParameterTable.Rows.Add(headerRow);

                foreach (var item in paramSettings)
                {
                    var bodyRow = new HtmlTableRow();
                    var delete = new Button
                    {
                        ID = Encoder.HtmlAttributeEncode(item.Id),
                        CommandName = Encoder.HtmlAttributeEncode(item.Id),
                        Text = ResourceHelper.GetCommerceServerContrib("ServiceEditorPart_Delete")
                    };
                    delete.Click += (this.DeleteSettingClick);
                    bodyRow.Cells.Add(UIHelper.CreateControlCell(delete));
                    bodyRow.Cells.Add(UIHelper.CreateCell(Encoder.HtmlEncode(item.Id)));
                    var friendlyName = UIHelper.GetAttributeValue(Type.GetType(item.SetType), AttributeProperty.FriendlyName);
                    bodyRow.Cells.Add(UIHelper.CreateCell(Encoder.HtmlEncode(friendlyName)));
                    bodyRow.Cells.Add(UIHelper.CreateCell(Encoder.HtmlEncode(item.ContextValue)));
                    bodyRow.Cells.Add(UIHelper.CreateCell(Encoder.HtmlEncode(item.DefaultValue)));
                    existingParameterTable.Rows.Add(bodyRow);
                }
            }

            return existingParameterTable;
        }

        /// <summary>
        /// Toggles the existing parameters table.
        /// </summary>
        /// <param name="visible">if set to <c>true</c> [visible].</param>
        private void ToggleExistingParametersTable(bool visible)
        {
            this._existingParameterLabel.Visible = visible;
            this._existingParameterTable.Visible = visible;
            this._resetSettings.Visible = visible;
        }

        /// <summary>
        /// Toggles the parameter selector controls.
        /// </summary>
        /// <param name="visible">if set to <c>true</c> [visible].</param>
        private void ToggleParameterSelectorControls(bool visible)
        {
            this._parameterValueLabel.Visible = visible;
            this._saveParameterChange.Visible = visible;

            if (visible)
            {
                this.ClearErrors();
            }
        }

        /// <summary>
        /// Resets the selector controls.
        /// </summary>
        private void ResetSelectorControls()
        {
            this._xsltParameterName.Text = string.Empty;
            this._xsltParameterSettingType.SelectedIndex = 0;
            this._xsltDefaultParameterValue.Text = string.Empty;
            this.ToggleParameterSelectorControls(false);
            this.ResetAllParameterTypes();
        }

        /// <summary>
        /// Deletes the settings click.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void DeleteSettingClick(object sender, EventArgs e)
        {
            var paramenterId = ((Button)sender).CommandName;
            var workingParameterCollection = SerializationHelper.Deserialize<List<ParameterSelection>>(this.XsltParameterSettings, StringConstants.OutputTypeXmlDataContract);
            var parameterSelected = workingParameterCollection.Where(s => s.Id == paramenterId).FirstOrDefault();
            workingParameterCollection.Remove(parameterSelected);
            this.XsltParameterSettings = SerializationHelper.Serialize(workingParameterCollection, StringConstants.OutputTypeXmlDataContract);

            this.RenderExistingParametersTable(this._outerTable);
        }

        /// <summary>
        /// Resets all parameter types.
        /// </summary>
        private void ResetAllParameterTypes()
        {
            for (var i = 1; i < this._xsltParameterSettingType.Items.Count; i++)
            {
                var parameterTypeControl = this._outerTable.Rows[this._parameterTypeRowStartIndex + (i - 1)].Controls[0].Controls[0] as Control;
                var parameterType = Type.GetType(this._xsltParameterSettingType.Items[i].Value);
                var parameterImplementation = PresentationServiceLocator.Current.GetImplementation(parameterType) as IParameterType;

                if (parameterImplementation != null)
                {
                    parameterImplementation.ResetEditorControl(parameterTypeControl);
                }
            }

            this.DisplaySelectedParameterType();
        }

        /// <summary>
        /// Set the error message label.
        /// </summary>
        /// <param name="errorText">The error text.</param>
        private void SetError(string errorText)
        {
            if (this._errorSummary == null)
            {
                return;
            }

            this._errorSummary.Visible = true;
            this._errorSummary.Text = this._errorSummary.Text + @"<br>" + errorText;
            this._errorSummary.Visible = true;
        }

        /// <summary>
        /// Clears the error messages.
        /// </summary>
        private void ClearErrors()
        {
            if (this._errorSummary == null)
            {
                return;
            }

            this._errorSummary.Visible = false;
            this._errorSummary.Text = ResourceHelper.GetCommerceServerContrib("XsltParameterEditorPart_CannotSave");
        }

        #endregion
    }
}
