﻿//---------------------------------------------------------------------
// <copyright file="XsltUriEditorPart.cs" company="commerceserver.net">
//     Copyright © commerceserver.net.  All rights reserved.
// </copyright>
// <summary>XsltUriEditorPart class</summary>
//---------------------------------------------------------------------

namespace CommerceServerContrib.Web
{
    using System;
    using System.Linq;
    using System.Drawing;
    using System.Web.UI;
    using System.Web.UI.HtmlControls;
    using System.Web.UI.WebControls;
    using System.Web.UI.WebControls.WebParts;
    using CommerceServerContrib.CrossTier.Interfaces;

    /// <summary>
    /// An editor part for configuring xslt uri property.
    /// </summary>
    public class XsltUriEditorPart : EditorPart
    {
        #region Private Fields

        private const string TitleKey = "XsltUriEditorPartTitle";
        private const string XsltUriKey = "xsltUri";
        private const string RenderModeKey = "renderMode";

        private HtmlTable _outerTable;
        private TextBox _xsltUri;
        private Label _xsltUriError;
        private DropDownList _renderModeDdl;

        #endregion

        /// <summary>
        /// Initializes a new instance of the <see cref="XsltUriEditorPart"/> class.
        /// </summary>
        public XsltUriEditorPart()
        {
            this.Title = ResourceHelper.GetCommerceServerContrib(TitleKey);
        }

        #region Public Properties

        /// <summary>
        /// Gets or sets the parameter settings for the request template.
        /// </summary>
        public string XsltUri
        {
            get
            {
                return this.CurrentViewState[XsltUriKey] == null ? string.Empty : this.CurrentViewState[XsltUriKey].ToString();
            }

            set
            {
                this.CurrentViewState[XsltUriKey] = value;
            }
        }

        /// <summary>
        /// Gets or sets the parameter settings for the request template.
        /// </summary>
        public RenderModes RenderMode
        {
            get
            {
                return (this.CurrentViewState[RenderModeKey] as RenderModes?) ?? RenderModes.Auto;
            }

            set
            {
                this.CurrentViewState[RenderModeKey] = value;
            }
        }

        /// <summary>
        /// Gets the webpart associated with this editor webpart
        /// </summary>
        protected virtual BaseWebPart AssociatedWebPart
        {
            get
            {
                return this.WebPartToEdit as BaseWebPart;
            }
        }

        /// <summary>
        /// Gets the current view state.
        /// </summary>
        /// <value>
        /// The current StateBag
        /// </value>
        protected virtual StateBag CurrentViewState
        {
            get
            {
                return this.ViewState;
            }
        }

        #endregion

        #region Overrides

        /// <summary>
        /// Saves the values in an <see cref="T:System.Web.UI.WebControls.WebParts.EditorPart"/> control to the corresponding properties in the associated <see cref="T:System.Web.UI.WebControls.WebParts.WebPart"/> control.
        /// </summary>
        /// <returns>
        /// true if the action of saving values from the <see cref="T:System.Web.UI.WebControls.WebParts.EditorPart"/> control to the <see cref="T:System.Web.UI.WebControls.WebParts.WebPart"/> control is successful; otherwise (if an error occurs), false.
        /// </returns>
        public override bool ApplyChanges()
        {
            this.RenderMode = (RenderModes)Enum.Parse(typeof(RenderModes), this._renderModeDdl.SelectedValue);
            var isXsltUriValid = this.ValidateXsltUri();
            if (!isXsltUriValid)
            {
                return false;
            }

            var wp = this.AssociatedWebPart;
            if (wp != null)
            {
                wp.XsltUri = this.XsltUri;
                wp.SpecifiedRenderMode = this.RenderMode;
            }

            return true;
        }

        /// <summary>
        /// Retrieves the property values from a <see cref="T:System.Web.UI.WebControls.WebParts.WebPart"/> control for its associated <see cref="T:System.Web.UI.WebControls.WebParts.EditorPart"/> control.
        /// </summary>
        public override void SyncChanges()
        {
            var wp = this.AssociatedWebPart;

            if (wp != null)
            {
                this.XsltUri = wp.XsltUri;
                this.RenderMode = wp.SpecifiedRenderMode;
            }
        }

        /// <summary>
        /// Called by the ASP.NET page framework to notify server controls that use composition-based implementation to create any child controls they contain in preparation for posting back or rendering.
        /// </summary>
        protected override void CreateChildControls()
        {
            this._outerTable = new HtmlTable();
            this.SetupXsltUriControls(this._outerTable);
            this.Controls.Add(this._outerTable);
        }

        #endregion

        #region Private Methods

        /// <summary>
        /// Setups the parameter selection controls.
        /// </summary>
        /// <param name="controlTable">The control table.</param>
        private void SetupXsltUriControls(HtmlTable controlTable)
        {
            controlTable.Rows.Add(UIHelper.CreateControlRow(new Label { Text = ResourceHelper.GetCommerceServerContrib("WebPartProperty_SpecifiedRenderModeDisplayName") }, ResourceHelper.GetCommerceServerContrib("WebPartProperty_SpecifiedRenderModeDescription")));
            this._renderModeDdl = new DropDownList();

            var values = Enum.GetValues(typeof(RenderModes)).Cast<RenderModes>();

            foreach (var value in values)
            {
                var item = new ListItem(value.ToString(), value.ToString());

                if (item.Value == this.RenderMode.ToString())
                {
                    item.Selected = true;
                }

                this._renderModeDdl.Items.Add(item);
            }

            controlTable.Rows.Add(UIHelper.CreateControlRow(this._renderModeDdl, string.Empty));

            controlTable.Rows.Add(UIHelper.CreateControlRow(new Label { Text = ResourceHelper.GetCommerceServerContrib("XsltUriEditorPart_XsltUri") }, ResourceHelper.GetCommerceServerContrib("XsltUriEditorPart_XsltUriDescription")));
            this._xsltUri = new TextBox { Text = this.XsltUri };
            controlTable.Rows.Add(UIHelper.CreateControlRow(this._xsltUri, string.Empty));

            this._xsltUriError = new Label { ID = "xsltUriError", Visible = false, RenderTag = LabelTag.Span, ForeColor = Color.Red, Text = ClientFrameworkResources.InvalidXsltHasBeenSpecified };
            controlTable.Rows.Add(UIHelper.CreateControlRow(this._xsltUriError, string.Empty));
        }

        /// <summary>
        /// Saves the parameter change click.
        /// </summary>
        /// <returns>True if the xlst uri is valid, false otherwise</returns>
        private bool ValidateXsltUri()
        {
            this._xsltUriError.Visible = false;

            if (!string.IsNullOrEmpty(this._xsltUri.Text))
            {
                if (!RenderHelper.XsltFileExists(this._xsltUri.Text))
                {
                    this._xsltUriError.Visible = true;
                    return false;
                }
            }

            this.XsltUri = this._xsltUri.Text;
            return true;
        }

        #endregion
    }
}