﻿//---------------------------------------------------------------------
// <copyright file="AesEncryptionProvider.cs" company="commerceserver.net">
//     Copyright © commerceserver.net.  All rights reserved.
// </copyright>
// <summary>AesEncryptionProvider class</summary>
//---------------------------------------------------------------------

namespace CommerceServerContrib.Web
{
    using System;
    using System.Collections.Generic;
    using System.IO;
    using System.Linq;
    using System.Security.Cryptography;
    using System.Text;
    using CommerceServerContrib.CrossTier.Interfaces;
    using CommerceServerContrib.CrossTier.Exceptions;
    using CommerceServerContrib.CrossTier.Helpers;
    using CommerceServerContrib.Web.Helpers;
    using CommerceServer.Foundation;

    /// <summary>
    /// EncryptionHelper class
    /// </summary>
    public class AesEncryptionProvider : IEncryptionProvider
    {
        #region IEncryptionProvider Implementation

        /// <summary>
        /// Encrypts the specified input value.
        /// </summary>
        /// <param name="valueToEncrypt">The value to encrypt.</param>
        /// <param name="keys">The encryption keys.</param>
        /// <returns>
        /// The inputValue encrypted
        /// </returns>
        public string Encrypt(string valueToEncrypt, Dictionary<string, string> keys)
        {
            ParameterChecker.CheckForNullOrEmpty(valueToEncrypt, "valueToEncrypt");
            ParameterChecker.CheckForNull(keys, "keys");

            using (CommerceSegmentTrace.ClientPresentationTier(this, "Encrypt(string valueToEncrypt, string key)"))
            {
                using (var provider = new AesCryptoServiceProvider())
                {
                    provider.Key = GetKey(keys);
                    using (var stream = new MemoryStream())
                    {
                        using (var transform = provider.CreateEncryptor())
                        {
                            using (var stream2 = new CryptoStream(stream, transform, CryptoStreamMode.Write))
                            {
                                byte[] bytes = new UTF8Encoding().GetBytes(valueToEncrypt);
                                stream2.Write(bytes, 0, bytes.Length);
                                stream2.FlushFinalBlock();
                                string str = Convert.ToBase64String(provider.IV);
                                return ((str.Length).ToString(System.Globalization.CultureInfo.InvariantCulture).PadLeft(3, '0') + str + Convert.ToBase64String(stream.ToArray()));
                            }
                        }
                    }
                }
            }
        }

        /// <summary>
        /// Decrypts the specified encrypted value.
        /// </summary>
        /// <param name="encryptedValue">The encrypted value.</param>
        /// <param name="keys">The encryption keys.</param>
        /// <returns>
        /// The encryptedValue decrypted
        /// </returns>
        public string Decrypt(string encryptedValue, Dictionary<string, string> keys)
        {
            ParameterChecker.CheckForNullOrEmpty(encryptedValue, "encryptedValue");
            ParameterChecker.CheckForNull(keys, "keys");
            
            using (CommerceSegmentTrace.ClientPresentationTier(this, "Decrypt(string encryptedValue, string key)"))
            {
                using (var provider = new AesCryptoServiceProvider())
                {
                    provider.Key = GetKey(keys);
                    int num = Convert.ToInt32(encryptedValue.Substring(0, 3), System.Globalization.CultureInfo.InvariantCulture);
                    string str2 = encryptedValue.Substring(3, num);
                    string str3 = encryptedValue.Substring((num + 3));
                    provider.IV = Convert.FromBase64String(str2);
                    byte[] buffer = Convert.FromBase64String(str3);
                    using (var stream = new MemoryStream())
                    {
                        using (var transform = provider.CreateDecryptor())
                        {
                            using (var stream2 = new CryptoStream(stream, transform, CryptoStreamMode.Write))
                            {
                                stream2.Write(buffer, 0, buffer.Length);
                                stream2.FlushFinalBlock();
                                return new UTF8Encoding().GetString(stream.ToArray());
                            }
                        }
                    }
                }
            }
        }

        #endregion
        
        #region Private Methods
        
        private static byte[] GetKey(Dictionary<string, string> keys)
        {
            var key = string.Empty;
            if (keys.Count > 0)
            {
                key = keys.ToList().FirstOrDefault().Value;
            }

            if (string.IsNullOrEmpty(key))
            {
                throw new CommerceServerContribBaseException(EncryptionHelper.GetCurrentChannelName(), ClientFrameworkResources.AesEncryptionProviderRequiredKey);
            }

            if (!key.StartsWith("registry:", StringComparison.OrdinalIgnoreCase))
            {
                return Convert.FromBase64String(key);
            }

            string registryPath = key.Replace("registry:", string.Empty);
            string keyName = key.Split(new[] { ',' })[1];
            registryPath = registryPath.Replace("," + keyName, string.Empty);

            return EncryptionHelper.GetKeyFromRegistry(registryPath, keyName);
        }

        #endregion
    }
}
