﻿//---------------------------------------------------------------------
// <copyright file="RsaEncryptionProvider.cs" company="commerceserver.net">
//     Copyright © commerceserver.net.  All rights reserved.
// </copyright>
// <summary>RsaEncryptionProvider class</summary>
//---------------------------------------------------------------------

namespace CommerceServerContrib.Web
{
    using System;
    using System.Collections.Generic;
    using System.Security.Cryptography;
    using System.Text;
    using CommerceServerContrib.CrossTier.Interfaces;
    using CommerceServerContrib.CrossTier.Helpers;
    using CommerceServerContrib.Web.Helpers;
    using CommerceServerContrib.CrossTier.Exceptions;
    using CommerceServer.Foundation;

    /// <summary>
    /// EncryptionHelper class
    /// </summary>
    public class RsaEncryptionProvider : IEncryptionProvider
    {
        #region IEncryptionProvider Implementation

        /// <summary>
        /// Encrypts the specified input value.
        /// </summary>
        /// <param name="valueToEncrypt">The value to encrypt.</param>
        /// <param name="keys">The encryption keys.</param>
        /// <returns>
        /// The valueToEncrypt encrypted
        /// </returns>
        public string Encrypt(string valueToEncrypt, Dictionary<string, string> keys)
        {
            ParameterChecker.CheckForNullOrEmpty(valueToEncrypt, "valueToEncrypt");
            ParameterChecker.CheckForNull(keys, "keys");

            using (CommerceSegmentTrace.ClientPresentationTier(this, "Encrypt(string valueToEncrypt, string key)"))
            {
                RSACryptoServiceProvider provider = LoadProvider(keys, false);
                byte[] bytes = Encoding.UTF8.GetBytes(valueToEncrypt);

                return Convert.ToBase64String(provider.Encrypt(bytes, false));
            }
        }

        /// <summary>
        /// Decrypts the specified encrypted value.
        /// </summary>
        /// <param name="encryptedValue">The encrypted value.</param>
        /// <param name="keys">The encryption keys.</param>
        /// <returns>
        /// The encryptedValue decrypted
        /// </returns>
        public string Decrypt(string encryptedValue, Dictionary<string, string> keys)
        {
            ParameterChecker.CheckForNullOrEmpty(encryptedValue, "encryptedValue");
            ParameterChecker.CheckForNull(keys, "keys");

            using (CommerceSegmentTrace.ClientPresentationTier(this, "Decrypt(string encryptedValue, string key)"))
            {
                RSACryptoServiceProvider provider = LoadProvider(keys, true);
                byte[] buffer = Convert.FromBase64String(encryptedValue);
                byte[] buffer2 = provider.Decrypt(buffer, false);

                return Encoding.UTF8.GetString(buffer2);
            }
        }

        #endregion

        #region Private Methods

        private static RSACryptoServiceProvider LoadProvider(Dictionary<string, string> keys, bool usePrivateKey)
        {
            string key = string.Empty;
            if (keys.Count > 0)
            {
                string privateKey = keys.ContainsKey("public") ? keys["public"] : string.Empty;
                string publicKey = keys.ContainsKey("private") ? keys["private"] : string.Empty;
                key = usePrivateKey ? privateKey : publicKey;
            }
            
            if (string.IsNullOrEmpty(key))
            {
                throw new CommerceServerContribBaseException(EncryptionHelper.GetCurrentChannelName(), ClientFrameworkResources.RsaEncryptionProviderRequiredKeys);
            }

            byte[] keyFromRegistry;
            var parameters = new CspParameters(1, "Microsoft Enhanced Cryptographic Provider v1.0", "Profile Key Manager");
            var provider = new RSACryptoServiceProvider(0x800, parameters) { PersistKeyInCsp = false };
            if (key.StartsWith("registry:", StringComparison.OrdinalIgnoreCase))
            {
                string registryPath = key.Replace("registry:", string.Empty);
                string keyName = key.Split(new[] { ',' })[1];
                registryPath = registryPath.Replace("," + keyName, string.Empty);
                keyFromRegistry = EncryptionHelper.GetKeyFromRegistry(registryPath, keyName);
            }
            else
            {
                keyFromRegistry = EncryptionHelper.HexStringToBytes(key);
            }

            provider.ImportCspBlob(keyFromRegistry);
            return provider;
        }

        #endregion
    }
}
