﻿//---------------------------------------------------------------------
// <copyright file="EncryptionHelper.cs" company="commerceserver.net">
//     Copyright © commerceserver.net.  All rights reserved.
// </copyright>
// <summary>EncryptionHelper class</summary>
//---------------------------------------------------------------------

namespace CommerceServerContrib.Web.Helpers
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Security;
    using System.Security.Cryptography;
    using System.Text;
    using CommerceServerContrib.CrossTier.Configuration;
    using CommerceServerContrib.CrossTier.Exceptions;
    using CommerceServerContrib.CrossTier.Interfaces;
    using CommerceServerContrib.CrossTier.Helpers;
    using Microsoft.Win32;
    using CommerceServer.Foundation;
    using CommerceServerContrib.CrossTier.Diagnostics;

    /// <summary>
    /// EncryptionHelper class
    /// </summary>
    public static class EncryptionHelper
    {
        #region Public Methods

        /// <summary>
        /// Encrypts the specified value to encrypt.
        /// </summary>
        /// <param name="valueToEncrypt">The value to encrypt.</param>
        /// <param name="encryptionKeyName">Name of the encryption key.</param>
        /// <returns>The encrypted value</returns>
        public static string Encrypt(string valueToEncrypt, string encryptionKeyName)
        {
            ParameterChecker.CheckForNullOrEmpty(valueToEncrypt, "valueToEncrypt");
            ParameterChecker.CheckForNullOrEmpty(encryptionKeyName, "encryptionKeyName");
            
            Dictionary<string, string> keys;
            IEncryptionProvider encryptionProvider = GetProvider(out keys, encryptionKeyName);
            
            return encryptionProvider.Encrypt(valueToEncrypt, keys);
        }

        /// <summary>
        /// Decrypts the specified encrypted value.
        /// </summary>
        /// <param name="encryptedValue">The encrypted value.</param>
        /// <param name="encryptionKeyName">Name of the encryption key.</param>
        /// <returns>The decrypted value</returns>
        public static string Decrypt(string encryptedValue, string encryptionKeyName)
        {
            ParameterChecker.CheckForNullOrEmpty(encryptedValue, "encryptedValue");
            ParameterChecker.CheckForNullOrEmpty(encryptionKeyName, "encryptionKeyName");

            Dictionary<string, string> keys;
            IEncryptionProvider encryptionProvider = GetProvider(out keys, encryptionKeyName);
            
            return encryptionProvider.Decrypt(encryptedValue, keys);
        }

        /// <summary>
        /// Gets the key from registry.
        /// </summary>
        /// <param name="registryPath">The registry path.</param>
        /// <param name="keyName">Name of the key.</param>
        /// <returns>The key from the registry</returns>
        public static byte[] GetKeyFromRegistry(string registryPath, string keyName)
        {
            byte[] buffer3 = null;
            try
            {
                string subKey = registryPath.Replace(@"HKEY_LOCAL_MACHINE\", string.Empty);
                RegistryKey key = RegistryKey.OpenRemoteBaseKey(RegistryHive.LocalMachine, string.Empty);
                RegistryKey key2 = key.OpenSubKey(subKey, false);
                if (key2 == null)
                {
                    CommerceEventLog.PresentationTierSource.Error("message: {0}", "Registry key value is empty");
                }
                else
                {
                    byte[] cipherText = (byte[])key2.GetValue(keyName, null);
                    byte[] buffer2 = DecryptProtectedData(cipherText);
                    string str2 = Encoding.UTF8.GetString(buffer2);
                    buffer3 = HexStringToBytes(str2);
                }
            }
            catch (SecurityException exception)
            {
                CommerceServerContribExceptionManager.HandleException(exception);
            }

            return buffer3;
        }

        /// <summary>
        /// Hexes the string to bytes.
        /// </summary>
        /// <param name="value">The value.</param>
        /// <returns>The bytes from the string</returns>
        public static byte[] HexStringToBytes(string value)
        {
            if (value == null)
            {
                return null;
            }

            var buffer = new byte[value.Length / 2];
            int index = 0;
            for (int i = 0; i < value.Length; i = (i + 2))
            {
                string str = value.Substring(i, 2);
                buffer[index] = Convert.ToByte(str, 0x10);
                index = (index + 1);
            }

            return buffer;
        }

        /// <summary>
        /// Decrypts the protected data.
        /// </summary>
        /// <param name="cipherText">The cipher text.</param>
        /// <returns>The decryted data</returns>
        public static byte[] DecryptProtectedData(byte[] cipherText)
        {
            byte[] bytes = Encoding.UTF8.GetBytes("Commerce Server Contrib Registry Secrets");
            return ProtectedData.Unprotect(cipherText, bytes, DataProtectionScope.LocalMachine);
        }

        /// <summary>
        /// Gets the name of the current channel.
        /// </summary>
        /// <returns>The name of the current channel</returns>
        public static string GetCurrentChannelName()
        {
            return PresentationTypeLoader.ConfigurationProvider.CurrentConfiguration.Channel;
        }

        #endregion

        #region Private Methods

        private static IEncryptionProvider GetProvider(out Dictionary<string, string> keys, string encryptionKeyName)
        {
            var provider = PresentationServiceLocator.Current.BrandContext.EncryptionProviders[encryptionKeyName];
            var encryptionProviderType = provider.EncryptionProviderType;
            keys = provider.EncryptionKeys.Values.ToDictionary(x => x.Name.ToLowerInvariant(), x => x.Value);

            if (keys.Count == 0)
            {
                throw new CommerceServerContribBaseException(GetCurrentChannelName(), ClientFrameworkResources.EncryptionKeysRequired);
            }

            IEncryptionProvider encryptionProvider = PresentationServiceLocator.Current.EncryptionProviders.Find(x => x.ToString().Contains(encryptionProviderType));
            if (encryptionProvider == null)
            {
                throw new CommerceServerContribBaseException(GetCurrentChannelName(), ClientFrameworkResources.EncryptionProviderTypeRequiredAndValid);
            }

            return encryptionProvider;
        }

        #endregion
    }
}
