﻿//---------------------------------------------------------------------
// <copyright file="PresentationServiceLocator.cs" company="commerceserver.net">
//     Copyright © commerceserver.net.  All rights reserved.
// </copyright>
// <summary></summary>
//---------------------------------------------------------------------
namespace CommerceServerContrib.Web
{
    using System;
    using System.Collections.Generic;
    using System.Web;
    using System.Web.Caching;
    using CommerceServerContrib.CrossTier.Entities;
    using CommerceServerContrib.CrossTier.Helpers;
    using CommerceServerContrib.CrossTier.Interfaces;
    using Microsoft.Practices.Unity;
    using CommerceServerContrib.CrossTier.Exceptions;
    using CommerceServerContrib.CrossTier.Configuration;
    using CommerceServerContrib.CrossTier.Providers;
    using CommerceServerContrib.CrossTier;
    using CommerceServer.Foundation;
    using CommerceServer.Foundation.RequestTemplates;
    using CommerceServer.Foundation.Configuration;

    /// <summary>
    /// Used as an access point for alot of the presentation services
    /// </summary>
    public class PresentationServiceLocator : BaseServiceLocator
    {
        private const string AvailableServicesKey = "AvailableServices";
        private static PresentationServiceLocator _presentationServiceLocator = null;
        private static object _syncRoot = new object();
        private static object _serviceAgentsSyncRoot = new object();
        private static IBrandProvider _brandProvider;
        private static Dictionary<string, IAsyncOperationServiceAgent> _serviceAgents = new Dictionary<string, IAsyncOperationServiceAgent>();

        #region Constructors

        /// <summary>
        /// Prevents a default instance of the <see cref="PresentationServiceLocator"/> class from being created.
        /// </summary>
        private PresentationServiceLocator()
        {
            this.SetUnityContainerNameDelegate(this.GetUnityContainerName);
        }

        #endregion

        #region Static (public, static)

        /// <summary>
        /// Gets the current instance of the PresentationOperations class.
        /// </summary>
        public static PresentationServiceLocator Current
        {
            get
            {
                if (_presentationServiceLocator == null)
                {
                    lock (_syncRoot)
                    {
                        if (_presentationServiceLocator == null)
                        {
                            _presentationServiceLocator = new PresentationServiceLocator();
                        }
                    }
                }

                return _presentationServiceLocator;
            }
        }

        #endregion

        #region Properties (public, non-static)

        /// <summary>
        /// Gets the site context.
        /// </summary>
        [Dependency]
        public ICommerceContext SiteContext
        {
            get
            {
                return PresentationTypeLoader.SiteContext as ICommerceContext;
            }
        }

        /// <summary>
        /// Gets the site settings provider.
        /// </summary>
        public ISiteSettingsProvider SiteSettingProvider
        {
            get
            {
                return GetImplementation<ISiteSettingsProvider>();
            }
        }

        /// <summary>
        /// Gets the brand provider.
        /// </summary>
        public IBrandProvider BrandProvider
        {
            get
            {
                if (_brandProvider == null)
                {
                    lock (_syncRoot)
                    {
                        if (_brandProvider == null)
                        {
                            _brandProvider = (IBrandProvider)this.GetDefaultImplementation("BrandProvider", true);
                        }
                    }
                }

                return _brandProvider;
            }
        }

        /// <summary>
        /// Gets the current brand context.
        /// </summary>
        public Brand BrandContext
        {
            get
            {
                return this.BrandProvider.Current;
            }
        }

        /// <summary>
        /// Gets the operation service agent for the site.
        /// </summary>
        [Dependency]
        public CommerceServerContrib.CrossTier.Interfaces.IAuthorizationEvaluator AuthorizationEvaluator
        {
            get
            {
                return this.GetImplementation<CommerceServerContrib.CrossTier.Interfaces.IAuthorizationEvaluator>();
            }
        }

        /// <summary>
        /// Gets the service adapter.
        /// </summary>
        [Dependency]
        public IServiceAdapter ServiceAdapter
        {
            get
            {
                return PresentationTypeLoader.ServiceAdapter;
            }
        }

        /// <summary>
        /// Gets the available services.
        /// </summary>
        public List<ControllerInstance> AvailableServices
        {
            get
            {
                var cacheContext = PresentationCacheSettings.GetCacheContext(CommerceServerContrib.CrossTier.Helpers.CacheConstants.RequestTemplateCache);

                if (cacheContext != null && cacheContext.IsCachingEnabled)
                {
                    var provider = PresentationTypeLoader.CacheProvider(cacheContext);
                    var services = provider.GetData<List<ControllerInstance>>(this.BrandContext.Name, cacheContext.CacheName, AvailableServicesKey);

                    if (services == null)
                    {
                        lock (_syncRoot)
                        {
                            if (services == null)
                            {
                                services = ExternalServices.GetServices();

                                provider.AddData<List<ControllerInstance>>(this.BrandContext.Name, cacheContext.CacheName, AvailableServicesKey, services);
                            }
                        }
                    }

                    return services;
                }
                else
                {
                    return ExternalServices.GetServices();
                }
            }
        }

        /// <summary>
        /// Gets the encryption provider.
        /// </summary>
        [Dependency]
        public List<IEncryptionProvider> EncryptionProviders
        {
            get
            {
                return GetImplementations<IEncryptionProvider>(false);
            }
        }

        /// <summary>
        /// Gets the resource provider.
        /// </summary>
        public IResourceProvider ResourceProvider
        {
            get
            {
                return GetImplementation<IResourceProvider>();
            }
        }

        #endregion

        #region Methods (public, non-static)

        /// <summary>
        /// Gets the query string provider.
        /// </summary>
        /// <param name="cacheType">Type of the cache.</param>
        /// <returns>Cache provider is returned.  Method will throw and exception if a cache provider is not found.</returns>
        public ICacheProvider CacheProvider(string cacheType)
        {
            var context = PresentationCacheSettings.GetCacheContext(cacheType);
            
            return PresentationTypeLoader.CacheProvider(context);
        }

        /// <summary>
        /// Gets the service agent by name.
        /// </summary>
        /// <param name="serviceName">Name of the service.</param>
        /// <returns>The specific service agent instance</returns>
        public IAsyncOperationServiceAgent GetServiceAgent(string serviceName)
        {
            this.EnsureAgent(serviceName);

            return _serviceAgents[serviceName];
        }

        #endregion

        #region Methods (protected, non-static)

        /// <summary>
        /// Gets the name of the unity container.
        /// </summary>
        /// <returns>
        /// The name of the unit container that should be used.
        /// </returns>
        protected string GetUnityContainerName()
        {
            return this.BrandContext.Name;
        }

        #endregion

        #region Methods (private)

        /// <summary>
        /// Checks to see if an agent for a service has been created, and if no it creates it
        /// </summary>
        /// <param name="serviceName">Name of the service.</param>
        private void EnsureAgent(string serviceName)
        {
            if (!_serviceAgents.ContainsKey(serviceName))
            {
                lock (_serviceAgentsSyncRoot)
                {
                    if (!_serviceAgents.ContainsKey(serviceName))
                    {
                        var agent = this.GetImplementation<IAsyncOperationServiceAgent>();

                        agent.Initialize(serviceName);

                        _serviceAgents.Add(serviceName, agent);
                    }
                }
            }
        }

        #endregion
    }
}
