﻿//-----------------------------------------------------------------------
// <copyright file="RenderHelper.cs" company="commerceserver.net">
//     Copyright © commerceserver.net.  All rights reserved.
// </copyright>
// <summary>Render helper class</summary>
//-----------------------------------------------------------------------
namespace CommerceServerContrib.Web
{
    using System;
    using System.Collections.Generic;
    using System.Globalization;
    using System.IO;
    using System.Security;
    using System.Web.UI;
    using System.Xml;
    using System.Xml.Xsl;
    using CommerceServer.Foundation;
    using CommerceServerContrib.CrossTier.Entities;
    using CommerceServerContrib.CrossTier.Interfaces;

    /// <summary>
    /// Used to help with Common xslt funtionality
    /// </summary>
    [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1053:StaticHolderTypesShouldNotHaveConstructors", Justification = "If this was made static we would not be able to make XsltHelper extend from this, and then be able to mark it obsolete.")]
    public class RenderHelper
    {
        /// <summary>
        /// Transforms the specified XSLT path.
        /// </summary>
        /// <param name="xsltPath">The XSLT path.</param>
        /// <param name="xmlToTransform">The XML to transform.</param>
        /// <returns>The transformed string.</returns>
        public static string Transform(string xsltPath, string xmlToTransform)
        {
            return Transform(xsltPath, xmlToTransform, null);
        }

        /// <summary>
        /// Transforms the specified XSLT path.
        /// </summary>
        /// <param name="xsltPath">The XSLT path.</param>
        /// <param name="xmlToTransform">The XML to transform.</param>
        /// <param name="arguments">The xslt arguments.</param>
        /// <returns>
        /// The transformed string.
        /// </returns>
        public static string Transform(string xsltPath, string xmlToTransform, RenderArgumentBuilder arguments)
        {
            ParameterChecker.CheckForNullOrEmpty(xsltPath, "xsltPath");
            ParameterChecker.CheckForNullOrEmpty(xmlToTransform, "xmlToTransform");
            
            var xslt = GetXslt(xsltPath);

            if (xslt != null)
            {
                var renderArgumentBuilder = new RenderArgumentBuilder();

                List<IRenderParameterProvider> xsltParameterProviders = PresentationServiceLocator.Current.GetImplementations<IRenderParameterProvider>(true);
                xsltParameterProviders.ForEach(x => x.GetRenderParameters().CopyTo(renderArgumentBuilder.Parameters, false));

                List<IRenderExtensionObjectProvider> xsltExtensionObjectProviders = PresentationServiceLocator.Current.GetImplementations<IRenderExtensionObjectProvider>(true);
                xsltExtensionObjectProviders.ForEach(x => x.GetRenderExtensionObjects().CopyTo(renderArgumentBuilder.RenderExtensionObjects, false));

                List<IRenderParameterProvider> renderParameterProviders = PresentationServiceLocator.Current.GetImplementations<IRenderParameterProvider>(true);
                renderParameterProviders.ForEach(x => x.GetRenderParameters().CopyTo(renderArgumentBuilder.Parameters, false));

                List<IRenderExtensionObjectProvider> renderExtensionObjectProviders = PresentationServiceLocator.Current.GetImplementations<IRenderExtensionObjectProvider>(true);
                renderExtensionObjectProviders.ForEach(x => x.GetRenderExtensionObjects().CopyTo(renderArgumentBuilder.RenderExtensionObjects, false));

                if (arguments != null)
                {
                    if (arguments.RenderExtensionObjects.Count > 0)
                    {
                        arguments.RenderExtensionObjects.CopyTo(renderArgumentBuilder.RenderExtensionObjects, true);
                    }

                    if (arguments.Parameters.Count > 0)
                    {
                        arguments.Parameters.CopyTo(renderArgumentBuilder.Parameters, true);
                    }
                }

                var xslOut = new StringWriter(CultureInfo.InvariantCulture);

                try
                {
                    xslt.Transform(XmlReader.Create(new StringReader(xmlToTransform)), renderArgumentBuilder.ToXsltArgumentList(), xslOut);

                    var responseMessage = xslOut.ToString();

                    return responseMessage;
                }
                catch (System.Threading.ThreadAbortException)
                {
                    // don't do anything because it is most likely that a redirection has been called so no point in logging a message
                }
                catch (SecurityException ex)
                {
                    CommerceServerContribExceptionManager.HandleException(ex);
                }
                catch (XsltException ex)
                {
                    CommerceServerContribExceptionManager.HandleException(ex);
                }
            }

            return null;
        }

        /// <summary>
        /// Transforms the specified XSLT path.
        /// </summary>
        /// <param name="filePath">The file path.</param>
        /// <param name="response">The response data to render.</param>
        /// <param name="arguments">The xslt arguments.</param>
        /// <param name="writer">The writer.</param>
        /// <param name="currentPage">The current page.</param>
        public static void Render(string filePath, object response, RenderArgumentBuilder arguments, HtmlTextWriter writer, TemplateControl currentPage)
        {
            ParameterChecker.CheckForNullOrEmpty(filePath, "filePath");
            ParameterChecker.CheckForNull(writer, "writer");

            IBaseControl cntrl = null;

            if (!filePath.StartsWith("~", StringComparison.OrdinalIgnoreCase))
            {
                if (!filePath.StartsWith(@"\", StringComparison.OrdinalIgnoreCase))
                {
                    filePath = string.Concat(@"\", filePath);
                }

                filePath = string.Concat("~", filePath);
            }

            if (currentPage == null)
            {
                var page = new System.Web.UI.Page();
                cntrl = page.LoadControl(filePath) as IBaseControl;
            }
            else
            {
                cntrl = currentPage.LoadControl(filePath) as IBaseControl;
            }

            if (cntrl != null)
            {
                var viewBag = new Dictionary<string, object>();
                var renderArgumentBuilder = new RenderArgumentBuilder();

                List<IRenderParameterProvider> xsltParameterProviders = PresentationServiceLocator.Current.GetImplementations<IRenderParameterProvider>(true);
                xsltParameterProviders.ForEach(x => x.GetRenderParameters().CopyTo(renderArgumentBuilder.Parameters, false));

                List<IRenderExtensionObjectProvider> xsltExtensionObjectProviders = PresentationServiceLocator.Current.GetImplementations<IRenderExtensionObjectProvider>(true);
                xsltExtensionObjectProviders.ForEach(x => x.GetRenderExtensionObjects().CopyTo(renderArgumentBuilder.RenderExtensionObjects, false));

                List<IRenderParameterProvider> renderParameterProviders = PresentationServiceLocator.Current.GetImplementations<IRenderParameterProvider>(true);
                renderParameterProviders.ForEach(x => x.GetRenderParameters().CopyTo(renderArgumentBuilder.Parameters, false));

                List<IRenderExtensionObjectProvider> renderExtensionObjectProviders = PresentationServiceLocator.Current.GetImplementations<IRenderExtensionObjectProvider>(true);
                renderExtensionObjectProviders.ForEach(x => x.GetRenderExtensionObjects().CopyTo(renderArgumentBuilder.RenderExtensionObjects, false));

                if (arguments != null)
                {
                    if (arguments.RenderExtensionObjects.Count > 0)
                    {
                        arguments.RenderExtensionObjects.CopyTo(renderArgumentBuilder.RenderExtensionObjects, true);
                    }

                    if (arguments.Parameters.Count > 0)
                    {
                        arguments.Parameters.CopyTo(renderArgumentBuilder.Parameters, true);
                    }
                }

                renderArgumentBuilder.Parameters.CopyTo(viewBag, false);
                renderArgumentBuilder.RenderExtensionObjects.CopyTo(viewBag, false);

                if (response != null)
                {
                    cntrl.ResponseData = response as List<object>;
                }

                cntrl.SetViewBag(viewBag);

                ((Control)cntrl).RenderControl(writer);
            }
        }

        /// <summary>
        /// Indicates if the Xslt file exists or not.
        /// </summary>
        /// <param name="path">The path to the xslt file.</param>
        /// <returns>true if the file exists; otherwise false.</returns>
        public static bool XsltFileExists(string path)
        {
            var xsltProvider = PresentationServiceLocator.Current.GetImplementation<IXsltProvider>();

            if (xsltProvider == null)
            {
                CommerceTrace.Error(CommerceClientTraceSource.ApplicationTier, ClientFrameworkResources.XsltProviderIsNull);

                return false;
            }

            return xsltProvider.XsltFileExists(path);
        }

        /// <summary>
        /// Gets the XSLT path.
        /// </summary>
        /// <param name="xsltPath">The path of the XSLT.</param>
        /// <returns>The path to the xslt file</returns>
        private static XslCompiledTransform GetXslt(string xsltPath)
        {
            if (!string.IsNullOrEmpty(xsltPath))
            {
                var xsltProvider = PresentationServiceLocator.Current.GetImplementation<IXsltProvider>();

                if (xsltProvider != null)
                {
                    return xsltProvider.GetXslt(xsltPath);
                }
            }

            return null;
        }
    }
}
