﻿//-----------------------------------------------------------------------
// <copyright file="CheckoutFlowIdValidationModule.cs" company="commerceserver.net">
//     Copyright © commerceserver.net.  All rights reserved.
// </copyright>
// <summary>Defines the CheckoutFlowIdValidationModule class</summary>
//-----------------------------------------------------------------------

namespace CommerceServerContrib.Web
{
    using System;
    using System.Collections.Generic;
    using System.Collections.ObjectModel;
    using System.Linq;
    using System.Threading;
    using System.Web;
    using CommerceServer.Foundation;

    /// <summary>
    /// Module that is responsible for issuing checkout flow id (CFId) cookies to the user, and determining when a basket CFId should be validated.
    /// </summary>
    public class CheckoutFlowIdValidationModule : IHttpModule
    {
        private const string CheckoutFlowIdCookieName = "__CLCheckoutFlowId";
        private object _checkoutStepsLock = new object();
        private ReadOnlyCollection<string> _checkoutSteps = null;

        /// <summary>
        /// Gets the checkout pages that have been defined the site settings.
        /// </summary>
        protected ReadOnlyCollection<string> CheckoutSteps
        {
            get
            {
                if (this._checkoutSteps == null)
                {
                    lock (this._checkoutStepsLock)
                    {
                        if (this._checkoutSteps == null)
                        {
                            this._checkoutSteps = this.GetCheckoutSteps();
                        }
                    }
                }

                return this._checkoutSteps;
            }
        }

        /// <summary>
        /// Disposes of the resources (other than memory) used by the module that implements <see cref="T:System.Web.IHttpModule"/>.
        /// </summary>
        public void Dispose()
        {
            //// No resources to dispose.
        }

        /// <summary>
        /// Initializes the HTTP module.
        /// </summary>
        /// <param name="context">The HTTP application context.</param>
        public void Init(HttpApplication context)
        {
            ParameterChecker.CheckForNull(context, "context");
            context.PreRequestHandlerExecute += new EventHandler(this.OnPreRequestHandlerExecute);
            context.PostRequestHandlerExecute += new EventHandler(this.OnPostRequestHandlerExecute);
        }

        /// <summary>
        /// Handles the PostRequestHandlerExecute event of the current <see cref="HttpApplication"/>.
        /// </summary>
        /// <param name="sender">The object that triggered the event.</param>
        /// <param name="e">The event arguments.</param>
        protected virtual void OnPostRequestHandlerExecute(object sender, EventArgs e)
        {
            using (CommerceSegmentTrace.ClientApplicationTier(typeof(CheckoutFlowIdValidationModule), "OnPostRequestHandlerExecute"))
            {
                var currentContext = PresentationServiceLocator.Current.SiteContext.CurrentContext;
                if (currentContext == null ||
                    currentContext.Request == null ||
                    currentContext.Response == null)
                {
                    CommerceTrace.Warning(CommerceClientTraceSource.PresentationTier, "CheckoutFlowIdValidationModule: CFId cookie cannot be removed because PresentationServiceLocator.Current.SiteContext.CurrentContext, PresentationServiceLocator.Current.SiteContext.CurrentContext.Request, or PresentationServiceLocator.Current.SiteContext.CurrentContext.Response is null.");
                    return;
                }

                if (!this.IsLastCheckoutPage(currentContext.Request))
                {
                    CommerceTrace.Diagnostic(CommerceClientTraceSource.PresentationTier, "CheckoutFlowIdValidationModule: CFId cookie will not be removed because the page that was requessted is not the last page in the checkout process.");
                    return;
                }

                CommerceTrace.Information(
                    CommerceClientTraceSource.PresentationTier,
                    "CheckoutFlowIdValidationModule: Deleting CFId cookie for the user '{0}' - the last page in the checkout process ({1}) was requested.",
                    currentContext.User != null ? currentContext.User.Identity.Name : Thread.CurrentPrincipal.Identity.Name,
                    this.CheckoutSteps[this.CheckoutSteps.Count - 1]);

                UserCookieHelper.RemoveCheckoutFlowIdCookie();
            }
        }

        /// <summary>
        /// Handles the PreRequestHandlerExecute event of the current <see cref="HttpApplication"/>.
        /// </summary>
        /// <param name="sender">The object that triggered the event.</param>
        /// <param name="e">The event arguments.</param>
        protected virtual void OnPreRequestHandlerExecute(object sender, EventArgs e)
        {
            using (CommerceSegmentTrace.ClientApplicationTier(typeof(CheckoutFlowIdValidationModule), "OnPreRequestHandlerExecute"))
            {
                var siteContext = PresentationServiceLocator.Current.SiteContext;
                var currentContext = siteContext.CurrentContext;

                if (currentContext == null ||
                    currentContext.Request == null ||
                    currentContext.Response == null)
                {
                    CommerceTrace.Warning(CommerceClientTraceSource.PresentationTier, "CheckoutFlowIdValidationModule: CFId cookie cannot be isued because PresentationServiceLocator.Current.SiteContext.CurrentContext, PresentationServiceLocator.Current.SiteContext.CurrentContext.Request, or PresentationServiceLocator.Current.SiteContext.CurrentContext.Response is null.");
                    return;
                }

                if (this.IsCheckoutPage(currentContext.Request, siteContext.IsSecureConnection))
                {
                    string checkoutFlowId = UserCookieHelper.GetCheckoutFlowId();
                    if (string.IsNullOrEmpty(checkoutFlowId))
                    {
                        this.IssueCheckoutFlowIdCookie();
                    }
                    else
                    {
                        CommerceTrace.Diagnostic(
                            CommerceClientTraceSource.PresentationTier,
                            "CheckoutFlowIdValidationModule: no cookie will be issued because the user '{0}' already has a CFId cookie with value '{1}'.",
                            currentContext.User != null ? currentContext.User.Identity.Name : Thread.CurrentPrincipal.Identity.Name,
                            checkoutFlowId);
                    }
                }
            }
        }

        /// <summary>
        /// Issues a new cookie to the current user that contains the checkout flow id.
        /// </summary>
        protected virtual void IssueCheckoutFlowIdCookie()
        {
            // The user does not have a CFId cookie - issue a new one
            var checkoutFlowId = this.NewCheckoutFlowId();
            UserCookieHelper.IssueCheckoutFlowIdCookie(checkoutFlowId);

            var currentContext = PresentationServiceLocator.Current.SiteContext.CurrentContext;
            CommerceTrace.Information(
                CommerceClientTraceSource.PresentationTier,
                "CheckoutFlowIdValidationModule:  Issuing a new CFId cookie with value '{0}' for user '{1}'",
                checkoutFlowId,
                currentContext.User != null ? currentContext.User.Identity.Name : Thread.CurrentPrincipal.Identity.Name);
        }

        /// <summary>
        /// Creates a new checkout flow ID.
        /// </summary>
        /// <returns>A new checkout flow ID.</returns>
        protected virtual string NewCheckoutFlowId()
        {
            return Guid.NewGuid().ToString("B");
        }

        /// <summary>
        /// Checks if the current request page is the last page in the checkout process (i.e. the order confirmation page).
        /// </summary>
        /// <param name="request">The current see <see cref="HttpRequest"/>.</param>
        /// <returns><c>True</c> if this is the last page in the checkout process, otherwise <c>false</c>.</returns>
        protected virtual bool IsLastCheckoutPage(HttpRequest request)
        {
            if (request == null || this.CheckoutSteps.Count == 0)
            {
                return false;
            }

            if (PresentationServiceLocator.Current.BrandContext.SecurePageSettings.RequireSsl &&
                !PresentationServiceLocator.Current.SiteContext.IsSecureConnection)
            {
                // only handle pages over SSL.
                return false;
            }

            if (!string.Equals(this.CheckoutSteps[this.CheckoutSteps.Count - 1], request.FilePath, StringComparison.OrdinalIgnoreCase))
            {
                // CheckoutSteps is sorted, and the last item in the list is the last checkout page.
                return false;
            }

            return true;
        }

        /// <summary>
        /// Determines if the current request page is a checkout page.
        /// </summary>
        /// <param name="request">The current see <see cref="HttpRequest"/>.</param>
        /// <param name="isSecureConnection">if set to <c>true</c> [is secure connection].</param>
        /// <returns>
        ///   <c>True</c> if this is a checkout page, otherwise <c>false</c>.
        /// </returns>
        protected virtual bool IsCheckoutPage(HttpRequest request, bool isSecureConnection)
        {
            ParameterChecker.CheckForNull(request, "request");

            if (this.CheckoutSteps.Count == 0)
            {
                CommerceTrace.Diagnostic(CommerceClientTraceSource.PresentationTier, "CheckoutFlowIdValidationModule: No cookie will be issued because no checkout pages have been defined.");
                return false;
            }

            if (PresentationServiceLocator.Current.BrandContext.SecurePageSettings.RequireSsl && !isSecureConnection)
            {
                // only handle pages over SSL.
                CommerceTrace.Diagnostic(CommerceClientTraceSource.PresentationTier, "CheckoutFlowIdValidationModule: No cookie will be issued because the request for '{0}' is not secure.", request.FilePath);
                return false;
            }

            if (!this.CheckoutSteps.Contains(request.FilePath, StringComparer.OrdinalIgnoreCase))
            {
                // only handle checkout pages.
                CommerceTrace.Diagnostic(CommerceClientTraceSource.PresentationTier, "CheckoutFlowIdValidationModule: No cookie will be issued because the request for '{0}' is not a checkout page.", request.FilePath);
                return false;
            }

            return true;
        }

        /// <summary>
        /// Loads the checkout steps defined in site settings.
        /// </summary>
        /// <returns>The checkout pages sorted alphabetically by the checkout page key.</returns>
        private ReadOnlyCollection<string> GetCheckoutSteps()
        {
            var settingResponse = PresentationServiceLocator.Current.SiteSettingProvider.GetSiteSettings("CheckoutSteps");
            if (settingResponse.Count <= 0)
            {
                return null;
            }

            SortedList<string, string> sortedSteps = new SortedList<string, string>(StringComparer.OrdinalIgnoreCase);

            foreach (var checkoutStep in settingResponse)
            {
                sortedSteps.Add(checkoutStep.Key, checkoutStep.Value);
            }
            
            List<string> checkoutSteps = new List<string>();
            foreach (var item in sortedSteps)
            {
                checkoutSteps.Add(item.Value);
            }

            return checkoutSteps.AsReadOnly();
        }
    }
}
