﻿//-----------------------------------------------------------------------
// <copyright file="CommerceServerContribCsrfMitigationModule.cs" company="commerceserver.net">
//     Copyright © commerceserver.net.  All rights reserved.
// </copyright>
// <summary>Defines the CommerceServerContribCsrfMitigationModule class</summary>
//-----------------------------------------------------------------------

namespace CommerceServerContrib.Web
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Text;
    using System.Web;   
    using System.Web.UI;
    using System.Net;
    using CommerceServerContrib.CrossTier.Configuration;
    using System.Globalization;
    using System.Collections;
    using CLH = CommerceServerContrib.CrossTier.Helpers;
    using System.Security.Cryptography;
    using CommerceServer.Foundation;

    /// <summary>
    /// Http module that inspects page requests for potential CSRF attacks.
    /// </summary>
    public class CommerceServerContribCsrfMitigationModule : IHttpModule
    {
        private const int SaltValueLengthBytes = 16;
        private const int SaltValueLengthChars = 24;
        private const string ContextIndexName = "CommerceServerContrib.Web.CommerceServerContribCsrfMitigationModule.CsrfToken";
        private static RandomNumberGenerator _gen = new RNGCryptoServiceProvider();
        private Hashtable _typeCache = new Hashtable();

        /// <summary>
        /// Disposes of the resources (other than memory) used by the module that implements <see cref="T:System.Web.IHttpModule"/>.
        /// </summary>
        public virtual void Dispose()
        {
            //// nothing to dispose.
        }

        /// <summary>
        /// Inits the specified HTTP application.
        /// </summary>
        /// <param name="context">The HTTP application.</param>
        public virtual void Init(HttpApplication context)
        {
            CommerceServer.Foundation.ParameterChecker.CheckForNull(context, "context");
            context.PreRequestHandlerExecute += new EventHandler(this.OnPreRequestHandlerExecute);
        }

        /// <summary>
        /// Handles the PreRequestHandlerExecute HttpApplication event.
        /// </summary>
        /// <param name="sender">The event source.</param>
        /// <param name="eventArgs">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        /// <remarks>
        /// Occurs just before ASP.NET starts executing an event handler (for example, a page or an XML Web service).
        /// </remarks>
        protected virtual void OnPreRequestHandlerExecute(object sender, EventArgs eventArgs)
        {
            CommerceServer.Foundation.ParameterChecker.CheckForNull(sender, "sender");
            CommerceServer.Foundation.ParameterChecker.CheckIsInstanceOfType(sender, "sender", typeof(HttpApplication));

            HttpApplication application = (HttpApplication)sender;
            HttpContext context = application.Context;

            if (this.ShouldInspectContext(context))
            {
                this.InspectContext(context);
            }
        }

        /// <summary>
        /// Inspects the provided handler and context for a potential CSRF attack.
        /// </summary>
        /// <param name="context">The current <see cref="HttpContext"/>.</param>
        protected virtual void InspectContext(HttpContext context)
        {
            ParameterChecker.CheckForNull(context, "context");
            
            Page page = context.Handler as Page;

            if (page != null)
            {
                this.InspectPage(page, context);
                return;
            }

            this.InspectHttpHandler(context.Handler, context);
        }

        /// <summary>
        /// Inspects a HTTP handler request for a potential CSRF attack.
        /// </summary>
        /// <param name="httpHandler">The <see cref="IHttpHandler"/> to inspect.</param>
        /// <param name="context">The current <see cref="HttpContext"/>.</param>
        protected virtual void InspectHttpHandler(IHttpHandler httpHandler, HttpContext context)
        {
            if (context.Request.HttpMethod.Equals(WebRequestMethods.Http.Post, StringComparison.Ordinal))
            {
                string token = null;
                if (context.Request.Form != null)
                {
                    token = context.Request.Form[CommerceServerContribConfiguration.Presentation.CsrfSettings.FormFieldName];
                }

                if (string.IsNullOrEmpty(token))
                {
                    CommerceTrace.Error(
                        CommerceClientTraceSource.PresentationTier,
                        "CommerceServerContribCsrfMitigationModule.InspectHttpHandler: A potential CSRF attack was detected.  The request does not contain a Form, or the hidden CSRF token Form field is not present.  FormFieldName = '{0}', request.Url='{1}'",
                        CommerceServerContribConfiguration.Presentation.CsrfSettings.FormFieldName,
                        (context != null && context.Request != null && context.Request.Url != null) ? context.Request.Url.ToString() : "<NULL>");
                    this.HandleCsrfError(context);
                    return;
                }

                this.ValidateToken(token, context);
            }
        }

        /// <summary>
        /// Inspects a page request for a potential cSRF attack.
        /// </summary>
        /// <param name="page">The <see cref="Page"/> to inspect.</param>
        /// <param name="context">The current <see cref="HttpContext"/>.</param>
        protected virtual void InspectPage(Page page, HttpContext context)
        {
            ParameterChecker.CheckForNull(page, "page");

            // Attach a Page Load handler to check if this is a GET request masquerading as a postback
            if (page.IsPostBack && page.Request != null && page.Request.HttpMethod.Equals(WebRequestMethods.Http.Get))
            {
                CommerceTrace.Error(
                    CommerceClientTraceSource.PresentationTier,
                    "CommerceServerContribCsrfMitigationModule.OnPageLoad: Page post-backs are not allowed for HTTP requests using the 'GET' method. page.Request.Url='{0}'",
                    (page.Request != null && page.Request.Url != null) ? page.Request.Url.ToString() : "<NULL>");

                this.HandleCsrfError(HttpContext.Current);
                return;
            }

            // Attach our own PreRender event handler so we can write out the CSRF form field.
            string cookieToken = this.CreateToken(context);

            // Ensure that the cookie token is properly encoded
            cookieToken = HttpUtility.UrlEncode(cookieToken);

            // add a new filter to inject the new CSRF token
            context.Response.Filter = new CSRFPageFilterStream(context.Response.Filter, CommerceServerContribConfiguration.Presentation.CsrfSettings.FormFieldName, cookieToken);
        }

        /// <summary>
        /// Checks if the provided HTTP context should be examined for CSRF attacks.
        /// </summary>
        /// <param name="context">The <see cref="HttpContext"/> to examine.</param>
        /// <returns>True if the context should be examined for CSRF attacks, otherwise false.</returns>
        protected virtual bool ShouldInspectContext(HttpContext context)
        {
            IHttpHandler handler = (context == null) ? null : context.Handler;
            var siteContext = PresentationServiceLocator.Current.SiteContext;

            // do not inspect when the context is null or does not contain a handler
            if (handler == null)
            {
                return false;
            }

            // when SSL is required, only protect secure pages
            if (PresentationServiceLocator.Current.BrandContext.SecurePageSettings.RequireSsl &&
                !siteContext.IsSecureConnection)
            {
                return false;
            }

            // only inspect requests made by a logged-in user
            if (siteContext.User == null || string.IsNullOrEmpty(siteContext.User.Id))
            {
                return false;
            }

            // do not inspect pages that implement the ISuppressCsrfCheck interface
            if (handler is CommerceServerContrib.CrossTier.ISuppressCsrfCheck)
            {
                return false;
            }

            // do not inspect pages that are marked with the SuppressCsrfCheck attribute
            if (this.ContainsSuppressCsrfCheckAttribute(handler))
            {
                return false;
            }

            // otherwise, the page should be inspected.
            return true;
        }

        /// <summary>
        /// Raises a CSRF error based on the configuration settings.
        /// </summary>
        /// <param name="context">The current HttpContext.</param>
        protected virtual void HandleCsrfError(HttpContext context)
        {
            this.HandleCsrfError(context, null);
        }

        /// <summary>
        /// Raises a CSRF error based on the configuration settings.
        /// </summary>
        /// <param name="context">The current HttpContext.</param>
        /// <param name="innerException">The exception that caused the CSRF error.</param>
        protected virtual void HandleCsrfError(HttpContext context, Exception innerException)
        {
            if (CommerceServerContribConfiguration.Presentation.CsrfSettings.ErrorHandlingBehavior == CsrfErrorHandlingBehavior.RedirectToPage)
            {
                string errorPage = CommerceServerContribConfiguration.Presentation.CsrfSettings.ErrorPage;
                if (string.IsNullOrEmpty(errorPage))
                {
                    errorPage = @"~/";
                }

                PresentationServiceLocator.Current.SiteContext.RedirectTo(CommerceServerContribConfiguration.Presentation.CsrfSettings.ErrorPage, true);
            }
            else
            {
                string message = ClientFrameworkResources.ErrorPotentialCsrfAttack;
                Exception ex = new CommerceServerContrib.CrossTier.CommerceServerContribCsrfException(message, innerException);
                throw ex;
            }
        }

        /// <summary>
        /// Creates a new CSRF token.
        /// </summary>
        /// <param name="context">The current <see cref="HttpContext"/>.</param>
        /// <returns>A new CSRF token.</returns>
        protected virtual string CreateToken(HttpContext context)
        {
            string userId = PresentationServiceLocator.Current.SiteContext.User.Id;
            if (string.IsNullOrEmpty(userId))
            {
                return null;
            }

            string salt = this.GetRandomSaltValue();
            string hashInput = string.Concat(salt, userId);

            string key = CommerceServerContribConfiguration.Presentation.CsrfSettings.HashingKey ?? CsrfSettingsElement.HashingKeyDefaultValue;
            HMACSHA256 crypto = new HMACSHA256(Encoding.UTF32.GetBytes(key));
            byte[] hashOutputBytes = crypto.ComputeHash(Encoding.UTF32.GetBytes(hashInput));
            string hashOutput = Convert.ToBase64String(hashOutputBytes);
            string token = string.Concat(salt, hashOutput);

            CommerceTrace.Diagnostic(
                CommerceClientTraceSource.PresentationTier,
                "CommerceServerContribCsrfMitigationModule.CreateToken: create token for user.  userId='{0}', salt='{1}', key='{2}', request.Url='{3}'",
                userId,
                salt,
                key,
                (context != null && context.Request != null && context.Request.Url != null) ? context.Request.Url.ToString() : "<NULL>");

            return token;
        }

        /// <summary>
        /// Validates that the tokens retrieved from a request Cookie and Form are valid and equal.
        /// </summary>
        /// <param name="token">The serialized token from the request Form.</param>
        /// <param name="context">The current <see cref="HttpContext"/>.</param>
        protected virtual void ValidateToken(string token, HttpContext context)
        {
            string userId = PresentationServiceLocator.Current.SiteContext.User.Id;
            if (string.IsNullOrEmpty(userId))
            {
                // if there is no registered user ID, then there is no CSRF risk.
                // CSRF attacks are only dangerous if a logged-in user is being exploited.
                return;
            }

            if (string.IsNullOrEmpty(token))
            {
                CommerceTrace.Error(
                    CommerceClientTraceSource.PresentationTier,
                    "CommerceServerContribCsrfMitigationModule.ValidateToken: A potential CSRF attack was detected.  The provided token is null or empty.  request.Url='{0}'",
                    (context != null && context.Request != null && context.Request.Url != null) ? context.Request.Url.ToString() : "<NULL>");
                this.HandleCsrfError(context);
                return;
            }

            if (token.Length <= SaltValueLengthChars)
            {
                CommerceTrace.Error(
                    CommerceClientTraceSource.PresentationTier,
                    "CommerceServerContribCsrfMitigationModule.ValidateToken: A potential CSRF attack was detected.  The provided token is not long enough.  Expected token minimum length: {0} chars.  Actual token length: {1} chars, request.Url='{2}'",
                    SaltValueLengthChars,
                    token.Length,
                    (context != null && context.Request != null && context.Request.Url != null) ? context.Request.Url.ToString() : "<NULL>");
                this.HandleCsrfError(context);
                return;
            }

            string salt = token.Substring(0, SaltValueLengthChars);
            string actualHashOutput = token.Substring(SaltValueLengthChars);

            string key = CommerceServerContribConfiguration.Presentation.CsrfSettings.HashingKey ?? CsrfSettingsElement.HashingKeyDefaultValue;
            HMACSHA256 crypto = new HMACSHA256(Encoding.UTF32.GetBytes(key));

            string hashInput = string.Concat(salt, userId);
            byte[] expectedHashOutputBytes = crypto.ComputeHash(Encoding.UTF32.GetBytes(hashInput));
            string expectedHashOutput = Convert.ToBase64String(expectedHashOutputBytes);

            if (!string.Equals(expectedHashOutput, actualHashOutput, StringComparison.Ordinal))
            {
                CommerceTrace.Error(
                    CommerceClientTraceSource.PresentationTier,
                    "CommerceServerContribCsrfMitigationModule.CreateToken: A potential CSRF attack was detected. The hash value provided in the CSRF token does not match the expected hash value.  userId='{0}', salt='{1}', key='{2}', request.Url='{3}'",
                    userId,
                    salt,
                    key,
                    (context != null && context.Request != null && context.Request.Url != null) ? context.Request.Url.ToString() : "<NULL>");
                this.HandleCsrfError(context);
                return;
            }
        }

        private bool ContainsSuppressCsrfCheckAttribute(object handler)
        {
            Type handlerType = handler.GetType();
            bool? containsAttribute = this._typeCache[handler] as bool?;
            if (!containsAttribute.HasValue)
            {
                lock (this._typeCache.SyncRoot)
                {
                    containsAttribute = handlerType.GetCustomAttributes(typeof(CommerceServerContrib.CrossTier.SuppressCsrfCheckAttribute), true).Length > 0;
                    this._typeCache[handlerType] = containsAttribute;
                }
            }

            return containsAttribute.Value;
        }

        private string GetRandomSaltValue()
        {
            byte[] buffer = new byte[SaltValueLengthBytes];
            _gen.GetBytes(buffer);
            return Convert.ToBase64String(buffer);
        }
    }
}
