﻿//-----------------------------------------------------------------------
// <copyright file="SecurePageModule.cs" company="commerceserver.net">
//     Copyright © commerceserver.net.  All rights reserved.
// </copyright>
// <summary>HttpModule which will enfore that some pages are only access via SSL.</summary>
//-----------------------------------------------------------------------

namespace CommerceServerContrib.Web
{
    using System;
    using System.Collections.Generic;
    using System.Globalization;
    using System.Web;
    using CommerceServerContrib.CrossTier.Helpers;
    using CommerceServer.Foundation;

    /// <summary>
    /// HttpModule wwhich will enfore that some pages are only access via SSL.
    /// </summary>
    public class SecurePageModule : IHttpModule
    {
        #region Private members

        private const string AspxPageType = ".aspx";
        private const string ServicesPageType = "/services/";
        private const string WildCard = "*";
        private List<string> _pages;
        private List<string> _wildCardPages;
        private List<string> _ignorePages;
        private bool _enabled;
        private string _secureHostName = string.Empty;
        private int _securePort = 443;
        private string _nonSecureHostName = string.Empty;
        private int _nonSecurePort = 80;
        private int _sslOffloadPort;

        #endregion

        #region Public Methods

        /// <summary>
        /// Inits the specified HTTP application.
        /// </summary>
        /// <param name="httpApplication">The HTTP application.</param>
        public void Init(HttpApplication httpApplication)
        {
            ParameterChecker.CheckForNull(httpApplication, "httpApplication");
            this.Initialize();
            httpApplication.PreRequestHandlerExecute += this.SecurePageModulePreRequestHandlerExecute;
        }

        /// <summary>
        /// Releases unmanaged and - optionally - managed resources
        /// </summary>
        public void Dispose()
        {
        }

        #endregion

        #region Private Methods

        /// <summary>
        /// Determines if a page should be HTTPs and then redirects to the secure version if need be.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void SecurePageModulePreRequestHandlerExecute(object sender, EventArgs e)
        {
            if (!this._enabled)
            {
                return;
            }

            var isCurrentRequestSecure = PresentationServiceLocator.Current.SiteContext.IsSecureConnection;
            var lowerPath = HttpContext.Current.Request.Path.ToLower(CultureInfo.InvariantCulture);

            var requireSecure = this.DoesPageRequireSecureAccess(lowerPath, isCurrentRequestSecure);

            if (isCurrentRequestSecure)
            {
                if (!requireSecure)
                {
                    //// Instruct browser to reload page in HTTP
                    this.Redirect(this.BuildUri(false));
                }
            }
            else if (requireSecure)
            {
                //// Instruct browser to reload page in HTTPS
                this.Redirect(this.BuildUri(true));
            }
        }

        /// <summary>
        /// Redirects the to the newly built link.
        /// </summary>
        /// <param name="targetUrl">The target URL.</param>
        private void Redirect(Uri targetUrl)
        {
            PresentationServiceLocator.Current.SiteContext.RedirectTo(targetUrl.OriginalString, true);
        }

        /// <summary>
        /// Builds the URI.
        /// </summary>
        /// <param name="secure">if set to <c>true</c> [secure].</param>
        /// <returns>The updated Uri.</returns>
        private Uri BuildUri(bool secure)
        {
            var builder = new UriBuilder(HttpContext.Current.Request.Url);
            if (secure)
            {
                builder.Scheme = Uri.UriSchemeHttps;
                builder.Port = this._securePort;
                if (!String.IsNullOrEmpty(this._secureHostName))
                {
                    builder.Host = this._secureHostName;
                }
            }
            else
            {
                builder.Scheme = Uri.UriSchemeHttp;
                builder.Port = this._nonSecurePort;
                if (!String.IsNullOrEmpty(this._nonSecureHostName))
                {
                    builder.Host = this._nonSecureHostName;
                }
            }

            return builder.Uri;
        }

        /// <summary>
        /// Doeses the page require secure access.
        /// </summary>
        /// <param name="path">The path of the page.</param>
        /// <param name="isCurrentRequestSecure">if set to <c>true</c> [is current request secure].</param>
        /// <returns>
        /// A boolean value indicating if the current path requires SSL.
        /// </returns>
        private bool DoesPageRequireSecureAccess(string path, bool isCurrentRequestSecure)
        {
            var isRecognised = this.IsARecoginsedPath(path);
            var isIgnored = this.IsAnIgnoredPath(path);

            // if it is in the pages list but not in the ignore pages list then it is required to be secure
            if (isRecognised && !isIgnored)
            {
                return true;
            }

            // if it is an ignored path and the current page is secure, then we jsut pretend the page requires SSL to avoid a redirection
            if (isCurrentRequestSecure && isIgnored)
            {
                return true;
            }

            return false;
        }

        /// <summary>
        /// Determines whether [is an ignored path] [the specified path].
        /// </summary>
        /// <param name="path">The path to check.</param>
        /// <returns>
        ///   <c>true</c> if [is an ignored path] [the specified path]; otherwise, <c>false</c>.
        /// </returns>
        private bool IsAnIgnoredPath(string path)
        {
            if (this._ignorePages.Contains(path))
            {
                return true;
            }

            foreach (var page in this._ignorePages)
            {
                if (page.EndsWith(WildCard, StringComparison.OrdinalIgnoreCase))
                {
                    if (path.StartsWith(page, StringComparison.OrdinalIgnoreCase))
                    {
                        return true;
                    }
                }
            }

            return false;
        }

        /// <summary>
        /// Determines whether [is A recoginsed path] [the specified path].
        /// </summary>
        /// <param name="path">The path to check against.</param>
        /// <returns>
        ///   <c>true</c> if [is A recoginsed path] [the specified path]; otherwise, <c>false</c>.
        /// </returns>
        private bool IsARecoginsedPath(string path)
        {
            if (this._pages.Contains(path))
            {
                return true;
            }

            if (this._wildCardPages.Count > 0)
            {
                foreach (var wildCardPath in this._wildCardPages)
                {
                    if (path.StartsWith(wildCardPath, StringComparison.OrdinalIgnoreCase))
                    {
                        return true;
                    }
                }
            }

            return false;
        }

        /// <summary>
        /// Inits the secure pages modules by reading the web configuration section.
        /// </summary>
        private void Initialize()
        {
            var securePageSettings = PresentationServiceLocator.Current.BrandContext.SecurePageSettings;

            this._pages = new List<string>();
            this._ignorePages = new List<string>();
            this._wildCardPages = new List<string>();

            foreach (KeyValuePair<string, CommerceServerContrib.CrossTier.SecurePage> page in securePageSettings.Pages)
            {
                if (page.Value.IgnoreProtocolScheme)
                {
                    this._ignorePages.Add(page.Value.Path.ToLower(CultureInfo.InvariantCulture));
                }
                else if (page.Value.Path.EndsWith(WildCard, StringComparison.OrdinalIgnoreCase))
                {
                    this._wildCardPages.Add(page.Value.Path.ToLower(CultureInfo.InvariantCulture).Replace(WildCard, string.Empty));
                }
                else
                {
                    this._pages.Add(page.Value.Path.ToLower(CultureInfo.InvariantCulture));
                }
            }

            this._secureHostName = securePageSettings.SecureHostName;
            this._securePort = securePageSettings.SecurePort;
            this._nonSecureHostName = securePageSettings.NonSecureHostName;
            this._nonSecurePort = securePageSettings.NonSecurePort;
            this._enabled = securePageSettings.RequireSsl;
            this._sslOffloadPort = securePageSettings.SslOffloadPort;
        }

        #endregion
    }
}