﻿//---------------------------------------------------------------------
// <copyright file="UserSecurityManagementModule.cs" company="commerceserver.net">
//     Copyright © commerceserver.net.  All rights reserved.
// </copyright>
// <summary>UserSecurityManagementModule class</summary>
//---------------------------------------------------------------------

namespace CommerceServerContrib.Web
{
    using System.Web;
    using CommerceServerContrib.CrossTier.Entities;
    using CommerceServerContrib.CrossTier.Helpers;
    using CommerceServer.Foundation;

    /// <summary>
    /// UserSecurityManagementModule class
    /// </summary>
    public class UserSecurityManagementModule : IHttpModule
    {
        /// <summary>
        /// Disposes of the resources (other than memory) used by the module that implements <see cref="T:System.Web.IHttpModule"/>.
        /// </summary>
        public void Dispose()
        {
        }

        /// <summary>
        /// Initializes a module and prepares it to handle requests.
        /// </summary>
        /// <param name="context">An <see cref="T:System.Web.HttpApplication"/> that provides access to the methods, properties, and events common to all application objects within an ASP.NET application</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Security", "CA2116:AptcaMethodsShouldOnlyCallAptcaMethods", Justification = "Reviewed and considered safe.")]
        public void Init(HttpApplication context)
        {
            ParameterChecker.CheckForNull(context, "context");
            context.PostAuthenticateRequest += this.PostAuthenticateRequest;
        }

        /// <summary>
        /// Called when [post authenticate request].
        /// </summary>
        protected virtual void OnPostAuthenticateRequest()
        {
            if (HttpContext.Current == null)
            {
                return;
            }

            User user;
            if (!this.TryGetRegisteredUserInfo(out user))
            {
                user.AnonymousId = this.GetAnonymousUserId();
            }
            
            PresentationServiceLocator.Current.SiteContext.User = new User
                                                                    {
                                                                        Id = user.Id,
                                                                        Email = user.Email,
                                                                        Name = user.Name,
                                                                        AuthenticationType = user.AuthenticationType,
                                                                        AnonymousId = user.AnonymousId,
                                                                        IsWindowsIdentity = user.IsWindowsIdentity,
                                                                        RememberMe = UserCookieHelper.GetRememberMe()
                                                                    };
        }

        /// <summary>
        /// Tries the get registered user info.
        /// </summary>
        /// <param name="registeredUser">The registered user.</param>
        /// <returns>The current user</returns>
        protected virtual bool TryGetRegisteredUserInfo(out User registeredUser)
        {
            if (UserCookieHelper.TryGetRegisteredUser(out registeredUser))
            {
                return true;
            }
            
            System.Security.Principal.IIdentity identity = UserCookieHelper.CurrentPrincipal.Identity;
            if (identity.IsAuthenticated)
            {
                registeredUser.Name = identity.Name;
                registeredUser.Email = identity.Name;
                registeredUser.AuthenticationType = identity.AuthenticationType;
                registeredUser.IsWindowsIdentity = (identity is System.Security.Principal.WindowsIdentity);
                registeredUser.AnonymousId = this.GetAnonymousUserId();
                return true;
            }

            return false;
        }

        /// <summary>
        /// Gets the anonymous user id.
        /// </summary>
        /// <returns>The anonymous user id</returns>
        protected virtual string GetAnonymousUserId()
        {
            string anonymousUserId = UserCookieHelper.GetAnonymousUserId();
            if (string.IsNullOrEmpty(anonymousUserId))
            {
                anonymousUserId = System.Guid.NewGuid().ToString("B");
                UserCookieHelper.IssueAnonymousCookie(anonymousUserId);
            }

            return anonymousUserId;
        }

        /// <summary>
        /// Posts the authenticate request.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void PostAuthenticateRequest(object sender, System.EventArgs e)
        {
            this.OnPostAuthenticateRequest();
        }
    }
}