﻿//---------------------------------------------------------------------
// <copyright file="CommerceMembershipProvider.cs" company="commerceserver.net">
//     Copyright © commerceserver.net.  All rights reserved.
// </copyright>
// <summary></summary>
//---------------------------------------------------------------------

namespace CommerceServerContrib.Web
{
    using System;
    using System.Configuration;
    using System.Globalization;
    using System.ServiceModel;
    using System.Text.RegularExpressions;
    using System.Web.Security;
    using CommerceServerContrib.CrossTier.Helpers;
    using CommerceServer.Foundation;
    using CommerceServer.Foundation.Definitions;
    using CommerceServerContrib.CrossTier.Diagnostics;

    /// <summary>
    /// Used to handle registering and retrieving user info from the backend repository
    /// </summary>
    public class CommerceMembershipProvider : CommerceServer.Foundation.CommerceMembershipProvider
    {
        private const string CreateUserModelName = "UserProfile";
        private const string UserFirstName = "FirstName";
        private const string UserLastName = "LastName";
        private const string DefaultChannelKey = "Default_481EC306-4764-44ff-9D7C-BEE9FC88FA7F";
        private const string MembershipUserModelNameKey = "membershipUserModelName";
        private System.Collections.Generic.Dictionary<string, CommerceMembershipSettings> channelSettingsLookup = new System.Collections.Generic.Dictionary<string, CommerceMembershipSettings>();
        private object channelLookupLock = new object();

        /// <summary>
        /// Gets the user.
        /// </summary>
        /// <param name="userName">Name of the user.</param>
        /// <param name="userIsOnline">if set to <c>true</c> user is online.</param>
        /// <returns>The MembershipUser object.</returns>
        public override MembershipUser GetUser(string userName, bool userIsOnline)
        {
            MembershipUser membershipUser = base.GetUser(userName, userIsOnline);
            if (membershipUser != null && membershipUser is CommerceMembershipUser)
            {
                var csMembershipUser = membershipUser as CommerceMembershipUser;
            }

            return membershipUser;
        }

        /// <summary>
        /// This method overrides the commerce server ResetPassword
        /// </summary>
        /// <param name="userName">The user's username</param>
        /// <param name="passwordAnswer">The user's password answer</param>
        /// <param name="channelName">The channel name</param>
        /// <returns>Returns the newly created password</returns>
        public override string ResetPassword(string userName, string passwordAnswer, string channelName)
        {
            CommerceMembershipSettings settings = this.GetChannelMembershipSettings(channelName);

            if (!settings.EnablePasswordReset)
            {
                CommerceTrace.Error(CommerceClientTraceSource.ApplicationTier, ClientFrameworkResources.MembershipProvider_MembershipSettingNotEnabled, "EnablePasswordReset");
                throw new ConfigurationErrorsException(string.Format(CultureInfo.InvariantCulture, ClientFrameworkResources.MembershipProvider_MembershipSettingNotEnabled, "EnablePasswordReset"));
            }

            ParameterChecker.CheckForNullOrEmpty(userName, "userName");

            if (settings.RequiresQuestionAndAnswer)
            {
                if (string.IsNullOrEmpty(passwordAnswer))
                {
                    throw new ArgumentNullException("passwordAnswer");
                }

                if (!this.ValidateUserAnswer(userName, passwordAnswer, channelName))
                {
                    CommerceTrace.Error(CommerceClientTraceSource.ApplicationTier, ClientFrameworkResources.MembershipProvider_MembershipUserCouldNotBeValidated);
                    throw new MembershipPasswordException(ClientFrameworkResources.MembershipProvider_MembershipSettingNotEnabled);
                }
            }

            // Passwords reset by the membership provider
            // will meet min length and special characters,
            // but are not guaranteed to meet the 
            // password strength regex.  See MSDN docs.
            // This method will meet the regex
            string newPassword = this.GenerateAutoPassword();

            var update = new CommerceUpdate<CommerceEntity>(settings.MembershipUserModelName);

            update.Model.Properties[CommerceMembershipSettings.PasswordPropertyName] = newPassword;
            update.SearchCriteria.Model.Properties[settings.LogonPropertyName] = userName;

            try
            {
                this.ProcessRequest(this.GetRequestContext(), update);
            }
            catch (FaultException<ItemDoesNotExistFault> ex)
            {
                CommerceTrace.Error(CommerceClientTraceSource.ApplicationTier, ClientFrameworkResources.MembershipProvider_MembershipUserCouldNotBeValidated);
                CommerceEventLog.PresentationTierSource.WriteException(ex);

                throw new MembershipPasswordException(ClientFrameworkResources.MembershipProvider_MembershipSettingNotEnabled);
            }

            return newPassword;
        }

        /// <summary>
        /// Processes the request.
        /// </summary>
        /// <param name="requestContext">The request context.</param>
        /// <param name="requestBuilder">The request builder.</param>
        /// <returns>The response from the foundation</returns>
        protected override CommerceResponse ProcessRequest(CommerceRequestContext requestContext, CommerceBaseBuilder requestBuilder)
        {
            this.AdjustCreateUser(requestBuilder);

            return base.ProcessRequest(requestContext, requestBuilder);
        }

        /// <summary>
        /// Adjusts the create user request.
        /// </summary>
        /// <param name="requestBuilder">The request builder for the current request.</param>
        protected virtual void AdjustCreateUser(CommerceBaseBuilder requestBuilder)
        {
            if (requestBuilder != null && requestBuilder is CommerceCreate<CommerceEntity>)
            {
                var request = (CommerceCreate<CommerceEntity>)requestBuilder;

                if (request.Model.ModelName == CreateUserModelName)
                {
                    // first name and last name fields are required, so we need to pass in some default values.
                    request.Model.Properties.Add(UserFirstName, UserFirstName);
                    request.Model.Properties.Add(UserLastName, UserLastName);
                }
            }
        }

        /// <summary>
        /// This method generates a password
        /// </summary>
        /// <returns>The automatically generated password</returns>
        protected virtual string GenerateAutoPassword()
        {
            // Make sure length is valid, between 1 and 50
            if (MinRequiredPasswordLength <= 0 || MinRequiredPasswordLength > 50)
            {
                CommerceTrace.Error(CommerceClientTraceSource.ApplicationTier, ClientFrameworkResources.MembershipProvider_InvalidMinLength);
                throw new ArgumentException(ClientFrameworkResources.MembershipProvider_InvalidMinLength);
            }

            var newPassword = Membership.GeneratePassword(this.MinRequiredPasswordLength, this.MinRequiredNonAlphanumericCharacters);

            if (!Regex.IsMatch(newPassword, this.PasswordStrengthRegularExpression))
            {
                // Since the regex failed, we can assume there's either no decimals, or no capital letters, or no lowercase letters
                var rnd = new Random(0);

                // Get a random number
                int randomNumber = rnd.Next(9);
                
                // Get a random capital letter
                string randomCapitalLetter = Char.ConvertFromUtf32(rnd.Next(65, 91));
                
                // Get a random lower-case letter
                string randomLowerCaseLetter = Char.ConvertFromUtf32(rnd.Next(97, 123));
                newPassword = string.Concat(newPassword, randomNumber.ToString(CultureInfo.InvariantCulture), randomCapitalLetter, randomLowerCaseLetter);
            }

            return newPassword;
        }

        private CommerceMembershipSettings GetChannelMembershipSettings(string channelName)
        {
            string channelKey = channelName;

            if (string.IsNullOrEmpty(channelKey))
            {
                channelKey = CommerceMembershipProvider.DefaultChannelKey;
            }

            CommerceMembershipSettings settings;
            if (!this.channelSettingsLookup.ContainsKey(channelKey))
            {
                lock (this.channelLookupLock)
                {
                    if (!this.channelSettingsLookup.ContainsKey(channelKey))
                    {
                        CommerceEntity membershipSettings = this.QueryMembershipSettings(channelName);

                        // Get the model name for the membership user
                        string membershipUserModel = membershipSettings.Properties[CommerceMembershipProvider.MembershipUserModelNameKey] as string;

                        if (string.IsNullOrEmpty(membershipUserModel))
                        {
                            CommerceTrace.Error(CommerceClientTraceSource.ApplicationTier, ClientFrameworkResources.MembershipProvider_MembershipSettingsMissingRequiredProperty, CommerceMembershipProvider.MembershipUserModelNameKey);
                            throw new ConfigurationErrorsException(string.Format(CultureInfo.InvariantCulture, ClientFrameworkResources.MembershipProvider_MembershipSettingsMissingRequiredProperty, CommerceMembershipProvider.MembershipUserModelNameKey));
                        }

                        CommerceEntityDefinition membershipUserDefinition = this.QueryUserEntityDefinition(channelName, membershipUserModel);

                        settings = new CommerceMembershipSettings(this.Name, membershipUserDefinition, membershipSettings);

                        this.channelSettingsLookup.Add(channelKey, settings);
                    }
                    else
                    {
                        settings = this.channelSettingsLookup[channelKey];
                    }
                }
            }
            else
            {
                settings = this.channelSettingsLookup[channelKey];
            }

            return settings;
        }

        private CommerceEntity QueryMembershipSettings(string channel)
        {
            var query = new CommerceQuery<CommerceEntity, CommerceModelSearch<CommerceEntity>>("Channel");
            query.SearchCriteria.Model.Properties.Add("Id", channel);

            CommerceQueryRelatedItem<CommerceEntity> membershipSettings = new CommerceQueryRelatedItem<CommerceEntity>("MembershipSettings", "MembershipSettings");

            query.RelatedOperations.Add(membershipSettings);

            CommerceResponse response = this.ProcessRequest(this.GetRequestContext(channel), query);

            CommerceQueryOperationResponse queryResponse = response.OperationResponses[0] as CommerceQueryOperationResponse;

            if (queryResponse.CommerceEntities == null || queryResponse.CommerceEntities.Count < 1)
            {
                CommerceTrace.Error(CommerceClientTraceSource.ApplicationTier, ClientFrameworkResources.MembershipProvider_MembershipInvalidSettingsQueryResponse);
                throw new InvalidOperationException(ClientFrameworkResources.MembershipProvider_MembershipInvalidSettingsQueryResponse);
            }

            CommerceRelationshipList relationshipList = queryResponse.CommerceEntities[0].Properties["MembershipSettings"] as CommerceRelationshipList;

            if (relationshipList == null || relationshipList.Count < 1 || relationshipList[0].Target == null)
            {
                CommerceTrace.Error(CommerceClientTraceSource.ApplicationTier, ClientFrameworkResources.MembershipProvider_MembershipInvalidSettingsQueryResponse);
                throw new InvalidOperationException(ClientFrameworkResources.MembershipProvider_MembershipInvalidSettingsQueryResponse);
            }

            return relationshipList[0].Target;
        }

        private CommerceEntityDefinition QueryUserEntityDefinition(string channelName, string modelName)
        {
            CommerceRequestContext commerceContext = this.GetRequestContext(channelName);

            // Build the query
            var metadataQuery = new CommerceQuery<CommerceEntityDefinition, CommerceModelSearch<CommerceEntity>>();
            metadataQuery.SearchCriteria.Model.ModelName = modelName;

            var propertyDefinitionsQuery = new CommerceQueryRelatedItem<PropertyDefinition>(
                CommerceEntityDefinition.RelationshipName.PropertyDefinitions);
            metadataQuery.RelatedOperations.Add(propertyDefinitionsQuery);

            var constraintsQuery = new CommerceQueryRelatedItem<ConstraintBase>(
                PropertyDefinition.RelationshipName.Constraints);
            propertyDefinitionsQuery.RelatedOperations.Add(constraintsQuery);

            // Execute the query
            CommerceResponse response = this.ProcessRequest(commerceContext, metadataQuery);

            if (response.OperationResponses.Count < 1)
            {
                CommerceTrace.Error(CommerceClientTraceSource.ApplicationTier, ClientFrameworkResources.MembershipProvider_MembershipInvalidEntityDefinitionResponse);
                throw new InvalidOperationException(ClientFrameworkResources.MembershipProvider_MembershipInvalidEntityDefinitionResponse);
            }

            // Get the query operation response
            CommerceQueryOperationResponse queryResponse = response.OperationResponses[0] as CommerceQueryOperationResponse;

            if (queryResponse.CommerceEntities.Count < 1)
            {
                CommerceTrace.Error(CommerceClientTraceSource.ApplicationTier, ClientFrameworkResources.MembershipProvider_MembershipInvalidEntityDefinitionResponse);
                throw new InvalidOperationException(ClientFrameworkResources.MembershipProvider_MembershipInvalidEntityDefinitionResponse);
            }

            // Get the commerce entity definition
            CommerceEntityDefinition commerceEntityDefinition = queryResponse.CommerceEntities[0] as CommerceEntityDefinition;

            if (commerceEntityDefinition == null)
            {
                CommerceTrace.Error(CommerceClientTraceSource.ApplicationTier, ClientFrameworkResources.MembershipProvider_MembershipInvalidEntityDefinitionResponse);
                throw new InvalidOperationException(ClientFrameworkResources.MembershipProvider_MembershipInvalidEntityDefinitionResponse);
            }

            return commerceEntityDefinition;
        }

        private CommerceRequestContext GetRequestContext(string channel)
        {
            var context = base.GetRequestContext();
            context.Channel = channel;
            return context;
        }
    }
}
