﻿//---------------------------------------------------------------------
// <copyright file="siteContextParameterType.cs" company="commerceserver.net">
//     Copyright © commerceserver.net.  All rights reserved.
// </copyright>
// <summary></summary>
//---------------------------------------------------------------------

namespace CommerceServerContrib.Web
{
    using System.Collections.Generic;
    using System.Web.UI;
    using System.Web.UI.WebControls;
    using CommerceServerContrib.CrossTier.Exceptions;
    using CommerceServerContrib.CrossTier.Interfaces;
    using CommerceServer.Foundation;
    using System;
    using System.Globalization;

    /// <summary>
    /// A parameter type that maps to a collection of siteContexts.
    /// </summary>
    [FriendlyName(ResourceHelper.CommerceServerContribResourceLocation, "SiteContextParameterType")]
    public class SiteContextParameterType : IParameterType
    {
        #region IParameterType Members
        /// <summary>
        /// Private dictionary to store parameters
        /// </summary>
        private Dictionary<string, object> _siteContextParameterDictionary;

        /// <summary>
        /// Creates the editor control.
        /// </summary>
        /// <returns>A control for editing the parameter type.</returns>
        public System.Web.UI.Control CreateEditorControl()
        {
            var siteContextDropDown = new DropDownList { ID = "siteContextDropDown" };
            siteContextDropDown.Items.Clear();

            var siteContextsDictionary = this.GetSiteContextParameters();
            foreach (var key in siteContextsDictionary.Keys)
            {
                siteContextDropDown.Items.Add(new ListItem(key, key));
            }

            siteContextDropDown.Items.Insert(0, new ListItem(string.Empty));

            return siteContextDropDown;
        }

        /// <summary>
        /// Get the value of the parameter.
        /// </summary>
        /// <param name="contextValue">The context value.</param>
        /// <returns>The value of the parameter.</returns>
        public object GetParameterValue(string contextValue)
        {
            if (string.IsNullOrEmpty(contextValue))
            {
                throw new ArgumentNullException("contextValue");
            }

            if (!this.ValidateProperty(contextValue))
            {
                throw new ParameterValueNotFoundException(string.Format(CultureInfo.InvariantCulture, "The key {0} was not found in the SiteContext Parameter collection.", contextValue));
            }

            return this.GetSiteContextParameters()[contextValue];
        }

        /// <summary>
        /// Gets the context value from the editor control.
        /// </summary>
        /// <param name="editorControl">The editor control.</param>
        /// <returns>The context value.</returns>
        public string GetContextValueFromEditorControl(Control editorControl)
        {
            var contextValue = string.Empty;
            var ddl = editorControl as DropDownList;

            if (ddl != null)
            {
                contextValue = ddl.SelectedValue;
            }

            return contextValue;
        }

        /// <summary>
        /// Resets the editor control.
        /// </summary>
        /// <param name="editorControl">The editor control.</param>
        public void ResetEditorControl(Control editorControl)
        {
            var ddl = editorControl as DropDownList;

            if (ddl != null)
            {
                ddl.SelectedIndex = 0;
            }
        }

        /// <summary>
        /// Gets the complete list of site context parameters with their values
        /// </summary>
        /// <returns>Returns the complete list of site context parameters with their values</returns>
        public Dictionary<string, object> GetSiteContextParameters()
        {
            if (this._siteContextParameterDictionary == null)
            {
                this._siteContextParameterDictionary = new Dictionary<string, object>();
            
                // Other info only needs to be populated once
                this.PopulateOtherInfo();
            }

            // User information needs to be refreshed every time
            this.PopulateUserInfo();
            return this._siteContextParameterDictionary;
        }

        /// <summary>
        /// Validates that the requested property exists within the possible SiteContext properties
        /// </summary>
        /// <param name="property">The name of the property to verify</param>
        /// <returns>True if the property is valid, false if it isn't</returns>
        public bool ValidateProperty(string property)
        {
            return this.GetSiteContextParameters().ContainsKey(property);
        }

        /// <summary>
        /// This method is used to populate the dictionary of key/value pairs possible to return.
        /// </summary>
        protected virtual void PopulateOtherInfo()
        {
            var siteContext = PresentationServiceLocator.Current.SiteContext;

            if (siteContext != null)
            {
                // Other SiteContext properties
                this.AddPropertyIfNotNull("Browser", siteContext.Browser);
                this.AddPropertyIfNotNull("CultureCode", siteContext.CultureCode);
                this.AddPropertyIfNotNull("DefaultBasketName", siteContext.DefaultBasketName);
                this.AddPropertyIfNotNull("DefaultBasketType", siteContext.DefaultBasketType);
                this.AddPropertyIfNotNull("FullLocationPath", siteContext.FullLocationPath);
                this.AddPropertyIfNotNull("IsUserAuthenticated", siteContext.IsUserAuthenticated);
                this.AddPropertyIfNotNull("IsAnonymous", siteContext.IsAnonymous);
                this.AddPropertyIfNotNull("SiteId", siteContext.SiteId);
                this.AddPropertyIfNotNull("UserId", siteContext.UserId);
                this.AddPropertyIfNotNull("DefaultCatalogName", siteContext.DefaultCatalogName);
            }
        }

        /// <summary>
        /// This method is used to populate the dictionary of key/value pairs possible to return.
        /// </summary>
        protected virtual void PopulateUserInfo()
        {
            if (PresentationServiceLocator.Current.SiteContext != null && PresentationServiceLocator.Current.SiteContext.User != null)
            {
                var userInfo = PresentationServiceLocator.Current.SiteContext.User;

                this.AddPropertyIfNotNull("User ID", userInfo.Id);
                this.AddPropertyIfNotNull("User AnonymousId", userInfo.AnonymousId);
                this.AddPropertyIfNotNull("User AuthenticationType", userInfo.AuthenticationType);
                this.AddPropertyIfNotNull("User Email", userInfo.Email);
                this.AddPropertyIfNotNull("User IsWindowsIdentity", userInfo.IsWindowsIdentity);
                this.AddPropertyIfNotNull("User Name", userInfo.Name);
                this.AddPropertyIfNotNull("Remember Me", userInfo.RememberMe);
            }
        }

        /// <summary>
        /// This method adds a property to the property bag if it's not null
        /// </summary>
        /// <param name="propertyName">The name of the property</param>
        /// <param name="property">The property</param>
        /// <returns>True if addition was successful, false if it failed</returns>
        protected bool AddPropertyIfNotNull(string propertyName, object property)
        {
            if (!string.IsNullOrEmpty(propertyName))
            {
                if (this._siteContextParameterDictionary.ContainsKey(propertyName))
                {
                    this._siteContextParameterDictionary.Remove(propertyName);
                }

                this._siteContextParameterDictionary.Add(propertyName, property);
                return true;
            }

            return false;
        }
        #endregion
    }
}
