﻿//---------------------------------------------------------------------
// <copyright file="BaseWebPart.cs" company="commerceserver.net">
//     Copyright © commerceserver.net.  All rights reserved.
// </copyright>
// <summary></summary>
//---------------------------------------------------------------------

namespace CommerceServerContrib.Web
{
    using System;
    using System.Collections.Generic;
    using System.Globalization;
    using System.IO;
    using System.Text;
    using System.Web;
    using System.Web.UI;
    using System.Web.UI.WebControls.WebParts;
    using System.Xml.Linq;
    using System.Xml.Xsl;
    using System.Linq;
    using CommerceServerContrib.CrossTier.Entities;
    using CommerceServerContrib.CrossTier.Exceptions;
    using CommerceServerContrib.CrossTier.Helpers;
    using CommerceServerContrib.CrossTier.Interfaces;
    using CommerceServerContrib.CrossTier.Providers;
    using System.ComponentModel;
    using CommerceServer.Foundation;

    /// <summary>
    /// The base webpart inherited by all webparts
    /// </summary>
    public class BaseWebPart : WebPart, IServiceResponse, IWebEditable, ICacheableWebPart
    {
        #region Private Fields

        private const string CacheKeySeperator = "_";
        private const string ResponseCachePrefix = "Response_{0}";
        private const string FileTypeXslt = "xslt";
        private const string FileTypeAscx = "ascx";
        private const string PathStarter = @"\";

        private static object _cacheLock = new object();
        private XDocument _xmlMessage;
        private bool _cacheKeyShouldIncludeUrl = true, _loadOnce = true;
        private string _tabTitle = string.Empty;
        private string _xsltUri = string.Empty;
        private List<object> _renderData = new List<object>();

        /// <summary>
        /// Properties used for webpart output caching.  Optimization to ensure cache context is not rebuilt on every call IsDataInCache() method.
        /// </summary>
        private CacheContext _cacheContext;
        private TimeSpan _cacheExpiration = TimeSpan.Zero;
        private string _cachedHtml;

        #endregion

        #region WebPart Properties

        /// <summary>
        /// Gets or sets a value indicating whether to cache the web part content.
        /// </summary>
        /// <value>
        ///   <c>true</c> if the web part content is to be cached; otherwise, <c>false</c>.
        /// </value>
        [Personalizable(PersonalizationScope.Shared)]
        [WebBrowsable(false)]
        public bool CacheContent
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets a value indicating whether to use the current page url in the cache key
        /// </summary>
        /// <value>
        ///   <c>true</c> if the web part content is to be cached; otherwise, <c>false</c>.
        /// </value>
        [Personalizable(PersonalizationScope.Shared)]
        [WebBrowsable(false)]
        public bool CacheKeyShouldIncludeUrl
        {
            get
            {
                return this._cacheKeyShouldIncludeUrl;
            }

            set
            {
                this._cacheKeyShouldIncludeUrl = value;
            }
        }

        /// <summary>
        /// Gets or sets a value indicating whether specific querystrings should be used when caching with a url
        /// </summary>
        /// <value>
        ///   <c>true</c> if specific values should be used; otherwise, <c>false</c>.
        /// </value>
        [Personalizable(PersonalizationScope.Shared)]
        [WebBrowsable(false)]
        public bool CacheWithSpecificQueryStrings
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the cache expiration time.
        /// </summary>
        /// <value>
        /// The cache expiration time.
        /// </value>
        [Personalizable(PersonalizationScope.Shared)]
        [WebBrowsable(false)]
        public TimeSpan? CacheExpiration
        {
            get
            {
                return this._cacheExpiration;
            }

            set
            {
                if (value.HasValue)
                {
                    this._cacheExpiration = value.Value;
                }
                else
                {
                    this._cacheExpiration = TimeSpan.Zero;
                }
            }
        }

        /// <summary>
        /// Gets or sets the cache querystring settings.
        /// </summary>
        /// <value>
        /// The cache querystring settings.
        /// </value>
        [Personalizable(PersonalizationScope.Shared)]
        [WebBrowsable(false)]
        public string CacheQueryStringSettings
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets a value indicating whether the content of the response is cached.
        /// </summary>
        /// <value>
        /// <c>true</c> if the response content is to be cached; otherwise, <c>false</c>.
        /// </value>
        [Personalizable(PersonalizationScope.Shared)]
        [WebBrowsable(false)]
        public bool CacheResponseContent
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets a list of all of the XSLT parameters
        /// </summary>
        [Personalizable(PersonalizationScope.Shared)]
        [WebBrowsable(false)]
        [LocalizedWebDisplayName(ResourceHelper.CommerceServerContribResourceLocation, "WebPartProperty_XsltParameterSettingsDisplayName")]
        [LocalizedWebDescription(ResourceHelper.CommerceServerContribResourceLocation, "WebPartProperty_XsltParameterSettingsDescription")]
        [LocalizedWebCategory(CommonPropertyCategories.CommerceServerContrib)]
        public string XsltParameterSettings
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets a value indicating whether or not the request xml should be rendered out
        /// </summary>
        [Personalizable(PersonalizationScope.Shared)]
        [WebBrowsable(true)]
        [LocalizedWebDisplayName(ResourceHelper.CommerceServerContribResourceLocation, "WebPartProperty_ShowRequestXmlDisplayName")]
        [LocalizedWebDescription(ResourceHelper.CommerceServerContribResourceLocation, "WebPartProperty_ShowRequestXmlDescription")]
        [LocalizedWebCategory(CommonPropertyCategories.CommerceServerContrib)]
        public bool ShowRequestXml
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets a value indicating whether this instance is consumer.
        /// </summary>
        [Personalizable(PersonalizationScope.Shared)]
        [WebBrowsable(true)]
        [LocalizedWebDisplayName(ResourceHelper.CommerceServerContribResourceLocation, "WebPartProperty_IsConsumerDisplayName")]
        [LocalizedWebDescription(ResourceHelper.CommerceServerContribResourceLocation, "WebPartProperty_IsConsumerDescription")]
        [LocalizedWebCategory(CommonPropertyCategories.CommerceServerContrib)]
        public bool IsConsumer
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the Uri to be Xslt that will be used.
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1056:UriPropertiesShouldNotBeStrings", Justification = "Webparts do not support Uri type.")]
        [Personalizable(PersonalizationScope.Shared)]
        [WebBrowsable(false)]
        [LocalizedWebDisplayName(ResourceHelper.CommerceServerContribResourceLocation, "WebPartProperty_WebDisplayName")]
        [LocalizedWebDescription(ResourceHelper.CommerceServerContribResourceLocation, "WebPartProperty_WebDescription")]
        [LocalizedWebCategory(CommonPropertyCategories.CommerceServerContrib)]
        public string XsltUri
        {
            get
            {
                return this._xsltUri;
            }

            set
            {
                this._xsltUri = value;
            }
        }

        /// <summary>
        /// Gets or sets the render mode specified by the user
        /// </summary>
        [Personalizable(PersonalizationScope.Shared)]
        [WebBrowsable(false)]
        [LocalizedWebDisplayName(ResourceHelper.CommerceServerContribResourceLocation, "WebPartProperty_SpecifiedRenderModeDisplayName")]
        [LocalizedWebDescription(ResourceHelper.CommerceServerContribResourceLocation, "WebPartProperty_SpecifiedRenderModeDescription")]
        [LocalizedWebCategory(CommonPropertyCategories.CommerceServerContrib)]
        public RenderModes SpecifiedRenderMode
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets a value indicating whether or not the Xslt should be rendered
        /// even if there is no Xml to render
        /// </summary>
        [Personalizable(PersonalizationScope.Shared)]
        [WebBrowsable(true)]
        [LocalizedWebDisplayName(ResourceHelper.CommerceServerContribResourceLocation, "WebPartProperty_RenderXsltEvenIfNoXmlDisplayName")]
        [LocalizedWebDescription(ResourceHelper.CommerceServerContribResourceLocation, "WebPartProperty_RenderXsltEvenIfNoXmlDescription")]
        [LocalizedWebCategory(CommonPropertyCategories.CommerceServerContrib)]
        public bool RenderXsltEvenIfNoXml
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the file to populate the XML message.
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1056:UriPropertiesShouldNotBeStrings", Justification = "Webparts do not support Uri type.")]
        [Personalizable(PersonalizationScope.Shared)]
        [WebBrowsable(true)]
        [LocalizedWebDisplayName(ResourceHelper.CommerceServerContribResourceLocation, "WebPartProperty_XmlMessageUriDisplayName")]
        [LocalizedWebDescription(ResourceHelper.CommerceServerContribResourceLocation, "WebPartProperty_XmlMessageUriDescription")]
        [LocalizedWebCategory(CommonPropertyCategories.CommerceServerContrib)]
        public string XmlMessageUri
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets a value indicating which authorization name is required to act on this webpart
        /// </summary>
        [Personalizable(PersonalizationScope.Shared)]
        [WebBrowsable(false)]
        [LocalizedWebDisplayName(ResourceHelper.CommerceServerContribResourceLocation, "WebPartProperty_AuthorizationRoleNameDisplayName")]
        [LocalizedWebDescription(ResourceHelper.CommerceServerContribResourceLocation, "WebPartProperty_AuthorizationRoleNameDescription")]
        [LocalizedWebCategory(CommonPropertyCategories.CommerceServerContrib)]
        public string AuthorizationRoleName
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets a value indicating whether [load only once].
        /// </summary>
        /// <value>
        ///   <c>true</c> if [load only once]; otherwise, <c>false</c>.
        /// </value>
        [Personalizable(PersonalizationScope.Shared)]
        [WebBrowsable(true)]
        [LocalizedWebDisplayName(ResourceHelper.CommerceServerContribResourceLocation, "WebPartProperty_LoadOnceDisplayName")]
        [LocalizedWebDescription(ResourceHelper.CommerceServerContribResourceLocation, "WebPartProperty_LoadOnceDescription")]
        [LocalizedWebCategory(CommonPropertyCategories.TabConnection)]
        public bool LoadOnce
        {
            get
            {
                return this._loadOnce;
            }

            set
            {
                this._loadOnce = value;
            }
        }

        /// <summary>
        /// Gets or sets key of the tab title.
        /// </summary>
        [Personalizable(PersonalizationScope.Shared)]
        [WebBrowsable(true)]
        [LocalizedWebDisplayName(ResourceHelper.CommerceServerContribResourceLocation, "WebPartProperty_TabTitleResourceKeyDisplayName")]
        [LocalizedWebDescription(ResourceHelper.CommerceServerContribResourceLocation, "WebPartProperty_TabTitleResourceKeyDescription")]
        [LocalizedWebCategory(CommonPropertyCategories.TabConnection)]
        public string TabTitleResourceKey
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the location of the tab tile resource file.
        /// </summary>
        [Personalizable(PersonalizationScope.Shared)]
        [WebBrowsable(true)]
        [LocalizedWebDisplayName(ResourceHelper.CommerceServerContribResourceLocation, "WebPartProperty_TabTitleResourceLocationDisplayName")]
        [LocalizedWebDescription(ResourceHelper.CommerceServerContribResourceLocation, "WebPartProperty_TabTitleResourceLocationDescription")]
        [LocalizedWebCategory(CommonPropertyCategories.TabConnection)]
        public string TabTitleResourceLocation
        {
            get;
            set;
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets or sets the tab title.
        /// </summary>
        /// <value>
        /// The tab title.
        /// </value>
        public string TabTitle
        {
            get
            {
                if (!string.IsNullOrEmpty(this.TabTitleResourceLocation) && !string.IsNullOrEmpty(this.TabTitleResourceKey))
                {
                    return ResourceHelper.Get(this.TabTitleResourceLocation, this.TabTitleResourceKey);
                }

                return this.Title;
            }

            set
            {
                this._tabTitle = value;
            }
        }

        /// <summary>
        /// Gets or sets the post back location.
        /// </summary>
        /// <value>
        /// The post back location.
        /// </value>
        public string PostBackLocation { get; set; }

        /// <summary>
        /// Gets or sets the json data.
        /// </summary>
        /// <value>
        /// The json data.
        /// </value>
        public string JsonData { get; set; }

        /// <summary>
        /// Gets the Id of the element for the webpart
        /// </summary>
        /// <value>
        /// The Id of the main element
        /// </value>
        public string MainElementId
        {
            get
            {
                return this.UniqueID.Replace("$", "_");
            }
        }

        /// <summary>
        /// Gets or sets the Xml message to transform against
        /// </summary>
        public XDocument XmlMessage
        {
            get
            {
                if (this._xmlMessage == null)
                {
                    this._xmlMessage = new XDocument();
                    this._xmlMessage.AddFirst(new XElement(CommerceServerContrib.CrossTier.Helpers.StringConstants.XmlRootNodeName));
                }

                return this._xmlMessage;
            }

            set
            {
            }
        }

        /// <summary>
        /// Gets the current render mode.
        /// </summary>
        public virtual RenderModes RenderMode
        {
            get
            {
                if (this.SpecifiedRenderMode == RenderModes.Auto)
                {
                    if (!string.IsNullOrEmpty(this.XsltUri))
                    {
                        if (this.XsltUri.EndsWith(FileTypeXslt, StringComparison.OrdinalIgnoreCase))
                        {
                            return RenderModes.Xslt;
                        }

                        if (this.XsltUri.EndsWith(FileTypeAscx, StringComparison.OrdinalIgnoreCase))
                        {
                            return RenderModes.Ascx;
                        }
                    }

                    return RenderModes.Xslt;
                }
                else
                {
                    return this.SpecifiedRenderMode;
                }
            }
        }

        /// <summary>
        /// Gets or sets the data that should be passed to the rendering engine.
        /// </summary>
        /// <value>
        /// The render data.
        /// </value>
        public List<object> RenderData
        {
            get
            {
                return this._renderData;
            }

            set
            {
                this._renderData = value;
            }
        }

        /// <summary>
        /// Gets a reference to the <see cref="T:System.Web.UI.WebControls.WebParts.WebPart"/> control to enable it to be edited by custom <see cref="T:System.Web.UI.WebControls.WebParts.EditorPart"/> controls.
        /// </summary>
        /// <returns>A <see cref="T:System.Object"/> that consists of the child control of a <see cref="T:System.Web.UI.WebControls.WebParts.WebPart"/> control.</returns>
        object IWebEditable.WebBrowsableObject
        {
            get
            {
                return this;
            }
        }

        /// <summary>
        /// Gets or sets the output of the message transformation.
        /// </summary>
        protected string OutputMessage
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the compiled transform.
        /// </summary>
        protected XslCompiledTransform CompiledTransform
        {
            get;
            set;
        }

        /// <summary>
        /// Gets a value indicating whether [in display mode].
        /// </summary>
        /// <value>
        ///   <c>true</c> if [in display mode]; otherwise, <c>false</c>.
        /// </value>
        protected virtual bool InDisplayMode
        {
            get
            {
                if (this.WebPartManager == null)
                {
                    return true;
                }

                return (this.WebPartManager.DisplayMode == WebPartManager.BrowseDisplayMode);
            }
        }

        /// <summary>
        /// Gets the current page.
        /// </summary>
        protected virtual Page CurrentPage
        {
            get { return this.Page; }
        }

        #endregion

        #region IWebEditable Members

        /// <summary>
        /// Returns a collection of custom <see cref="T:System.Web.UI.WebControls.WebParts.EditorPart"/> controls that can be used to edit a <see cref="T:System.Web.UI.WebControls.WebParts.WebPart"/> control when it is in edit mode.
        /// </summary>
        /// <returns>
        /// An <see cref="T:System.Web.UI.WebControls.WebParts.EditorPartCollection"/> that contains custom <see cref="T:System.Web.UI.WebControls.WebParts.EditorPart"/> controls associated with a <see cref="T:System.Web.UI.WebControls.WebParts.WebPart"/> control.
        /// </returns>
        EditorPartCollection IWebEditable.CreateEditorParts()
        {
            return this.SetupEditorParts();
        }

        #endregion

        #region IServiceResponse Members

        /// <summary>
        /// Provides the XML message.
        /// </summary>
        /// <returns>An instance of the provider</returns>
        [ConnectionProvider("Service Response", "ServiceResponseProvider", AllowsMultipleConnections = true)]
        public IServiceResponse ProvideXmlMessage()
        {
            return this;
        }

        /// <summary>
        /// Consumes the XML message.
        /// </summary>
        /// <param name="provider">The provider.</param>
        [ConnectionConsumer("Service Response", "ServiceResponseConsumer", AllowsMultipleConnections = true)]
        public void ConsumeXmlMessage(IServiceResponse provider)
        {
            if (provider != null)
            {
                if (provider.RenderMode == RenderModes.Xslt && provider.XmlMessage != null && provider.XmlMessage.Root != null)
                {
                    if (provider.XmlMessage.Root.Name == this.XmlMessage.Root.Name)
                    {
                        this.XmlMessage.Root.Add(provider.XmlMessage.Root.Nodes());
                    }
                    else
                    {
                        this.XmlMessage.Root.Add(provider.XmlMessage.Root);
                    }
                }
                else if (provider.RenderMode != RenderModes.Xslt && provider.RenderData != null)
                {
                    this.RenderData.AddRange(provider.RenderData);
                }
            }
        }

        #endregion

        #region Methods

        /// <summary>
        /// Setups the editor parts.
        /// </summary>
        /// <returns>A collection of editor parts</returns>
        protected virtual EditorPartCollection SetupEditorParts()
        {
            var editors = new List<EditorPart>();
            
            var xsltParameterEditorPart = new XsltParameterEditorPart { ID = "xsltParameterEditorPart" };
            editors.Add(xsltParameterEditorPart);
            
            var authorizationEditorPart = new AuthorizationEditorPart { ID = "authorizationEditorPart" };
            editors.Add(authorizationEditorPart);
            
            var xsltUriEditorPart = new XsltUriEditorPart { ID = "xsltUriEditorPart" };
            editors.Add(xsltUriEditorPart);
            
            var cachingEditorPart = new CachingEditorPart { ID = "cachingEditorPart" };
            editors.Add(cachingEditorPart);

            return new EditorPartCollection(editors);
        }

        /// <summary>
        /// Gets the XML message from a URI that will be transformed
        /// </summary>
        /// <returns>The Xml message to transform</returns>
        protected virtual XDocument GetXmlMessageFromUri()
        {
            if (!string.IsNullOrEmpty(this.XmlMessageUri))
            {
                var xmlProvider = PresentationServiceLocator.Current.GetImplementation<IFileProvider>();

                if (xmlProvider != null)
                {
                    try
                    {
                        return XDocument.Parse(xmlProvider.GetFileAsString(this.XmlMessageUri));
                    }
                    catch (FileNotFoundException)
                    {
                        return null;
                    }
                }
            }

            return null;
        }

        /// <summary>
        /// Gets the arguments for the rendering mehod.
        /// </summary>
        /// <returns>The Base method returns only the Authorization pieces, gives an extensiblity point for extended web parts.</returns>
        protected virtual RenderParameters GetRenderParameters()
        {
            var returnList = new RenderParameters();

            var validationRules = new RenderParameter("_validationRules", string.Empty, this.GetValidationRules());
            returnList.Add(validationRules);
            returnList.Add(new RenderParameter("_IsAuthorized", string.Empty, this.IsAuthorized()));

            return returnList;
        }

        /// <summary>
        /// Gets the Render extension objects.
        /// </summary>
        /// <returns>The Base method returns null, gives an extensiblity point for extended web parts.</returns>
        protected virtual RenderExtensionObjects GetRenderExtensionObjects()
        {
            return null;
        }

        /// <summary>
        /// Gets the output cache, cache key.
        /// </summary>
        /// <returns>The web part output cache, cache key.</returns>
        protected virtual string GetOutputCacheKey()
        {
            var sb = new StringBuilder();

            sb.Append(this.UniqueID);

            if (this.CacheKeyShouldIncludeUrl)
            {
                if (this.CacheWithSpecificQueryStrings && !string.IsNullOrEmpty(this.CacheQueryStringSettings))
                {
                    sb.Append(CacheKeySeperator);

                    var paramSettings = SerializationHelper.Deserialize<List<ParameterSelection>>(this.CacheQueryStringSettings, CommerceServerContrib.CrossTier.Helpers.StringConstants.OutputTypeXmlDataContract);
                    var queryStringProvider = PresentationServiceLocator.Current.GetImplementation<IQueryStringProvider>(true);
                    var queryStrings = queryStringProvider.GetQueryStrings();
                    var url = UrlBuilder.CurrentUrl;
                    var currentStrings = UrlBuilder.CurrentUrl.QueryList;

                    url.QueryList.Clear();

                    foreach (var param in paramSettings)
                    {
                        var key = queryStrings[param.Id].QueryParameterKey;
                        url.QueryList.Add(key, currentStrings[key]);
                    }

                    sb.Append(url.ToString().ToLowerInvariant());
                }
                else
                {
                    var queryStringProvider = PresentationServiceLocator.Current.GetImplementation<IQueryStringProvider>(true);
                    var queryStrings = queryStringProvider.GetQueryStrings();
                    var url = UrlBuilder.CurrentUrl;
                    var queryKeysToRemove = new List<string>();
                    
                    foreach (string urlQueryKey in url.QueryList.Keys)
                    {
                        if (null == queryStrings.Values.FirstOrDefault(
                            x => string.Equals(x.QueryParameterKey, urlQueryKey, StringComparison.OrdinalIgnoreCase)))
                        {
                            queryKeysToRemove.Add(urlQueryKey);
                        }
                    }

                    foreach (var queryKey in queryKeysToRemove)
                    {
                        url.QueryList.Remove(queryKey);
                    }

                    sb.Append(CacheKeySeperator);
                    sb.Append(url.ToString().ToLowerInvariant());
                }
            }

            sb.Append(CacheKeySeperator);
            sb.Append(PresentationServiceLocator.Current.SiteContext.CultureCode);

            return sb.ToString();
        }

        /// <summary>
        /// Gets the data to pass to the rendering engine.
        /// </summary>
        /// <returns>The data to render in the render file</returns>
        protected virtual object GetData()
        {
            return null;
        }

        /// <summary>
        /// Raises the <see cref="E:System.Web.UI.Control.Load"/> event.
        /// </summary>
        /// <param name="e">The <see cref="T:System.EventArgs"/> object that contains the event data.</param>
        protected override void OnLoad(System.EventArgs e)
        {
            if (this.IsDataInCache())
            {
                return;
            }

            // only set things up if we are actually in display mode
            if (!this.InDisplayMode)
            {
                return;
            }

            if (this.IsConsumer)
            {
                return;
            }

            if (this.RenderMode != RenderModes.Xslt)
            {
                this.RenderData.Add(this.GetData());

                return;
            }

            var extendedXmlMessage = new List<XDocument>();

            if (string.IsNullOrEmpty(this.XmlMessageUri))
            {
                extendedXmlMessage = this.GetData() as List<XDocument>;
            }
            else
            {
                extendedXmlMessage.Add(this.GetXmlMessageFromUri());
            }

            if (extendedXmlMessage == null || extendedXmlMessage.Count <= 0 || this.XmlMessage.Root == null)
            {
                return;
            }

            foreach (var xmlDoc in extendedXmlMessage)
            {
                if (xmlDoc != null)
                {
                    if (xmlDoc.Root != null && xmlDoc.Root.Name == this.XmlMessage.Root.Name)
                    {
                        this.XmlMessage.Root.Add(xmlDoc.Root.Nodes());
                    }
                    else
                    {
                        this.XmlMessage.Root.Add(xmlDoc.Root);
                    }
                }
            }
        }

        /// <summary>
        /// Indicates if the web part content is in the output cache.
        /// </summary>
        /// <returns>
        ///   <c>true</c> if the web part content is in the cache; otherwise, <c>false</c>.
        /// </returns>
        protected bool IsDataInCache()
        {
            if (!this.CacheContent)
            {
                return false;
            }

            var cacheContext = this.GetCacheContext();

            if (this.CacheContent && cacheContext.IsCachingEnabled)
            {
                if (string.IsNullOrEmpty(this._cachedHtml))
                {
                    var provider = PresentationServiceLocator.Current.CacheProvider(cacheContext.CacheType);
                    this._cachedHtml = provider.GetData<string>(PresentationServiceLocator.Current.BrandContext.Name, cacheContext.CacheName, cacheContext.CacheKey);

                    if (this.CacheResponseContent && this._xmlMessage == null)
                    {
                        this._xmlMessage = provider.GetData<XDocument>(PresentationServiceLocator.Current.BrandContext.Name, cacheContext.CacheName, String.Format(CultureInfo.InvariantCulture, ResponseCachePrefix, cacheContext.CacheKey));
                    }
                }

                return (this._cachedHtml != null);
            }

            return false;
        }

        /// <summary>
        /// Renders the control to the specified HTML writer.
        /// 
        /// This is where the web part output caching takes place.  Do not override this method otherwise this will break the output caching.
        /// </summary>
        /// <param name="writer">The <see cref="T:System.Web.UI.HtmlTextWriter"/> object that receives the control content.</param>
        protected override void Render(HtmlTextWriter writer)
        {
            var cacheContext = this.GetCacheContext();

            if (this.CacheContent && cacheContext.IsCachingEnabled)
            {
                if (!this.IsDataInCache())
                {
                    // This has been moved before the lock as it created a situation where we had a top level lock and many many children locks that
                    // stopped the system from scaling properly under load due to resource contentions other data being cached.
                    StringWriter sw = new StringWriter(CultureInfo.InvariantCulture);
                    using (HtmlTextWriter htmlWriter = new HtmlTextWriter(sw))
                    {
                        base.Render(htmlWriter);

                        lock (_cacheLock)
                        {
                            // DaIsDataInCache sets this._cachedHtml.
                            if (!this.IsDataInCache())
                            {
                                this._cachedHtml = sw.ToString();

                                var provider = PresentationServiceLocator.Current.CacheProvider(cacheContext.CacheType);

                                TimeSpan? cacheTime = null;

                                if (cacheContext.CacheExpiration.HasValue && cacheContext.CacheExpiration.Value.TotalSeconds > 0)
                                {
                                    cacheTime = cacheContext.CacheExpiration;
                                }

                                provider.AddData<string>(PresentationServiceLocator.Current.BrandContext.Name, cacheContext.CacheName, cacheContext.CacheKey, this._cachedHtml, cacheTime);
                                if (this.CacheResponseContent)
                                {
                                    provider.AddData<XDocument>(PresentationServiceLocator.Current.BrandContext.Name, cacheContext.CacheName, String.Format(CultureInfo.InvariantCulture, ResponseCachePrefix, cacheContext.CacheKey), this._xmlMessage, cacheTime);
                                }
                            }
                        }
                    }
                }
                else
                {
                    CommerceTrace.Information(CommerceClientTraceSource.PresentationTier, "BaseWebPart.Render(): Data has been retrieved from the cache.  ID: {0}", this.UniqueID);
                }

                writer.Write(this._cachedHtml);
            }
            else
            {
                base.Render(writer);
            }
        }

        /// <summary>
        /// Renders the control to the specified HTML writer.
        /// </summary>
        /// <param name="writer">The <see cref="T:System.Web.UI.HtmlTextWriter"/> object that receives the control content.</param>
        protected override void RenderContents(System.Web.UI.HtmlTextWriter writer)
        {
            // only set things up if we are actually in display mode
            if (this.InDisplayMode)
            {
                if (this.RenderMode == RenderModes.Xslt)
                {
                    if (this.ShowRequestXml)
                    {
                        this.OutputMessage = this.RenderRequestXml();
                    }
                    else if ((this.XmlMessage.Root != null && this.XmlMessage.Root.FirstNode != null) || (this.XmlMessage.Root != null && this.RenderXsltEvenIfNoXml))
                    {
                        if (!string.IsNullOrEmpty(this.XsltUri))
                        {
                            if (this.XmlMessage != null)
                            {
                                var arguments = this.GetRenderArguments();
                                this.OutputMessage = RenderHelper.Transform(this.XsltUri, this.XmlMessage.ToString(), arguments);
                            }
                        }
                    }
                }
                else if (this.RenderMode == RenderModes.Ascx && !string.IsNullOrEmpty(this.XsltUri) && (this.RenderXsltEvenIfNoXml || this.RenderData.Count > 0))
                {
                    var arguments = this.GetRenderArguments();

                    RenderHelper.Render(this.XsltUri, this.RenderData, arguments, writer, this.Page);
                }
            }

            if (!string.IsNullOrEmpty(this.OutputMessage))
            {
                writer.Write(this.OutputMessage);
            }
        }

        /// <summary>
        /// Goes through unit block and ensure that all the IRenderArgumentList and IExtensionObject implementations in the Argument List
        /// </summary>
        /// <returns>A list of RenderArgumentBuilder <see cref="RenderArgumentBuilder" /></returns>
        protected virtual RenderArgumentBuilder GetRenderArguments()
        {
            var renderArgumentBuilder = new RenderArgumentBuilder();

            var parameters = this.GetRenderParameters();

            if (parameters != null)
            {
                parameters.CopyTo(renderArgumentBuilder.Parameters, false);
            }

            var renderExtensionObjects = this.GetRenderExtensionObjects();
            if (renderExtensionObjects != null)
            {
                renderExtensionObjects.CopyTo(renderArgumentBuilder.RenderExtensionObjects, false);
            }

            this.TranslateXsltParameterSettingsIntoRenderParameters().CopyTo(renderArgumentBuilder.Parameters, true);

            return renderArgumentBuilder;
        }

        /// <summary>
        /// Translates the XSLT parameter settings provided in the edit web part into Render parameters.
        /// </summary>
        /// <returns>Returns a list of RenderParameters Object <see cref="RenderParameters" /></returns>
        protected virtual RenderParameters TranslateXsltParameterSettingsIntoRenderParameters()
        {
            var renderParameters = new RenderParameters();

            if (string.IsNullOrEmpty(this.XsltParameterSettings))
            {
                return renderParameters;
            }

            List<ParameterSelection> selectedParametersFromEditorWebpart = SerializationHelper.Deserialize<List<ParameterSelection>>(this.XsltParameterSettings, CommerceServerContrib.CrossTier.Helpers.StringConstants.OutputTypeXmlDataContract);
            List<IParameterType> parameterTypes = PresentationServiceLocator.Current.GetImplementations<IParameterType>(true);
            foreach (ParameterSelection parameterSelection in selectedParametersFromEditorWebpart)
            {
                ParameterSelection selection = parameterSelection;
                var xsltParameter = parameterTypes.Find(x => x.ToString().Equals(selection.SetType) || x.GetType().AssemblyQualifiedName.Equals(selection.SetType));

                if (xsltParameter != null)
                {
                    var xsltParamValue = string.Empty;

                    try
                    {
                        xsltParamValue = xsltParameter.GetParameterValue(parameterSelection.ContextValue) as string;
                    }
                    catch (ParameterValueNotFoundException)
                    {
                    }

                    if (string.IsNullOrEmpty(xsltParamValue))
                    {
                        xsltParamValue = parameterSelection.DefaultValue;
                    }

                    renderParameters.Add(new RenderParameter(parameterSelection.Id, string.Empty, xsltParamValue), true);
                }
            }

            return renderParameters;
        }

        /// <summary>
        /// Returns a value indicating whether or not the web part is valid and has all the required parameters.
        /// </summary>
        /// <returns>A value indicating whether or not the web part is valid and has all the required parameters</returns>
        protected virtual bool IsValid()
        {
            return true;
        }

        /// <summary>
        /// Gets the validation rules.
        /// </summary>
        /// <returns>The request template validation rules</returns>
        protected virtual object GetValidationRules()
        {
            return new object();
        }

        #endregion

        #region Private Methods

        /// <summary>
        /// Checks whether or not the current user is authorized based on the AuthorizationRoleName
        /// </summary>
        /// <returns>True if authorized, false if not</returns>
        private bool IsAuthorized()
        {
            var evaluator = PresentationServiceLocator.Current.AuthorizationEvaluator;
            if (evaluator != null)
            {
                return evaluator.EvaluateAuthorization(this.AuthorizationRoleName);
            }

            return false;
        }

        /// <summary>
        /// Renders the request xml and the XSLT parameters including its values.
        /// </summary>
        /// <returns>The request XML and the XSLT parameters including its values.</returns>
        private string RenderRequestXml()
        {
            var requestXml = new StringBuilder();

            requestXml.AppendLine("<b>Xslt Parameters:</b>");
            requestXml.AppendLine("<br/>");
            requestXml.AppendLine("<br/>");

            var xsltParameters = this.GetRenderParameters();

            if (xsltParameters != null)
            {
                foreach (var parameter in xsltParameters)
                {
                    requestXml.AppendLine(parameter.RenderParameterKey + ": " + parameter.RenderParameterValue + "<br/>");
                }
            }

            requestXml.AppendLine("<br/>");
            requestXml.AppendLine("<b>Request XML:</b>");
            requestXml.AppendLine("<br/>");
            requestXml.AppendLine("<br/>");

            if (this._xmlMessage != null)
            {
                requestXml.Append(HttpUtility.HtmlEncode(this.XmlMessage.ToString()));
            }

            return requestXml.ToString();
        }

        /// <summary>
        /// Gets the cache context.
        /// </summary>
        /// <returns>the cache context</returns>
        private CacheContext GetCacheContext()
        {
            if (this._cacheContext == null)
            {
                this._cacheContext = PresentationCacheSettings.GetCacheContext(CommerceServerContrib.CrossTier.Helpers.CacheConstants.WebPartOutputCache);

                this._cacheContext.CacheKey = this.GetOutputCacheKey();

                if (!this._cacheContext.CacheExpiration.HasValue)
                {
                    this._cacheContext.CacheExpiration = this.CacheExpiration.Value;
                }
            }

            return this._cacheContext;
        }

        #endregion
    }
}
