﻿//---------------------------------------------------------------------
// <copyright file="Html.cs" company="commerceserver.net">
//     Copyright © commerceserver.net.  All rights reserved.
// </copyright>
// <summary></summary>
//---------------------------------------------------------------------
namespace CommerceServerContrib.Web
{
    using System;
    using System.Globalization;
    using System.Xml.XPath;
    using CommerceServerContrib.CrossTier.Entities;
    using CommerceServerContrib.CrossTier.Helpers;
    using CommerceServerContrib.CrossTier.Interfaces;
    using System.Web;
    using System.Text.RegularExpressions;
    using CommerceServerContrib.CrossTier.Exceptions;
    using Microsoft.Security.Application;
    using System.Collections.Generic;
    using CommerceServer.Foundation;

    /// <summary>
    /// XSLT Extension which holds multiple helpers which make working with XSLTs easier. 
    /// </summary>
    public class Html : IRenderExtensionObjectProvider
    {
        #region Fields and Constants

        private const string SchemeHostSeperator = "://";
        private const string AspxPageType = ".aspx";
        private readonly ICatalogImageManager _catalogImageManager = PresentationServiceLocator.Current.GetImplementation<ICatalogImageManager>();

        #endregion

        #region Public Properties

        /// <summary>
        /// Gets the name of the extension object.
        /// </summary>
        public string ExtensionObjectName
        {
            get { return "html"; }
        }

        #endregion

        /// <summary>
        /// Converts the given string to uppercase using the current culture.
        /// </summary>
        /// <param name="value">The string value.</param>
        /// <returns>The uppercase string.</returns>
        public string ToUpper(string value)
        {
            ParameterChecker.CheckForNull(value, "value");

            return value.ToUpper(CultureInfo.CurrentCulture);
        }

        /// <summary>
        /// Converts the given string to uppercase using the invariant culture.
        /// </summary>
        /// <param name="value">The value.</param>
        /// <returns>The uppercase string.</returns>
        public string ToInvariantUpper(string value)
        {
            ParameterChecker.CheckForNull(value, "value");

            return value.ToUpper(CultureInfo.InvariantCulture);
        }

        /// <summary>
        /// Converts the given string to lowercase using the current culture.
        /// </summary>
        /// <param name="value">The string value.</param>
        /// <returns>The lowercase string.</returns>
        public string ToLower(string value)
        {
            ParameterChecker.CheckForNull(value, "value");

            return value.ToLower(CultureInfo.CurrentCulture);
        }

        /// <summary>
        /// Converts the given string to lowercase using the invariant culture.
        /// </summary>
        /// <param name="value">The string value.</param>
        /// <returns>The lowercase string.</returns>
        public string ToInvariantLower(string value)
        {
            ParameterChecker.CheckForNull(value, "value");

            return value.ToLower(CultureInfo.InvariantCulture);
        }

        /// <summary>
        /// Does a case insensitive string compare using current culture.
        /// </summary>
        /// <param name="value1">The value1.</param>
        /// <param name="value2">The value2.</param>
        /// <returns><c>true</c> if the strings are equal; otherwise <c>false</c></returns>
        public bool StringInsensitiveEqual(string value1, string value2)
        {
            ParameterChecker.CheckForNull(value1, "value1");
            ParameterChecker.CheckForNull(value2, "value2");

            return value1.Equals(value2, StringComparison.CurrentCultureIgnoreCase);
        }

        /// <summary>
        /// Does a case insensitive string compare using the ordinal culture.
        /// </summary>
        /// <param name="value1">The first string.</param>
        /// <param name="value2">The second string.</param>
        /// <returns><c>true</c> if the strings are equal; otherwise <c>false</c></returns>
        public bool StringOrdinalIgnoreCaseEqual(string value1, string value2)
        {
            return value1.Equals(value2, StringComparison.OrdinalIgnoreCase);
        }

        /// <summary>
        /// Adjusts the for Http or Https depending on the current page mode
        /// </summary>
        /// <param name="url">The URL to adjust.</param>
        /// <returns>The url adjusted based on the current scheme</returns>
        public string AdjustUrlScheme(string url)
        {
            return CdnHelper.AdjustUrlScheme(url);
        }

        /// <summary>
        /// Sets the cookie.
        /// </summary>
        /// <param name="cookieName">Name of the cookie.</param>
        /// <param name="value">The value of the cookie.</param>
        public void SetCookie(string cookieName, string value)
        {
            UserCookieHelper.SetCookie(cookieName, value);
        }

        /// <summary>
        /// Returns CDNized path using the SiteCDN base URL.
        /// </summary>
        /// <param name="relativePath">The relative path used to create the image tag.</param>
        /// <returns>An image tag with CDN url.</returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1704:IdentifiersShouldBeSpelledCorrectly", MessageId = "Cdn", Justification = "Cdn is an acronym")]
        public string CdnSiteImagePath(string relativePath)
        {
            return CdnHelper.CreateSiteImageUrl(relativePath);
        }

        /// <summary>
        /// Creates a CDNized Image element using the Site base URL.
        /// </summary>
        /// <param name="relativePath">The relative path used to create the image tag.</param>
        /// <returns>An image tag with CDN url.</returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1704:IdentifiersShouldBeSpelledCorrectly", MessageId = "Cdn", Justification = "Cdn is an acronym")]
        public string CdnSiteImage(string relativePath)
        {
            return this.CdnSiteImage(relativePath, string.Empty);
        }

        /// <summary>
        /// Creates a CDNized image element using the Site base URL.
        /// </summary>
        /// <param name="relativePath">The relative path used to create the image tag.</param>
        /// <param name="attributes">Encoded string of attributes to add to the element.</param>
        /// <returns>An image tag with CDN url and attributes.</returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1704:IdentifiersShouldBeSpelledCorrectly", MessageId = "Cdn", Justification = "Cdn is an acronym")]
        public string CdnSiteImage(string relativePath, string attributes)
        {
            ParameterChecker.CheckForNullOrEmpty(relativePath, "relativePath");

            GenericHtmlElement image = new GenericHtmlElement("img");
            image.EncodedAttributeString = attributes;
            image.AddAttribute("src", CdnHelper.CreateSiteImageUrl(relativePath));

            return image.ToString();
        }

        /// <summary>
        /// Creates a CDNized Image element using the Catalog base URL.
        /// </summary>
        /// <param name="relativePath">The relative path used to create the image tag.</param>
        /// <returns>An image tag with CDN url.</returns>
        public string CdnCatalogImage(string relativePath)
        {
            return this.CdnCatalogImage(relativePath, string.Empty);
        }

        /// <summary>
        /// Returns CDNized path using the MarketingCDN base URL.
        /// </summary>
        /// <param name="htmlData">The full html source from the Marketing Ad.</param>
        /// <returns>The modified html with the img src replaced.</returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1704:IdentifiersShouldBeSpelledCorrectly", MessageId = "Cdn", Justification = "Cdn is an acronym")]
        public string CdnMarketingImagePath(string htmlData)
        {
            string path = CdnHelper.CreateMarketingImageUrl("/");
            return htmlData.Replace("src=\"", "src=\"" + path);
        }

        /// <summary>
        /// Creates a CDNized image element using the Catalog base URL.
        /// </summary>
        /// <param name="relativePath">The relative path used to create the image tag.</param>
        /// <param name="attributes">Encoded string of attributes to add to the element.</param>
        /// <returns>An image tag with CDN url and attributes.</returns>
        public string CdnCatalogImage(string relativePath, string attributes)
        {
            ParameterChecker.CheckForNullOrEmpty(relativePath, "relativePath");

            GenericHtmlElement image = new GenericHtmlElement("img");
            image.EncodedAttributeString = attributes;
            image.AddAttribute("src", CdnHelper.CreateCatalogImageUrl(relativePath));

            return image.ToString();
        }

        /// <summary>
        /// Creates a CDNized image element using the Marketing base URL.
        /// </summary>
        /// <param name="relativePath">The relative path used to create the CDNized image tag.</param>
        /// <returns>An image element with CDN url and attributes.</returns>
        public string CdnMarketingImage(string relativePath)
        {
            return this.CdnMarketingImage(relativePath, string.Empty);
        }

        /// <summary>
        /// Creates a CDNized image element using the Marketing base URL and specified attributes.
        /// </summary>
        /// <param name="relativePath">The relative path used to create the url.</param>
        /// <param name="attributes">An encoded string of attributes to be added to the image element.</param>
        /// <returns>An image element with CDN url.</returns>
        public string CdnMarketingImage(string relativePath, string attributes)
        {
            ParameterChecker.CheckForNullOrEmpty(relativePath, "Relative Path");

            GenericHtmlElement image = new GenericHtmlElement("img");
            image.EncodedAttributeString = attributes;

            image.AddAttribute("src", CdnHelper.CreateMarketingImageUrl(relativePath));

            return image.ToString();
        }

        /// <summary>
        /// Creates a CDN script element based on the "Script" area base url.
        /// </summary>
        /// <param name="relativePath">The relative path used to create the CDNized script element.</param>
        /// <returns>A script element with an absolute url pointing to the CDN.</returns>
        public string CdnScript(string relativePath)
        {
            ParameterChecker.CheckForNullOrEmpty(relativePath, "Relative Path");

            GenericHtmlElement script = new GenericHtmlElement("script");
            script.AddAttribute("type", "text/javascript");
            script.AddAttribute("src", CdnHelper.CreateScriptUrl(relativePath));
            return script.ToString();
        }

        /// <summary>
        /// Creates a CDN URL based on the relative url passed and the area passed.
        /// </summary>
        /// <param name="relativePath">The relative path used to construct the CDNized url.</param>
        /// <param name="area">The area used to look up the CDN base url.</param>
        /// <returns>An absolute URL using the base URL of the specified area.</returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1055:UriReturnValuesShouldNotBeStrings", Justification = "No need for URL, need a string.")]
        public string CdnUrl(string relativePath, string area)
        {
            ParameterChecker.CheckForNullOrEmpty(relativePath, "Path");
            ParameterChecker.CheckForNullOrEmpty(area, "area");

            return CdnHelper.CdnUrl(relativePath, area);
        }

        /// <summary>
        /// Find and format money within a string.
        /// </summary>
        /// <param name="value">The string to search within</param>
        /// <returns>The input string with all numbers formatted as money</returns>
        public string FindAndPrintMoney(string value)
        {
            var pattern = @"\d+(\.\d{1,2})?";
            var evaluator = new MatchEvaluator(this.Replacer);
            var response = Regex.Replace(value, pattern, evaluator);

            return response;
        }

        /// <summary>
        /// Replacers the specified match.
        /// </summary>
        /// <param name="match">The match found in the string.</param>
        /// <returns>The formatted string</returns>
        public string Replacer(Capture match)
        {
            var currency = this.PrintMoney(match.Value, PresentationServiceLocator.Current.SiteContext.CultureUICode);

            return currency;
        }

        /// <summary>
        /// Prints the money based on the current culture.
        /// </summary>
        /// <param name="value">The money value</param>
        /// <returns>The money value formatted correctly</returns>
        public string PrintMoney(string value)
        {
            return this.PrintMoney(value, PresentationServiceLocator.Current.SiteContext.CultureUICode);
        }

        /// <summary>
        /// Prints the money based on the current culture.
        /// </summary>
        /// <param name="value">The money value</param>
        /// <param name="cultureFormat">The culture format to format to.</param>
        /// <returns>
        /// The money value formatted correctly
        /// </returns>
        public string PrintMoney(string value, string cultureFormat)
        {
            return CommerceServerContrib.CrossTier.Money.PrintMoney(value, cultureFormat);
        }

        /// <summary>
        /// Gets the resources value.
        /// </summary>
        /// <param name="location">The location.</param>
        /// <param name="key">The key to search for.</param>
        /// <returns>
        /// The value associated to the key and the area.
        /// </returns>
        public string GetResource(string location, string key)
        {
            return ResourceHelper.Get(location, key);
        }

        /// <summary>
        /// Gets the first catalog image path.
        /// </summary>
        /// <param name="delimitedCatalogImageName">Name of the delimited catalog image.</param>
        /// <param name="imageSize">Size of the image.</param>
        /// <returns>The url of the first catalog image</returns>
        public string GetFirstCatalogImagePath(string delimitedCatalogImageName, string imageSize)
        {
            return this._catalogImageManager.GetFirstCatalogImagePath(delimitedCatalogImageName, imageSize);
        }

        /// <summary>
        /// Gets the alternative catalog image paths.
        /// </summary>
        /// <param name="delimitedCatalogImageName">Name of the delimited catalog image.</param>
        /// <param name="imageSize">Size of the image.</param>
        /// <returns>The a XPathNodeIterator with a node per alternative image </returns>
        public XPathNodeIterator GetAlternativeCatalogImagePaths(string delimitedCatalogImageName, string imageSize)
        {
            return this._catalogImageManager.GetAlternativeCatalogImagePaths(delimitedCatalogImageName, imageSize);
        }

        /// <summary>
        /// Gets the alternative catalog image paths.
        /// </summary>
        /// <param name="delimitedCatalogImageName">Name of the delimited catalog image.</param>
        /// <param name="imageSize">Size of the image.</param>
        /// <returns>The a List[string] with a node per alternative image </returns>
        public List<string> GetAlternativeCatalogImagePathsNoXml(string delimitedCatalogImageName, string imageSize)
        {
            return this._catalogImageManager.GetAlternativeCatalogImagePathsNoXml(delimitedCatalogImageName, imageSize);
        }

        /// <summary>
        /// Sets the URL value.
        /// </summary>
        /// <param name="key">The query string key.</param>
        /// <param name="value">The query string value.</param>
        /// <returns>The current url with the specified key set</returns>
        public string SetAndReturnNewUrl(string key, string value)
        {
            UrlBuilder currentUrl = UrlBuilder.CurrentUrl;
            currentUrl.QueryList.AddOrSet(key, value);
            return currentUrl.ToString();
        }

        /// <summary>
        /// Sets the and return new URL.
        /// </summary>
        /// <param name="url">The URL to modify.</param>
        /// <param name="key">The key to set.</param>
        /// <param name="value">The value to set.</param>
        /// <returns>The url specified with the specified key set</returns>
        public string SetAndReturnNewUrl(string url, string key, string value)
        {
            var urlBuilder = new UrlBuilder(url);
            urlBuilder.QueryList.AddOrSet(key, value);
            return urlBuilder.ToString();
        }

        /// <summary>
        /// Validates if the current user falls within this webpart's required role
        /// </summary>
        /// <param name="requiredRole">The required role to determine if user is authorized or not</param>
        /// <returns>True if authorized, false if not</returns>
        public bool IsAuthorized(string requiredRole)
        {
            var evaluator = PresentationServiceLocator.Current.AuthorizationEvaluator;
            return evaluator.EvaluateAuthorization(requiredRole);
        }

        /// <summary>
        /// Gets the current date.
        /// </summary>
        /// <returns>The current date in the user ui culture</returns>
        public string CurrentDate()
        {
            return this.CurrentDate("D");
        }

        /// <summary>
        /// Currents the date.
        /// </summary>
        /// <param name="format">The format.</param>
        /// <returns>The current date in the user ui culture with the specified format</returns>
        public string CurrentDate(string format)
        {
            return DateTime.Today.Date.ToString(format, PresentationServiceLocator.Current.SiteContext.UICulture);
        }

        /// <summary>
        /// Currents the now.
        /// </summary>
        /// <returns>The current date and time in the user ui culture with sortable date/time format</returns>
        public string CurrentNow()
        {
            return this.CurrentNow("s");
        }

        /// <summary>
        /// Currents the now.
        /// </summary>
        /// <param name="format">The format.</param>
        /// <returns>The current date and time in the user ui culture with specified format</returns>
        public string CurrentNow(string format)
        {
            return DateTime.Now.ToString(format, PresentationServiceLocator.Current.SiteContext.UICulture);
        }

        /// <summary>
        /// Date the earlier than now.
        /// </summary>
        /// <param name="date">The date to compare with.</param>
        /// <returns>Returns true if the specified date\time is earlier than now, otherwise false</returns>
        public bool IsDateEarlierThanNow(string date)
        {
            var siteContext = PresentationServiceLocator.Current.SiteContext;

            return Convert.ToDateTime(date, siteContext.UICulture).CompareTo(siteContext.CurrentDateTime) < 0;
        }

        /// <summary>
        /// Formats the string.
        /// </summary>
        /// <param name="value">The string to format.</param>
        /// <param name="arg">The argument to format with.</param>
        /// <returns>The stringToFormat in the user ui culture</returns>
        public string FormatString(string value, string arg)
        {
            return string.Format(CultureInfo.InvariantCulture, value, arg);
        }

        /// <summary>
        /// Formats the date.
        /// </summary>
        /// <param name="dateToFormat">The date to format.</param>
        /// <returns>The dateToFormat in the user ui culture</returns>
        public string FormatDate(string dateToFormat)
        {
            return this.FormatDate(dateToFormat, "D");
        }

        /// <summary>
        /// Formats the date.
        /// </summary>
        /// <param name="dateToFormat">The date to format.</param>
        /// <param name="format">The format.</param>
        /// <returns>he dateToFormat in the user ui culture</returns>
        public string FormatDate(string dateToFormat, string format)
        {
            CultureInfo currentUiCulture = PresentationServiceLocator.Current.SiteContext.UICulture;
            DateTime dateFormatted = Convert.ToDateTime(dateToFormat, currentUiCulture);
            return dateFormatted.ToString(format, currentUiCulture);
        }

        /// <summary>
        /// Gets the date format.
        /// </summary>
        /// <param name="longFormat">if set to <c>true</c> [long format].</param>
        /// <returns>The current ui culture's date format</returns>
        public string GetDateFormat(bool longFormat)
        {
            var dateFormat = PresentationServiceLocator.Current.SiteContext.UICulture.DateTimeFormat;
            return longFormat ? dateFormat.LongDatePattern : dateFormat.ShortDatePattern;
        }

        /// <summary>
        /// Gets the current year.
        /// </summary>
        /// <param name="numberOfYears">The number of years to add to the current year.</param>
        /// <returns>The current year in the user ui culture</returns>
        public string CurrentYear(int numberOfYears)
        {
            return DateTime.Now.AddYears(numberOfYears).Year.ToString("0000", PresentationServiceLocator.Current.SiteContext.UICulture);
        }

        /// <summary>
        /// Returns a new GUID
        /// </summary>
        /// <param name="format">The format of the GUID.</param>
        /// <returns>A new GUID.</returns>
        public string NewGuid(string format)
        {
            return System.Guid.NewGuid().ToString(format);
        }

        /// <summary>
        /// Integers to string.
        /// </summary>
        /// <param name="number">The number.</param>
        /// <returns>The string representation in the current ui culture of the number</returns>
        public string IntegerToString(string number)
        {
            int integer;
            return Int32.TryParse(number, NumberStyles.Number, PresentationServiceLocator.Current.SiteContext.UICulture, out integer) ? integer.ToString(PresentationServiceLocator.Current.SiteContext.UICulture) : number;
        }

        /// <summary>
        /// Ensures the checkout authentication.
        /// </summary>
        /// <param name="isBasketEmpty">Must be set to <c>true</c> if the cart is empty.</param>
        /// <param name="shippingSpecified">Must be set to <c>true</c> if the cart shipping information has been specified.</param>
        /// <param name="billingSpecified">Must be set to <c>true</c> if the cart billing information has been specified.</param>
        public virtual void EnsureCheckoutAuthentication(bool isBasketEmpty, bool shippingSpecified, bool billingSpecified)
        {
            var siteSettingsProvider = PresentationServiceLocator.Current.SiteSettingProvider;
            var siteContext = PresentationServiceLocator.Current.SiteContext;
            var cartPageUrl = siteSettingsProvider.GetFromWebSetting("CartPageUrl");

            if (isBasketEmpty && !shippingSpecified && !billingSpecified)
            {
                siteContext.RedirectTo(cartPageUrl, true);
            }

            bool isUserAuthenticated = siteContext.IsUserAuthenticated;

            bool isGuestCheckout = false;
            var isGuestCheckoutCookieName = string.Concat(PresentationServiceLocator.Current.BrandContext.Name, "_isGuestCheckoutCookieName");
            var isGuestCheckoutCookie = siteContext.CurrentContext.Request.Cookies[isGuestCheckoutCookieName];
            if (isGuestCheckoutCookie != null)
            {
                isGuestCheckout = Boolean.Parse(isGuestCheckoutCookie.Value);
            }

            var fromCheckoutKey = PresentationServiceLocator.Current.GetImplementation<IQueryStringProvider>().GetQueryString("FromCheckoutUrlKey");
            string signInPageUrl = siteSettingsProvider.GetFromWebSetting("SignInPageUrl");

            if (isUserAuthenticated || isGuestCheckout)
            {
                var isFromValidPage = false;

                if (siteContext.CurrentContext.Request.QueryString[fromCheckoutKey.QueryParameterKey] != null)
                {
                    isFromValidPage = Convert.ToBoolean(siteContext.CurrentContext.Request.QueryString[fromCheckoutKey.QueryParameterKey], CultureInfo.InvariantCulture);
                }

                string previousPageUrl = siteContext.CurrentContext.Request.UrlReferrer != null ? siteContext.CurrentContext.Request.UrlReferrer.AbsolutePath : string.Empty;
                
                if (!string.IsNullOrEmpty(previousPageUrl) && !previousPageUrl.EndsWith(AspxPageType, StringComparison.OrdinalIgnoreCase))
                {
                    previousPageUrl = siteSettingsProvider.ConvertFriendlyToPhysicalUrl(previousPageUrl);
                }

                if (!string.IsNullOrEmpty(previousPageUrl))
                {
                    var steps = siteSettingsProvider.GetSiteSettings("CheckoutSteps");
                    string createAccountPageUrl = siteSettingsProvider.GetFromWebSetting("CreateAccountPageUrl");

                    isFromValidPage = (steps.Find(ssr => ssr.Value.Equals(previousPageUrl, StringComparison.OrdinalIgnoreCase)) != null) ||
                                       (previousPageUrl.Equals(cartPageUrl, StringComparison.OrdinalIgnoreCase)) ||
                                       (previousPageUrl.Equals(createAccountPageUrl, StringComparison.OrdinalIgnoreCase)) ||
                                       (previousPageUrl.Equals(signInPageUrl, StringComparison.OrdinalIgnoreCase));
                }

                if (!isFromValidPage)
                {
                    siteContext.RedirectTo(cartPageUrl, true);
                }

                return;
            }

            string firstStepUrl = siteSettingsProvider.GetFromWebSetting("Step1");
            var fromCheckout = string.Concat(fromCheckoutKey.QueryParameterKey, "=true");
            var returnUrl = string.Concat("ReturnUrl=", firstStepUrl);
            string redirectPageUrl = string.Concat(signInPageUrl, "?", returnUrl, "&", fromCheckout);
            siteContext.RedirectTo(redirectPageUrl, true);
        }

        /// <summary>
        /// Gets the UI culture.
        /// </summary>
        /// <returns>The name of the current UI culture</returns>
        public string GetUiCulture()
        {
            return PresentationServiceLocator.Current.SiteContext.UICulture.Name;
        }

        /// <summary>
        /// Switches the URL host.
        /// </summary>
        /// <param name="url">The URL to change.</param>
        /// <param name="newHost">The new host.</param>
        /// <returns>The url with the host replaced</returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1055:UriReturnValuesShouldNotBeStrings", Justification = "We use urls as strings throughout the product."), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1054:UriParametersShouldNotBeStrings", Justification = "We use urls as strings throughout the product.", MessageId = "0#")]
        public string SwitchUrlHost(string url, string newHost)
        {
            if (url.IndexOf(SchemeHostSeperator, StringComparison.OrdinalIgnoreCase) > -1)
            {
                var newUri = new Uri(url, UriKind.RelativeOrAbsolute);

                var redirectHost = string.Concat(newUri.Scheme, SchemeHostSeperator, newUri.Authority);

                if (newUri.Port != 80 && newUri.Port != 443)
                {
                    redirectHost = string.Concat(redirectHost, ":", newUri.Port);
                }

                url = url.Replace(redirectHost, newHost.TrimEnd('/'));
            }

            return url;
        }

        /// <summary>
        /// Getsthe name of the page.
        /// </summary>
        /// <returns>The name of the page.</returns>
        public string CurrentPageName()
        {
            return UrlBuilder.CurrentUrl.PageName;
        }

        /// <summary>
        /// Gets url domain.
        /// </summary>
        /// <returns>The url domain.</returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1055:UriReturnValuesShouldNotBeStrings", Justification = "We use urls as strings throughout the product.")]
        public string GetUrlDomain()
        {
            return UrlBuilder.CurrentUrl.UrlDomain;
        }

        /// <summary>
        /// Redirects to url.
        /// </summary>        
        /// <param name="url">The URL to redirect to.</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1054:UriParametersShouldNotBeStrings", Justification = "We use urls as strings throughout the product.", MessageId = "0#")]
        public void Redirect(string url)
        {
            PresentationServiceLocator.Current.SiteContext.RedirectTo(url, true);
        }

        /// <summary>
        /// Redirects to the application 404 page.
        /// </summary>
        public void Redirect404()
        {
            PresentationServiceLocator.Current.SiteContext.RedirectTo404();
        }

        /// <summary>
        /// Gets base url.
        /// </summary>
        /// <returns>The base url.</returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1055:UriReturnValuesShouldNotBeStrings", Justification = "We use urls as strings throughout the product.")]
        public string GetBaseUrl()
        {
            return UrlBuilder.CurrentUrl.ToString().Replace(UrlBuilder.CurrentUrl.QueryList.ToString(), string.Empty);
        }

        /// <summary>
        /// Gets the current url.
        /// </summary>
        /// <returns>The query list from url.</returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1055:UriReturnValuesShouldNotBeStrings", Justification = "We use urls as strings throughout the product.")]
        public string GetCurrentUrl()
        {
            return UrlBuilder.CurrentUrl.ToString();
        }

        /// <summary>
        /// Gets query list from url.
        /// </summary>
        /// <returns>The query list from url.</returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1055:UriReturnValuesShouldNotBeStrings", Justification = "We use urls as strings throughout the product.")]
        public string GetQueryListFromUrl()
        {
            return UrlBuilder.CurrentUrl.QueryList.ToString();
        }

        /// <summary>
        /// Encodes a value for safe use as a JavaScript string.
        /// </summary>
        /// <param name="value">The value to encode.</param>
        /// <returns>The encoded string value.</returns>
        public string JavaScriptEncode(string value)
        {
            return Encoder.JavaScriptEncode(value, false);
        }

        /// <summary>
        /// UrlEncodes the given value.
        /// </summary>
        /// <param name="value">The value.</param>
        /// <returns>Encoded value.</returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1055:UriReturnValuesShouldNotBeStrings", Justification = "xslt extension.  Must return a string.")]
        public string UrlEncode(string value)
        {
            return Encoder.UrlEncode(value);
        }

        /// <summary>
        /// Encodes a value for safe use as HTML text.
        /// </summary>
        /// <param name="value">The value to encode.</param>
        /// <returns>The encoded string value.</returns>
        public string HtmlEncode(string value)
        {
            return Encoder.HtmlEncode(value);
        }

        /// <summary>
        /// Encodes a value for safe use as HTML attribute text.
        /// </summary>
        /// <param name="value">The value to encode.</param>
        /// <returns>The encoded string value.</returns>
        public string HtmlAttributeEncode(string value)
        {
            return Encoder.HtmlAttributeEncode(value);
        }

        /// <summary>
        /// Encodes a value for save use as a JavaScript string.
        /// </summary>
        /// <param name="value">The value to encode.</param>
        /// <returns>The encoded string value.</returns>
        public string GetSafeHtmlFragment(string value)
        {
            return Sanitizer.GetSafeHtmlFragment(value);
        }

        /// <summary>
        /// Ratings to image key.
        /// </summary>
        /// <param name="rating">The rating.</param>
        /// <returns>The image that represents the rating</returns>
        public string RatingToImageKey(string rating)
        {
            if (string.IsNullOrEmpty(rating))
            {
                return string.Empty;
            }

            try
            {
                var ratingDecimal = Convert.ToDecimal(rating, CultureInfo.InvariantCulture);
                var tens = (int)ratingDecimal;
                var ones = (ratingDecimal - ((decimal)tens)) < 0.5M ? 0 : 5;

                return string.Concat(tens, ones);
            }
            catch (FormatException ex)
            {
                CommerceTrace.Warning(CommerceClientTraceSource.PresentationTier, "Html.RatingToImageKey: FormatException while converting \"{0}\" to a decimal value: {1}", rating, ex);
                return string.Empty;
            }
            catch (OverflowException ex)
            {
                CommerceTrace.Warning(CommerceClientTraceSource.PresentationTier, "Html.RatingToImageKey: OverflowException while converting \"{0}\" to a decimal value: {1}", rating, ex);
                return string.Empty;
            }
        }

        #region Implementation of IRenderExtensionObjectProvider

        /// <summary>
        /// Gets the XSLT extension objects <see cref="CommerceServerContrib.CrossTier.Entities.RenderExtensionObjects"/>
        /// </summary>
        /// <returns>
        /// XsltExtensionObjects which is a collection of XsltExtensionObject <see cref="RenderExtensionObject"/>
        /// </returns>
        public RenderExtensionObjects GetRenderExtensionObjects()
        {
            var extensionObjects = new RenderExtensionObjects();
            extensionObjects.Add(new RenderExtensionObject(this.ExtensionObjectName, this));

            return extensionObjects;
        }

        #endregion
    }
}
