﻿//---------------------------------------------------------------------
// <copyright file="HtmlElement.cs" company="commerceserver.net">
//     Copyright © commerceserver.net.  All rights reserved.
// </copyright>
// <summary></summary>
//---------------------------------------------------------------------

namespace CommerceServerContrib.Web
{
    using System;
    using System.Collections.Generic;
    using System.Text;
    using System.Text.RegularExpressions;
    using System.Web;
    using CommerceServerContrib.CrossTier.Helpers;
    using CommerceServerContrib.CrossTier.Interfaces;
    using CommerceServer.Foundation;

    /// <summary>
    /// Is used to build an HTML element from, including th validation attributes etc.
    /// </summary>
    public abstract class HtmlElement
    {
        #region Fields and constants
        private bool selfClose = true;
        private string tag;
        private Dictionary<string, string> attributes = new Dictionary<string, string>();
        #endregion

        /// <summary>
        /// Initializes a new instance of the <see cref="HtmlElement"/> class.
        /// </summary>
        /// <param name="tag">The tag of the element, "input" for example.</param>
        protected HtmlElement(string tag)
        {
            ParameterChecker.CheckForNullOrEmpty(tag, "tag");

            this.Tag = tag;
        }

        /// <summary>
        /// Gets or sets the id attribute value.
        /// </summary>
        /// <value>
        /// The id attribute value.
        /// </value>
        public string Id
        {
            get
            {
                string returnValue;
                if (this.attributes.TryGetValue("id", out returnValue))
                {
                    return returnValue;
                }

                return string.Empty;
            }

            set
            {
                this.attributes["id"] = value;
            }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the self close property.  Default true.  If true the element is closed with />.  If false it is closed with > and it is the responsibility of the developer to close the element.  Handy for making selects.
        /// </summary>
        /// <value>
        /// Set to <c>true</c> to close the element with />; <c>false</c> to close the element with >.
        /// </value>
        public bool SelfCloseElement
        {
            get
            {
                return this.selfClose;
            }

            set
            {
                this.selfClose = value;
            }
        }

        /// <summary>
        /// Gets or sets the HTML attribute encoded string containing attributes most likely passed from the XSLT helpers.
        /// </summary>
        /// <value>
        /// The html attribute encoded string of attributes.
        /// </value>
        public string EncodedAttributeString { get; set; }

        /// <summary>
        /// Gets or sets the tag - something like input, select, etc..
        /// </summary>
        /// <value>
        /// The tag value of the tag.
        /// </value>
        protected string Tag
        {
            get
            {
                return this.tag;
            }

            set
            {
                value = value.Trim();
                if (!Regex.IsMatch(value, "^[0-9a-zA-Z]+$"))
                {
                    throw new FormatException(ResourceHelper.GetCommerceServerContrib("InvalidTagFormat"));
                }

                this.tag = value;
            }
        }

        private string Name
        {
            get
            {
                return this.Id;
            }
        }

        /// <summary>
        /// A helper method which is used to create a closing tag.
        /// </summary>
        /// <param name="tag">The closing tag of the type passed.</param>
        /// <returns>String that represents the closing tag.</returns>
        public static string ClosingElement(string tag)
        {
            ParameterChecker.CheckForNullOrEmpty(tag, "tag");
            return string.Join(string.Empty, new string[] { "</", tag, ">" });
        }

        /// <summary>
        /// Adds the attribute to the collection of attributes which will be rendered on the element.  If an element of the same key already exist it will be overwritten.  
        /// </summary>
        /// <param name="key">The key of the HTML element.</param>
        /// <param name="value">The value of the HTML element.</param>
        public virtual void AddAttribute(string key, string value)
        {
            ParameterChecker.CheckForNullOrEmpty(key, "key");
            key = key.Trim();
            if (!Regex.IsMatch(key, "^[0-9a-zA-Z]+$"))
            {
                throw new FormatException(ResourceHelper.GetCommerceServerContrib("InvalidAttributeFormat"));
            }

            this.attributes[key] = value;
        }

        /// <summary>
        /// Gets the attribute value by key.
        /// </summary>
        /// <param name="key">The key or the html element requested.</param>
        /// <returns>The string value of the attribute requested.</returns>
        public virtual string GetAttributeValue(string key)
        {
            ParameterChecker.CheckForNullOrEmpty(key, "key");
            string returnValue = string.Empty;
            if (this.attributes.TryGetValue(key, out returnValue))
            {
                return returnValue;
            }

            return string.Empty;
        }

        /// <summary>
        /// Returns a <see cref="System.String"/> that represents this instance.  Used to render the element.
        /// </summary>
        /// <returns>
        /// A <see cref="System.String"/> that represents this instance.
        /// </returns>
        public override string ToString()
        {
            StringBuilder sb = new StringBuilder();
            sb.Append("<").Append(this.Tag);
            this.AddAttributesToElementDuringRender(sb);

            if (this.selfClose)
            {
                sb.Append("/>");
            }
            else
            {
                sb.Append(">");
            }

            return sb.ToString();
        }

        /// <summary>
        /// Adds the attributes to the element - id, name, validation, encoded passed string, etc..
        /// </summary>
        /// <param name="existingHtml">The string builder used to build the element.</param>
        protected virtual void AddAttributesToElementDuringRender(StringBuilder existingHtml)
        {
            ParameterChecker.CheckForNull(existingHtml, "StringBuilder");

            foreach (var item in this.attributes)
            {
                existingHtml.Append(" ").Append(item.Key).Append("=\"").Append(HttpUtility.HtmlAttributeEncode(item.Value)).Append('"');
            }

            if (!string.IsNullOrEmpty(this.Name))
            {
                existingHtml.Append(" name=\"").Append(this.Id).Append('"');
            }

            if (!string.IsNullOrEmpty(this.EncodedAttributeString))
            {
                existingHtml.Append(" ").Append(HttpUtility.HtmlDecode(this.EncodedAttributeString));
            }

            this.AddValidationAttributesToElementDuringRender(existingHtml);
        }

        /// <summary>
        /// Adds the validation attributes.
        /// </summary>
        /// <param name="existingHtml">The string builder used to create the element.</param>
        protected virtual void AddValidationAttributesToElementDuringRender(StringBuilder existingHtml)
        {
            ParameterChecker.CheckForNull(existingHtml, "StringBuilder");
        }
    }
}
