﻿//-----------------------------------------------------------------------
// <copyright file="BasePage.cs" company="commerceserver.net">
//     Copyright © commerceserver.net.  All rights reserved.
// </copyright>
//-----------------------------------------------------------------------
namespace CommerceServerContrib.WebFormsSiteTemplate.PageBase
{
    using CommerceServer.Foundation;
    using CommerceServerContrib.CrossTier.Interfaces;
    using CommerceServerContrib.CrossTier.Interfaces.Controllers;
    using CommerceServerContrib.Web;
    using CommerceServerContrib.WebFormsSiteTemplate.Interfaces;
    using CommerceServerContrib.WebFormsSiteTemplate.Utils;
    using System.Collections.Generic;
    using System.Configuration;
    using System.Globalization;
    using System.Web.Mvc;

    /// <summary>
    /// The base classf or all pages in the project
    /// </summary>
    /// <typeparam name="T">Represents the CommerceEntity used bu the page</typeparam>
    public class BasePage<T> : System.Web.UI.Page, IBasePage
                where T : class 
    {
        /// <summary>
        /// The value to tell the app tier to recalculate the basket
        /// </summary>
        public const string DoRefreshBasket = "true";

        private ICommerceContext _context;
        private string _userId = string.Empty; 
        private DynamicViewDataDictionary _dynamicViewData = null;
        private ViewDataDictionary _viewData;
        private IBaseController _controller = null;

        /// <summary>
        /// Initializes a new instance of the <see cref="BasePage{T}" /> class.
        /// </summary>
        public BasePage()
        {
            this._context = PresentationServiceLocator.Current.SiteContext;
            this._userId = string.IsNullOrWhiteSpace(this._context.User.Id) ? this._context.User.AnonymousId : this._context.User.Id;
        }

        /// <summary>
        /// Gets the view bag used by the page
        /// </summary>
        public dynamic ViewBag
        {
            get
            {
                if (_dynamicViewData == null)
                {
                    _dynamicViewData = new DynamicViewDataDictionary(() => ViewData);
                }

                return _dynamicViewData;
            }
        }

        /// <summary>
        /// Gets or sets the view bag used by the page
        /// </summary>
        public IBaseController Controller
        {
            get
            {
                return this._controller;
            }

            set
            {
                this._controller = value;
            }
        }

        /// <summary>
        /// Gets or sets the view data for the page
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Usage", "CA2227:CollectionPropertiesShouldBeReadOnly", Justification = "By design.")]
        public ViewDataDictionary ViewData
        {
            get
            {
                if (this._viewData == null)
                {
                    this.SetViewData(new ViewDataDictionary());
                }

                return this._viewData;
            }

            set
            {
                this.SetViewData(value);
            }
        }

        /// <summary>
        /// Gets or sets the  model for the page
        /// </summary>
        public T Model
        {
            get;
            set;
        }

        /// <summary>
        /// Gets the default catalog for the site
        /// </summary>
        public string DefaultCatalog
        {
            get
            {
                return ConfigurationManager.AppSettings["DefaultCommerceCatalog"];
            }
        }

        /// <summary>
        /// Gets the current context for the site
        /// </summary>
        public ICommerceContext CommerceContext
        {
            get
            {
                return this._context;
            }
        }

        /// <summary>
        /// Gets the current user id
        /// </summary>
        public string UserId
        {
            get
            {
                return this._userId;
            }
        }

        /// <summary>
        /// Gets the id of a cart basket type
        /// </summary>
        public string CartBasketType
        {
            get
            {
                return this.CommerceContext.DefaultBasketType.ToString(CultureInfo.InvariantCulture);
            }
        }

        /// <summary>
        /// Gets a key from the General resource
        /// </summary>
        /// <param name="key">The key to return</param>
        /// <returns>The value associated to the key</returns>
        public string Resource(string key)
        {
            return Resource("General", key);
        }

        /// <summary>
        /// Gets a key from the General resource
        /// </summary>
        /// <param name="source">The file to pull the key from</param>
        /// <param name="key">The key to return</param>        
        /// <returns>The value associated to the key</returns>
        public string Resource(string source, string key)
        {
            return (string)GetGlobalResourceObject(source, key);
        }

        /// <summary>
        /// Used to set the view data
        /// </summary>
        /// <param name="viewData">The view data object to set</param>
        protected virtual void SetViewData(ViewDataDictionary viewData)
        {
            this._viewData = viewData;
        }

        /// <summary>
        /// Executes the queued call
        /// </summary>
        /// <returns>The response value as a list of commerce entity</returns>
        protected virtual List<CommerceEntity> GetQueuedResponse()
        {
            if (this.Controller != null)
            {
                var response = this.Controller.ProcessRequest();
                var entities = response.Value as List<CommerceEntity>;

                return entities;
            }

            return null;
        }
    }
}