﻿//-----------------------------------------------------------------------
// <copyright file="Cart.aspx.cs" company="commerceserver.net">
//     Copyright © commerceserver.net.  All rights reserved.
// </copyright>
//-----------------------------------------------------------------------
namespace CommerceServerContrib.WebFormsSiteTemplate.Pages
{
    using System;
    using System.Globalization;
    using System.IO;
    using System.Linq;
    using System.Web;
    using System.Web.Services;
    using System.Web.UI;
    using CommerceServerContrib.Web;
    using CommerceServerContrib.WebFormsSiteTemplate.Interfaces;
    using CommerceServerContrib.WebFormsSiteTemplate.Models;
    using CommerceServerContrib.WebFormsSiteTemplate.Models.Generated;
    using CommerceServerContrib.WebFormsSiteTemplate.PageBase;
    using CommerceServerContrib.WebFormsSiteTemplate.RequestTemplates;
    using Generated = CommerceServerContrib.WebFormsSiteTemplate.Models.Generated;

    /// <summary>
    /// Handles all actions on the cart page
    /// </summary>
    public partial class Cart : BasePage<BasketViewModel>
    {
        /// <summary>
        /// Used to refresh the mini cart via AJAX
        /// </summary>
        /// <returns>A rendered version of the mini cart</returns>
        [WebMethod]
        public static string MiniCart()
        {
            try
            {
                return RenderControl("controls/cart/minicart.ascx");
            }
            catch (Exception ex)
            {
                CommerceServerContrib.Web.CommerceServerContribExceptionManager.HandleException(ex);
                return string.Empty;
            }
        }

        /// <summary>
        /// Used to add an item to cart via AJAX
        /// </summary>
        /// <param name="productId">The id of the product to add</param>
        /// <param name="variantId">The id of the products variant</param>
        /// <param name="quantity">The quantity of items to add</param>
        /// <returns>A boolean value indicating whether or not the add to cart succeeded</returns>
        [WebMethod]
        public static bool AddToCart(string productId, string variantId, int quantity)
        {
            try
            {
                var cartPage = new Cart();
                var basket = (Generated.Basket)Orders.CreateLineItemWithPagedLineItems(
                    cartPage.CommerceContext.DefaultBasketName, 
                    cartPage.UserId, 
                    cartPage.CartBasketType, 
                    DoRefreshBasket, 
                    cartPage.DefaultCatalog, 
                    productId, 
                    variantId, 
                    quantity.ToString(CultureInfo.InvariantCulture),
                    "0",
                    "10",
                    "true").FirstOrDefault();
                BasketCachingHelper.AddBasketToCache(basket);
                return true;
            }
            catch (Exception ex)
            {
                CommerceServerContrib.Web.CommerceServerContribExceptionManager.HandleException(ex);
                return false;
            }
        }

        /// <summary>
        /// Removes an item from the basket
        /// </summary>
        /// <param name="lineItemId">The id of the line item to remove from the basket</param>
        /// <returns>A rendered cart control</returns>
        [WebMethod]
        public static string DeleteLineItem(string lineItemId)
        {
            try
            {
                var cartPage = new Cart();
                var basket = (Basket)Orders.DeleteLineItem(cartPage.CommerceContext.DefaultBasketName, cartPage.UserId, cartPage.CartBasketType, DoRefreshBasket, lineItemId)[0];
                BasketCachingHelper.AddBasketToCache(basket);
                var cart = ViewModelBuilder.CreateBasketViewModel(basket);

                return RenderControl<BasketViewModel>("controls/cart/cart.ascx", cart);
            }
            catch (Exception ex)
            {
                CommerceServerContribExceptionManager.HandleException(ex);
            }

            return null;
        }

        /// <summary>
        /// Updates a line item in the basket
        /// </summary>
        /// <param name="lineItemId">The id of the line item to update</param>
        /// <param name="quantity">The quantity to update the lineitem to</param>
        /// <returns>A rendered cart page</returns>
        [WebMethod]
        public static string UpdateLineItem(string lineItemId, int quantity)
        {
            try
            {
                var cartPage = new Cart();
                var basket = (Basket)Orders.UpdateLineItem(cartPage.CommerceContext.DefaultBasketName, cartPage.UserId, cartPage.CartBasketType, DoRefreshBasket, lineItemId, quantity.ToString(CultureInfo.InvariantCulture))[0];
                BasketCachingHelper.AddBasketToCache(basket);
                var cart = ViewModelBuilder.CreateBasketViewModel(basket);

                return RenderControl<BasketViewModel>("controls/cart/cart.ascx", cart);
            }
            catch (Exception ex)
            {
                CommerceServerContribExceptionManager.HandleException(ex);
            }

            return null;
        }

        private static string RenderControl(string path)
        {
            return RenderControl<object>(path, null);
        }

        private static string RenderControl<T>(string path, T model)
        {
            using (var page = new BasePage<BasketViewModel>())
            {
                var userControl = (UserControl)page.LoadControl(path);

                using (var writer = new StringWriter(CultureInfo.InvariantCulture))
                {
                    page.Controls.Add(userControl);

                    if (model != null)
                    {
                        ((IBaseControl<T>)userControl).Model = model; 
                    }

                    HttpContext.Current.Server.Execute(page, writer, false);
                    return writer.ToString();
                }
            }
        }
    }
}